/*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

function parallax_init()
{
        if (! window.WebGLRenderingContext) return;

        var list   = document.getElementsByTagName("*");
        var object = null;
        var data   = "";
        var c      = list.length;

        for (var i = 0; i < c; i++)
        {
                object = list.item(i);

                if (object.hasAttribute("data-parallax-distance"))
                {
                        data = object.getAttribute("data-parallax-distance");
                        if (data.match(/^-?\d+$/)) parallax_add(object, parseInt(data));
                };
        };

        parallax_initialize();
};

var parallax_list = new Array();

function parallax_add(object,
                      distance)
{
        parallax_list.push(new Array(object, distance / 100, 0, 0, 1));
};

function parallax_process(i)
{
        var object          = parallax_list[i][0];
        var distance        = parallax_list[i][1];

        var x               = fx_left(object);
        var y               = fx_top(object);
        var size_x          = fx_width(object);
        var size_y          = fx_height(object);

        var root            = document.documentElement;
        var scroll_x        = fx_width(root)  - fx_window_width;
        var scroll_y        = fx_height(root) - fx_window_height;

        parallax_list[i][2] = 0;
        parallax_list[i][3] = 0;
        var scale_x         = 0;
        var scale_y         = 0;

        if (scroll_x > 0)
        {
                var offset_x        = Math.min(scroll_x - x, 0);
                var _size_x         = Math.min(size_x + x, fx_window_width);
                parallax_list[i][2] = x + (offset_x + size_x - _size_y) / 2;
                scale_x             = 1 / size_x * (offset_x - size_x + _size_y) * distance;
        };

        if (scroll_y > 0)
        {
                var offset_y        = Math.min(scroll_y - y, 0);
                var _size_y         = Math.min(size_y + y, fx_window_height);
                parallax_list[i][3] = y + (offset_y + size_y - _size_y) / 2;
                scale_y             = 1 / size_y * (offset_y - size_y + _size_y) * distance;
        };

        var scale = 1 + ((scale_x > scale_y) ? scale_x : scale_y);

        parallax_list[i][4] = scale;
};

function parallax_event()
{
        var object     = null;
        var distance   = 0;
        var object_x   = 0;
        var object_y   = 0;
        var scale      = 1;
        var distance_x = 0;
        var distance_y = 0;
        var screen_x   = fx_page_left();
        var screen_y   = fx_page_top();
        var i          = 0;

        for (i = 0; i < parallax_list.length; i++)
        {
                parallax_process(i);

                object     = parallax_list[i][0];
                distance   = parallax_list[i][1];
                object_x   = parallax_list[i][2];
                object_y   = parallax_list[i][3];
                scale      = parallax_list[i][4];
                distance_x = screen_x - object_x;
                distance_y = screen_y - object_y;

                with (object.style)
                        transform = MozTransform = webkitTransform =
                                "translateZ(0) " + //hack
                                "matrix(" + scale + ", 0, 0, " + scale + ", " +
                                        (distance_x * distance) + ", " +
                                        (distance_y * distance) + ")";
        };
};

function parallax_initialize()
{
        if (parallax_list.length == 0) return;

        fx_event_listen(window, "load", parallax_event);
        fx_event_listen(window, "scroll", parallax_event);
        fx_event_listen(window, "resize", parallax_event);
        parallax_event();
};