<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

function http_post($url,
                   $data)
{
        /*
         * posts optionally encoded data to specified url
         * returns body data on success
         *
         * binary safe, utf-8 compliant
         */

        if (! is_array($data)) return FALSE; //no post data

        //break address into components
        $url = analyze_url($url);
        if ($url["host"] === NULL) return FALSE; //no host

        //scheme
        switch (strtolower($url["scheme"]))
        {
        case "":
        case "http":
                $transport    = "tcp://";
                $port_default = 80;
                break;

        case "https":
                $transport    = "ssl://";
                $port_default = 443;
                break;

        default:
                return FALSE; //impermissible scheme
        };

        //address components
        $host  = $url["host"];
        $path  = ($url["path"] === NULL)  ? "/"           : $url["path"];
        $port  = ($url["port"] === NULL)  ? $port_default : (int)$url["port"];
        $query = ($url["query"] === NULL) ? ""            : "?" . $url["query"];

        //authorization
        $authorization = "";

        if ($url["user"] !== NULL)
        {
                $authorization = $url["user"];
                if (isset($url["pass"])) $authorization .= ":" + $url["pass"];
                $authorization = "Authorization: Basic " + base64_encode($authorization) . "\r\n";
        };

        //connect
        if (! $hfile = fsockopen($transport . $host, $port, $error_number, $error, 20)) return FALSE;

        //create name-value-pair string
        $buffer = "";

        reset($data);
        while (list($key, $value) = each($data))
                $buffer .= r($key) . "=" . r($value) . "&";

        $buffer = substr($buffer, 0, -1);

        //post data
        fputs($hfile, "POST $path$query HTTP/1.0\r\n" .
                      "Host: $host\r\n" .
                      "User-Agent: " . CMS_SOFTWARE . ", " . CMS_HOST . "\r\n" .
                      "Content-Type: application/x-www-form-urlencoded; charset=utf-8\r\n" .
                      "Content-Length: " . strlen($buffer) . "\r\n" .
                      $authorization .
                      "Connection: close\r\n" .
                      "\r\n" .
                      $buffer);

        //receive reply
        socket_set_timeout($hfile, 10);
        $header = trim(fgets($hfile, 1024));
        if (! preg_match("/^HTTP\/[0-9]\.[0-9]\s+2[0-9]{2}/i", $header)) return FALSE;

        //receive header
        socket_set_timeout($hfile, 10);
        while (! feof($hfile) && trim(fgets($hfile, 1024)))
                socket_set_timeout($hfile, 10);

        //receive content
        $buffer = "";

        socket_set_timeout($hfile, 10);
        while (! feof($hfile) && ($_buffer = fgets($hfile, 65536)))
        {
                $buffer .= $_buffer;
                socket_set_timeout($hfile, 10);
        };

        return $buffer;
};
//-----------------------------------------------------------------------------------------------------
function notify_admin($string)
{
        if (! cms_load("core_resource")) return FALSE;

        //create core object
        $ims    = new core_resource(CMS_DATA_PATH . "#desktop/ims.core",
                                    array("id"       => "string[8]",
                                          "thread"   => "string[8]",
                                          "time"     => "string[20]",
                                          "owner"    => "_string[40]",
                                          "receiver" => "_string[40]",
                                          "sender"   => "_string[40]",
                                          "status"   => "byte",
                                          "text"     => "_string[1500]",
                                          "hash"     => "string[32]"));
        $thread = unique_id(8);
        $time   = time();
        $string = CMS_L_COMMON_001 . ": $string";
        $hash   = md5($thread . $time . "admin" . $string);

        //create message
        $ims->seek(array("owner" => NULL), TRUE);
        $ims->set(array("id"       => unique_id(8),
                        "thread"   => $thread,
                        "time"     => $time,
                        "owner"    => "admin",
                        "receiver" => "admin",
                        "sender"   => "admin",
                        "status"   => 0,
                        "text"     => $string,
                        "hash"     => $hash));

        //create notification file
        if ($hfile = fopen(CMS_DATA_PATH . "#desktop/admin/ims.txt", "wb"))
        {
                fputs($hfile, 1);
                fclose($hfile);
        };

        return TRUE;
};
?>