<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

function weekday($number,
                 $abbr = TRUE)
{
        /*
         * returns name of weekday
         *
         * binary safe, utf-8 compliant
         */

        $number %= 7;

        switch ($number)
        {
        case 0: return $abbr ? CMS_L_SUNDAY_ABBR    : CMS_L_SUNDAY;
        case 1: return $abbr ? CMS_L_MONDAY_ABBR    : CMS_L_MONDAY;
        case 2: return $abbr ? CMS_L_TUESDAY_ABBR   : CMS_L_TUESDAY;
        case 3: return $abbr ? CMS_L_WEDNESDAY_ABBR : CMS_L_WEDNESDAY;
        case 4: return $abbr ? CMS_L_THURSDAY_ABBR  : CMS_L_THURSDAY;
        case 5: return $abbr ? CMS_L_FRIDAY_ABBR    : CMS_L_FRIDAY;
        case 6: return $abbr ? CMS_L_SATURDAY_ABBR  : CMS_L_SATURDAY;
        };

        return FALSE;
};
//-----------------------------------------------------------------------------------------------------
function month($number,
               $abbr = TRUE)
{
        /*
         * returns name of month
         *
         * binary safe, utf-8 compliant
         */

        switch ($number)
        {
        case  1: return $abbr ? CMS_L_JANUARY_ABBR   : CMS_L_JANUARY;
        case  2: return $abbr ? CMS_L_FEBRUARY_ABBR  : CMS_L_FEBRUARY;
        case  3: return $abbr ? CMS_L_MARCH_ABBR     : CMS_L_MARCH;
        case  4: return $abbr ? CMS_L_APRIL_ABBR     : CMS_L_APRIL;
        case  5: return $abbr ? CMS_L_MAY_ABBR       : CMS_L_MAY;
        case  6: return $abbr ? CMS_L_JUNE_ABBR      : CMS_L_JUNE;
        case  7: return $abbr ? CMS_L_JULY_ABBR      : CMS_L_JULY;
        case  8: return $abbr ? CMS_L_AUGUST_ABBR    : CMS_L_AUGUST;
        case  9: return $abbr ? CMS_L_SEPTEMBER_ABBR : CMS_L_AUGUST;
        case 10: return $abbr ? CMS_L_OCTOBER_ABBR   : CMS_L_OCTOBER;
        case 11: return $abbr ? CMS_L_NOVEMBER_ABBR  : CMS_L_NOVEMBER;
        case 12: return $abbr ? CMS_L_DECEMBER_ABBR  : CMS_L_DECEMBER;
        };

        return FALSE;
};
//-----------------------------------------------------------------------------------------------------
function friendly_date($time)
{
        if ($time === NULL) $time = time();
        $date   = getdate($time);
        $_time  = time();
        $now    = getdate($_time);
        $future = $time > $_time;

        //interval in seconds
        $seconds = $_time - $time;

        //within the minute
        if ((int)($seconds / 60) === 0)
                return sprintf($future ? CMS_L_COMMON_012 : CMS_L_COMMON_002,
                               abs($seconds));

        //interval in hours
        $hours = (int)($seconds / 3600);

        //within the hour
        if (abs($hours) === 0)
                return sprintf($future ? CMS_L_COMMON_013 : CMS_L_COMMON_003,
                               abs((int)($seconds / 60)));

        //within 6 hours
        if (abs($hours) <= 6)
        {
                //interval in minutes
                $minutes = (int)($seconds % 3600 / 60);

                //first minute of hour
                if ($minutes === 0)
                        return sprintf($future ? CMS_L_COMMON_019 : CMS_L_COMMON_018,
                                       abs($hours));

                //ony other minute
                return sprintf($future ? CMS_L_COMMON_017 : CMS_L_COMMON_015,
                               abs($hours),
                               abs($minutes));
        };

        //interval in days
        $days = (int)($_time / 86400) - (int)($time / 86400);

        //today
        if ($days === 0)
                return date(CMS_L_SHORT_TIME_FORMAT, $time);

        //yesterday / tomorrow
        if (abs($days) === 1)
                return sprintf($future ? CMS_L_COMMON_014 : CMS_L_COMMON_004,
                               date(CMS_L_SHORT_TIME_FORMAT, $time));

        //week interval
        $weeks = (int)($_time / 604800) - (int)($time / 604800);

        //this week
        if ($weeks === 0)
                return sprintf(CMS_L_COMMON_005,
                               weekday($date["wday"]),
                               date(CMS_L_SHORT_TIME_FORMAT, $time));

        //this year
        if ($date["year"] === $now["year"])
        {
                //this month
                if ($date["mon"] === $now["mon"])
                        return sprintf($future ? CMS_L_COMMON_016 : CMS_L_COMMON_006,
                                       weekday($date["wday"]),
                                       abs($days),
                                       date(CMS_L_SHORT_TIME_FORMAT, $time));

                //any other month
                return sprintf(CMS_L_COMMON_007,
                               $date["mday"],
                               month($date["mon"]),
                               date(CMS_L_SHORT_TIME_FORMAT, $time));
        };

        //any other year
        return date(CMS_L_DATE_FORMAT . ", " . CMS_L_SHORT_TIME_FORMAT, $time);
};
?>
