<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

function mkpath($path)
{
        /*
         * creates full path
         *
         * binary safe, utf-8 compliant
         */

        if (is_dir($path)) return TRUE;

        $path  = explode("/", preg_replace("/^" . preg_quote(CMS_ROOT_PATH, "/") . "/iu", "", $path));
        $_path = CMS_ROOT_PATH;

        reset($path);
        while (list($key, $value) = each($path))
        {
                if (! $value) continue;

                $_path .= $value;

                if (! (is_dir($_path) || mkdir($_path))) return FALSE;

                $_path .= "/";
        };

        return TRUE;
};
//-----------------------------------------------------------------------------------------------------
function read_file($file)
{
        /*
         * reads file content
         *
         * binary safe, utf-8 compliant
         */

        return is_file($file) ? file_get_contents($file) : FALSE;
};
//-----------------------------------------------------------------------------------------------------
function download($file)
{
    $size = 0;

    //open file
    $hfile = fopen($file, 'rb');
    if ($hfile === FALSE) return FALSE; //error

    while (ob_get_level()) ob_end_clean();
    ob_start();

    //send file in chunks
    do
    {
            set_time_limit(600); //10 minutes
            $buffer = fread($hfile, 524288);
            echo($buffer);
            ob_flush();
            flush();

            $size += strlen($buffer);
    }
    while (! feof($hfile));

    set_time_limit(ini_get("max_execution_time"));

    return fclose($hfile) ? $size : FALSE;
};
//-----------------------------------------------------------------------------------------------------
function translate_path($address)
{
        /*
         * expands internally used identifiers to full path:
         * image://{index}
         *
         * binary safe, utf-8 compliant
         */

        if (($i = strpos($address, "://")) === FALSE) return $address;

        $type  = substr($address, 0, $i);
        $i    += 3;

        switch ($type)
        {
        case "image":
//.....................................................................................................
                return CMS_DATA_PATH . "image/" . substr($address, $i);
                break;
        };

        return $address;
};
//-----------------------------------------------------------------------------------------------------
function stringtofilename($string,
                          $replacement = "-")
{
        /*
         * converts a string to a filename
         *
         * binary safe, utf-8 compliant
         */

        //ansi transliteration
        $translation = array("À" => "A", "Á" => "A",  "Â" => "A", "Ã" => "A", "Ä" => "Ae", "Å" => "A", "Æ" => "Ae",
                             "Ç" => "C",
                             "È" => "E", "É" => "E",  "Ê" => "E", "Ë" => "E",
                             "Ì" => "I", "Í" => "I",  "Î" => "I", "Ï" => "I",
                             "Ð" => "D", "Ñ" => "N",
                             "Ò" => "O", "Ó" => "O",  "Ô" => "O", "Õ" => "O", "Ö" => "Oe", "Ø" => "Oe",
                             "Ù" => "U", "Ú" => "U",  "Û" => "U", "Ü" => "Ue",
                             "Ý" => "Y", "Þ" => "Th", "ß" => "ss",

                             "à" => "a", "á" => "a",  "â" => "a", "ã" => "a", "ä" => "ae", "å" => "a", "æ" => "ae",
                             "ç" => "c",
                             "è" => "e", "é" => "e",  "ê" => "e", "ë" => "e",
                             "ì" => "i", "í" => "i",  "î" => "i", "ï" => "i",
                             "ð" => "d", "ñ" => "n",
                             "ò" => "o", "ó" => "o",  "ô" => "o", "õ" => "o", "ö" => "oe", "ø" => "oe",
                             "ù" => "u", "ú" => "u",  "û" => "u", "ü" => "ue",
                             "ý" => "y", "þ" => "th", "ÿ" => "y");

        $string  = strtr($string, $translation);

        //strip everything but letters and numbers
        $search  = array("/^[^\p{L}\p{N}]+/u", "/[^\p{L}\p{N}]+$/u", "/[^\p{L}\p{N}]+/u");
        $replace = array("",                   "",                   $replacement);

        $string  = preg_replace($search, $replace, $string);

        //convert to lowercase
        $string  = utf8_strtolower($string);

        return $string;
};
//-----------------------------------------------------------------------------------------------------
function basename($path,
                  $suffix = NULL)
{
        /*
         * workaround for bug #37268 (http://bugs.php.net/bug.php?id=37268):
         * if a file starts with a non-latin letter (cyrillic) and not with a non-letter character,
         * basename will cut off the beginning of name until a latin letter or a non-letter character
         * is found in the name
         *
         * binary safe, utf-8 compliant
         */

        //remove trailing path separators
        $path  = preg_replace("/\/+$/", "", $path);

        //remove preceding path
        $value = (($value = strrchr($path, "/")) === FALSE) ? $path : substr($value, 1);

        //remove suffix
        if ($suffix !== NULL)
        {
                $length = strlen($suffix);
                if (streq(substr($value, -$length), $suffix))
                        $value = substr($value, 0, -$length);
        };

        return $value;
};
?>