<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

function l($text)
{
        /*
         * short alias for language_get
         *
         * binary safe, utf-8 compliant
         */

         return language_get($text);
};
//-----------------------------------------------------------------------------------------------------
function language_get($text,
                      $language = NULL,
                      $explicit = NULL)
{
        /*
         * returns text in specified language
         *
         * binary safe, utf-8 compliant
         */

        $i        = strcspn($text, CMS_LANGUAGE_SEPARATOR);
        $default  = substr($text, 0, $i);
        if (! $language)
        {
                if (($language === FALSE) || $explicit) return $default;
                $language = CMS_LANGUAGE;
        };
        if (($i = strpos($text, CMS_LANGUAGE_SEPARATOR . "$language:", $i)) === FALSE)
                return $explicit ? NULL : $default;
        $i       += strlen($language) + 2;
        if (($_i = strpos($text, CMS_LANGUAGE_SEPARATOR, $i)) === FALSE)
                $buffer = substr($text, $i);
        else
                $buffer = substr($text, $i, $_i - $i);
        return ($buffer || $explicit) ? $buffer : $default;
};
//-----------------------------------------------------------------------------------------------------
function language_get_array($text)
{
        /*
         * returns text as array indexed by language code
         *
         * binary safe, utf-8 compliant
         */

        $array  = explode(CMS_LANGUAGE_SEPARATOR, $text);

        //initialize return array
        $return = array("" => $array[0]);

        if (CMS_LANGUAGE_ENABLED)
        {
                $language = explode(",", CMS_LANGUAGE_ENABLED);

                reset($array);
                while (list($key, $value) = each($language))
                        $return[$value] = NULL;
        };

        //fill return array
        $c      = count($array);

        for ($i = 1; $i < $c; $i++)
        {
                $text = $array[$i];

                if (($_i = strpos($text, ":")) !== FALSE)
                {
                        $key  = substr($text, 0, $_i);
                        $text = substr($text, $_i + 1);
                        $return[$key] = $text;
                };
        };

        return $return;
};
//-----------------------------------------------------------------------------------------------------
function language_set($text,
                      $value    = NULL,
                      $language = NULL)
{
        /*
         * sets text in specified language
         *
         * binary safe, utf-8 compliant
         */

        $i       = strcspn($text, CMS_LANGUAGE_SEPARATOR);
        $default = substr($text, 0, $i);
        if (! $language) return substr_replace($text, $value, 0, $i);
        if ($value)
                $value = CMS_LANGUAGE_SEPARATOR . "$language:$value";
        if (($i = strpos($text, CMS_LANGUAGE_SEPARATOR . "$language:", $i)) === FALSE)
                return $text . $value;
        $c       = strlen($language) + 2;
        if (($_i = strpos($text, CMS_LANGUAGE_SEPARATOR, $i + $c)) === FALSE)
                return substr_replace($text, $value, $i);
        return substr_replace($text, $value, $i, $_i - $i);
};
//-----------------------------------------------------------------------------------------------------
function language_set_array($array)
{
        /*
         * returns text from array indexed by language code
         *
         * binary safe, utf-8 compliant
         */

        $return = "";

        reset($array);
        while (list($key, $value) = each($array))
        {
                if (stre($value)) continue;
                if (stre($key))   $return  = $value . $return;
                else              $return .= CMS_LANGUAGE_SEPARATOR . "$key:$value";
        };

        return $return;
};
//-----------------------------------------------------------------------------------------------------
function language_detect($text)
{
        /*
         * returns detected language code of text using stop words
         *
         * binary safe, utf-8 compliant
         */

        $data     = new data("#system/language");
        $text     = preg_split("/[\p{C}\p{Ps}\p{Pe}\p{Pi}\p{Pf}\p{Po}\p{Z}]+/u",
                               utf8_strtolower($text), -1, PREG_SPLIT_NO_EMPTY);
        $max      = 0;
        $language = "";

        $data->move("first");
        while ($key = $data->move("next"))
        {
                $stopword = utf8_strtolower($data->get($key, "stopword"));
                $stopword = preg_split("/\s/", $stopword, -1, PREG_SPLIT_NO_EMPTY);
                $match    = count(array_intersect($text, $stopword));

                if ($match > $max)
                {
                        $max      = $match;
                        $language = $key;
                };
        };

        return $language;
};
//-----------------------------------------------------------------------------------------------------
function language_strip_stopword($text,
                                 $language)
{
        /*
         * removes stop words from text according to language
         *
         * binary safe, utf-8 compliant
         */

        $data     = new data("#system/language");
        $stopword = $data->get($language, "stopword");
        $stopword = preg_split("/\s/", $stopword, -1, PREG_SPLIT_NO_EMPTY);

        reset($stopword);
        while (list(, $value) = each($stopword))
                $text = preg_replace("/(?<=^|[\p{C}\p{Ps}\p{Pe}\p{Pi}\p{Pf}\p{Po}\p{Z}])" .
                                     preg_quote($value, "/") .
                                     "(?=$|[\p{C}\p{Ps}\p{Pe}\p{Pi}\p{Pf}\p{Po}\p{Z}])/iu",
                                     "", $text);

        return $text;
};
//-----------------------------------------------------------------------------------------------------
function language_name($string)
{
        /*
         * retrieves language name by code
         *
         * binary safe, utf-8 compliant
         */

        $data = new data("#system/language");

        return $data->get($string, "name");
};
?>