<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

define("CMS_MYSQL_IMPROVED", extension_loaded("mysqli"));
$cms_mysql_connection = NULL;

//result type constants
define("MYSQL_ASSOC", 1);
define("MYSQL_NUM",   2);
define("MYSQL_BOTH",  3);

function mysql_get_link_identifier($link_identifier = NULL)
{
        global $cms_mysql_connection;

        //mysqli
        if (CMS_MYSQL_IMPROVED)
        {
                if (
                     is_object($link_identifier)
                     &&
                     streq(get_class($link_identifier), "mysqli")
                   )
                        return $link_identifier;

                if (
                     is_object($cms_mysql_connection)
                     &&
                     streq(get_class($cms_mysql_connection), "mysqli")
                   )
                        return $cms_mysql_connection;

                return NULL;
        };

        //mysql
        if (is_resource($link_identifier))      return $link_identifier;
        if (is_resource($cms_mysql_connection)) return $cms_mysql_connection;
        return NULL;
};
//-----------------------------------------------------------------------------------------------------
function mysql_affected_rows($link_identifier = NULL)
{
        $link_identifier = mysql_get_link_identifier($link_identifier);
        return CMS_MYSQL_IMPROVED ?
               mysqli_affected_rows($link_identifier) :
               \mysql_affected_rows($link_identifier);
};                                 
//-----------------------------------------------------------------------------------------------------
function mysql_client_encoding($link_identifier = NULL)
{
        $link_identifier = mysql_get_link_identifier($link_identifier);
        return CMS_MYSQL_IMPROVED ?
               mysqli_character_set_name($link_identifier) :
               \mysql_client_encoding($link_identifier);
};
//-----------------------------------------------------------------------------------------------------
function mysql_close($link_identifier = NULL)
{
        $link_identifier = mysql_get_link_identifier($link_identifier);
        return CMS_MYSQL_IMPROVED ?
               mysqli_close($link_identifier) :
               \mysql_close($link_identifier);
};
//-----------------------------------------------------------------------------------------------------
function mysql_connect($server      = NULL,
                       $username    = NULL,
                       $password    = NULL,
                       $new_link    = NULL,
                       $client_flag = NULL)
{
        global $cms_mysql_connection;

        if (CMS_MYSQL_IMPROVED)
        {
                $port   = NULL;
                $socket = NULL;

                if ($extension = strstr($server, ":"))
                {
                        $server    = substr($server, 0, -strlen($extension));
                        $extension = substr($extension, 1);

                        if (is_numeric($extension)) $port   = $extension;
                        else                        $socket = $extension;
                };

                $cms_mysql_connection = mysqli_connect($server, $username, $password, NULL, $port, $socket);
                return $cms_mysql_connection;
        };

        $cms_mysql_connection = \mysql_connect($server, $username, $password, $new_link, $client_flag);
        return $cms_mysql_connection;
};
//-----------------------------------------------------------------------------------------------------
function mysql_data_seek($result,
                         $row_number)
{
        return CMS_MYSQL_IMPROVED ?
               mysqli_data_seek($result, $row_number) :
               \mysql_data_seek($result, $row_number);
};
//-----------------------------------------------------------------------------------------------------
function mysql_db_name($result,
                       $row,
                       $field = NULL)
{
        if (CMS_MYSQL_IMPROVED)
        {
                mysqli_data_seek($result, $row);
                $resultrow = mysqli_fetch_array($result);
                return ($field === NULL) ? $resultrow[0] : $resultrow[$field];
        };

        return \mysql_db_name($result, $row, $field);
};
//-----------------------------------------------------------------------------------------------------
function mysql_errno($link_identifier = NULL)
{
        $link_identifier = mysql_get_link_identifier($link_identifier);
        return CMS_MYSQL_IMPROVED ?
               mysqli_errno($link_identifier) :
               \mysql_errno($link_identifier);
};
//-----------------------------------------------------------------------------------------------------
function mysql_error($link_identifier = NULL)
{
        $link_identifier = mysql_get_link_identifier($link_identifier);
        return CMS_MYSQL_IMPROVED ?
               mysqli_error($link_identifier) :
               \mysql_error($link_identifier);
};
//-----------------------------------------------------------------------------------------------------
function mysql_fetch_array($result,
                           $result_type = MYSQL_BOTH)
{
        return CMS_MYSQL_IMPROVED ?
               mysqli_fetch_array($result, $result_type) :
               \mysql_fetch_array($result, $result_type);
};
//-----------------------------------------------------------------------------------------------------
function mysql_fetch_assoc($result)
{
        return CMS_MYSQL_IMPROVED ?
               mysqli_fetch_assoc($result) :
               \mysql_fetch_assoc($result);
};
//-----------------------------------------------------------------------------------------------------
function mysql_fetch_field($result,
                           $field_offset = NULL)
{
        if (CMS_MYSQL_IMPROVED)
        {
                if ($field_offset) mysqli_field_seek($result, $field_offset);
                return mysqli_fetch_field($result);
        };

        return \mysql_fetch_field($result, $field_offset);
};
//-----------------------------------------------------------------------------------------------------
function mysql_fetch_lengths($result)
{
        return CMS_MYSQL_IMPROVED ?
               mysqli_fetch_lengths($result) :
               \mysql_fetch_lengths($result);
};
//-----------------------------------------------------------------------------------------------------
function mysql_fetch_object($result,
                            $class_name = NULL,
                            $params     = NULL)
{
        return CMS_MYSQL_IMPROVED ?
               mysqli_fetch_object($result, $class_name, $params) :
               \mysql_fetch_object($result, $class_name, $params);
};
//-----------------------------------------------------------------------------------------------------
function mysql_fetch_row($result)
{
        return CMS_MYSQL_IMPROVED ?
               mysqli_fetch_row($result) :
               \mysql_fetch_row($result);
};
//-----------------------------------------------------------------------------------------------------
function mysql_field_flags($result,
                           $field_offset)
{
        if (CMS_MYSQL_IMPROVED)
        {
                mysqli_field_seek($result, $field_offset);
                $field = mysqli_fetch_field($result);
                return $field->flags;
        };

        return \mysql_field_flags($result, $field_offset);
};
//-----------------------------------------------------------------------------------------------------
function mysql_field_len($result,
                         $field_offset)
{
        if (CMS_MYSQL_IMPROVED)
        {
                mysqli_field_seek($result, $field_offset);
                $field = mysqli_fetch_field($result);
                return $field->max_length;
        };

        return \mysql_field_len($result, $field_offset);
};
//-----------------------------------------------------------------------------------------------------
function mysql_field_name($result,
                          $field_offset)
{
        if (CMS_MYSQL_IMPROVED)
        {
                mysqli_field_seek($result, $field_offset);
                $field = mysqli_fetch_field($result);
                return $field->name;
        };

        return \mysql_field_name($result, $field_offset);
};
//-----------------------------------------------------------------------------------------------------
function mysql_field_seek($result,
                          $field_offset)
{
        return CMS_MYSQL_IMPROVED ?
               mysqli_field_seek($result, $field_offset) :
               \mysql_field_seek($result, $field_offset);
};
//-----------------------------------------------------------------------------------------------------
function mysql_field_table($result,
                           $field_offset)
{
        if (CMS_MYSQL_IMPROVED)
        {
                mysqli_field_seek($result, $field_offset);
                $field = mysqli_fetch_field($result);
                return $field->table;
        };

        return \mysql_field_table($result, $field_offset);
};
//-----------------------------------------------------------------------------------------------------
function mysql_field_type($result,
                          $field_offset)
{
        if (CMS_MYSQL_IMPROVED)
        {
                mysqli_field_seek($result, $field_offset);
                $field = mysqli_fetch_field($result);
                return $field->type;
        };

        return \mysql_field_type($result, $field_offset);
};
//-----------------------------------------------------------------------------------------------------
function mysql_free_result($result)
{
        return CMS_MYSQL_IMPROVED ?
               mysqli_free_result($result) :
               \mysql_free_result($result);
};
//-----------------------------------------------------------------------------------------------------
function mysql_get_client_info()
{
        return CMS_MYSQL_IMPROVED ?
               mysqli_get_client_info() :
               \mysql_get_client_info();
};
//-----------------------------------------------------------------------------------------------------
function mysql_get_host_info($link_identifier = NULL)
{
        $link_identifier = mysql_get_link_identifier($link_identifier);
        return CMS_MYSQL_IMPROVED ?
               mysqli_get_host_info($link_identifier) :
               \mysql_get_host_info($link_identifier);
};
//-----------------------------------------------------------------------------------------------------
function mysql_get_proto_info($link_identifier = NULL)
{
        $link_identifier = mysql_get_link_identifier($link_identifier);
        return CMS_MYSQL_IMPROVED ?
               mysqli_get_proto_info($link_identifier) :
               \mysql_get_proto_info($link_identifier);
};
//-----------------------------------------------------------------------------------------------------
function mysql_get_server_info($link_identifier = NULL)
{
        $link_identifier = mysql_get_link_identifier($link_identifier);
        return CMS_MYSQL_IMPROVED ?
               mysqli_get_server_info($link_identifier) :
               \mysql_get_server_info($link_identifier);
};
//-----------------------------------------------------------------------------------------------------
function mysql_info($link_identifier = NULL)
{
        $link_identifier = mysql_get_link_identifier($link_identifier);
        return CMS_MYSQL_IMPROVED ?
               mysqli_info($link_identifier) :
               \mysql_info($link_identifier);
};
//-----------------------------------------------------------------------------------------------------
function mysql_insert_id($link_identifier = NULL)
{
        $link_identifier = mysql_get_link_identifier($link_identifier);
        return CMS_MYSQL_IMPROVED ?
               mysqli_insert_id($link_identifier) :
               \mysql_insert_id($link_identifier);
};
//-----------------------------------------------------------------------------------------------------
function mysql_list_dbs($link_identifier = NULL)
{
        $link_identifier = mysql_get_link_identifier($link_identifier);
        return CMS_MYSQL_IMPROVED ?
               mysqli_query($link_identifier, "SHOW DATABASES") :
               \mysql_list_dbs($link_identifier);
};
//-----------------------------------------------------------------------------------------------------
function mysql_list_processes($link_identifier = NULL)
{
        $link_identifier = mysql_get_link_identifier($link_identifier);
        return CMS_MYSQL_IMPROVED ?
               mysqli_query($link_identifier, "SHOW PROCESSLIST") :
               \mysql_list_processes($link_identifier);
};
//-----------------------------------------------------------------------------------------------------
function mysql_num_fields($result)
{
        return CMS_MYSQL_IMPROVED ?
               mysqli_num_fields($result) :
               \mysql_num_fields($result);
};
//-----------------------------------------------------------------------------------------------------
function mysql_num_rows($result)
{
        return CMS_MYSQL_IMPROVED ?
               mysqli_num_rows($result) :
               \mysql_num_rows($result);
};
//-----------------------------------------------------------------------------------------------------
function mysql_pconnect($server      = NULL,
                        $username    = NULL,
                        $password    = NULL,
                        $client_flag = NULL)
{
        if (CMS_MYSQL_IMPROVED)
        {
                $port   = NULL;
                $socket = NULL;

                if ($extension = strstr($server, ":"))
                {
                        $server    = substr($server, 0, -strlen($extension));
                        $extension = substr($extension, 1);

                        if (is_numeric($extension)) $port   = $extension;
                        else                        $socket = $extension;
                };

                $cms_mysql_connection = mysqli_connect("p:$server", $username, $password, NULL, $port, $socket);
                return $cms_mysql_connection;
        };

        $cms_mysql_connection = \mysql_pconnect($server, $username, $password, $client_flag);
        return $cms_mysql_connection;
};
//-----------------------------------------------------------------------------------------------------
function mysql_ping($link_identifier = NULL)
{
        $link_identifier = mysql_get_link_identifier($link_identifier);
        return CMS_MYSQL_IMPROVED ?
               mysqli_ping($link_identifier) :
               \mysql_ping($link_identifier);
};
//-----------------------------------------------------------------------------------------------------
function mysql_query($query,
                     $link_identifier = NULL)
{
        $link_identifier = mysql_get_link_identifier($link_identifier);
        $result = CMS_MYSQL_IMPROVED ?
                  mysqli_query($link_identifier, $query) :
                  \mysql_query($query, $link_identifier);

        if ($result === FALSE) trigger_error(mysql_error($link_identifier) . "; " . $query);
        return $result;
};
//-----------------------------------------------------------------------------------------------------
function mysql_real_escape_string($unescaped_string,
                                  $link_identifier = NULL)
{
        $link_identifier = mysql_get_link_identifier($link_identifier);
        return CMS_MYSQL_IMPROVED ?
               mysqli_real_escape_string($link_identifier, $unescaped_string) :
               \mysql_real_escape_string($unescaped_string, $link_identifier);
};
//-----------------------------------------------------------------------------------------------------
function mysql_result($result,
                      $row,
                      $field = NULL)
{
        if (CMS_MYSQL_IMPROVED)
        {
                mysqli_data_seek($result, $row);
                $resultrow = mysqli_fetch_array($result);
                return ($field === NULL) ? $resultrow[0] : $resultrow[$field];
        };

        return \mysql_result($result, $row, $field);
};
//-----------------------------------------------------------------------------------------------------
function mysql_select_db($database_name,
                         $link_identifier = NULL)
{
        $link_identifier = mysql_get_link_identifier($link_identifier);
        return CMS_MYSQL_IMPROVED ?
               mysqli_select_db($link_identifier, $database_name) :
               \mysql_select_db($database_name, $link_identifier);
};
//-----------------------------------------------------------------------------------------------------
function mysql_stat($link_identifier = NULL)
{
        $link_identifier = mysql_get_link_identifier($link_identifier);
        return CMS_MYSQL_IMPROVED ?
               mysqli_stat($link_identifier) :
               \mysql_stat($link_identifier);
};
//-----------------------------------------------------------------------------------------------------
function mysql_thread_id($link_identifier = NULL)
{
        $link_identifier = mysql_get_link_identifier($link_identifier);
        return CMS_MYSQL_IMPROVED ?
               mysqli_thread_id($link_identifier) :
               \mysql_thread_id($link_identifier);
};
//-----------------------------------------------------------------------------------------------------
function mysql_unbuffered_query($query,
                                $link_identifier = NULL)
{
        $link_identifier = mysql_get_link_identifier($link_identifier);

        if (CMS_MYSQL_IMPROVED)
        {
                mysqli_real_query($link_identifier, $query);
                return mysqli_use_result($link_identifier);
        };

        return \mysql_unbuffered_query($query, $link_identifier);
};
?>
