<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

function permission($array,
                    $return = NULL)
{
        /*
         * displays permission management button
         *
         * binary safe, utf-8 compliant
         */

        if (cms_permission("interface.permission", NULL, NULL))
        {
                $application = cms_application();
                $buffer      = "<div style=\"PADDING:5px;TEXT-ALIGN:right\">" .
                               image("permission/icon_access", NULL, NULL, CMS_L_COMMON_010) . " ";

                reset($array);
                while (list($key, $value) = each($array))
                {
                        if ($key) $key = ".$key";
                        $buffer .= "<a href=\"javascript:load_page('" .
                                       qx(CMS_MODULES_URL) . "interface.php" .
                                      "?ifc_page=permission" .
                                      "&amp;ifc_message=_set" .
                                      "&amp;ifc_option=external" .
                                      "&amp;ifc_param1=" . rqx($application . $key) . "');\" " .
                                      "title=\"" . x($application . $key) . "\">" .
                                   x($value) .
                                   "</a>" .
                                   " | ";
                };

                $buffer      = substr($buffer, 0, -3) . "</div>";
                if ($return) return $buffer;
                echo($buffer);
        };
};
//-----------------------------------------------------------------------------------------------------
function insert($position = NULL)
{
        /*
         * outputs predefined insert text
         * used in modules, edited by interface
         *
         * binary safe, utf-8 compliant
         */

        $key    = cms_application();
        if ($position) $key .= ".$position";

        $data   = new data("#system/insert");
        $insert = $data->get($key, "insert");

        if (
             cms_permission("interface.insert", NULL, NULL)
             &&

        //only display in editing mode
             (
               (! defined("CMS_TEMPLATE_OPTION"))
               ||
               CMS_TEMPLATE_OPTION
             )
           )
                echo("<div>" .
                     "<a href=\"javascript:load_page('" .
                         qx(CMS_MODULES_URL) . "interface.php" .
                        "?ifc_page=insert" .
                        "&amp;ifc_message=_add" .
                        "&amp;ifc_option=external" .
                        "&amp;ifc_param1=" . rqx($key) .
                        "&amp;object');\">" .
                     image("insert/button_edit", NULL, NULL, CMS_L_COMMON_011) .
                     "</a>" .
                     "</div>");

        if ($insert)
        {
                $data = new data("#system/insert.code");
                $code = l($data->get($insert, "code"));

                if (nstre($code))
                        echo($data->get($insert, "html") ?
                             parse_token($code) :
                             "<div>" . parse_text($code) . "</div>");
        };
};
//-----------------------------------------------------------------------------------------------------
function class_varied($option = NULL,
                      $index  = 0)
{
        /*
         * switches style tag
         * resets switch if option is TRUE
         * appends class name if option is a string
         *
         * index allows nesting
         *
         * binary safe, utf-8 compliant
         */

        static $flag  = NULL;

        $index        = (int)$index;
        if (($option === TRUE) || (! isset($flag[$index]))) $flag[$index] = TRUE;
        $flag[$index] = ! $flag[$index];

        if (is_string($option)) return $flag[$index] ? " class=\"varied $option\"" : " class=\"$option\"";
        if ($flag[$index])      return " class=\"varied\"";
        return "";
};
//-----------------------------------------------------------------------------------------------------
function jscript($code)
{
        /*
         * javascript code
         *
         * binary safe, utf-8 compliant
         */

        return "<script>" .
               str_replace("</", "<\/", $code) .
               "</script>";
};
//-----------------------------------------------------------------------------------------------------
function stylesheet($url,
                    $asynchronously = TRUE)
{
        /*
         * load stylesheet (asynchronously) with fallback
         *
         * binary safe, utf-8 compliant
         */

        $url = x($url);
        return ($asynchronously ?
               "<link rel=\"stylesheet\" " .
                     "href=\"$url\" " .
                     "media=\"not all\" " .
                     "onload=\"this.media='all';\">" .
               "<noscript>" : "") .
               "<link rel=\"stylesheet\" " .
                     "href=\"$url\">" .
               ($asynchronously ?
               "</noscript>" : "");
};
//-----------------------------------------------------------------------------------------------------
function select($option,
                $preset    = NULL,
                $name      = NULL,
                $set_value = FALSE,
                $disabled  = FALSE,
                $height    = NULL)
{
        /*
         * outputs select box
         *
         * binary safe, utf-8 compliant
         */

        if (! is_array($option)) return FALSE;

        if ($height != NULL)     $name .= "[]";

        echo("<select" .
             option($name     != NULL, " name=\"" . x($name) . "\"") .
             option($height   != NULL, " size=\"" . (int)$height . "\" multiple") .
             option($disabled != NULL, " disabled") . ">");

        reset($option);
        while(list($key, $value) = each($option))
        {
                $_value   = $set_value                  ? $value      : $key;
                $selected = is_array($preset) ?
                            (in_array($_value, $preset) ? " selected" : "") :
                            (streq($_value, $preset)    ? " selected" : "");

                echo("<option value=\"" . x($_value) . "\"$selected>" .
                     whitechartostring(x($key), "&nbsp;") .
                     "</option>");
        };
        echo("</select>");

        return TRUE;
};
//-----------------------------------------------------------------------------------------------------
function info($text)
{
        /*
         * info message
         *
         * binary safe, utf-8 compliant
         */

        return "<p>" .
               image("icon_info") . "&nbsp;" .
               "<strong>" .
               $text .
               "</strong>" .
               "</p>";
};
//-----------------------------------------------------------------------------------------------------
function alert($text)
{
        /*
         * alert message
         *
         * binary safe, utf-8 compliant
         */

        return "<p>" .
               image("icon_alert") . "&nbsp;" .
               "<strong style=\"COLOR:#ff0000\">" .
               $text .
               "</strong>" .
               "</p>";
};
?>