<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

function parse_text($text,
                    $token    = TRUE,
                    $base_url = CMS_HOST)
{
        /*
         * parses formatted text for HTML output
         *
         * binary safe, utf-8 compliant
         */

        if ((! $base_url) || (! $base_url = analyze_url($base_url))) $base_url = analyze_url(CMS_HOST);
        $base_url    = "/^" . preg_quote($base_url["scheme"], "/") . ":\/\/" .
                       "([^\.]+\.)?" . preg_quote($base_url["host"], "/") . "/i";

        $format      = array("+"     => array("<b>",
                                              "</b>"),
                             "/"     => array("<i>",
                                              "</i>"),
                             "_"     => array("<u>",
                                              "</u>"),
                             "<"     => array("<span style=\"FONT-SIZE:larger\">",
                                              "</span>"),
                             ">"     => array("<small>",
                                              "</small>"),
                             "<-"    => array("<span style=\"DISPLAY:block;TEXT-ALIGN:left\">",
                                              "</span>"),
                             "<->"   => array("<span style=\"DISPLAY:block;TEXT-ALIGN:center\">",
                                              "</span>"),
                             "->"    => array("<span style=\"DISPLAY:block;TEXT-ALIGN:right\">",
                                              "</span>"),
                             "*"     => array("<h1 style=\"DISPLAY:inline\">",
                                              "</h1>"),
                             "\""    => array("<q>",
                                              "</q>"),
                             "-"     => array("<pre style=\"DISPLAY:inline\">",
                                              "</pre>"),
                             "IMG"   => array("",
                                              ""),
                             "<-IMG" => array("",
                                              ""),
                             "IMG->" => array("",
                                              ""),
                             "#"     => array("<table class=\"layout\">" .
                                              "<tr>" .
                                              "<td style=\"PADDING:0 1em 1em 0\">",
                                              "</td>" .
                                              "</tr>" .
                                              "</table>"));

        $html_chars  = array("<"  => "&lt;",
                             ">"  => "&gt;",
                             "\"" => "&quot;",
                             "&"  => "&amp;");

        $image       = cms_load("image") ? new image() : NULL;
        $depth       = 0;                   //output buffer level
        $buffer      = array($depth => ""); //output buffer
        $mode        = 0;                   //current parsing mode
        $stack       = array();             //format marker stack
        $marker      = "";                  //format marker
        $table_depth = 0;                   //table nesting depth
        $table_char  = "";                  //table character
        $char        = "";                  //current character
        $_char       = "";                  //previous character
        $__char      = "";                  //antepenultimate character
        $whitespace  = "";                  //whitespace buffer
        $i           = 0;                   //text position
        $c           = strlen($text);       //length of input text

        while (TRUE)
        {
                for (; $i < $c; $i++)
                {
                        $char = $text[$i];

                        switch ($mode)
                        {
                        case "1": //token
//.....................................................................................................

                                //append current character
                                $buffer[$depth] .= $char;

                                //unescaped token delimeter
                                if (($char === "%") && ($_char === "%") && ($__char !== "\\"))
                                        $mode = 0; //reset mode
                                break;

                        case "2": //formatting
//.....................................................................................................
                                if ($char === "\r") break;

                                //unescaped space or linebreak
                                if (
                                     (($char === " ")                       && ($_char  !== "\\"))
                                     ||
                                     (($char === "\n") && ($_char !== "\r") && ($_char  !== "\\"))
                                     ||
                                     (($char === "\n") && ($_char === "\r") && ($__char !== "\\"))
                                   )
                                {
                                        //marker
                                        if (isset($format[$marker]))
                                        {
                                                //memorize format marker
                                                array_push($stack, $marker);

                                                //table
                                                if ($marker === "#")
                                                {
                                                        $table_char = ""; //reset table character
                                                        $table_depth++;   //increase table nesting depth
                                                };

                                                //append format initiation
                                                $buffer[$depth] .= $format[$marker][0];
                                        }

                                        //hyperlink, marker represents url
                                        else
                                        {
                                                //memorize format completion
                                                array_push($stack, "</a>");

                                                //evaluate url
                                                $marker = translate_url($marker);

                                                //append format initiation
                                                $buffer[$depth] .= "<a href=\"" . x($marker) . "\"";

                                                //not internal or javascript
                                                if (
                                                     (! preg_match($base_url, $marker))
                                                     &&
                                                     (! preg_match("/^(javascript|mailto):/i", $marker))
                                                   )
                                                        $buffer[$depth] .=
                                                                //" rel=\"nofollow\"" .
                                                                " onclick=\"window.open(this.href);return false;\"";

                                                $buffer[$depth] .= ">";
                                        };

                                        $char             = ""; //consume current character
                                        $buffer[++$depth] = ""; //increase buffer level
                                        $mode             = 0;  //reset mode
                                        break;
                                };

                                //append current character
                                $marker .= $char;
                                break;

                        default:
//.....................................................................................................

                                //replace html/xml characters with entities
                                if (isset($html_chars[$char])) $char = $html_chars[$char];

                                //collect adjacent whitespace
                                if ($_char === " ")
                                        $whitespace .= ($char === " ") ? "&nbsp;" : " ";
                                if ($char === "\n")
                                {
                                        if (
                                                                   (($_char  === "") || ($_char  === "\n"))
                                             ||
                                             (($_char === "\r") && (($__char === "") || ($__char === "\n")))
                                           )
                                                $whitespace .= "&nbsp;";
                                        $whitespace .= ($_char === "\n") ? "<br class=\"multiple\">" : "<br>";
                                        break;
                                };
                                if ($char === "\r") break;
                                if ($char === " ")  break;

                                //table
                                if ($table_depth > 0)
                                {
                                        if ($char === "|") //table delimiter
                                        {
                                                if ($table_char === "|") //table row delimiter
                                                {
                                                        $buffer[$depth] .=
                                                                "</td>" .
                                                                "</tr>" .
                                                                "<tr>" .
                                                                "<td style=\"PADDING:0 1em 1em 0\">";

                                                        //consume memorized table delimiter
                                                        $table_char = "";
                                                }
                                                else
                                                {
                                                        //memorize table delimiter
                                                        $table_char = $char;
                                                };

                                                $char       = ""; //consume current character
                                                $whitespace = ""; //discard whitespace
                                        }
                                        else
                                        {
                                                if ($table_char === "|") //table cell delimiter
                                                {
                                                        $buffer[$depth] .=
                                                                "</td>" .
                                                                "<td style=\"PADDING:0 1em 1em 0\">";
                                                        $whitespace = ""; //discard whitespace
                                                }
                                                elseif ($table_char === "") //initial table cell
                                                {
                                                        $whitespace = ""; //discard whitespace
                                                };

                                                //memorize table character
                                                $table_char = $char;
                                        };
                                };

                                switch ($char)
                                {
                                case "%": //token

                                        if (

                                        //unescaped token delimiter
                                             (($_char === "%") && ($__char !== "\\"))
                                             &&

                                        //token support enabled
                                             $token
                                           )
                                        {
                                                //append current character
                                                $buffer[$depth] .= $whitespace . $char;
                                                $whitespace      = ""; //consume whitespace
                                                $mode            = 1;  //switch to token mode
                                                break 2;
                                        };

                                        break;

                                case "[": //start formatting

                                        if ($_char === "\\") //escaped
                                        {
                                                $_char = ""; //consume escape character
                                                break;
                                        };

                                        //append whitespace
                                        $buffer[$depth] .= $whitespace;
                                        $whitespace      = ""; //consume whitespace
                                        $marker          = ""; //reset marker
                                        $mode            = 2;  //switch to formatting mode
                                        break 2;

                                case "]": //end formatting

                                        if ($_char === "\\") //escaped
                                        {
                                                $_char = ""; //consume escape character
                                                break;
                                        };

                                        //retrieve memorized marker
                                        if ($marker = array_pop($stack))
                                        {
                                                //format marker
                                                if (isset($format[$marker]))
                                                {
                                                        switch ($marker)
                                                        {

                                                        //image
                                                        case "IMG":
                                                        case "<-IMG":
                                                        case "IMG->":

                                                                $_buffer = preg_split("/\s+/", $buffer[$depth], 3);
                                                                $url     = $_buffer[0];
                                                                $title   = NULL;
                                                                if (streq(substr($url, 0, 8), "image://"))
                                                                {
                                                                        if ($image !== NULL) $title = l($image->data->get($url, "name"));
                                                                        $url = translate_url($url);
                                                                };
                                                                $width   = (isset($_buffer[1]) && nstreq($_buffer[1], "*")) ?
                                                                           $_buffer[1] : NULL;
                                                                $height  = (isset($_buffer[2]) && nstreq($_buffer[2], "*")) ?
                                                                           $_buffer[2] : NULL;

                                                                switch ($marker)
                                                                {
                                                                case "IMG":   $style = "MARGIN:0 1em 1em 0;MAX-WIDTH:100%"; break;
                                                                case "<-IMG": $style = "FLOAT:left;MARGIN:0 1em 1em 0;MAX-WIDTH:100%"; break;
                                                                case "IMG->": $style = "FLOAT:right;MARGIN:0 0 1em 1em;MAX-WIDTH:100%"; break;
                                                                };

                                                                $buffer[$depth] = image($url, $width, $height, $title, $style, TRUE);
                                                                break;

                                                        //table
                                                        case "#":
                                                                $whitespace = ""; //discard whitespace
                                                                $table_depth--;   //decrease table nesting depth
                                                                break;
                                                        };

                                                        //replace marker with format completion
                                                        $marker = $format[$marker][1];
                                                };

                                                //decrease buffer level and append collected data
                                                $buffer[--$depth] .= array_pop($buffer) . $whitespace . $marker;
                                        }

                                        //no formatting initiated
                                        else
                                        {
                                                //append current character
                                                $buffer[$depth] .= $whitespace . $char;
                                        };

                                        $whitespace = ""; //consume whitespace
                                        break 2;
                                };

                                //append unconsumed backslash from previous pass
                                if ($_char === "\\")
                                {
                                        $buffer[$depth] .= $_char;
                                        $whitespace      = ""; //consume whitespace
                                };

                                //collect backslash
                                if ($char === "\\")  break;

                                //append current character
                                $buffer[$depth] .= $whitespace . $char;
                                $whitespace      = ""; //consume whitespace
                        };

                        //shift character
                        $__char = $_char;
                        $_char  = $char;
                };

                if ($depth === 0) break; //end parsing

                switch ($mode)
                {
                case 1: //token
                        $mode = 0; //reset mode
                        break;

                case 2: //formatting
                        $text .= " ]"; //complete formatting
                        break;
                };

                //close open formattings
                $text .= str_repeat("]", $depth);
                $c     = strlen($text);
        };

        //append remaining whitespace
        $buffer[0] .= $whitespace;
        if ($_char === " ")
                $buffer[0] .= " ";

        //append unconsumed backslash
        elseif ($_char === "\\")
                $buffer[0] .= $_char;

        //token
        if ($token)
                $buffer[0]  = parse_token($buffer[0]);

        return $buffer[0];
};
//-----------------------------------------------------------------------------------------------------
function remove_format($text,
                       $token         = TRUE,
                       $discard_links = FALSE)
{
        /*
         * removes text format markers
         *
         * binary safe, utf-8 compliant
         */

        $format      = array("+"     => TRUE,
                             "/"     => TRUE,
                             "_"     => TRUE,
                             "<"     => TRUE,
                             ">"     => TRUE,
                             "<-"    => TRUE,
                             "<->"   => TRUE,
                             "->"    => TRUE,
                             "*"     => TRUE,
                             "\""    => TRUE,
                             "-"     => TRUE,
                             "IMG"   => TRUE,
                             "<-IMG" => TRUE,
                             "IMG->" => TRUE,
                             "#"     => TRUE);

        $depth       = 0;                   //output buffer level
        $buffer      = array($depth => ""); //output buffer
        $mode        = 0;                   //current parsing mode
        $stack       = array();             //format marker stack
        $marker      = "";                  //format marker
        $table_depth = 0;                   //table nesting depth
        $table_char  = "";                  //table character
        $char        = "";                  //current character
        $_char       = "";                  //previous character
        $__char      = "";                  //antepenultimate character
        $whitespace  = "";                  //whitespace buffer
        $i           = 0;                   //text position
        $c           = strlen($text);       //length of input text

        while (TRUE)
        {
                for (; $i < $c; $i++)
                {
                        $char = $text[$i];

                        switch ($mode)
                        {
                        case "1": //token
//.....................................................................................................

                                //append current character
                                $buffer[$depth] .= $char;

                                //unescaped token delimeter
                                if (($char === "%") && ($_char === "%") && ($__char !== "\\"))
                                        $mode = 0; //reset mode
                                break;

                        case "2": //formatting
//.....................................................................................................
                                if ($char === "\r") break;

                                //unescaped space or linebreak
                                if (
                                     (($char === " ")                       && ($_char  !== "\\"))
                                     ||
                                     (($char === "\n") && ($_char !== "\r") && ($_char  !== "\\"))
                                     ||
                                     (($char === "\n") && ($_char === "\r") && ($__char !== "\\"))
                                   )
                                {
                                        //marker
                                        if (isset($format[$marker]))
                                        {
                                                //memorize format marker
                                                array_push($stack, $marker);

                                                //table
                                                if ($marker === "#")
                                                {
                                                        $buffer[$depth] .= "\n" . str_repeat("\t", $table_depth);
                                                        $table_char      = ""; //reset table character
                                                        $table_depth++;        //increase table nesting depth
                                                };
                                        }

                                        //hyperlink, marker represents url
                                        else
                                        {
                                                //discard url
                                                if ($discard_links || preg_match("/^javascript:/i", $marker))
                                                {
                                                        array_push($stack, "");
                                                }

                                                else
                                                {
                                                        //evaluate url
                                                        $marker = translate_url($marker);
                                                        $marker = preg_replace("/^mailto:/i", "", $marker);
                                                        array_push($stack, " ($marker)");
                                                };
                                        };

                                        $char             = ""; //consume current character
                                        $buffer[++$depth] = ""; //increase buffer level
                                        $mode             = 0;  //reset mode
                                        break;
                                };

                                //append current character
                                $marker .= $char;
                                break;

                        default:
//.....................................................................................................

                                //collect adjacent whitespace
                                if (
                                     ($char === " ")
                                     ||
                                     ($char === "\r")
                                     ||
                                     ($char === "\n")
                                   )
                                {
                                        $whitespace .= $char;
                                        break;
                                };

                                //table
                                if ($table_depth > 0)
                                {
                                        if ($char === "|") //table delimiter
                                        {
                                                if ($table_char === "|") //table row delimiter
                                                {
                                                        $buffer[$depth] .= "\n" . str_repeat("\t", $table_depth - 1);

                                                        //consume memorized table delimiter
                                                        $table_char      = "";
                                                }
                                                else
                                                {
                                                        //memorize table delimiter
                                                        $table_char      = $char;
                                                };

                                                $char       = ""; //consume current character
                                                $whitespace = ""; //discard whitespace
                                        }
                                        else
                                        {
                                                if ($table_char === "|") //table cell delimiter
                                                {
                                                        $buffer[$depth] .= "\t";
                                                        $whitespace      = ""; //discard whitespace
                                                }
                                                elseif ($table_char === "") //initial table cell
                                                {
                                                        $whitespace      = ""; //discard whitespace
                                                };

                                                //memorize table character
                                                $table_char = $char;
                                        };
                                };

                                switch ($char)
                                {
                                case "%": //token

                                        if (

                                        //unescaped token delimiter
                                             (($_char === "%") && ($__char !== "\\"))
                                             &&

                                        //token support enabled
                                             $token
                                           )
                                        {
                                                //append current character
                                                $buffer[$depth] .= $whitespace . $char;
                                                $whitespace      = ""; //consume whitespace
                                                $mode            = 1;  //switch to token mode
                                                break 2;
                                        };

                                        break;

                                case "[": //start formatting

                                        if ($_char === "\\") //escaped
                                        {
                                                $_char = ""; //consume escape character
                                                break;
                                        };

                                        //append whitespace
                                        $buffer[$depth] .= $whitespace;
                                        $whitespace      = ""; //consume whitespace
                                        $marker          = ""; //reset marker
                                        $mode            = 2;  //switch to formatting mode
                                        break 2;

                                case "]": //end formatting

                                        if ($_char === "\\") //escaped
                                        {
                                                $_char = ""; //consume escape character
                                                break;
                                        };

                                        //retrieve memorized marker
                                        if (($marker = array_pop($stack)) !== NULL)
                                        {
                                                //format marker
                                                if (isset($format[$marker]))
                                                {
                                                        switch ($marker)
                                                        {

                                                        //image
                                                        case "IMG":
                                                        case "<-IMG":
                                                        case "IMG->":

                                                                //discard url
                                                                $buffer[$depth] = "";
                                                                break;

                                                        //table
                                                        case "#":
                                                                $whitespace = "\n"; //overwrite whitespace
                                                                $table_depth--;     //decrease table nesting depth
                                                                break;
                                                        };

                                                        $marker = ""; //clear marker
                                                };

                                                //decrease buffer level and append collected data
                                                $buffer[--$depth] .= array_pop($buffer) . $whitespace . $marker;
                                        }

                                        //no formatting initiated
                                        else
                                        {
                                                //append current character
                                                $buffer[$depth] .= $whitespace . $char;
                                        };

                                        $whitespace = ""; //consume whitespace
                                        break 2;
                                };

                                //append unconsumed backslash from previous pass
                                if ($_char === "\\")
                                {
                                        $buffer[$depth] .= $whitespace . $_char;
                                        $whitespace      = ""; //consume whitespace
                                };

                                //collect backslash
                                if ($char === "\\")  break;

                                //append current character
                                $buffer[$depth] .= $whitespace . $char;
                                $whitespace      = ""; //consume whitespace
                        };

                        //shift character
                        $__char = $_char;
                        $_char  = $char;
                };

                if ($depth === 0) break; //end parsing

                switch ($mode)
                {
                case 1: //token
                        $mode = 0; //reset mode
                        break;

                case 2: //formatting
                        $text .= " ]"; //complete formatting
                        break;
                };

                //close open formattings
                $text .= str_repeat("]", $depth);
                $c     = strlen($text);
        };

        //append unconsumed backslash
        if ($_char === "\\")
                $buffer[0] .= $whitespace . $_char;

        //token
        if ($token)
                $buffer[0]  = parse_token($buffer[0]);

        return $buffer[0];
};
//-----------------------------------------------------------------------------------------------------
function get_first_image($text)
{
        /*
         * returns first image from text
         *
         * binary safe, utf-8 compliant
         */

        return preg_match("/\[(?:IMG|<-IMG|IMG->)\s+([^\s]+)[^\]]*\]/s", $text, $text) ?
               $text[1] : NULL;
};
//-----------------------------------------------------------------------------------------------------
function parse_token($text)
{
        /*
         * parses and returns token
         *
         * binary safe, utf-8 compliant
         */

        static $token = NULL;

        if (! cms_load("token")) return $text;
        if ($token === NULL)     $token = new token();

        return $token->apply($text);
};
//-----------------------------------------------------------------------------------------------------
function quote_text($text,
                    $keyword,
                    $minimum_keyword_length = 3)
{
        /*
         * returns highlighted quotation from text containing keyword
         * used for search result display
         *
         * binary safe, utf-8 compliant
         */

        $keyword  = preg_split("/[\p{C}\p{P}\p{Z}]+/u", //extract keywords
                               $keyword,
                               -1,
                               PREG_SPLIT_NO_EMPTY);
        $keyword  = array_unique($keyword);             //remove repetitive keywords
        $range    = array();                            //match range list
        $_keyword = array();                            //preg quoted keyword list

        //go through keyword list to find relevant passages
        reset($keyword);
        while (list(, $value) = each($keyword))
        {
                if (strlen($value) <= $minimum_keyword_length) continue; //keyword length too low
                $value   = preg_quote($value, "/");

                //find quotation
                $pattern = "/(?:^|[\p{C}\p{P}\p{Z}]+)(.{0,50}%s.{0,150})(?:$|[\p{C}\p{P}\p{Z}]+)/iu";

                if (

                //match exact word
                     preg_match(sprintf($pattern, "(?<=^|[\p{C}\p{P}\p{Z}])$value(?=$|[\p{C}\p{P}\p{Z}])"),
                                $text, $match, PREG_OFFSET_CAPTURE)
                     ||

                //match beginning of word
                     preg_match(sprintf($pattern, "(?<=^|[\p{C}\p{P}\p{Z}])$value"),
                                $text, $match, PREG_OFFSET_CAPTURE)
                     ||

                //match word ending
                     preg_match(sprintf($pattern, "$value(?=$|[\p{C}\p{P}\p{Z}])"),
                                $text, $match, PREG_OFFSET_CAPTURE)
                     ||

                //match any part of word
                     preg_match(sprintf($pattern, $value),
                                $text, $match, PREG_OFFSET_CAPTURE)
                   )
                {
                        $_keyword[] = $value;                         //add keyword to highlight list
                        $offset     = $match[1][1];                   //get offset position
                        $end        = $offset + strlen($match[1][0]); //get end position

                        //store quotation range
                        if ((! isset($range[$offset])) || ($range[$offset] < $end))
                                $range[$offset] = $end;
                };
        };

        if (count($range) === 0) return ""; //no matches found

        //merge relevant passages
        $buffer = "";
        $offset = 0;
        $end    = 0;

        ksort($range); //sort by offset

        //iterate quotation ranges
        reset($range);
        while (list($key, $value) = each($range))
        {
                if ($key > $end) //offset outside of previous quotation range
                {
                        if ($end) $buffer .= substr($text, $offset, $end - $offset) . "\xE2\x80\xA6 ";
                        $offset = $key; //set new offset
                };

                $end = $value; //set current end
        };
        $buffer .= substr($text, $offset, $end - $offset) . "\xE2\x80\xA6";

        //highlight keywords
        $buffer = preg_replace("/(" . implode("|", $_keyword) . ")/iu",
                               "<strong>\\1</strong>",
                               x($buffer));

        return $buffer;
};
//-----------------------------------------------------------------------------------------------------
function htmltoplain($string,
                     $format = FALSE)
{
        /*
         * converts HTML to plain text
         *
         * binary safe, utf-8 compliant
         */

        //convert whitespace to space
        $search[]  = "/\s+/su";
        $replace[] = " ";

        //comment
        $search[]  = "/<!--.*?-->/su";
        $replace[] = "";

        //head
        $search[]  = "/<head(?:\s[^>]*)?>.*?<\/head\s*>/isu";
        $replace[] = "";

        //script
        $search[]  = "/<script(?:\s[^>]*)?>.*?<\/script\s*>/isu";
        $replace[] = "";

        //style
        $search[]  = "/<style(?:\s[^>]*)?>.*?<\/style\s*>/isu";
        $replace[] = "";

        if ($format)
        {
                //br, div, ol, p, table, ul, h1 - h6
                $search[]  = "/<\/?(?:br|div|h[1-6]|ol|p|table|ul)(?:\s[^>]*)?>/iu";
                $replace[] = "\n";

                //li
                $search[]  = "/<li(?:\s[^>]*)?>/isu";
                $replace[] = "* ";

                $search[]  = "/<\/li\s*>/iu";
                $replace[] = "\n";

                //a
                $search[]  = "/<a\s[^>]*href=\"\s*([^\"]+?)\s*\"[^>]*>(.*?)<\/a\s*>/isu";
                $replace[] = "\\2 &lt;\\1&gt;";

                //hr
                $search[]  = "/<hr(?:\s[^>]*)?>/iu";
                $replace[] = "\n--------------------------\n";

                //tr
                $search[]  = "/<\/tr\s*>/iu";
                $replace[] = "\n";

                //td
                $search[]  = "/<td(?:\s[^>]*)?>/isu";
                $replace[] = "\t";

                //th
                $search[]  = "/<th(?:\s[^>]*)?>/isu";
                $replace[] = "\t";
        };

        //img
        $search[]  = "/<img\s[^>]*alt=\"\s*([^\"]{0,49}[^\"\s])(?=\"|[\p{C}\p{P}\p{Z}])[^\"]*\"[^>]*>/isu";
        $replace[] = " [\\1] ";

        //strip markup
        $search[]  = "/<[^>]+>/u";
        $replace[] = " ";

        //convert entities
        $search[]  = "/&nbsp;/iu";
        $replace[] = " ";
        $search[]  = "/&quot;/iu";
        $replace[] = "\"";
        $search[]  = "/&gt;/iu";
        $replace[] = ">";
        $search[]  = "/&lt;/iu";
        $replace[] = "<";
        $search[]  = "/&amp;/iu";
        $replace[] = "&";

        //strip entities
        $search[]  = "/&[^&;]+;/u";
        $replace[] = " ";

        //strip whitespace
        $search[]  = "/ {2,}/u";
        $replace[] = " ";

        if ($format)
        {
                $search[]  = "/[ \t]*([\r\n]+)[ \t]*/u";
                $replace[] = "\\1";
                $search[]  = "/(?:\r?\n){3,}/u";
                $replace[] = "\n\n";
        };

        return utf8_trim(preg_replace($search, $replace, $string));
};
//-----------------------------------------------------------------------------------------------------
function linebreak($string,
                   $length  = 64,
                   $break   = "\n",
                   $comment = ">")
{
        /*
         * breaks lines after specified length
         * lines beginning with comment string are ignored
         * used for plain text emails and forum entries
         *
         * binary safe, utf-8 compliant
         */

        $array  = preg_split("/\r?\n/", $string);
        $string = NULL;
        $c      = strlen($comment);

        reset($array);
        while (list(, $value) = each($array))
        {
                if (nstreq(substr($value, 0, $c), $comment))
                {
                        $i       = 0;
                        $_array  = explode(" ", $value);

                        reset($_array);
                        while (list(, $_value) = each($_array))
                        {
                                $_i = strlen($_value);
                                if ($i)
                                {
                                        if ($i + $_i >= $length)
                                        {
                                                $i       = $_i;
                                                $string .= $break . $_value;
                                        }
                                        else
                                        {
                                                $i      += $_i + 1;
                                                $string .= " $_value";
                                        };
                                }
                                else
                                {
                                        $i       = $_i;
                                        $string .= $_value;
                                };
                        };
                }
                else
                {
                        $string .= $value;
                };

                $string .= $break;
        };

        return $string;
};
//-----------------------------------------------------------------------------------------------------
function text_similarity($text1,
                         $text2)
{
        /*
         * returns similarity between text1 and text2 in percent
         *
         * binary safe, utf-8 compliant
         */

        $fingerprint1 = fingerprint($text1);
        $fingerprint2 = fingerprint($text2);
        $count_total  = count($fingerprint1) + count($fingerprint1);
        if ($count_total === 0) return 100; //both texts are empty and therefore identical
        $count_match  = count(array_intersect($fingerprint1, $fingerprint2)) +
                        count(array_intersect($fingerprint2, $fingerprint1));
        return 100 / $count_total * $count_match;
};
?>