<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

//-----------------------------------------------------------------------------------------------------
//===< CATEGORY CLASS >================================================================================
//-----------------------------------------------------------------------------------------------------
define("CMS_CATEGORY_CLASS_SPAM",          "#spam");
define("CMS_CATEGORY_LEARNING_THRESHOLD",  50);
define("CMS_CATEGORY_TOKEN_LIMIT",         15);
define("CMS_CATEGORY_DEVIATION_THRESHOLD", 10);
define("CMS_CATEGORY_PROBABILITY_DEFAULT", 40);

//database
define("CMS_DB_CATEGORY_META",             CMS_DB_PREFIX . "category_meta");
define("CMS_DB_CATEGORY_META_INDEX",       "id");
define("CMS_DB_CATEGORY_META_CLASS",       "class");
define("CMS_DB_CATEGORY_META_COUNT_YES",   "count_yes");
define("CMS_DB_CATEGORY_META_COUNT_NO",    "count_no");

define("CMS_DB_CATEGORY",                  CMS_DB_PREFIX . "category");
define("CMS_DB_CATEGORY_INDEX",            "id");
define("CMS_DB_CATEGORY_TOKEN",            "token");
define("CMS_DB_CATEGORY_CLASS",            "class");
define("CMS_DB_CATEGORY_COUNT_YES",        "count_yes");
define("CMS_DB_CATEGORY_COUNT_NO",         "count_no");
define("CMS_DB_CATEGORY_PROBABILITY_YES",  "probability_yes");
define("CMS_DB_CATEGORY_PROBABILITY_NO",   "probability_no");
define("CMS_DB_CATEGORY_RATING",           "rating");
define("CMS_DB_CATEGORY_TIME",             "time");

//related functions
//-----------------------------------------------------------------------------------------------------
function category_tokenize_text($text)
{
        $array   = array();
        $string  = NULL;
        $_string = NULL;
        $count   = 0;
        $_count  = 0;
        $char    = NULL;
        $_char   = NULL;
        $__count = 0;

        for ($i = 0, $c = utf8_strlen($text); $i < $c; $i++)
        {
                $char = utf8_substr($text, $i, 1);

                //other, punctuation, separator
                if (preg_match("/[\p{C}\p{P}\p{Z}]/u", $char))
                {
                        //primary string length is greater than 2
                        if ($count > 2)
                        {
                                $array[] = sqlesc($string);

                                //secondary string length is more than 2
                                if ($_count > 2) $array[] = sqlesc($_string);
                                $_string = NULL;
                                $_count  = 0;

                        }

                        //primary string length is less than 3
                        else
                        {
                                //append to secondary string
                                $_string .= $string;
                                $_count  += $count;
                        };

                        $string  = NULL;
                        $count   = 0;
                }

                //visible character
                else
                {
                        //character repetition counter
                        if (streq($char, $_char))
                        {
                                $__count++;
                        }
                        else
                        {
                                $_char    = $char;
                                $__count  = 1;
                        };

                        //append to primary string if repetition counter is less than 3
                        if ($__count < 3)
                        {
                                $string  .= $char;
                                $count   += 1;
                        };
                };
        };

        if ($count > 2)  $array[] = sqlesc($string);
        if ($_count > 2) $array[] = sqlesc($_string);

        return array_unique($array);
};

class category
{
//properties
//-----------------------------------------------------------------------------------------------------
        public $mysql   = NULL;
        public $enabled = NULL;
//constructor
//-----------------------------------------------------------------------------------------------------
        function __construct()
        {
                $this->mysql = new mysql();

                if (
                     $this->mysql->verify_table(CMS_DB_CATEGORY_META,
                                                array(CMS_DB_CATEGORY_META_INDEX      => "INT UNSIGNED PRIMARY KEY AUTO_INCREMENT NOT NULL",
                                                      CMS_DB_CATEGORY_META_CLASS      => "VARCHAR(40) UNIQUE NOT NULL DEFAULT ''",
                                                      CMS_DB_CATEGORY_META_COUNT_YES  => "INT UNSIGNED NOT NULL DEFAULT '0'",
                                                      CMS_DB_CATEGORY_META_COUNT_NO   => "INT UNSIGNED NOT NULL DEFAULT '0'"))
                     &&
                     $this->mysql->verify_table(CMS_DB_CATEGORY,
                                                array(CMS_DB_CATEGORY_INDEX           => "INT UNSIGNED PRIMARY KEY AUTO_INCREMENT NOT NULL",
                                                      CMS_DB_CATEGORY_TOKEN           => "VARCHAR(40) NOT NULL DEFAULT ''",
                                                      CMS_DB_CATEGORY_CLASS           => "VARCHAR(40) NOT NULL DEFAULT ''",
                                                      CMS_DB_CATEGORY_COUNT_YES       => "INT UNSIGNED NOT NULL DEFAULT '0'",
                                                      CMS_DB_CATEGORY_COUNT_NO        => "INT UNSIGNED NOT NULL DEFAULT '0'",
                                                      CMS_DB_CATEGORY_PROBABILITY_YES => "TINYINT UNSIGNED NOT NULL DEFAULT '0'",
                                                      CMS_DB_CATEGORY_PROBABILITY_NO  => "TINYINT UNSIGNED NOT NULL DEFAULT '0'",
                                                      CMS_DB_CATEGORY_RATING          => "TINYINT UNSIGNED NOT NULL DEFAULT '" .
                                                                                         CMS_CATEGORY_PROBABILITY_DEFAULT . "'",
                                                      CMS_DB_CATEGORY_TIME            => "INT UNSIGNED NOT NULL DEFAULT '0', " .
                                                                                         "ADD UNIQUE (" .
                                                                                         CMS_DB_CATEGORY_TOKEN . "(40), " .
                                                                                         CMS_DB_CATEGORY_CLASS . "(40))"))
                   )
                        $this->enabled = TRUE;
        }
//methods
//-----------------------------------------------------------------------------------------------------
        function train($text,
                       $class = "",
                       $valid = TRUE,
                       $undo  = FALSE)
        {
                if (! $this->enabled) return FALSE;

                //tokenize text
                $token = category_tokenize_text($text);
                if (count($token) === 0) return TRUE; //no tokens

                $field_meta = $valid ? CMS_DB_CATEGORY_META_COUNT_YES : CMS_DB_CATEGORY_META_COUNT_NO;
                $field      = $valid ? CMS_DB_CATEGORY_COUNT_YES      : CMS_DB_CATEGORY_COUNT_NO;
                $class      = sqlesc($class);

                if (

                //add meta class
                     mysql_query("INSERT IGNORE INTO " . CMS_DB_CATEGORY_META . " " .
                                                   "(" . CMS_DB_CATEGORY_CLASS . ")" .
                                 "VALUES ('" .           $class . "')")
                     &&

                //adjust meta count
                     mysql_query("UPDATE " . CMS_DB_CATEGORY_META . " " .

                //increase/decrease counter
                                 "SET " .    $field_meta .
                                 "=" .       $field_meta . ($undo ? "-" : "+") . "1 " .

                //restrict to class
                                 "WHERE " .  CMS_DB_CATEGORY_META_CLASS .
                                 "='" .      $class . "' " .

                //only subtract if counter is positive
                                 ($undo ? "AND $field_meta>0 " : "") .

                //limit to one operation
                                 "LIMIT 1")
                     &&

                //add token/class combinations
                     mysql_query("INSERT IGNORE INTO " . CMS_DB_CATEGORY . " " .
                                                   "(" . CMS_DB_CATEGORY_TOKEN . ", " .
                                                         CMS_DB_CATEGORY_CLASS . ") " .
                                 "VALUES (" .
                                                   "'" . implode("', '$class'), ('", $token) . "', " .
                                                   "'" . $class . "')")
                     &&

                //adjust count
                     mysql_query("UPDATE " . CMS_DB_CATEGORY . " " .

                //increase/decrease counter
                                 "SET " .    $field .
                                 "=" .       $field . ($undo ? "-" : "+") . "1, " .

                //set update timestamp
                                             CMS_DB_CATEGORY_TIME .
                                 "='" .      sqlesc(time()) . "' " .

                //restrict to tokens
                                 "WHERE " .  CMS_DB_CATEGORY_TOKEN . " " .
                                 "IN ('" .   implode("', '", $token) . "') " .

                //restrict to class
                                 "AND " .    CMS_DB_CATEGORY_CLASS .
                                 "='" .      $class . "' " .

                //only subtract if counter is positive
                                 ($undo ? "AND $field>0 " : "") .

                //limit to number of tokens
                                 "LIMIT " .  sqlesc(count($token)))
                   )
                        return TRUE;

                return FALSE;
        }
//-----------------------------------------------------------------------------------------------------
        function train_valid($text,
                             $class = "")
        {
                return $this->train($text, $class);
        }
//-----------------------------------------------------------------------------------------------------
        function undo_valid($text,
                            $class = "")
        {
                return $this->train($text, $class, TRUE, TRUE);
        }
//-----------------------------------------------------------------------------------------------------
        function train_invalid($text,
                               $class = "")
        {
                return $this->train($text, $class, FALSE);
        }
//-----------------------------------------------------------------------------------------------------
        function undo_invalid($text,
                              $class = "")
        {
                return $this->train($text, $class, FALSE, TRUE);
        }
//-----------------------------------------------------------------------------------------------------
        function train_spam($text)
        {
                return $this->train($text, CMS_CATEGORY_CLASS_SPAM);
        }
//-----------------------------------------------------------------------------------------------------
        function undo_spam($text)
        {
                return $this->train($text, CMS_CATEGORY_CLASS_SPAM, TRUE, TRUE);
        }
//-----------------------------------------------------------------------------------------------------
        function train_nospam($text)
        {
                return $this->train($text, CMS_CATEGORY_CLASS_SPAM, FALSE);
        }
//-----------------------------------------------------------------------------------------------------
        function undo_nospam($text)
        {
                return $this->train($text, CMS_CATEGORY_CLASS_SPAM, FALSE, TRUE);
        }
//-----------------------------------------------------------------------------------------------------
        function update()
        {
                if (! $this->enabled) return FALSE;

                //retrieve total count per class
                if ($result = mysql_query("SELECT " . CMS_DB_CATEGORY_META_CLASS . ", " .
                                                      CMS_DB_CATEGORY_META_COUNT_YES . ", " .
                                                      CMS_DB_CATEGORY_META_COUNT_NO . " " .
                                          "FROM " .   CMS_DB_CATEGORY_META))
                {
                        //process classes
                        while (list($class, $count_yes, $count_no) = mysql_fetch_row($result))

                                //update probability
                                mysql_query("UPDATE " . CMS_DB_CATEGORY . " " .

                                //compute relative positive rating
                                            "SET " .    CMS_DB_CATEGORY_PROBABILITY_YES .
                                            "=" .       (($count_yes === 0) ? "0, " :
                                            "100*" .    CMS_DB_CATEGORY_COUNT_YES .  "/" . (int)$count_yes . ", ") .

                                //compute relative negative rating
                                                        CMS_DB_CATEGORY_PROBABILITY_NO .
                                            "=" .       (($count_no  === 0) ? "0, " :
                                            "100*" .    CMS_DB_CATEGORY_COUNT_NO .   "/" . (int)$count_no . " " .

                                //restrict to selected current class
                                            "WHERE " .  CMS_DB_CATEGORY_CLASS .
                                            "='" .      sqlesc($class) . "'"));

                        //update rating
                        mysql_query("UPDATE " . CMS_DB_CATEGORY . " " .

                        //apply bayes formula
                                    "SET " .    CMS_DB_CATEGORY_RATING .
                                    "=100*" .   CMS_DB_CATEGORY_PROBABILITY_YES .
                                    "/(" .      CMS_DB_CATEGORY_PROBABILITY_YES .
                                    "+" .       CMS_DB_CATEGORY_PROBABILITY_NO . ")");

                        //clean up
                        mysql_query("DELETE FROM " .  CMS_DB_CATEGORY . " " .

                        //restrict to entries older than 30 days
                                    "WHERE " .        CMS_DB_CATEGORY_TIME .
                                    "<" .             sqlesc(time() - 2592000) . " " .

                        //restrict to insignificant ratings
                                    "AND (50-CAST(" . CMS_DB_CATEGORY_RATING . " AS SIGNED)) " .
                                    "<" .             CMS_CATEGORY_DEVIATION_THRESHOLD);

                        return TRUE;
                };

                return FALSE;
        }
//-----------------------------------------------------------------------------------------------------
        function evaluate($text,
                          $class = "")
        {
                if (! $this->enabled) return FALSE;

                //tokenize text
                $token = category_tokenize_text($text);
                $count = count($token);
                if ($count === 0) return CMS_CATEGORY_PROBABILITY_DEFAULT; //no tokens

                //check for minimum required number of positive and negative ratings
                if (! $result = mysql_query("SELECT " . CMS_DB_CATEGORY_META_COUNT_YES . ", " .
                                                        CMS_DB_CATEGORY_META_COUNT_NO . " " .
                                            "FROM " .   CMS_DB_CATEGORY_META . " " .
                                            "WHERE " .  CMS_DB_CATEGORY_META_CLASS .
                                            "='" .      sqlesc($class) . "' " .
                                            "LIMIT 1"))
                        return FALSE; //error

                if (
                     (! list($count_yes, $count_no) = mysql_fetch_row($result))
                     ||
                     ($count_yes < CMS_CATEGORY_LEARNING_THRESHOLD)
                     ||
                     ($count_no < CMS_CATEGORY_LEARNING_THRESHOLD)
                   )
                        return CMS_CATEGORY_PROBABILITY_DEFAULT; //learning mode

                //limit number of tokens to be used
                if ($count > CMS_CATEGORY_TOKEN_LIMIT) $count = CMS_CATEGORY_TOKEN_LIMIT;

                //retrieve most important matching tokens
                if ($result = mysql_query("SELECT " .      CMS_DB_CATEGORY_RATING . ", " .

                //compute deviation to indicate significance
                                          "ABS(50-CAST(" . CMS_DB_CATEGORY_RATING . " AS SIGNED)) AS `#relevance` " .
                              
                                          "FROM " .        CMS_DB_CATEGORY . " " .
                              
                //restrict to tokens
                                          "WHERE " .       CMS_DB_CATEGORY_TOKEN . " " .
                                          "IN ('" .        implode("', '", $token) . "') " .

                //restrict to class
                                          "AND " .         CMS_DB_CATEGORY_CLASS .
                                          "='" .           sqlesc($class) . "' " .

                //use significant ratings only
                                          "HAVING " .      "`#relevance`" .
                                          ">=" .           CMS_CATEGORY_DEVIATION_THRESHOLD . " " .

                //order by significance
                                          "ORDER BY " .    "`#relevance` DESC " .

                //limit to first n most significant ratings
                                          "LIMIT " .       $count))
                {
                        $_count = mysql_num_rows($result);
                        if ($_count === 0) return CMS_CATEGORY_PROBABILITY_DEFAULT; //no match

                        //add up ratings retrieved from matching tokens
                        $rating_avg = 0;
                        while (list($rating) = mysql_fetch_row($result))
                                $rating_avg += $rating;

                        //add default rating for remaining tokens
                        $rating_avg += ($count - $_count) * CMS_CATEGORY_PROBABILITY_DEFAULT;

                        //compute average rating
                        $rating_avg /= $count;

                        return round($rating_avg);
                };

                return FALSE; //error
        }
//-----------------------------------------------------------------------------------------------------
        function evaluate_spam($text)
        {
                return $this->evaluate($text, CMS_CATEGORY_CLASS_SPAM);
        }
};
?>
