<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

//-----------------------------------------------------------------------------------------------------
//===< COMMENT CLASS >=================================================================================
//-----------------------------------------------------------------------------------------------------

//permission
define("CMS_COMMENT_PERMISSION_OPERATOR", "operator");
define("CMS_COMMENT_PERMISSION_WRITER",   "writer");
define("CMS_COMMENT_PERMISSION_READER",   "reader");

//status
define("CMS_DB_COMMENT_STATUS_INACTIVE",  0);
define("CMS_DB_COMMENT_STATUS_ACTIVE",    1);
define("CMS_DB_COMMENT_STATUS_HIDDEN",    2);

//database
define("CMS_DB_COMMENT",                  CMS_DB_PREFIX . "comment");
define("CMS_DB_COMMENT_INDEX",            "id");
define("CMS_DB_COMMENT_INSTANCE",         "instance");
define("CMS_DB_COMMENT_STATUS",           "status");
define("CMS_DB_COMMENT_TIME",             "time");
define("CMS_DB_COMMENT_NAME",             "name");
define("CMS_DB_COMMENT_EMAIL",            "email");
define("CMS_DB_COMMENT_URL",              "url");
define("CMS_DB_COMMENT_TEXT",             "text");
define("CMS_DB_COMMENT_MD5",              "md5");
define("CMS_DB_COMMENT_RATING_VALUE",     "rating_value");
define("CMS_DB_COMMENT_RATING_COUNT",     "rating_count");
define("CMS_DB_COMMENT_RATING_USERID",    "rating_userid");
define("CMS_DB_COMMENT_SPAM_PROBABILITY", "spam_probability");

class comment
{
//properties
//-----------------------------------------------------------------------------------------------------
        public $instance       = "";
        public $enabled        = FALSE;
        public $operator       = FALSE;
        public $writer         = FALSE;
        public $reader         = FALSE;
        public $default_status = CMS_DB_COMMENT_STATUS_INACTIVE;
        public $spam_threshold = 95;
//constructor
//-----------------------------------------------------------------------------------------------------
        function __construct($instance = "")
        {
                $mysql          = new mysql();
                $this->instance = $instance;

                if ($mysql->verify_table(
                        CMS_DB_COMMENT,
                        array(CMS_DB_COMMENT_INDEX            => "INT UNSIGNED PRIMARY KEY AUTO_INCREMENT NOT NULL",
                              CMS_DB_COMMENT_INSTANCE         => "VARCHAR(40) NOT NULL DEFAULT ''",
                              CMS_DB_COMMENT_STATUS           => "CHAR(1) NOT NULL DEFAULT '" .
                                                                 sqlesc(CMS_DB_COMMENT_STATUS_INACTIVE) . "'",
                              CMS_DB_COMMENT_TIME             => "INT UNSIGNED NOT NULL DEFAULT '0'",
                              CMS_DB_COMMENT_NAME             => "VARCHAR(40) NOT NULL DEFAULT ''",
                              CMS_DB_COMMENT_EMAIL            => "VARCHAR(40) NOT NULL DEFAULT ''",
                              CMS_DB_COMMENT_URL              => "VARCHAR(40) NOT NULL DEFAULT ''",
                              CMS_DB_COMMENT_TEXT             => "TEXT NOT NULL DEFAULT ''",
                              CMS_DB_COMMENT_MD5              => "CHAR(32) NOT NULL DEFAULT ''",
                              CMS_DB_COMMENT_RATING_VALUE     => "INT NOT NULL DEFAULT '0'",
                              CMS_DB_COMMENT_RATING_COUNT     => "INT UNSIGNED NOT NULL DEFAULT '0'",
                              CMS_DB_COMMENT_RATING_USERID    => "TEXT NOT NULL DEFAULT ''",
                              CMS_DB_COMMENT_SPAM_PROBABILITY => "TINYINT UNSIGNED NOT NULL DEFAULT '0'")))
                {
                        $this->operator = cms_permission(CMS_COMMENT_PERMISSION_OPERATOR . ".$instance");
                        $this->writer   = cms_permission(CMS_COMMENT_PERMISSION_WRITER . ".$instance");
                        $this->reader   = cms_permission(CMS_COMMENT_PERMISSION_READER . ".$instance");
                        $this->enabled  = TRUE;
                };
        }
//methods
//-----------------------------------------------------------------------------------------------------
        function add($name,
                     $email,
                     $url,
                     $text)
        {
                if (! $this->enabled) return FALSE;
                if (! $this->writer)  return FALSE;

                //determine spam probability
                $spam_probability = 0;
                if ($flag = cms_load("category"))
                {
                        $category         = new category();
                        $spam_probability = $category->evaluate_spam($text);
                        if ($spam_probability > $this->spam_threshold) return -1; //spam
                };

                $time = time();
                $md5  = md5($text);

                //check for previous comment with same content
                if (! ($result = mysql_query("SELECT " . CMS_DB_COMMENT_INDEX . " " .
                                             "FROM " .   CMS_DB_COMMENT . " " .
                                             "WHERE " .  CMS_DB_COMMENT_INSTANCE .
                                             "='" .      sqlesc($this->instance) . "' " .
                                             "AND " .    CMS_DB_COMMENT_TIME .
                                             ">='" .     sqlesc($time - 3600) . "' " . //1 hour
                                             "AND " .    CMS_DB_COMMENT_MD5 .
                                             "='" .      sqlesc($md5) . "' " .
                                             "LIMIT 1")))
                        return FALSE;

                if (mysql_num_rows($result) > 0) return -2;

                //create comment
                if (mysql_query("INSERT " . CMS_DB_COMMENT . " " .
                                      "(" . CMS_DB_COMMENT_INSTANCE . ", " .
                                            CMS_DB_COMMENT_STATUS . ", " .
                                            CMS_DB_COMMENT_TIME . ", " .
                                            CMS_DB_COMMENT_NAME . ", " .
                                            CMS_DB_COMMENT_EMAIL . ", " .
                                            CMS_DB_COMMENT_URL . ", " .
                                            CMS_DB_COMMENT_TEXT . ", " .
                                            CMS_DB_COMMENT_MD5 . ", " .
                                            CMS_DB_COMMENT_SPAM_PROBABILITY . ") " .
                                "VALUES " .
                                     "('" . sqlesc($this->instance) . "', " .
                                      "'" . sqlesc($this->default_status) . "', " .
                                      "'" . sqlesc($time) . "', " .
                                      "'" . sqlesc($name) . "', " .
                                      "'" . sqlesc($email) . "', " .
                                      "'" . sqlesc($url) . "', " .
                                      "'" . sqlesc($text) . "', " .
                                      "'" . sqlesc($md5) . "', " .
                                      "'" . sqlesc($spam_probability) . "')"))
                {
                        $index = mysql_insert_id();

                        //automatically rate positively if active by default
                        if ($flag && ($this->default_status == CMS_DB_COMMENT_STATUS_ACTIVE))
                        {
                                $category->train_nospam($text);
                                $category->update();
                        };

                        //log
                        $log = new log();
                        $log->access(sprintf(CMS_L_COMMENT_001, $index, $name));
                        $log->user($name, NULL, NULL, NULL, NULL, $email);

                        return $index;
                };

                return FALSE;
        }
//-----------------------------------------------------------------------------------------------------
        function edit($index,
                      $name,
                      $email,
                      $url,
                      $text)
        {
                if (! $this->enabled)  return FALSE;
                if (! $this->operator) return FALSE;

                //edit comment
                return mysql_query("UPDATE " . CMS_DB_COMMENT . " " .
                                   "SET " .    CMS_DB_COMMENT_NAME .  "='" . sqlesc($name) . "', " .
                                               CMS_DB_COMMENT_EMAIL . "='" . sqlesc($email) . "', " .
                                               CMS_DB_COMMENT_URL .   "='" . sqlesc($url) . "', " .
                                               CMS_DB_COMMENT_TEXT .  "='" . sqlesc($text) . "', " .
                                               CMS_DB_COMMENT_MD5 .   "='" . sqlesc(md5($text)) . "' " .
                                   "WHERE " .  CMS_DB_COMMENT_INDEX . "='" . sqlesc($index) . "' " .
                                   "LIMIT 1") ?
                       TRUE : FALSE;
        }
//-----------------------------------------------------------------------------------------------------
        function status($index,
                        $status)
        {
                if (! $this->enabled)  return FALSE;
                if (! $this->operator) return FALSE;

                if ($status == CMS_DB_COMMENT_STATUS_INACTIVE) return FALSE; //invalid value

                //check if current status equals designated status
                if (
                     ($result = mysql_query("SELECT " . CMS_DB_COMMENT_STATUS . ", " .
                                                        CMS_DB_COMMENT_EMAIL . ", " .
                                                        CMS_DB_COMMENT_TEXT . " " .
                                            "FROM " .   CMS_DB_COMMENT . " " .
                                            "WHERE " .  CMS_DB_COMMENT_INDEX .
                                            "='" .      sqlesc($index) . "' " .
                                            "LIMIT 1"))
                     &&
                     ($resultrow = mysql_fetch_assoc($result))
                   )
                {
                        if ($resultrow[CMS_DB_COMMENT_STATUS] == $status) return TRUE; //no change

                        if (cms_load("category"))
                        {
                                $category = new category();

                                switch ($status)
                                {
                                case CMS_DB_COMMENT_STATUS_HIDDEN:

                                        //undo previous training
                                        $category->undo_nospam($resultrow[CMS_DB_COMMENT_TEXT]);
                                        $category->update();
                                        break;

                                case CMS_DB_COMMENT_STATUS_ACTIVE:

                                        //train as good
                                        $category->train_nospam($resultrow[CMS_DB_COMMENT_TEXT]);
                                        $category->update();
                                        break;
                                };
                        };

                        if (mysql_query("UPDATE " . CMS_DB_COMMENT . " " .
                                        "SET " .    CMS_DB_COMMENT_STATUS .
                                        "='" .      sqlesc($status) . "' " .
                                        "WHERE " .  CMS_DB_COMMENT_INDEX  .
                                        "='" .      sqlesc($index) . "' " .
                                        "LIMIT 1"))
                                return TRUE;
                };

                return FALSE;
        }
//-----------------------------------------------------------------------------------------------------
        function rate_good($index)
        {
                if (! $this->enabled) return FALSE;
                if (! $this->reader)  return FALSE;

                $userid = sqlesc("/" . CMS_USERID . "/");
                return mysql_query("UPDATE " .     CMS_DB_COMMENT . " " .
                                   "SET " .        CMS_DB_COMMENT_RATING_VALUE .
                                   "=(" .          CMS_DB_COMMENT_RATING_VALUE . " + 1), " .
                                                   CMS_DB_COMMENT_RATING_COUNT .
                                   "=(" .          CMS_DB_COMMENT_RATING_COUNT . " + 1), " .
                                                   CMS_DB_COMMENT_RATING_USERID .
                                   "=CONCAT('" .   $userid . "', " . CMS_DB_COMMENT_RATING_USERID . ") " .
                                   "WHERE " .      CMS_DB_COMMENT_INDEX  .
                                   "='" .          sqlesc($index) . "' " .
                                   "AND " .        CMS_DB_COMMENT_RATING_USERID . " " .
                                   "NOT LIKE '%" . $userid . "%' " .
                                   "LIMIT 1") ?
                       TRUE : FALSE;
        }
//-----------------------------------------------------------------------------------------------------
        function rate_bad($index)
        {
                if (! $this->enabled) return FALSE;
                if (! $this->reader)  return FALSE;

                $userid = sqlesc("/" . CMS_USERID . "/");
                return mysql_query("UPDATE " .     CMS_DB_COMMENT . " " .
                                   "SET " .        CMS_DB_COMMENT_RATING_VALUE .
                                   "=(" .          CMS_DB_COMMENT_RATING_VALUE . " - 1), " .
                                                   CMS_DB_COMMENT_RATING_COUNT .
                                   "=(" .          CMS_DB_COMMENT_RATING_COUNT . " + 1), " .
                                                   CMS_DB_COMMENT_RATING_USERID .
                                   "=CONCAT('" .   $userid . "', " . CMS_DB_COMMENT_RATING_USERID . ") " .
                                   "WHERE " .      CMS_DB_COMMENT_INDEX  .
                                   "='" .          sqlesc($index) . "' " .
                                   "AND " .        CMS_DB_COMMENT_RATING_USERID . " " .
                                   "NOT LIKE '%" . $userid . "%' " .
                                   "LIMIT 1") ?
                       TRUE : FALSE;
        }
//-----------------------------------------------------------------------------------------------------
        function delete($index,
                        $spam = FALSE)
        {
                if (! $this->enabled)  return FALSE;
                if (! $this->operator) return FALSE;

                //train spam filter
                if (
                     $spam
                     &&
                     cms_load("category")
                     &&
                     ($result = mysql_query("SELECT " . CMS_DB_COMMENT_STATUS . ", " .
                                                        CMS_DB_COMMENT_TEXT . " " .
                                            "FROM " .   CMS_DB_COMMENT . " " .
                                            "WHERE " .  CMS_DB_COMMENT_INDEX .
                                            "='" .      sqlesc($index) . "' " .
                                            "LIMIT 1"))
                     &&
                     ($resultrow = mysql_fetch_assoc($result))
                   )
                {
                        $category = new category();

                        //undo previous spam filter training
                        if ($resultrow[CMS_DB_COMMENT_STATUS] == CMS_DB_COMMENT_STATUS_ACTIVE)
                                $category->undo_nospam($resultrow[CMS_DB_COMMENT_TEXT]);

                        $category->train_spam($resultrow[CMS_DB_COMMENT_TEXT]);
                        $category->update();
                };

                return mysql_query("DELETE FROM " . CMS_DB_COMMENT . " " .
                                   "WHERE " .       CMS_DB_COMMENT_INDEX .
                                   "='" .           sqlesc($index) . "' " .
                                   "LIMIT 1") ?
                       TRUE : FALSE;
        }
};
?>