<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

//-----------------------------------------------------------------------------------------------------
//===< CORE_RESOURCE CLASS >===========================================================================
//-----------------------------------------------------------------------------------------------------
class core_resource
{
//properties
//-----------------------------------------------------------------------------------------------------
        public $file             = "";
        public $hfile            = NULL;
        public $field_type       = NULL;
        public $field_offset     = NULL;
        public $field_length     = NULL;
        public $recordset_length = 0;
        public $offset           = 0;
//constructor
//-----------------------------------------------------------------------------------------------------
        function __construct($file,
                             $structure)
        {
                $offset = 0;

                reset($structure);
                while (list($key, $value) = each($structure))
                {
                        if (preg_match("/^(string|_string|byte)(?:\[([0-9]+)\])?$/", $value, $_value))
                        {
                                switch ($_value[1])
                                {
                                case "string":
//.....................................................................................................
                                        $length = ($_value[2] != "") ? (int)$_value[2]     : 1;
                                        break;

                                case "_string": //utf-8
//.....................................................................................................
                                        $length = ($_value[2] != "") ? (int)$_value[2] * 4 : 4;
                                        break;

//.....................................................................................................
                                case "byte":
                                        $length = 1;
                                        break;
                                };

                                $this->field_type[$key]    = $_value[1];
                                $this->field_offset[$key]  = $offset;
                                $this->field_length[$key]  = $length;
                                $offset                   += $length;
                        };
                };

                $this->recordset_length = $offset;
                $this->file             = $file;

                if (is_file($file))
                {
                        $this->hfile = fopen($file, "r+b");
                }
                else
                {
                        $path        = dirname($file);
                        mkpath($path);
                        $this->hfile = fopen($file, "w+b");
                };

                set_file_buffer($this->hfile, 0);

                register_shutdown_function(array(&$this, "close"));
        }
//methods
//-----------------------------------------------------------------------------------------------------
        function current()
        {
                return ($this->offset < 0) ? 0 : $this->offset;
        }
//-----------------------------------------------------------------------------------------------------
        function reset()
        {
                $this->offset = -$this->recordset_length;
        }
//-----------------------------------------------------------------------------------------------------
        function next($filter = NULL,
                      $reset  = FALSE)
        {
                $filesize = filesize($this->file);
                $count    = is_array($filter) ? count($filter) : 0;

                if ($reset) $this->reset();

                while ($this->offset < $filesize)
                {
                        $this->offset += $this->recordset_length;

                        if ($count > 0)
                        {
                                $_count = 0;

                                reset($filter);
                                while (
                                        (list($key, $value) = each($filter))
                                        &&
                                        ($this->get($key) == $value)
                                      )
                                        $_count++;

                                if ($count === $_count) return TRUE;
                        }
                        else
                        {
                                return TRUE;
                        };
                };

                return FALSE;
        }
//-----------------------------------------------------------------------------------------------------
        function seek($filter,
                      $next = FALSE)
        {
                $filesize                 = filesize($this->file);
                $this->offset             = $this->current();
                if ($next) $this->offset += $this->recordset_length;
                $count                    = count($filter);
                $offset                   = $this->offset;
                $flag                     = TRUE;

                while ($flag || ($this->offset < $offset))
                {
                        $_count = 0;

                        reset($filter);
                        while (
                                (list($key, $value) = each($filter))
                                &&
                                ($this->get($key) == $value)
                              )
                                $_count++;

                        if ($count === $_count) return TRUE;

                        if ($this->offset >= $filesize)
                        {
                                $this->offset  = 0;
                                $flag          = FALSE;
                        }
                        else
                        {
                                $this->offset += $this->recordset_length;
                        };
                };

                return FALSE;
        }
//-----------------------------------------------------------------------------------------------------
        function get($key = NULL)
        {
                $offset = $this->current();

                if ($key !== NULL)
                {
                        if (isset($this->field_type[$key]))
                        {
                                fseek($this->hfile, $offset + $this->field_offset[$key]);
                                $value = fread($this->hfile, $this->field_length[$key]);
                                return $this->field_decode($key, $value);
                        };

                        return NULL;
                };

                reset($this->field_type);
                while (list($key) = each($this->field_type))
                {
                        fseek($this->hfile, $offset + $this->field_offset[$key]);
                        $value       = fread($this->hfile, $this->field_length[$key]);
                        $array[$key] = $this->field_decode($key, $value);
                };

                return $array;
        }
//-----------------------------------------------------------------------------------------------------
        function set($value)
        {
                $offset = $this->current();

                reset($value);
                while (list($key, $_value) = each($value))
                {
                        if (isset($this->field_type[$key]))
                        {
                                $_value = $this->field_encode($key, $_value);
                                $_value = pack("a" . $this->field_length[$key], $_value);
                                fseek($this->hfile, $offset + $this->field_offset[$key]);
                                fputs($this->hfile, $_value);
                        };
                };

                return TRUE;
        }
//-----------------------------------------------------------------------------------------------------
        function del()
        {
                fseek($this->hfile, $this->current());
                fputs($this->hfile, str_repeat("\x00", $this->recordset_length));

                return TRUE;
        }
//-----------------------------------------------------------------------------------------------------
        function field_encode($key,
                              $value)
        {
                switch ($this->field_type[$key])
                {
                case "string":  return trim($value);
                case "_string": return utf8_trim($value);
                case "byte":    return chr($value);
                };

                return $value;
        }
//-----------------------------------------------------------------------------------------------------
        function field_decode($key,
                              $value)
        {
                switch ($this->field_type[$key])
                {
                case "string":  return trim($value);
                case "_string": return utf8_trim($value);
                case "byte":    return ord($value);
                };

                return $value;
        }
//-----------------------------------------------------------------------------------------------------
        function close()
        {
                fclose($this->hfile);
        }
};
?>
