<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

//-----------------------------------------------------------------------------------------------------
//===< DESKTOP CLASS >=================================================================================
//-----------------------------------------------------------------------------------------------------
define("CMS_DESKTOP_TYPE_NONE",          0);
define("CMS_DESKTOP_TYPE_LINK",          1);
define("CMS_DESKTOP_TYPE_NOTE",          2);
define("CMS_DESKTOP_TYPE_APPOINTMENT",   4);
define("CMS_DESKTOP_TYPE_ADDRESS",       8);
define("CMS_DESKTOP_TYPE_CONTAINER",    16);
define("CMS_DESKTOP_TYPE_MAILBOX",      32);
define("CMS_DESKTOP_TYPE_ALL",         255);

class desktop
{
//properties
//-----------------------------------------------------------------------------------------------------
        public $user = NULL;
        public $data = NULL;
//constructor
//-----------------------------------------------------------------------------------------------------
        function __construct($user = NULL)
        {
                if ($user == NULL) $user = CMS_SUPERUSER;
                $this->user = $user;

                $this->data = new data("#desktop/" . r($user) . "/desktop");

                $flag       = 0;

                $this->data->move("first");

                while (
                        ($flag !== 3)
                        &&
                        ($key = $this->data->move("next"))
                      )
                {
                        switch ($this->data->get($key, "#subtype"))
                        {
                        case "mailbox":     $flag |= 1; break;
                        case "appointment": $flag |= 2; break;
                        case "address":     $flag |= 4; break;
                        };
                };


                if (! flag($flag, 1))
                {
                        $this->data->set_buffer(array(array("name"        => CMS_L_DESKTOP_001,
                                                            "#type"       => "container",
                                                            "#subtype"    => "mailbox",
                                                            "quickaccess" => TRUE),
                                                      array("#type"       => "/container")));
                        $this->data->insert();
                };

                if (! flag($flag, 2))
                {
                        $this->data->set_buffer(array(array("name"        => CMS_L_DESKTOP_002,
                                                            "#type"       => "container",
                                                            "#subtype"    => "appointment",
                                                            "quickaccess" => TRUE),
                                                      array("#type"       => "/container")));
                        $this->data->insert();
                };

                if (! flag($flag, 4))
                {
                        $this->data->set_buffer(array(array("name"        => CMS_L_DESKTOP_003,
                                                            "#type"       => "container",
                                                            "#subtype"    => "address",
                                                            "quickaccess" => TRUE),
                                                      array("#type"       => "/container")));
                        $this->data->insert();
                };

                if ($flag != 7) $this->data->save();
        }
//methods
//-----------------------------------------------------------------------------------------------------
        function object_get($index,
                            $property)
        {
                return $this->data->get($index, $property);
        }
//-----------------------------------------------------------------------------------------------------
        function object_set($index,
                            $property,
                            $value = NULL)
        {
                return $this->data->set($value, $index, $property);
        }
//-----------------------------------------------------------------------------------------------------
        function save()
        {
                return $this->data->save();
        }
//-----------------------------------------------------------------------------------------------------
        function create_object($index,
                               $type,
                               $name)
        {
                if (streq($type, "container"))
                        $this->data->set_buffer(array(array("name"  => $name,
                                                            "#type" => "container"),
                                                      array("#type" => "/container")));
                else
                        $this->data->set_buffer(array(array("name"  => $name,
                                                            "#type" => $type)));

                if (
                     ($return = $this->data->insert($index))
                     &&
                     $this->data->save()
                   )
                        return $return;

                return FALSE;
        }
//-----------------------------------------------------------------------------------------------------
        function move_object($source,
                             $target)
        {
                if (
                     $this->data->cut($source)
                     &&
                     (($return = $this->data->insert($target)) !== FALSE)
                     &&
                     $this->data->save()
                   )
                        return $return;

                return FALSE;
        }
//-----------------------------------------------------------------------------------------------------
        function delete_object($index)
        {
                if (! $this->data->del($index)) return FALSE;
                if (! $this->data->save())      return FALSE;

                $subtype = $this->data->get($index, "#subtype");

                //cleanup
                switch ($subtype)
                {
                case "mailbox":
//.....................................................................................................
                        if (cms_load("filemanager"))
                        {
                                $path = CMS_DATA_PATH . "#desktop/" . r($this->user) . "/$index/";
                                filemanager_delete($path);
                        };
                        break;

//.....................................................................................................
                default:
                        break;
                };

                return TRUE;
        }
//-----------------------------------------------------------------------------------------------------
        function object_type($index)
        {
                return $this->data->get($index, "#type");
        }
//-----------------------------------------------------------------------------------------------------
        function get_parent($index)
        {
                return $this->data->move("parent", $index);
        }
};
?>