<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

//-----------------------------------------------------------------------------------------------------
//===< DIRECTORY CLASS >===============================================================================
//-----------------------------------------------------------------------------------------------------
//related functions
//-----------------------------------------------------------------------------------------------------
function directory_get_canonical($index = NULL)
{
        if ($index === NULL) $index = 0;
        $language = (CMS_LANGUAGE != "") ? CMS_LANGUAGE . "." : "";
        $map      = new map("#system/" . $language . "directory.canonical");
        return $map->get_value($index);
};
//-----------------------------------------------------------------------------------------------------
function directory_flexview_display_function($flexview,
                                             $index,
                                             $open)
{
        if ($flag = streq($index, $flexview->index)) $open = TRUE;

        $title       = x($flexview->get_name($index));
        $name        = $title;
        $hidden      = ! empty($flexview->object[$index]["#data"]["hidden"]);
        $placeholder = ! empty($flexview->object[$index]["#data"]["placeholder"]);
        $used        = ! empty($flexview->object[$index]["#data"]["used"]);

        if ($hidden)
        {
                if ($placeholder)
                {
                        if ($used)
                        {
                                if ($flexview->icon_custom === FALSE)
                                        $icon = NULL;
                                elseif (count($flexview->object[$index]) > 2)
                                        $icon = $open ?
                                                "directory/icon_entryopen_hpu" :
                                                "directory/icon_entrycontainer_hpu";
                                else
                                        $icon = "directory/icon_entry_hpu";

                                $name = "<span style=\"COLOR:#008000;FONT-STYLE:italic\">$name</span>";
                        }
                        else
                        {
                                if ($flexview->icon_custom === FALSE)
                                        $icon = NULL;
                                elseif (count($flexview->object[$index]) > 2)
                                        $icon = $open ?
                                                "directory/icon_entryopen_hp" :
                                                "directory/icon_entrycontainer_hp";
                                else
                                        $icon = "directory/icon_entry_hp";

                                $name = "<span style=\"COLOR:#800000;FONT-STYLE:italic\">$name</span>";
                        };
                }
                else
                {
                        if ($flexview->icon_custom === FALSE)
                                $icon = NULL;
                        elseif (count($flexview->object[$index]) > 2)
                                $icon = $open ?
                                        "directory/icon_entryopen_h" :
                                        "directory/icon_entrycontainer_h";
                        else
                                $icon = "directory/icon_entry_h";

                        $name = "<span style=\"FONT-STYLE:italic\">$name</span>";
                };
        }
        elseif ($placeholder)
        {
                if ($used)
                {
                        if ($flexview->icon_custom === FALSE)
                                $icon = NULL;
                        elseif (count($flexview->object[$index]) > 2)
                                $icon = $open ?
                                        "directory/icon_entryopen_pu" :
                                        "directory/icon_entrycontainer_pu";
                        else
                                $icon = "directory/icon_entry_pu";

                        $name = "<span style=\"COLOR:#008000\">$name</span>";
                }
                else
                {
                        if ($flexview->icon_custom === FALSE)
                                $icon = NULL;
                        elseif (count($flexview->object[$index]) > 2)
                                $icon = $open ?
                                        "directory/icon_entryopen_p" :
                                        "directory/icon_entrycontainer_p";
                        else
                                $icon = "directory/icon_entry_p";

                        $name = "<span style=\"COLOR:#800000\">$name</span>";
                };
        }
        else
        {
                $icon = $flexview->get_icon($index, $open);
        };

        $display = $flexview->display;
        $array   = array("action"   => x($flexview->get_action($index)),
                         "title"    => $title,
                         "checkbox" => (strpos($flexview->display, "%checkbox%") !== FALSE) ?
                                       $flexview->get_checkbox($index) : "",
                         "mark"     => (($temp = $flexview->get_mark($index)) != NULL) ?
                                       image($temp) : "",
                         "icon"     => ($icon != NULL) ? image($icon) : "",
                         "name"     => $flag ? "<strong>$name</strong>" : $name,
                         "class"    => $open ? "active" : "");

        reset($array);
        while (list($key, $value) = each($array))
                $display = ($value != "") ?
                           preg_replace("/(\\[([^\\[\\]]*))?%$key%(([^\\[\\]]*)\\])?/", "\${2}$value\\4", $display) :
                           preg_replace("/(\\[[^\\[\\]]*)?%$key%([^\\[\\]]*\\])?/",     "",               $display);

        echo($display);
};
//-----------------------------------------------------------------------------------------------------
function directory_get_select()
{
        $data = new data("#system/directory");

        $array["[" . CMS_L_DIRECTORY_002 . "]"] = NULL;
        $indentation                            = 0;

        $data->move("first");
        while (($key = $data->move("next")) !== NULL)
        {
                $name = l($data->get($key, "name"));
                if ($name == "") $name = CMS_L_UNKNOWN;

                switch ($data->get($key, "#type"))
                {
                case "container":
//.....................................................................................................
                        if ($data->get($key, "url") == "") $name  = "($name)";

                        $name           = str_repeat(" ", $indentation) . $name;
                        $_name          = $name;
                        $i              = 1;

                        while (isset($array[$_name]))      $_name = $name . " (" . $i++ . ")";

                        $array[$_name]  = $key;
                        $indentation   += 2;
                        break;

                case "/container":
//.....................................................................................................
                        $indentation   -= 2;
                };
        };

        return $array;
};
//-----------------------------------------------------------------------------------------------------
function directory_get_type()
{
        $data  = new data("#system/directory.type");

        $array = array();

        $data->move("first");
        while(($key = $data->move("next")) !== NULL)
        {
                if (($format = $data->get($key, "format"))  != "")
                        $array[$key]    = CMS_DATA_URL . "directory.type/$key$format";
                if (($format = $data->get($key, "+format")) != "")
                        $array["+$key"] = CMS_DATA_URL . "directory.type/+$key$format";
        };

        return $array;
};
//-----------------------------------------------------------------------------------------------------
function directory_get_type_select()
{
        $data       = new data("#system/directory.type");

        $array[" "] = NULL;

        $data->move("first");
        while (($key = $data->move("next")) !== NULL)
        {
                $name          = $data->get($key, "name");
                if ($name == "") $name = CMS_L_UNKNOWN;

                $_name         = $name;
                $i             = 1;
                while (isset($array[$_name]))
                        $_name = "$name (" . $i++ . ")";

                $array[$_name] = $key;
        };

        ksort($array);

        return $array;
};
//-----------------------------------------------------------------------------------------------------
function directory_get_visible($index = NULL)
{
        if (($index === NULL) && defined("CMS_CONTENT_DIRECTORY_INDEX"))
                $index = CMS_CONTENT_DIRECTORY_INDEX;

        $data   = new data("#system/directory");
        if ($data->get($index) === NULL) return 0; //entry does not exist
        $stack  = array(0 => FALSE);
        $key    = 0;
        $hidden = FALSE;

        $data->move("first");
        while (($_key = $data->move("next")) !== NULL)
        {
                switch ($data->get($_key, "#type"))
                {
                case "container":
//.....................................................................................................
                        array_push($stack, $hidden);
                        $hidden |= (bool)$data->get($_key, "hidden");
                        if (! $hidden)            $key = $_key;
                        if (streq($_key, $index)) break 2;
                        break;

                case "/container":
//.....................................................................................................
                        $hidden  = array_pop($stack);
                };
        };

        return $key;
};
//-----------------------------------------------------------------------------------------------------
function directory_get_flexview($remove_hidden = TRUE)
{
        if (! cms_load("flexview")) return FALSE;

        $flexview = new flexview();
        $data     = new data("#system/directory");
        $language = (CMS_LANGUAGE != "") ? CMS_LANGUAGE . "." : "";
        $map      = new map("#system/" . $language . "directory");
        $array    = array(0 => 0);
        $count    = 0;

        $flexview->set_map($map);

        $data->move("first");
        while (($key = $data->move("next")) !== NULL)
        {
                switch ($data->get($key, "#type"))
                {
                case "container":
//.....................................................................................................
                        if (
                             (
                               $remove_hidden
                               &&
                               ($data->get($key, "hidden") == TRUE)
                             )
                             ||
                             (
                               ($data->get($key, "placeholder") == TRUE)
                               &&
                               ($data->get($key, "used") == FALSE)
                             )
                             ||
                             (($name = l($data->get($key, "name"))) == "")
                           )
                        {
                                $data->del($key);
                        }
                        else
                        {
                                $flexview->set($key, $data->get($key), $array[$count]);
                                $array[++$count] = $key;
                        };

                        break;

                case "/container":
//.....................................................................................................
                        unset($array[$count]);
                        $count--;
                };
        };

        return $flexview;
};
//-----------------------------------------------------------------------------------------------------
function directory_create_filesystem()
{
        //revert logged entries
        directory_remove_filesystem();

        //get auxiliary files
        $auxiliary = array();
        $path_aux  = CMS_DATA_PATH . "#directory/auxiliary/";

        if (
             is_dir($path_aux)
             &&
             (($hdir = opendir($path_aux)) !== FALSE)
           )
        {
                while (($file_object = readdir($hdir)) !== FALSE)
                {
                        if (! is_file($path_aux . $file_object))     continue;
                        if (preg_match("/^\.{1,2}$/", $file_object)) continue;

                        $auxiliary[] = $file_object;
                };

                closedir($hdir);
        };

        //open log
        if (($hfile = fopen(CMS_ROOT_PATH . "directory.log", "wb")) === FALSE) return FALSE;

        //initialize
        $data           = new data("#system/directory");
        $key            = $data->move("first");
        $path           = array();
        $map_path       = array();
        $canonical      = array();
        $_canonical     = $data->get($key, "canonical");
        $list_url       = array("index.php" => 0);
        $list_index     = array(0 => 0);
        $list_canonical = array();
        $list_depth     = array();
        $list_length    = array();
        $reference      = array();
        $language       = (CMS_LANGUAGE_ENABLED != "") ? explode(",", CMS_LANGUAGE_ENABLED) : array(0);
        $directory      = array();
        $file           = array();
        $content        = substr(CMS_MODULES_PATH, strlen(CMS_ROOT_PATH)) . "content.php";

        //initialize by language
        reset($language);
        while (list(, $value) = each($language))
        {
                $path[$value]           = ($value === 0) ? array() : array(stringtofilename($value));
                $map_path[$value]       = new map();
                $map_path[$value]->set("0", "index.php");
                $__canonical            = language_get($_canonical, $value);
                if (stre($__canonical)) $__canonical = CMS_ROOT_URL;
                $canonical[$value]      = $__canonical;
                $list_canonical[$value] = array(0 => $__canonical);
                $list_depth[$value]     = array(0 => substr_count($__canonical, "/"));
                $list_length[$value]    = array(0 => strlen($__canonical));
        };

        //create index file
        if (($_hfile = fopen(CMS_ROOT_PATH . "index.php", "wb")) !== FALSE)
        {
                fputs($_hfile,
                      "<?php " .
                      "namespace cms; " .
                      "define(__NAMESPACE__ . \"\\CMS_CONTENT_EXTERNAL\", TRUE); " .
                      "require(\"$content\"); " .
                      "?>");
                fclose($_hfile);
        };

        while (($key = $data->move("next")) !== NULL)
        {
                $type = $data->get($key, "#type");

                switch ($type)
                {
                case "container":
//.....................................................................................................

                        //skip unused placeholder entries
                        if (
                             ($data->get($key, "placeholder") == TRUE)
                             &&
                             ($data->get($key, "used") == FALSE)
                           )
                        {
                                $depth = 1;

                                while (($depth > 0) && (($key = $data->move("next")) !== NULL))
                                {
                                        $type = $data->get($key, "#type");

                                        switch ($type)
                                        {
                                        case "container":  $depth++; break;
                                        case "/container": $depth--; break;
                                        };
                                };

                                break;
                        };

                        //proceed with entry
                        $name       = $data->get($key, "name");
                        $url        = $data->get($key, "url");
                        $_path      = $data->get($key, "path");
                        $_canonical = $data->get($key, "canonical");
                        $flag       = FALSE;

                        //directory reference
                        if (streq(substr($url, 0, 12), "directory://"))
                        {
                                $_url            = analyze_url($url);
                                $reference[$key] = $_url["host"];
                                $flag            = TRUE;
                        };

                        reset($path);
                        while (list($_key, $value) = each($path))
                        {
                                if ($_key === 0) $_key = FALSE;
                                $value = implode("/", $value);

                                //create directory
                                if (($value !== "") && (! isset($directory[$value])))
                                {
                                        //generate unique name
                                        $buffer = $value;
                                        $i      = 1;

                                        while (
                                                (! isset($directory[$buffer]))
                                                &&
                                                is_dir(CMS_ROOT_PATH . $buffer)
                                              )
                                                $buffer = $value . "-" . $i++;

                                        if (
                                             (! isset($directory[$buffer]))
                                             &&
                                             nstreq($type, "/container")
                                           )
                                        {
                                                //create directory
                                                if (! mkdir(CMS_ROOT_PATH . $buffer)) break 3;

                                                //write log entry
                                                fputs($hfile, "$buffer\n");

                                                //memorize directory
                                                $directory[$buffer] = TRUE;

                                                //copy auxiliary files
                                                reset($auxiliary);
                                                while (list(, $value) = each($auxiliary))
                                                {
                                                        if (copy($path_aux . $value,
                                                                 CMS_ROOT_PATH . "$buffer/$value"))

                                                                //write log entry
                                                                fputs($hfile, "$buffer/$value\n");
                                                };
                                        };

                                        //modify directory name in path
                                        $value           = $buffer;
                                        $i               = count($path[$_key]) - 1;
                                        $path[$_key][$i] = basename($value);
                                };

                                //retrieve entry name
                                $_name         = language_get($name, $_key);
                                $_name         = stringtofilename($_name);

                                //add entry name to path
                                $path[$_key][] = $_name;
                                $depth         = count($path[$_key]);

                                //override path
                                if (($__path = language_get($_path, $_key)) != "")
                                {
                                        $__path = explode("/", $__path);
                                        $depth  = count($__path);
                                        $value  = "";
                                        $_name  = array_pop($__path);
                                        $_name  = stringtofilename($_name);
                                        $_name  = strtolower($_name);

                                        //create directory
                                        reset($__path);
                                        while (list($__key, $_value) = each($__path))
                                        {
                                                $_value  = stringtofilename($_value);
                                                $_value  = strtolower($_value);
                                                $value  .= ($value != "") ? "/$_value" : $_value;

                                                //generate unique name
                                                $buffer  = $value;
                                                $i       = 1;

                                                while (
                                                        (! isset($directory[$buffer]))
                                                        &&
                                                        is_dir(CMS_ROOT_PATH . $buffer)
                                                      )
                                                        $buffer = $value . "-" . $i++;

                                                if (! isset($directory[$buffer]))
                                                {
                                                        //create directory
                                                        if (! mkdir(CMS_ROOT_PATH . $buffer)) break 3;

                                                        //write log entry
                                                        fputs($hfile, "$buffer\n");

                                                        //memorize directory
                                                        $directory[$buffer] = TRUE;

                                                        //copy auxiliary files
                                                        reset($auxiliary);
                                                        while (list(, $value) = each($auxiliary))
                                                        {
                                                                if (copy($path_aux . $value,
                                                                         CMS_ROOT_PATH . "$buffer/$value"))

                                                                        //write log entry
                                                                        fputs($hfile, "$buffer/$value\n");
                                                        };
                                                };

                                                $value = $buffer;
                                        };
                                };

                                //override canonical
                                if (nstre($__canonical = language_get($_canonical, $_key)))
                                        $canonical[$_key] = $__canonical;
                                else
                                        $__canonical      = $canonical[$_key];

                                if ($flag) continue; //skip reference processing

                                //generate unique name
                                $value  = ($value != "") ? "$value/$_name" : $_name;
                                $buffer = $value;
                                $i      = 1;

                                while (
                                        isset($file[$buffer])
                                        ||
                                        is_file(CMS_ROOT_PATH . "$buffer.php")
                                      )
                                        $buffer = $value . "-" . $i++;

                                //create file
                                if (($_hfile = fopen(CMS_ROOT_PATH . "$buffer.php", "wb")) === FALSE) break 3;

                                //memorize file
                                $file[$buffer] = TRUE;

                                //map entry path
                                $map_path[$_key]->set($key, "$buffer.php");

                                fputs($_hfile,
                                      "<?php " .
                                      "namespace cms; " .
                                      "define(__NAMESPACE__ . \"\\CMS_CONTENT_EXTERNAL\", TRUE); " .
                                      "\$_GET[\"content_directory_index\"] = \"$key\"; " .
                                      "\$_GET[\"content_display\"] = \"directory\"; " .
                                      "\$cms_select_language = \"$_key\"; " .
                                      "require(\"" . str_repeat("../", $depth - 1) . $content . "\"); " .
                                      "?>");
                                fclose($_hfile);

                                //write log entry
                                fputs($hfile, "$buffer.php\n");

                                //canonical
                                $__canonical = absolute_path($__canonical, cms_path_urlencode($buffer) . ".php");
                                $depth       = substr_count($__canonical, "/");
                                $length      = strlen($__canonical);

                                if (isset($list_url[$url])) //repeating url
                                {
                                        $__key            = $list_url[$url];
                                        $list_index[$key] = $__key;

                                        if (isset($list_canonical[$_key][$__key]))
                                        {
                                                if ($list_depth[$_key][$__key]  <  $depth)  continue;
                                                if ($list_length[$_key][$__key] <= $length) continue;
                                        };
                                }
                                else //new url occurrence
                                {
                                        $__key            = count($list_url);
                                        $list_url[$url]   = $__key;
                                        $list_index[$key] = $__key;
                                };

                                $list_canonical[$_key][$__key] = $__canonical;
                                $list_depth[$_key][$__key]     = $depth;
                                $list_length[$_key][$__key]    = $length;
                        };

                        break;

                case "/container":
//.....................................................................................................
                        reset($path);
                        while (list($_key) = each($path))
                                array_pop($path[$_key]);
                };
        };

        //directory dereferencing
        reset($reference);
        while (list($key, $value) = each($reference))
        {
                while(isset($reference[$value]) && nstreq($value, $key))
                        $value = $reference[$value];

                if (streq($key, $value)) continue; //self reference

                //map entry
                reset($language);
                while (list(, $_value) = each($language))
                {
                        //path
                        $__value = $map_path[$_value]->get_value($value);
                        if ($__value !== NULL) $map_path[$_value]->set($key, $__value);
                };

                //canonical
                $list_index[$key] = $list_index[$value];
        };

        //save maps
        reset($language);
        while (list(, $value) = each($language))
        {
                //path
                $_value = ($value !== 0) ? "$value." : "";
                $map_path[$value]->save("#system/" . $_value . "directory");

                //canonical
                $map_canonical = new map();
                reset($list_index);
                while (list($key, $__value) = each($list_index))
                        $map_canonical->set($key, $list_canonical[$value][$__value]);
                $map_canonical->save("#system/" . $_value . "directory.canonical");
        };

        //close log
        fclose($hfile);

        return TRUE;
};
//-----------------------------------------------------------------------------------------------------
function directory_remove_filesystem()
{
        //revert logged entries
        if (($array = @file(CMS_ROOT_PATH . "directory.log")) !== FALSE)
        {
                $path = end($array);

                while (($path = rtrim($path)) != "")
                {
                        $path = CMS_ROOT_PATH . $path;

                        switch (@filetype($path))
                        {
                        case "file":
//.....................................................................................................
                                unlink($path);
                                break;

                        case "dir":
//.....................................................................................................
                                rmdir($path);
                        };

                        $path = prev($array);
                };
        };

        clearstatcache();
};

class directory
{
//properties
//-----------------------------------------------------------------------------------------------------
        public $data = NULL;
//constructor
//-----------------------------------------------------------------------------------------------------
        function __construct()
        {
                $this->data = new data("#system/directory");
        }
//methods
//-----------------------------------------------------------------------------------------------------
        function append($key,
                        $name,
                        $description  = "",
                        $url          = "",
                        $subtype      = "",
                        $hidden       = FALSE,
                        $placeholder  = FALSE,
                        $dynamic      = TRUE,
                        $image_button = "",
                        $image_hover  = "",
                        $image_active = "",
                        $path         = "",
                        $canonical    = "")
        {
                $this->data->set_buffer(array(array("name"         => $name,
                                                    "#type"        => "container",
                                                    "#subtype"     => $subtype,
                                                    "description"  => $description,
                                                    "url"          => $url,
                                                    "hidden"       => (bool)$hidden,
                                                    "placeholder"  => (bool)$placeholder,
                                                    "dynamic"      => (bool)$dynamic,
                                                    "image_button" => $image_button,
                                                    "image_hover"  => $image_hover,
                                                    "image_active" => $image_active,
                                                    "path"         => $path,
                                                    "canonical"    => $canonical),
                                              array("#type"        => "/container")));

                return $this->data->append($key);
        }
//-----------------------------------------------------------------------------------------------------
        function insert($key,
                        $name,
                        $description  = "",
                        $url          = "",
                        $subtype      = "",
                        $hidden       = FALSE,
                        $placeholder  = FALSE,
                        $dynamic      = TRUE,
                        $image_button = "",
                        $image_hover  = "",
                        $image_active = "",
                        $path         = "",
                        $canonical    = "")
        {
                $this->data->set_buffer(array(array("name"         => $name,
                                                    "#type"        => "container",
                                                    "#subtype"     => $subtype,
                                                    "description"  => $description,
                                                    "url"          => $url,
                                                    "hidden"       => (bool)$hidden,
                                                    "placeholder"  => (bool)$placeholder,
                                                    "dynamic"      => (bool)$dynamic,
                                                    "image_button" => $image_button,
                                                    "image_hover"  => $image_hover,
                                                    "image_active" => $image_active,
                                                    "path"         => $path,
                                                    "canonical"    => $canonical),
                                              array("#type"        => "/container")));

                return $this->data->insert($key);
        }
//-----------------------------------------------------------------------------------------------------
        function set($key,
                     $name         = NULL,
                     $description  = NULL,
                     $url          = NULL,
                     $subtype      = NULL,
                     $hidden       = NULL,
                     $placeholder  = NULL,
                     $dynamic      = NULL,
                     $image_button = NULL,
                     $image_hover  = NULL,
                     $image_active = NULL,
                     $path         = NULL,
                     $canonical    = NULL)
        {
                if ($this->data->get($key) === NULL) return FALSE;

                if ($name         !== NULL) $this->data->set($name,              $key, "name");
                if ($description  !== NULL) $this->data->set($description,       $key, "description");
                if ($url          !== NULL) $this->data->set($url,               $key, "url");
                if ($subtype      !== NULL) $this->data->set($subtype,           $key, "#subtype");
                if ($hidden       !== NULL) $this->data->set((bool)$hidden,      $key, "hidden");
                if ($placeholder  !== NULL) $this->data->set((bool)$placeholder, $key, "placeholder");
                if ($dynamic      !== NULL) $this->data->set((bool)$dynamic,     $key, "dynamic");
                if ($image_button !== NULL) $this->data->set($image_button,      $key, "image_button");
                if ($image_hover  !== NULL) $this->data->set($image_hover,       $key, "image_hover");
                if ($image_active !== NULL) $this->data->set($image_active,      $key, "image_active");
                if ($path         !== NULL) $this->data->set($path,              $key, "path");
                if ($canonical    !== NULL) $this->data->set($canonical,         $key, "canonical");

                return $key;
        }
//-----------------------------------------------------------------------------------------------------
        function del($key)
        {
                return $this->data->del($key);
        }
//-----------------------------------------------------------------------------------------------------
        function parse_placeholder()
        {
                $array = array();
                $depth = 0;
                $limit = 0;

                $key   = $this->data->move("first");
                while ($key !== NULL)
                {
                        switch ($this->data->get($key, "#type"))
                        {
                        case "container":
                                $array[++$depth] = $key;
                                if ($this->data->get($key, "url") != "")               $limit = $depth;
                                break;

                        case "/container":
                                $key             = $array[$depth];
                                if ($depth > $limit)                                   $this->data->del($key, "used");
                                elseif ($this->data->get($key, "placeholder") == TRUE) $this->data->set(TRUE, $key, "used");
                                else                                                   $this->data->del($key, "used");
                                if (--$depth < $limit)                                 $limit = $depth;
                        };

                        $key = $this->data->move("next");
                };
        }
//-----------------------------------------------------------------------------------------------------
        function save()
        {
                $this->parse_placeholder();

                if ($this->data->save())
                {
                        directory_create_filesystem();
                        return TRUE;
                };

                return FALSE;
        }
};
?>