<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

//-----------------------------------------------------------------------------------------------------
//===< DOWNLOAD CLASS >================================================================================
//-----------------------------------------------------------------------------------------------------

//permission
define("CMS_DOWNLOAD_PERMISSION_OPERATOR", "operator");

//related functions
//-----------------------------------------------------------------------------------------------------
function download_get_array()
{
        $data  = new data("#system/download");
        $array = array("" => array());

        $data->move("first");
        while (($key = $data->move("next")) !== NULL)
        {
                $category = $data->get($key, "category");
                if ($category == "") $category = "";
                $name     = l($data->get($key, "name"));
                if ($name == "")     $name     = $key;

                $_name    = $name;
                $i        = 1;
                while (isset($array[$category][$_name]))
                        $_name = $name . " (" . $i++ . ")";

                $array[$category][$_name] = $key;
        };

        ksort($array);
        reset($array);
        while (list($key) = each($array))
                if (is_array($array[$key])) ksort($array[$key]);

        return $array;
};
//-----------------------------------------------------------------------------------------------------
function download_get_select()
{
        $data  = new data("#system/download");
        $array = array("" => "");

        $data->move("first");
        while (($key = $data->move("next")) !== NULL)
        {
                $category = $data->get($key, "category");
                $array[($category != "") ? $category : ""] = $category;
        };

        ksort($array);

        return $array;
};

class download
{
//properties
//-----------------------------------------------------------------------------------------------------
        public $data     = NULL;
        public $operator = FALSE;
//constructor
//-----------------------------------------------------------------------------------------------------
        function __construct()
        {
                $this->data     = new data("#system/download");
                $this->operator = cms_permission(CMS_DOWNLOAD_PERMISSION_OPERATOR);

                mkpath(CMS_DATA_PATH . "#download");
        }
//methods
//-----------------------------------------------------------------------------------------------------
        function add($file,
                     $filename,
                     $name        = "",
                     $description = "",
                     $category    = "")
        {
                if (
                     $this->operator
                     &&
                     is_file($file)
                     &&
                     (($extension = utf8_strtolower(strrchr($filename, "."))) !== "")
                   )
                {
                        if (language_get($name, FALSE) == "")
                                $name = language_set($name, $filename, FALSE);

                        $path     = CMS_DATA_PATH . "#download/";
                        $filename = unique_id() . $extension;

                        if (move_uploaded_file($file, $path . $filename))
                        {
                                $category = utf8_trim($category);

                                $this->data->set(array("name"        => $name,
                                                       "description" => $description,
                                                       "category"    => $category),
                                                 $filename);

                                if ($this->data->save()) return $filename;
                        };
                };

                return FALSE;
        }
//-----------------------------------------------------------------------------------------------------
        function replace($index,
                         $file,
                         $filename)
        {
                if (
                     $this->operator
                     &&
                     is_file($file)
                     &&
                     (($extension = utf8_strtolower(strrchr($filename, "."))) !== "")
                   )
                {
                        if (nstreq($extension, strrchr($index, "."))) return FALSE;

                        $path = CMS_DATA_PATH . "#download/";

                        if (move_uploaded_file($file, $path . $index))
                                return $this->data->save();
                };

                return FALSE;
        }
//-----------------------------------------------------------------------------------------------------
        function set($index,
                     $name        = NULL,
                     $description = NULL,
                     $category    = NULL)
        {
                if (! $this->operator) return FALSE;

                if ($name !== NULL)
                {
                        if (language_get($name, FALSE) == "")
                                $name = language_set($name, CMS_L_UNKNOWN, FALSE);
                        $this->data->set($name, $index, "name");
                };

                if ($description !== NULL)
                        $this->data->set($description, $index, "description");

                if ($category !== NULL)
                {
                        $category = utf8_trim($category);
                        $this->data->set($category, $index, "category");
                };

                return $this->data->save();
        }
//-----------------------------------------------------------------------------------------------------
        function unlink($index)
        {
                if (! $this->operator) return FALSE;

                $this->data->del($index);

                if ($this->data->save())
                {
                        $path = CMS_DATA_PATH . "#download/";

                        if (is_file($path . $index)) unlink($path . $index);
                        return TRUE;
                };

                return FALSE;
        }
};
?>