<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

//-----------------------------------------------------------------------------------------------------
//===< FILEMANAGER FUNCTIONS >=========================================================================
//-----------------------------------------------------------------------------------------------------
function flexview_compare($value1,
                          $value2)
{
        if (substr($value1, -1) === "/")
        {
                if (substr($value2, -1) !== "/") return -1;
        }
        else
        {
                if (substr($value2, -1) === "/") return  1;
        };

        if (($return = strcmp(strrchr($value1, "."), strrchr($value2, "."))) !== 0) return $return;
        return utf8_strcasecmp($value1, $value2);
};
//-----------------------------------------------------------------------------------------------------
function filemanager_sort(&$array)
{
        uasort($array, __NAMESPACE__ . "\\flexview_compare");
};
//-----------------------------------------------------------------------------------------------------
function filemanager_flexview($root = NULL,
                              $path = NULL)
{
        if ($root == NULL)                   $root = CMS_ROOT_PATH;
        if ($path == NULL)                   $path = $root;
        if (! $hdir[$root] = opendir($root)) return FALSE;
        $_path    = $root;

        $flexview = new flexview();
        while (TRUE)
        {
                while (
                        ($file_object = readdir($hdir[$root]))
                        &&
                        (
                          preg_match("/^\.{1,2}$/", $file_object)
                          ||
                          (
                            streq("$root$file_object/", CMS_PATH)
                            &&
                            nstreq(CMS_USER, "admin")
                          )
                        )
                      );

                if (! $file_object)
                {
                        closedir($hdir[$root]);

                        uksort($flexview->object[$parent], __NAMESPACE__ . "\\flexview_compare");

                        if (streq($root, $_path)) break;
                        $root = dirname($root) . "/";

                        continue;
                };

                if (streq($root, $_path)) $parent = "";
                else                      $parent = $root;

                $_root = "$root$file_object";

                if (is_dir($_root))
                {
                        $_root .= "/";

                        if ($hdir[$_root] = opendir($_root))
                        {
                                $flexview->set($_root,
                                               array("name"  => $file_object,
                                                     "#type" => "container"),
                                               $parent);

                                if (streq(substr($path, 0, strlen($_root)), $_root))
                                        $root .= "$file_object/";
                        };
                }
                else
                {
                        $extension = utf8_strtolower(substr(strrchr($file_object, "."), 1));
                        $flexview->set($root . $file_object,
                                       array("name"     => $file_object,
                                             "#type"    => "file",
                                             "#subtype" => $extension),
                                       $parent);
                };
        };

        return $flexview;
};
//-----------------------------------------------------------------------------------------------------
function filemanager_flexview_directory($root = NULL,
                                        $path = NULL)
{
        if (! $root)                         $root = CMS_ROOT_PATH;
        if (! $path)                         $path = $root;
        if (! $hdir[$root] = opendir($root)) return FALSE;
        $_path    = $root;

        $flexview = new flexview();
        while (TRUE)
        {
                while (
                        ($file_object = readdir($hdir[$root]))
                        &&
                        (
                          preg_match("/^\.{1,2}$/", $file_object)
                          ||
                          (! is_dir($root . $file_object))
                          ||
                          (
                            streq("$root$file_object/", CMS_PATH)
                            &&
                            (! streq(CMS_USER, "admin"))
                          )
                        )
                      );

                if (! $file_object)
                {
                        closedir($hdir[$root]);

                        uksort($flexview->object[$parent], __NAMESPACE__ . "\\flexview_compare");

                        if (streq($root, $_path)) break;
                        $root = dirname($root) . "/";

                        continue;
                };

                if (streq($root, $_path)) $parent = "";
                else                      $parent = $root;

                $_root = "$root$file_object/";

                if ($hdir[$_root] = opendir($_root))
                {
                        $flexview->set($_root,
                                       array("name"  => $file_object,
                                             "#type" => "container"),
                                       $parent);

                        if (streq(substr($path, 0, strlen($_root)), $_root))
                                $root .= "$file_object/";
                };
        };

        return $flexview;
};
//-----------------------------------------------------------------------------------------------------
function filemanager_get_select($root = NULL)
{
        if (! $root)                         $root = CMS_ROOT_PATH;
        if (! $hdir[$root] = opendir($root)) return FALSE;

        $i                                = 0;
        $c                                = strlen(CMS_ROOT_PATH);
        $path                             = $root;
        $array[(string)substr($path, $c)] = $root;

        while (TRUE)
        {
                while (
                        ($file_object = readdir($hdir[$path]))
                        &&
                        (
                          preg_match("/^\.{1,2}$/", $file_object)
                          ||
                          (! is_dir($path . $file_object))
                          ||
                          (
                            streq("$path$file_object/", CMS_PATH)
                            &&
                            (! streq(CMS_USER, "admin"))
                          )
                        )
                      );

                if (! $file_object)
                {
                        closedir($hdir[$path]);
                        if (streq($path, $root)) break;
                        $path  = dirname($path) . "/";
                        $i    -= 2;

                        continue;
                };

                $path = "$path$file_object/";

                if ($hdir[$path] = opendir($path))
                {
                        $file_object           = str_repeat(" ", $i) . $file_object;
                        $_file_object          = $file_object;
                        $_i                    = 1;

                        while (isset($array[$_file_object]))
                                $_file_object = "$file_object (" . $_i++ . ")";

                        $array[$_file_object]  = $path;
                        $i                    += 2;
                };
        };

        return $array;
};
//-----------------------------------------------------------------------------------------------------
function filemanager_copy($source_path,
                          $target_path)
{
        if (is_dir($source_path))
        {
                if (! $hdir = opendir($source_path)) return FALSE;

                $stack = array();

                while ($file_object = readdir($hdir))
                {
                        if (preg_match("/^\.{1,2}$/", $file_object)) continue;
                        array_push($stack, $file_object);
                };

                closedir($hdir);

                $mode = fileperms($source_path);
                if (! mkdir($target_path, $mode)) return FALSE;

                $error = FALSE;

                while ($file_object = array_pop($stack))
                        $error |= ! filemanager_copy("$source_path/$file_object",
                                                     "$target_path/$file_object");

                return ! $error;
        };

        return (! is_file($target_path)) && copy($source_path, $target_path);
};
//-----------------------------------------------------------------------------------------------------
function filemanager_delete($path)
{
        if (is_dir($path))
        {
                if (! $hdir = opendir($path)) return FALSE;

                $error = FALSE;

                while ($file_object = readdir($hdir))
                {
                        if (preg_match("/^\.{1,2}$/", $file_object)) continue;
                        $error |= ! filemanager_delete("$path/$file_object");
                };

                closedir($hdir);

                if (! rmdir($path)) return FALSE;
                return ! $error;
        };

        return unlink($path);
};
//-----------------------------------------------------------------------------------------------------
function filemanager_gzcompress($source,
                                $target = NULL)
{
        $flag = FALSE;

        if (
             is_file($source)
             &&
             ($hfile_source = fopen($source, "rb"))
           )
        {
                if (! $target)                         $target  = $source;
                if (! preg_match("/\.gz$/i", $target)) $target .= ".gz";

                if ($hfile_target = gzopen($target, "wb9"))
                {
                        while ($buffer = fread($hfile_source, 65536))
                                gzwrite($hfile_target, $buffer);

                        gzclose($hfile_target);

                        $flag = TRUE;
                };

                fclose($hfile_source);
        };

        return $flag ? $target : FALSE;
};
//-----------------------------------------------------------------------------------------------------
function filemanager_gzdecompress($source,
                                  $target = NULL)
{
        $flag = FALSE;

        if (
             is_file($source)
             &&
             preg_match("/\.gz$/i", $source)
             &&
             ($hfile_source = gzopen($source, "rb"))
           )
        {
                if (! $target) $target = substr($source, 0, -3);

                if ($hfile_target = fopen($target, "wb"))
                {
                        while ($buffer = gzread($hfile_source, 65536))
                                fputs($hfile_target, $buffer);

                        fclose($hfile_target);

                        $flag = TRUE;
                };

                gzclose($hfile_source);
        };

        return $flag ? $target : FALSE;
};
?>
