<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

//-----------------------------------------------------------------------------------------------------
//===< FLEXVIEW CLASS >================================================================================
//-----------------------------------------------------------------------------------------------------
define("CMS_FLEXVIEW_ENTRY_TYPE_NONE",  0);
define("CMS_FLEXVIEW_ENTRY_TYPE_BASE",  1);
define("CMS_FLEXVIEW_ENTRY_TYPE_ENTRY", 2);
define("CMS_FLEXVIEW_ENTRY_TYPE_END",   3);

class flexview_entry
{
//properties
//-----------------------------------------------------------------------------------------------------
        public $type        = CMS_FLEXVIEW_ENTRY_TYPE_NONE;
        public $index       = NULL;
        public $parent      = NULL;
        public $position    = 0;
        public $count       = 0;
        public $subcount    = 0;
        public $indentation = 0;
        public $open        = FALSE;
};

class flexview
{
//properties
//-----------------------------------------------------------------------------------------------------
        public $object              = array("" => array("#data" => array("#type" => "base")));
        public $icon_default        = NULL;
        public $encoding_function   = NULL;
        public $display_function    = NULL;
        public $display             = "%checkbox%<a[ href=\"%action%\"][ class=\"%class%\"]>%mark%[%icon% ]%name%</a>";
        public $index               = "";
        public $checkbox_identifier = NULL;
        public $checkbox_index      = 0;
        public $checkbox_list       = NULL;
        public $mark                = NULL;
        public $icon_custom         = NULL;
        public $action              = "%index%";
        public $name_key            = "name";
        public $image_button_key    = "image_button";
        public $image_hover_key     = "image_hover";
        public $image_active_key    = "image_active";
        public $description_key     = "description";
        public $base                = "";
        public $map                 = NULL;
        public $param               = NULL;
//constructor
//-----------------------------------------------------------------------------------------------------
        function __construct()
        {
                $data                             = new data("#system/flexview.icon");
                $application                      = cms_application();
                $this->icon_default["#base"]      = $data->get($application, "#base");
                $this->icon_default["entry"]      = $data->get($application, "entry");
                $this->icon_default["container"]  = $data->get($application, "container");
                $this->icon_default["+container"] = $data->get($application, "+container");
                if (! $this->icon_default["#base"])      $this->icon_default["#base"]      = "flexview/icon_base";
                if (! $this->icon_default["entry"])      $this->icon_default["entry"]      = "flexview/icon_entry";
                if (! $this->icon_default["container"])  $this->icon_default["container"]  = "flexview/icon_entrycontainer";
                if (! $this->icon_default["+container"]) $this->icon_default["+container"] = "flexview/icon_entryopen";
                $this->encoding_function          = __NAMESPACE__ . "\\x";
        }
//methods
//-----------------------------------------------------------------------------------------------------
        function set_encoding_function($callback_function)
        {
                if (function_exists($callback_function)) $this->encoding_function = $callback_function;
        }
//-----------------------------------------------------------------------------------------------------
        function set_display_function($callback_function)
        {
                if (function_exists($callback_function)) $this->display_function = $callback_function;
        }
//-----------------------------------------------------------------------------------------------------
        function set_index($value)
        {
                if (! isset($this->object[$value]))
                {
                        if (! isset($this->object[$this->base])) $this->base = "";
                        $value = $this->base;
                };

                $this->index = $value;
        }
//-----------------------------------------------------------------------------------------------------
        function set_checkbox_identifier($value)
        {
                $this->checkbox_identifier = $value;
                $this->checkbox_index      = 0;
        }
//-----------------------------------------------------------------------------------------------------
        function set_checkbox_list($value)
        {
                $this->checkbox_list = is_array($value) ? array_flip($value) : NULL;
        }
//-----------------------------------------------------------------------------------------------------
        function get_checkbox($index)
        {
                static $flag = TRUE;

                if (! $this->checkbox_identifier) return NULL;

                if ($flag)
                {
//javascript
?>
<script>
function flexview_checkbox_load(name)
{
        var input           = document.getElementsByName(name)[0];
        var label           = input.nextSibling;
        var image           = label.firstChild;
        var checked         = input.checked;

        label.onclick       = function() { this.previousSibling.click(); return false; };
        label.onmouseenter  = function()
        {
                if ((fx_keyboard_key == 16) || //shift
                    (fx_keyboard_key == 17) || //ctrl
                    (fx_keyboard_key == 18))   //alt
                        this.previousSibling.click();
        };
        label.className     = checked ? "fv-cb-on" : "fv-cb-off";

        input.onclick       = function() { flexview_checkbox_click(this); };
        input.className     = "fv-cb";

        image.className     = "fv-cb-space";
        image.style.display = "";
};
//................................
function flexview_checkbox_click(trigger)
{
        var label       = trigger.nextSibling;
        var checked     = trigger.checked;

        label.className = checked ? "fv-cb-on" : "fv-cb-off";
}
</script>
<?php
                        $flag = FALSE;
                };

                $name    = $this->checkbox_identifier . "[" . $this->checkbox_index++ . "]";
                $checked = isset($this->checkbox_list[$index]);
                $index   = $this->map ? $this->map->get_value($index) : $index;

                return "<input name=\"" . x($name) . "\" " .
                              "type=\"checkbox\" " .
                              "value=\"" . x($index) . "\"" .
                              ($checked ? " checked>" : ">") .
                       "<label>" .
                       "<img src=\"" . x(CMS_IMAGES_URL) . "blank.png\" " .
                            "alt=\"\" " .
                            "style=\"DISPLAY:none\">" .
                       "</label>" .
                       jscript("flexview_checkbox_load(\"" . q($name) . "\");");
        }
//-----------------------------------------------------------------------------------------------------
        function set_mark($value)
        {
                $this->mark = $value;
        }
//-----------------------------------------------------------------------------------------------------
        function get_mark($index)
        {
                return isset($this->mark[$index]) ? $this->mark[$index] : NULL;
        }
//-----------------------------------------------------------------------------------------------------
        function set_icon($value)
        {
                $this->icon_custom = $value;
        }
//-----------------------------------------------------------------------------------------------------
        function get_icon($index,
                          $open = FALSE)
        {
                if ($this->icon_custom === FALSE) return NULL;

                if (streq($index, $this->base))
                {
                        if (isset($this->icon_custom["#base"]))           return $this->icon_custom["#base"];
                        return $this->icon_default["#base"];
                };

                $open |= streq($index, $this->index);

                if (isset($this->object[$index]["#data"]["#subtype"]))
                {
                        $type = $this->object[$index]["#data"]["#subtype"];
                        if ($open && isset($this->icon_custom["+$type"])) return $this->icon_custom["+$type"];
                        if (isset($this->icon_custom[$type]))             return $this->icon_custom[$type];
                };

                if (isset($this->object[$index]["#data"]["#type"]))
                {
                        $type = $this->object[$index]["#data"]["#type"];
                        if ($open && isset($this->icon_custom["+$type"])) return $this->icon_custom["+$type"];
                        if (isset($this->icon_custom[$type]))             return $this->icon_custom[$type];
                };

                if (count($this->object[$index]) > 2)
                {
                        if ($open)                                        return $this->icon_default["+container"];
                        return $this->icon_default["container"];
                };

                return $this->icon_default["entry"];
        }
//-----------------------------------------------------------------------------------------------------
        function set_action($value)
        {
                $this->action = $value;
        }
//-----------------------------------------------------------------------------------------------------
        function get_action($index)
        {
                return str_replace("%index%", $this->get_value($index), $this->action);
        }
//-----------------------------------------------------------------------------------------------------
        function set_name_key($value)
        {
                $this->name_key = $value;
        }
//-----------------------------------------------------------------------------------------------------
        function get_name($index)
        {
                if (
                     isset($this->object[$index]["#data"][$this->name_key])
                     &&
                     ($name = l($this->object[$index]["#data"][$this->name_key]))
                   )
                        return $name;

                return streq($index, $this->base) ? "[" . CMS_L_FLEXVIEW_002 . "]" : CMS_L_UNKNOWN;
        }
//-----------------------------------------------------------------------------------------------------
        function set_image_key($button,
                               $hover,
                               $active)
        {
                $this->image_button_key = $button;
                $this->image_hover_key  = $hover;
                $this->image_active_key = $active;
        }
//-----------------------------------------------------------------------------------------------------
        function get_image($index,
                           $open = FALSE)
        {
                if (
                     $open
                     &&
                     isset($this->object[$index]["#data"][$this->image_active_key])
                     &&
                     ($image = l($this->object[$index]["#data"][$this->image_active_key]))
                   )
                        return translate_url($image);

                if (
                     isset($this->object[$index]["#data"][$this->image_button_key])
                     &&
                     ($image = l($this->object[$index]["#data"][$this->image_button_key]))
                   )
                        return translate_url($image);

                return NULL;
        }
//-----------------------------------------------------------------------------------------------------
        function set_base($value)
        {
                if (! isset($this->object[$value])) $value = "";
                $this->base = $value;
        }
//-----------------------------------------------------------------------------------------------------
        function set_map(&$map)
        {
                if (streq(get_class($map), __NAMESPACE__ . "\\map")) $this->map = $map;
        }
//-----------------------------------------------------------------------------------------------------
        function display($index,
                         $open = FALSE)
        {
                if ($flag = streq($index, $this->index)) $open = TRUE;

                if ($this->display_function)
                {
                        call_user_func($this->display_function, $this, $index, $open);
                        return;
                };

                $display   = $this->display;
                $name      = $this->get_name($index);
                $title     = x($name);
                $container = count($this->object[$index]) > 2;

                $array     = array("index"        => x($index),
                                   "action"       => x($this->get_action($index)),
                                   "title"        => $title,
                                   "image"        => ($temp = $this->get_image($index, $open)) ?
                                                     image($temp, NULL, NULL, $name) : NULL,
                                   "image_url"    => $temp ? x($temp) : NULL,
                                   "image_button" => isset($this->object[$index]["#data"][$this->image_button_key]) ?
                                                     x(translate_url(l($this->object[$index]["#data"][$this->image_button_key]))) : NULL,
                                   "image_hover"  => isset($this->object[$index]["#data"][$this->image_hover_key]) ?
                                                     x(translate_url(l($this->object[$index]["#data"][$this->image_hover_key])))  : NULL,
                                   "image_active" => isset($this->object[$index]["#data"][$this->image_active_key]) ?
                                                     x(translate_url(l($this->object[$index]["#data"][$this->image_active_key]))) : NULL,
                                   "checkbox"     => (strpos($this->display, "%checkbox%") === FALSE) ?
                                                     NULL : $this->get_checkbox($index),
                                   "mark"         => ($temp = $this->get_mark($index)) ?
                                                     image($temp) : NULL,
                                   "icon"         => ($temp = $this->get_icon($index, $open)) ?
                                                     image($temp) : NULL,
                                   "name"         => $flag ? "<strong>$title</strong>" : $title,
                                   "description"  => isset($this->object[$index]["#data"][$this->description_key]) ?
                                                     parse_text(l($this->object[$index]["#data"][$this->description_key])) : NULL,
                                   "class"        => ltrim(($open      ? "active"     : NULL) .
                                                           ($container ? " container" : NULL)));

                reset($array);
                while (list($key, $value) = each($array))
                        $display = stre($value) ?
                                   preg_replace("/(\\[[^\\[\\]]*)?%$key%([^\\[\\]]*\\])?/",     "",               $display) :
                                   preg_replace("/(\\[([^\\[\\]]*))?%$key%(([^\\[\\]]*)\\])?/", "\${2}$value\\4", $display);


                echo($display);
        }
//-----------------------------------------------------------------------------------------------------
        function get_value($index)
        {
                if ($this->map)               $index = $this->map->get_value($index);
                if ($this->encoding_function) $index = call_user_func($this->encoding_function, $index);

                return $index;
        }
//-----------------------------------------------------------------------------------------------------
        function set($index,
                     $data   = NULL,
                     $parent = "")
        {
                $this->object[$index]["#data"]   = $data;
                $this->object[$index]["#parent"] = $parent;
                $this->object[$parent][$index]   = &$this->object[$index];
        }
//-----------------------------------------------------------------------------------------------------
        function get_path()
        {
                $index = $this->index;
                $path  = array(0 => $index);

                while (isset($this->object[$index]["#parent"]))
                {
                        $index  = $this->object[$index]["#parent"];
                        if (streq($index, $this->base)) return array_reverse($path);
                        $path[] = $index;
                };

                return array();
        }
//-----------------------------------------------------------------------------------------------------
        function import_data(&$data)
        {
                $array = array(0 => "");
                $count = 0;

                $data->move("first");
                while ($key = $data->move("next"))
                {
                        switch ($data->get($key, "#type"))
                        {
                        case "container":
//.....................................................................................................
                                $this->set($key, $data->get($key), $array[$count]);
                                $array[++$count] = $key;
                                break;

                        default:
//.....................................................................................................
                                $this->set($key, $data->get($key), $array[$count]);
                                break;

                        case "/container":
//.....................................................................................................
                                unset($array[$count]);
                                $count--;
                        };
                };
        }
//-----------------------------------------------------------------------------------------------------
        function import_database(&$result,
                                 $index_key  = "id",
                                 $parent_key = "container")
        {
                while ($array = mysql_fetch_assoc($result))
                {
                        if ($index = $array[$index_key])
                        {
                                unset($array[$index_key]);

                                if ($parent_key)
                                {
                                        $parent = $array[$parent_key];
                                        unset($array[$parent_key]);
                                }
                                else
                                {
                                        $parent = "";
                                };
                        };

                        $this->set($index, $array, $parent);
                };
        }
//-----------------------------------------------------------------------------------------------------
        function show_custom($callback_function)
        {
                $key                         = $this->base;

                $flexview_entry              = new flexview_entry();
                $flexview_entry->type        = CMS_FLEXVIEW_ENTRY_TYPE_BASE;
                $flexview_entry->index       = $key;
                if (isset($this->object[$key]["#parent"]))
                {
                        $flexview_entry->parent   = $this->object[$key]["#parent"];
                        $flexview_entry->subcount = count($this->object[$key]) - 2;
                }
                else
                {
                        $flexview_entry->parent   = NULL;
                        $flexview_entry->subcount = count($this->object[$key]);
                };
                $flexview_entry->position    = 0;
                $flexview_entry->count       = 0;
                $flexview_entry->indentation = 0;
                $flexview_entry->open        = TRUE;

                $position                    = array(0 => 0);
                $count                       = array(0 => $flexview_entry->subcount);
                $indentation                 = 0;
                $path                        = $this->get_path();

                if (! call_user_func($callback_function, $flexview_entry))
                {
                        reset($this->object[$key]);
                        while (TRUE)
                        {
                                while (list($_key, $value) = each($this->object[$key]))
                                {
                                        if (preg_match("/^#(?:data|parent)$/", $_key)) continue;

                                        $position[$indentation]++;

                                        $flexview_entry->type        = CMS_FLEXVIEW_ENTRY_TYPE_ENTRY;
                                        $flexview_entry->index       = $_key;
                                        $flexview_entry->parent      = $key;
                                        $flexview_entry->position    = $position[$indentation];
                                        $flexview_entry->count       = $count[$indentation];
                                        $flexview_entry->subcount    = count($this->object[$_key]) - 2;
                                        $flexview_entry->indentation = $indentation;
                                        $flexview_entry->open        = isset($path[$indentation]) &&
                                                                       streq($path[$indentation], $_key);

                                        if (! call_user_func($callback_function, $flexview_entry))
                                        {
                                                $key                      = $_key;
                                                $position[++$indentation] = 0;
                                                $count[$indentation]      = $flexview_entry->subcount;

                                                reset($this->object[$key]);
                                        };
                                };

                                if (! $indentation--) break;

                                $key = $this->object[$key]["#parent"];
                        };
                };

                $flexview_entry->type        = CMS_FLEXVIEW_ENTRY_TYPE_END;
                $flexview_entry->index       = NULL;
                $flexview_entry->parent      = NULL;
                $flexview_entry->position    = 0;
                $flexview_entry->count       = 0;
                $flexview_entry->subcount    = 0;
                $flexview_entry->indentation = 0;
                $flexview_entry->open        = FALSE;

                call_user_func($callback_function, $flexview_entry);
        }
//-----------------------------------------------------------------------------------------------------
        function show_select($index    = "",
                             $action   = NULL,
                             $name_key = "name",
                             $base     = "")
        {
                $this->set_base($base);
                $this->set_index($index);
                $this->set_action($action);
                $this->set_name_key($name_key);

                $this->show_custom(array($this, "_show_select"));
        }
//-----------------------------------------------------------------------------------------------------
        function _show_select($flexview_entry)
        {
                switch ($flexview_entry->type)
                {
                case CMS_FLEXVIEW_ENTRY_TYPE_BASE:
//.....................................................................................................
                        echo("<select ");

                        //execution
                        if (streq(substr($this->action, 0, 11), "javascript:"))
                        {
                                echo("onchange=\"eval(this[this.selectedIndex].value);\">");
                                $action = substr($this->action, 11);
                        }
                        else
                        {
                                echo("onchange=\"location.replace(this[this.selectedIndex].value);\">");
                        };

                        //base
                        $this->display = "<option[ value=\"%action%\"]" .
                                                "[ class=\"%class%\"]>" .
                                         "%name%" .
                                         "</option>";

                        $this->display($flexview_entry->index);

                        return;

                case CMS_FLEXVIEW_ENTRY_TYPE_ENTRY:
//.....................................................................................................

                        //display
                        $this->display = "<option[ value=\"%action%\"]" .
                                                "[ class=\"%class%\"]" .
                                         (streq($flexview_entry->index, $this->index) ? " selected>" : ">") .
                                         str_repeat("&nbsp;&nbsp;", $flexview_entry->indentation) . "%name%" .
                                         "</option>";

                        $this->display($flexview_entry->index, $flexview_entry->open);

                        return;

                case CMS_FLEXVIEW_ENTRY_TYPE_END:
//.....................................................................................................
                        echo("</select>");
                };
        }
//-----------------------------------------------------------------------------------------------------
        function show_tree($index    = "",
                           $action   = NULL,
                           $name_key = "name",
                           $mark     = NULL,
                           $icon     = NULL,
                           $base     = "")
        {
                $this->set_base($base);
                $this->set_index($index);
                $this->set_action($action);
                $this->set_name_key($name_key);
                $this->set_mark($mark);
                $this->set_icon($icon);

                $this->show_custom(array($this, "_show_tree"));
        }
//-----------------------------------------------------------------------------------------------------
        function _show_tree($flexview_entry)
        {
                static $indentation = "";

                switch ($flexview_entry->type)
                {
                case CMS_FLEXVIEW_ENTRY_TYPE_BASE:
//.....................................................................................................

                        //empty
                        if (! $flexview_entry->subcount)
                        {
                                echo(CMS_L_FLEXVIEW_001 . "<br>");
                                return TRUE;
                        };

                        //base
                        if (nstreq($flexview_entry->index, $this->index))
                        {
                                //display
                                $this->display = "<a[ href=\"%action%\"]" .
                                                   "[ class=\"%class%\"]>" .
                                                 "%mark%[%icon% ]<small>%name%</small>" .
                                                 "</a><br>";

                                $this->display($flexview_entry->index);
                        };

                        $this->display = "%checkbox%" .
                                         "<a[ href=\"%action%\"]" .
                                           "[ class=\"%class%\"]>" .
                                         "%mark%[%icon% ]%name%" .
                                         "</a><br>";

                        return;

                case CMS_FLEXVIEW_ENTRY_TYPE_ENTRY:
//.....................................................................................................

                        //indentation
                        if ($flexview_entry->indentation)
                        {
                                for ($i = 1; $i < $flexview_entry->indentation; $i++)
                                        echo($indentation[$i] ?
                                             "<img src=\"" . x(CMS_IMAGES_URL . "flexview/tree_indentation.png") . "\" " .
                                                  "alt=\"\" " .
                                                  "class=\"fv-tree\">" :
                                             $this->space());

                                $image = ($flexview_entry->position == $flexview_entry->count) ?
                                         CMS_IMAGES_URL . "flexview/tree_branchclose.png" :
                                         CMS_IMAGES_URL . "flexview/tree_branchopen.png";
                                echo("<img src=\"" . x($image) . "\" " .
                                          "alt=\"\" " .
                                          "class=\"fv-tree\">");
                        };

                        //open
                        if ($flexview_entry->open && $flexview_entry->subcount)
                                $indentation .= ($flexview_entry->position == $flexview_entry->count) ? "0" : "1";

                        //display
                        $this->display($flexview_entry->index, $flexview_entry->open);

                        return ! $flexview_entry->open;

                case CMS_FLEXVIEW_ENTRY_TYPE_END:
//.....................................................................................................

                };
        }
//-----------------------------------------------------------------------------------------------------
        function show_treetable($index    = "",
                                $action   = NULL,
                                $name_key = "name",
                                $mark     = NULL,
                                $icon     = NULL,
                                $base     = "")
        {
                $this->set_base($base);
                $this->set_index($index);
                $this->set_action($action);
                $this->set_name_key($name_key);
                $this->set_mark($mark);
                $this->set_icon($icon);
                $this->param["flag"] = NULL;

                $this->show_custom(array($this, "_show_treetable"));
        }
//-----------------------------------------------------------------------------------------------------
        function _show_treetable($flexview_entry)
        {
                static $indentation = 0;

                //indentation
                if ($this->param["flag"])
                {
                        $c = ($flexview_entry->indentation > 2) ? $flexview_entry->indentation : 2;
                        for ($i = $indentation; $i > $c; $i--)
                                echo("</div>");

                        if ($flexview_entry->indentation < 2)
                                echo(($indentation ? "</td>" : "</th>") .
                                     "</tr>");
                };

                switch ($flexview_entry->type)
                {
                case CMS_FLEXVIEW_ENTRY_TYPE_BASE:
//.....................................................................................................

                        //empty
                        if (! $flexview_entry->subcount)
                        {
                                echo(CMS_L_FLEXVIEW_001 . "<br>");
                                return TRUE;
                        };

                        echo("<table style=\"WIDTH:100%\">");

                        //base
                        if (nstreq($flexview_entry->index, $this->index))
                        {
                                //display
                                $this->display = "<a[ href=\"%action%\"]" .
                                                   "[ class=\"%class%\"]>" .
                                                 "%mark%[%icon% ]<small>%name%</small>" .
                                                 "</a>";

                                echo("<tr>" .
                                     "<td>");
                                $this->display($flexview_entry->index);
                                echo("</td>" .
                                     "</tr>");
                        };

                        $this->display = "%checkbox%" .
                                         "<a[ href=\"%action%\"]" .
                                           "[ class=\"%class%\"]>" .
                                         "%mark%[%icon% ]%name%" .
                                         "</a>";

                        return;

                case CMS_FLEXVIEW_ENTRY_TYPE_ENTRY:
//.....................................................................................................

                        //indentation
                        switch ($flexview_entry->indentation)
                        {
                        case 0:
                                echo("<tr>" .
                                     "<th>");
                                break;

                        case 1:
                                echo("<tr>" .
                                     "<td class=\"varied\">");
                                break;

                        case 2:
                                if ($flexview_entry->indentation > $indentation)
                                        echo("<tr>" .
                                             "<td>");
                                break;

                        default:
                                if ($flexview_entry->indentation > $indentation)
                                        echo("<div style=\"PADDING-LEFT:18px\">");
                                break;
                        };

                        $indentation = $flexview_entry->indentation;

                        //display
                        $this->display($flexview_entry->index, $flexview_entry->open);
                        if ($flexview_entry->subcount)
                                echo("<small>" .
                                     "&nbsp;[" . $flexview_entry->subcount . "]" .
                                     "</small>");
                        if (($indentation > 1) && ($flexview_entry->position != $flexview_entry->count))
                                echo("<br>");

                        $this->param["flag"] = TRUE;

                        return ! $flexview_entry->open;

                case CMS_FLEXVIEW_ENTRY_TYPE_END:
//.....................................................................................................
                        if ($this->param["flag"])
                                echo("</table>");

                };
        }
//-----------------------------------------------------------------------------------------------------
        function show_target($index          = "",
                             $action         = NULL,
                             $action_insert  = NULL,
                             $action_append  = NULL,
                             $name_key       = "name",
                             $base           = "",
                             $type_insert    = NULL,
                             $subtype_insert = NULL,
                             $type_append    = NULL,
                             $subtype_append = NULL)
        {
                $this->set_base($base);
                $this->set_index($index);
                $this->set_action($action);
                $this->param["action_insert"]  = $action_insert;
                $this->param["action_append"]  = $action_append;
                $this->set_name_key($name_key);
                $this->param["type_insert"]    = $type_insert;
                $this->param["subtype_insert"] = $subtype_insert;
                $this->param["type_append"]    = $type_append;
                $this->param["subtype_append"] = $subtype_append;

                $this->show_custom(array($this, "_show_target"));
        }
//-----------------------------------------------------------------------------------------------------
        function _show_target($flexview_entry)
        {
                static $stack_enabled  = array(),
                       $stack_index    = array(),
                       $stack_position = array(),
                       $stack_count    = array();

                //verify insert target type
                $action_insert = (bool)$this->param["action_insert"];

                if ($action_insert)
                {
                        if ($this->param["type_insert"])
                        {
                                if (isset($this->object[$flexview_entry->index]["#data"]["#type"]))
                                {
                                        $type           = $this->object[$flexview_entry->index]["#data"]["#type"];
                                        $action_insert  = isset($this->param["type_insert"][$type]) &&
                                                          $this->param["type_insert"][$type];
                                }
                                else
                                {
                                        $action_insert  = FALSE;
                                };
                        };

                        if ($action_insert && $this->param["subtype_insert"])
                        {
                                if (isset($this->object[$flexview_entry->index]["#data"]["#subtype"]))
                                {
                                        $subtype        = $this->object[$flexview_entry->index]["#data"]["#subtype"];
                                        $action_insert |= isset($this->param["subtype_insert"][$subtype]) &&
                                                          $this->param["subtype_insert"][$subtype];
                                }
                                else
                                {
                                        $action_insert  = FALSE;
                                };
                        };
                };

                //verify append target type
                $action_append = (bool)$this->param["action_append"];

                if ($action_append)
                {
                        if ($this->param["type_append"])
                        {
                                if (isset($this->object[$flexview_entry->index]["#data"]["#type"]))
                                {
                                        $type           = $this->object[$flexview_entry->index]["#data"]["#type"];
                                        $action_append  = isset($this->param["type_append"][$type]) &&
                                                          $this->param["type_append"][$type];
                                }
                                else
                                {
                                        $action_append  = FALSE;
                                };
                        };

                        if ($action_append && $this->param["subtype_append"])
                        {
                                if (isset($this->object[$flexview_entry->index]["#data"]["#subtype"]))
                                {
                                        $subtype        = $this->object[$flexview_entry->index]["#data"]["#subtype"];
                                        $action_append |= isset($this->param["subtype_append"][$subtype]) &&
                                                          $this->param["subtype_append"][$subtype];
                                }
                                else
                                {
                                        $action_append  = FALSE;
                                };
                        };
                };

                //append
                if (($c = count($stack_enabled)) > $flexview_entry->indentation)
                {
                        $this->display = "<a[ href=\"%action%\"]" .
                                           "[ class=\"%class%\"]>" .
                                         image("flexview/icon_target_small") . " ..." .
                                         "</a><br>";
                        $action        = $this->action;
                        $this->set_action($this->param["action_append"]);

                        do
                        {
                                $enabled = array_pop($stack_enabled);
                                $index   = array_pop($stack_index);

                                if ($enabled)
                                {
                                        //indentation
                                        if ($c > 1)
                                        {
                                                for ($i = 1; $i < $c; $i++)
                                                        echo(($stack_enabled[$i] || ($stack_position[$i] < $stack_count[$i])) ?
                                                             "<img src=\"" . x(CMS_IMAGES_URL . "flexview/tree_indentation_small.png") . "\" " .
                                                                  "alt=\"\" " .
                                                                  "class=\"fv-tree\">" :
                                                             $this->space(1, 12));
                                        };

                                        //display
                                        $this->display($index);
                                };
                        }
                        while (($c = count($stack_index)) > $flexview_entry->indentation);

                        $this->set_action($action);
                };

                switch ($flexview_entry->type)
                {
                case CMS_FLEXVIEW_ENTRY_TYPE_BASE:
//.....................................................................................................
                        echo("<div style=\"FONT-SIZE:0.85em;LINE-HEIGHT:100%\">");

                        //base
                        $stack_enabled  = array(1 => $action_append);
                        $stack_index    = array(1 => $flexview_entry->index);
                        $stack_position = array(0 => $flexview_entry->position);
                        $stack_count    = array(0 => $flexview_entry->count);

                        if ($action_insert && (! $action_append))
                        {
                                //display
                                $this->display = "<a[ href=\"%action%\"]" .
                                                   "[ class=\"%class%\"]>" .
                                                 image("flexview/icon_target_small") . " ..." .
                                                 "</a><br>";
                                $action        = $this->action;
                                $this->set_action($this->param["action_insert"]);

                                $this->display($flexview_entry->index);

                                $this->set_action($action);
                        };

                        return;

                case CMS_FLEXVIEW_ENTRY_TYPE_ENTRY:
//.....................................................................................................
                        $stack_position[$flexview_entry->indentation] = $flexview_entry->position;
                        $stack_count[$flexview_entry->indentation]    = $flexview_entry->count;

                        //indentation
                        if ($flexview_entry->indentation)
                        {
                                for ($i = 1; $i < $flexview_entry->indentation; $i++)
                                        echo(($stack_enabled[$i] ||
                                             ($stack_position[$i] < $stack_count[$i])) ?
                                             "<img src=\"" . x(CMS_IMAGES_URL . "flexview/tree_indentation_small.png") . "\" " .
                                                  "alt=\"\" " .
                                                  "class=\"fv-tree\">" :
                                             $this->space(1, 12));
                                $image = ($stack_enabled[$flexview_entry->indentation] ||
                                         ($flexview_entry->position < $flexview_entry->count)) ?
                                         CMS_IMAGES_URL . "flexview/tree_branchopen_small.png" :
                                         CMS_IMAGES_URL . "flexview/tree_branchclose_small.png";
                                echo("<img src=\"" . x($image) . "\" " .
                                          "alt=\"\" " .
                                          "class=\"fv-tree\">");
                        };

                        $open_container = $flexview_entry->open && $flexview_entry->subcount;

                        //display
                        $this->display = "<a[ href=\"%action%\"]" .
                                           "[ class=\"%class%\"]>" .
                                         image($flexview_entry->subcount ?
                                               "flexview/icon_entrycontainer_small" :
                                               "flexview/icon_entry_small") .
                                         " %name%" .
                                         "</a><br>";

                        $this->display($flexview_entry->index, $flexview_entry->open);

                        //insert
                        if ($action_insert)
                        {
                                //indentation
                                if ($flexview_entry->indentation)
                                {
                                        for ($i = 1; $i <= $flexview_entry->indentation; $i++)
                                                echo(($stack_enabled[$i] ||
                                                     ($stack_position[$i] < $stack_count[$i])) ?
                                                     "<img src=\"" . x(CMS_IMAGES_URL . "flexview/tree_indentation_small.png") . "\" " .
                                                          "alt=\"\" " .
                                                          "class=\"fv-tree\">" :
                                                     $this->space(1, 12));
                                };

                                if ($action_append)
                                {
                                        echo("<img src=\"" . x(CMS_IMAGES_URL . "flexview/tree_branchopen_small.png") . "\" " .
                                                  "alt=\"\" " .
                                                  "class=\"fv-tree\">");
                                }
                                else
                                {
                                        $image = $open_container ?
                                                 CMS_IMAGES_URL . "flexview/tree_branchopen_small.png" :
                                                 CMS_IMAGES_URL . "flexview/tree_branchclose_small.png";
                                        echo("<img src=\"" . x($image) . "\" " .
                                                  "alt=\"\" " .
                                                  "class=\"fv-tree\">");
                                };

                                //display
                                $this->display = "<a[ href=\"%action%\"]" .
                                                   "[ class=\"%class%\"]>" .
                                                 image("flexview/icon_target_small") . " ..." .
                                                 "</a><br>";
                                $action        = $this->action;
                                $this->set_action($this->param["action_insert"]);

                                $this->display($flexview_entry->index);

                                $this->set_action($action);
                        };

                        array_push($stack_enabled, $action_append);
                        array_push($stack_index,   $flexview_entry->index);

                        if (! $open_container) return TRUE;

                        return;

                case CMS_FLEXVIEW_ENTRY_TYPE_END:
//.....................................................................................................
                        echo("</div>");
                };
        }
//-----------------------------------------------------------------------------------------------------
        function show_path($index     = "",
                           $action    = NULL,
                           $name_key  = "name",
                           $delimeter = "\xC2\xBB",
                           $base      = "")
        {
                $this->set_base($base);
                $this->set_index($index);
                $this->set_action($action);
                $this->set_name_key($name_key);

                if (streq($this->index, $this->base)) return;

                //path
                $path = $this->get_path();

                //base
                $this->display = "<a[ href=\"%action%\"]" .
                                   "[ class=\"%class%\"]>" .
                                 "%mark%[%icon% ]<small>%name%</small>" .
                                 "</a>";

                $this->display($this->base);

                //display
                $this->display = " " . x($delimeter) . " " .
                                 "<a[ href=\"%action%\"]" .
                                   "[ class=\"%class%\"]>" .
                                 "%mark%[%icon% ]%name%" .
                                 "</a>";

                reset($path);
                while (list($key, $value) = each($path))
                        $this->display($value);
        }
//-----------------------------------------------------------------------------------------------------
        function show_column($index    = "",
                             $action   = NULL,
                             $name_key = "name",
                             $mark     = NULL,
                             $icon     = NULL,
                             $base     = "")
        {
                $this->set_base($base);
                $this->set_index($index);
                $this->set_action($action);
                $this->set_name_key($name_key);
                $this->set_mark($mark);
                $this->set_icon($icon);

                $this->_show_column();
        }
//-----------------------------------------------------------------------------------------------------
        function _show_column()
        {
                //empty
                if (count($this->object) == 1)
                {
                        echo(CMS_L_FLEXVIEW_001 . "<br>");
                        return;
                };

                $index = $this->index;

                if (nstreq($index, $this->base))
                {
                        //base
                        $this->display = "<a[ href=\"%action%\"]" .
                                            " class=\"fv-col[ %class%]\">" .
                                         "%mark%[%icon% ]<small>%name%</small>" .
                                         "</a>";

                        $this->display($this->base);

                        //path
                        if (count($this->object[$index]) < 3)
                                $index = $this->object[$index]["#parent"];

                        if (nstreq($index, $this->base))
                        {
                                $_index = $index;
                                $path   = array(0 => $_index);

                                while (
                                        isset($this->object[$_index]["#parent"])
                                        &&
                                        nstreq($_index = $this->object[$_index]["#parent"], $this->base)
                                      )
                                        $path[] = $_index;

                                $path   = array_reverse($path);

                                $this->display = "<a[ href=\"%action%\"]" .
                                                   "[ title=\"%title%\"]" .
                                                    " class=\"fv-col[ %class%]\">" .
                                                 "%mark%[%icon% ]%name%" .
                                                 "</a>";

                                reset($path);
                                while (list($key, $value) = each($path))
                                        $this->display($value, TRUE);
                        };
                };

                //display
                $this->display = "<div class=\"fv-col\">" .
                                 "%checkbox%" .
                                 "<a[ href=\"%action%\"]" .
                                   "[ title=\"%title%\"]" .
                                   "[ class=\"%class%\"]>" .
                                 "%mark%[%icon% ]%name%" .
                                 "</a>" .
                                 "</div>";

                reset($this->object[$index]);
                while (list($key) = each($this->object[$index]))
                {
                        if (! preg_match("/^#(?:data|parent)$/", $key))
                                $this->display($key);
                };

                echo("<br style=\"CLEAR:both\">");
        }
//-----------------------------------------------------------------------------------------------------
        function show_folder($index    = "",
                             $action   = NULL,
                             $name_key = "name",
                             $mark     = NULL,
                             $icon     = NULL,
                             $base     = "")
        {
                $this->set_base($base);
                $this->set_index($index);
                $this->set_action($action);
                $this->set_name_key($name_key);
                $this->set_mark($mark);
                $this->set_icon($icon);

                $this->_show_folder();
        }
//-----------------------------------------------------------------------------------------------------
        function _show_folder()
        {
                //empty
                if (count($this->object) == 1)
                {
                        echo(CMS_L_FLEXVIEW_001 . "<br>");
                        return;
                };

                $index = $this->index;

                if (nstreq($index, $this->base))
                {
                        //base
                        $this->display = "<a[ href=\"%action%\"]" .
                                           "[ class=\"%class%\"]>" .
                                         "%mark%[%icon% ]<small>%name%</small>" .
                                         "</a><br>";

                        $this->display($this->base);

                        //path
                        if (count($this->object[$index]) < 3)
                                $index = $this->object[$index]["#parent"];

                        if (nstreq($index, $this->base))
                        {
                                $_index = $index;
                                $path   = array(0 => $_index);

                                while (
                                        isset($this->object[$_index]["#parent"])
                                        &&
                                        nstreq($_index = $this->object[$_index]["#parent"], $this->base)
                                      )
                                        $path[] = $_index;

                                $path   = array_reverse($path);

                                $this->display = "<a[ href=\"%action%\"]" .
                                                   "[ class=\"%class%\"]>" .
                                                 "%mark%[%icon% ]%name%" .
                                                 "</a><br>";

                                reset($path);
                                while (list($key, $value) = each($path))
                                        $this->display($value, TRUE);
                        };
                };

                //display
                $this->display = "%checkbox%" .
                                 "<a[ href=\"%action%\"]" .
                                   "[ class=\"%class%\"]>" .
                                 "%mark%[%icon% ]%name%" .
                                 "</a><br>";

                reset($this->object[$index]);
                while (list($key) = each($this->object[$index]))
                {
                        if (! preg_match("/^#(?:data|parent)$/", $key))
                                $this->display($key);
                };
        }
//-----------------------------------------------------------------------------------------------------
        function show_table($index     = "",
                            $action    = NULL,
                            $name_key  = "name",
                            $mark      = NULL,
                            $icon      = NULL,
                            $column    = 3,
                            $delimeter = "\xC2\xBB",
                            $base      = "")
        {
                //empty
                if (count($this->object) == 1)
                {
                        echo(CMS_L_FLEXVIEW_001 . "<br>");
                        return;
                };

                $this->set_mark($mark);
                $this->set_icon($icon);

                //path
                $this->show_path($index, $action, $name_key, $delimeter, $base);

                $array = array();
                $count = 0;

                reset($this->object[$this->index]);
                while (list($key, $value) = each($this->object[$this->index]))
                {
                        if (! preg_match("/^#(?:data|parent)$/", $key))
                        {
                                $array[]  = $key;
                                $count   += count($value) - 1;
                        };
                };

                if (! $count) return;

                $count = $column ? ceil($count / $column) : NULL;
                $i     = 0;

                echo("<table class=\"layout\" " .
                            "style=\"WIDTH:100%\">" .
                     "<tr>" .
                     "<td>");

                reset($array);
                while (list($key, $value) = each($array))
                {
                        if ($count && ($i++ == $count))
                        {
                                echo("</td>" .
                                     "<td style=\"WIDTH:1px\">" .
                                     "&nbsp;" .
                                     "</td>" .
                                     "<td>");
                                $i = 1;
                        };

                        $this->display = "%checkbox%" .
                                         "<a[ href=\"%action%\"]" .
                                           "[ class=\"%class%\"]>" .
                                         "%mark%[%icon% ]%name%" .
                                         "</a><br>";

                        $this->display($value);

                        $this->display = "%checkbox%" .
                                         "<a[ href=\"%action%\"]" .
                                           "[ class=\"%class%\"]>" .
                                         "%mark%[%icon% ]<small>%name%</small>" .
                                         "</a><br>";

                        reset($this->object[$value]);
                        while (list($key) = each($this->object[$value]))
                        {
                                if (! preg_match("/^#(?:data|parent)$/", $key))
                                {
                                        if ($count && ($i++ == $count))
                                        {
                                                echo("</td>" .
                                                     "<td style=\"WIDTH:1px\">" .
                                                     "&nbsp;" .
                                                     "</td>" .
                                                     "<td>");
                                                $i = 1;
                                        };

                                        $this->display($key);
                                };
                        };
                };

                echo("</td>" .
                     "</tr>" .
                     "</table>");
        }
//-----------------------------------------------------------------------------------------------------
        function show_expand($index      = "",
                             $action     = NULL,
                             $name_key   = "name",
                             $mark       = NULL,
                             $icon       = NULL,
                             $horizontal = NULL,
                             $base       = "")
        {
                static $flag = TRUE;

                $this->set_base($base);
                $this->set_index($index);
                $this->set_action($action);
                $this->set_name_key($name_key);
                $this->set_mark($mark);
                $this->set_icon($icon);
                $this->param["horizontal"] = (bool)$horizontal;

                if ($flag)
                {
?>
<script>
var flexview_expand_stack   = new Array();
var flexview_expand_count   = 0;
var flexview_expand_timeout = null;

function flexview_expand_mousein(objectid,
                                 parentid,
                                 trigger) //arguments: 3 => horizontal
{
        clearTimeout(flexview_expand_timeout);
        var _object = document.getElementById(objectid);

        while (flexview_expand_count > 0)
        {
                if (flexview_expand_stack[flexview_expand_count][0] == parentid) break;
                fx_visible(flexview_expand_stack[flexview_expand_count--][0], false);
        };

        if (arguments.length > 3)
                fx_move(_object, fx_left(trigger, true), fx_top(trigger, true) + fx_height(trigger, true));
        else
                fx_move(_object, fx_left(trigger, true) + fx_width(trigger, true), fx_top(trigger, true));

        var x1 = fx_left(_object);
        var y1 = fx_top(_object);
        var x2 = x1 + fx_width(_object, true);
        var y2 = y1 + fx_height(_object, true);
        flexview_expand_stack[++flexview_expand_count] = new Array(objectid, x1, y1, x2, y2);

        fx_visible(_object, true);
        flexview_expand_timeout = null;
};
//................................
function flexview_expand_mouseout()
{
        if (! flexview_expand_count) return;

        flexview_expand_stack[flexview_expand_count][5] = true;
        if (! flexview_expand_timeout) flexview_expand_timeout = setTimeout(_flexview_expand_mouseout, 1500);
};
//................................
function _flexview_expand_mouseout()
{
        for (var i = flexview_expand_count; i > 0; i--)
        {
                if (
                     (fx_mouse_x < flexview_expand_stack[i][1])
                     ||
                     (fx_mouse_x > flexview_expand_stack[i][3])
                     ||
                     (fx_mouse_y < flexview_expand_stack[i][2])
                     ||
                     (fx_mouse_y > flexview_expand_stack[i][4])
                   )
                {
                        if (flexview_expand_stack[i][5])
                                fx_visible(flexview_expand_stack[flexview_expand_count--][0], false);

                        continue;
                };

                break;
        };

        flexview_expand_timeout = null;
};
//................................
function flexview_expand_mousemove(event)
{
        if (
             (flexview_expand_count > 0)
             &&
             (! flexview_expand_timeout)
             &&
             (event == "mousemove")
           )
                flexview_expand_timeout = setTimeout(_flexview_expand_mouseout, 1500);
};

fx_register_callback(flexview_expand_mousemove);
</script>
<?php
                        $flag = FALSE;
                };

                $this->show_custom(array($this, "_show_expand"));
        }
//-----------------------------------------------------------------------------------------------------
        function _show_expand($flexview_entry)
        {
                static $indentation = 0;

                switch ($flexview_entry->type)
                {
                case CMS_FLEXVIEW_ENTRY_TYPE_BASE:
//.....................................................................................................
                        return;

                case CMS_FLEXVIEW_ENTRY_TYPE_ENTRY:
//.....................................................................................................

                        //close open panels
                        for (; $indentation > $flexview_entry->indentation; $indentation--)
                                echo("</div>");

                        //sublevel
                        if ($flexview_entry->indentation)
                        {
                                //open new panel
                                if ($flexview_entry->indentation > $indentation)
                                {
                                        echo("<div id=\"fv-" . x($flexview_entry->parent) . "\" " .
                                                  "class=\"menu\" " .
                                                  "style=\"POSITION:absolute;VISIBILITY:hidden\">");
                                        $indentation++;
                                };

                                $this->display = $flexview_entry->subcount ?

                                //contains subentries
                                                 "%checkbox%" .
                                                 "<a[ href=\"%action%\"]" .
                                                   "[ class=\"%class%\"]" .
                                                    " onmouseenter=\"flexview_expand_mousein(" .
                                                     "'fv_" . qx($flexview_entry->index) . "'," .
                                                     "'fv_" . qx($flexview_entry->parent) . "'," .
                                                     "this);\"" .
                                                    " onmouseleave=\"flexview_expand_mouseout();\">" .
                                                 "%mark%[%icon% ]%name%" .
                                                 "</a>" :

                                //contains no subentries
                                                 "%checkbox%" .
                                                 "<a[ href=\"%action%\"]" .
                                                   "[ class=\"%class%\"]>" .
                                                 "%mark%[%icon% ]%name%" .
                                                 "</a>";
                        }

                        //base level
                        else
                        {
                                $this->display = $flexview_entry->subcount ?

                                //contains subentries
                                                 "%checkbox%" .
                                                 "<a[ href=\"%action%\"]" .
                                                   "[ class=\"%class%\"]" .
                                                    " onmouseenter=\"flexview_expand_mousein(" .
                                                     "'fv_" . qx($flexview_entry->index) . "','0',this" .
                                                     ($this->param["horizontal"] ? ",true" : NULL) . ");\"" .
                                                    " onmouseleave=\"flexview_expand_mouseout();\">" .
                                                 "%mark%[%icon% ]%name%" .
                                                 "</a>" :

                                //contains no subentries
                                                 "%checkbox%" .
                                                 "<a[ href=\"%action%\"]" .
                                                   "[ class=\"%class%\"]>" .
                                                 "%mark%[%icon% ]%name%" .
                                                 "</a>";
                        };

                        $this->display($flexview_entry->index, $flexview_entry->open);
                        return;

                case CMS_FLEXVIEW_ENTRY_TYPE_END:
//.....................................................................................................

                        //close open panels
                        for (; $indentation > 0; $indentation--)
                                echo("</div>");
                };
        }
//.....................................................................................................
        function space($count = 1,
                       $size  = 18)
        {
                return "<img src=\"" . x(CMS_IMAGES_URL) . "blank.png\" " .
                            "alt=\" \" " .
                            "class=\"fv-space\" " .
                            "style=\"HEIGHT:" . x($size) . "px;" .
                                    "WIDTH:" . x($size * $count) . "px\">";
        }
};
?>
