<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

//-----------------------------------------------------------------------------------------------------
//===< IFC CLASS >=====================================================================================
//-----------------------------------------------------------------------------------------------------
global $ifc_message,       //control message
       $ifc_page,          //selected page
       $ifc_option,        //'external' => externally opened windows
       $ifc_select,        //ifc fieldname of return value
       $ifc_select_action, //url of return call, %return% will be replaced by value
       $ifc_response,      //response message: CMS_MSG_DONE, CMS_MSG_ERROR, {text}
       $ifc_param;         //default parameter

//constants
define("CMS_IFC_MESSAGE",       $ifc_message);
define("CMS_IFC_PAGE",          CMS_INSTANCE);
define("CMS_IFC_OPTION",        $ifc_option);
define("CMS_IFC_SELECT",        $ifc_select);
define("CMS_IFC_SELECT_ACTION", $ifc_select_action);

//related functions
//-----------------------------------------------------------------------------------------------------
function ifc_permission($array = NULL)
{
        static $permission = NULL;

        if (! $array) return $permission;

        $permission = $array;
};
//-----------------------------------------------------------------------------------------------------
function ifc_available($module)
{
        return is_file(CMS_MODULES_PATH . "#interface/ifc.$module.inc");
};
//-----------------------------------------------------------------------------------------------------
function ifc_default($ifc_page = NULL)
{
        ifc_permission(array("" => CMS_L_ACCESS));

        $ifc = new ifc(NULL, $ifc_page, NULL);

        echo(x(CMS_SOFTWARE) . " " .
             x(CMS_VERSION) . "<br>" .
             x(CMS_COPYRIGHT) . ", " .
             "<a href=\"http://" . x(CMS_HOMEPAGE) . "\">" .
             x(CMS_HOMEPAGE) .
             "</a><br>" .
             "<small>" .
             "PHP " . x(phpversion()) . " (" . x(php_sapi_name()) . "), " . x(php_uname()) .
             "</small>" .
             "<div id=\"ifc-licence\">" .
             x(file_get_contents(CMS_PATH . "licence.txt")) .
             "</div>");

        $ifc->close();
};
//-----------------------------------------------------------------------------------------------------
function ifc_inactive($ifc_page = NULL)
{
        $ifc = new ifc(CMS_L_IFC_001, $ifc_page, NULL);

        echo(x(CMS));

        $ifc->close();
};
//-----------------------------------------------------------------------------------------------------
function ifc_table_open($width = "100%")
{
        echo("<table class=\"ifc\"" .
                     option(nstre($width), " style=\"WIDTH:$width\"") . ">");
};
//-----------------------------------------------------------------------------------------------------
function ifc_table_close()
{
        echo("</table>");
};
//-----------------------------------------------------------------------------------------------------
function ifc_close_external()
{
        echo(CMS_DOCTYPE_HTML .
             "<html>" .
             "<head>" .
             "<title>" .
             CMS_L_IFC_003 .
             "</title>" .
             jscript("window.close();") .
             "</head>" .
             "<body>" .
             "<div>" .
             "</div>" .
             "</body>" .
             "</html>");

        exit();
};
//-----------------------------------------------------------------------------------------------------
function ifc_varied($option = NULL,
                    $index  = 0)
{
        return class_varied($option, $index);
};

class ifc
{
//constructor
//-----------------------------------------------------------------------------------------------------
        function __construct($ifc_response         = NULL,
                             $ifc_page             = NULL,
                             $menu                 = TRUE,
                             $param                = NULL,
                             $message              = NULL,
                             $subpage              = NULL,
                             $content_container_id = NULL)
        {
                //title
                $title = isset($ifc_page[CMS_IFC_PAGE]) ? $ifc_page[CMS_IFC_PAGE] : "";
                if (stre($title))
                {
                        $constant = "CMS_L_IFC_" . strtoupper(CMS_IFC_PAGE);
                        if (defined($constant)) $title = constant($constant);
                        if (stre($title))
                        {
                                $title = CMS_IFC_PAGE;
                                if (stre($title)) $title = ucfirst(CMS_APPLICATION);
                        };
                };
                if (nstre($subpage)) $title .= " > " . $subpage;

                echo(CMS_DOCTYPE_HTML .
                     "<html>" .
                     "<head>" .
                     "<title>" .
                     x($title) .
                     "</title>" .
                     "<meta name=\"robots\" " .
                           "content=\"noindex, nofollow\">" .
                     "<meta http-equiv=\"content-type\" " .
                           "content=\"text/html; charset=utf-8\">" .
                     "<base href=\"" . htmlspecialchars(CMS_ROOT_URL) . "\">" .
                     CMS_JAVASCRIPT .
                     stylesheet(CMS_URL . "stylesheet/" . CMS_IFC_EDITION . ".css") .
                     "<link rel=\"shortcut icon\" " .
                           "type=\"image/png\" " .
                           "href=\"" . htmlspecialchars(CMS_URL) . "favicon.png\">" .
                     "<script src=\"" . x(CMS_JAVASCRIPT_URL) . "ifc.js\">" .
                     "</script>" .
                     jscript("fx_register_callback(ifc_loading_event);") .
                     "</head>" .

                     "<body>" .

                     "<form name=\"ifc\" " .
                           "action=\"" . x(CMS_ACTIVE_URL . querystring($_GET)) . "\" " .
                           "method=\"post\" " .
                           "enctype=\"multipart/form-data\" " .
                           "accept-charset=\"utf-8\">" .

                //page parameter
                     "<input name=\"ifc_page\" " .
                            "type=\"hidden\" " .
                            "value=\"" . x(CMS_IFC_PAGE) . "\">" .

                //message parameter
                     "<input name=\"ifc_message\" " .
                            "type=\"hidden\" " .
                            "value=\"" . x($message) . "\">" .

                //left scroll position
                     "<input name=\"ifc_left\" " .
                            "type=\"hidden\" " .
                            "value=\"0\">" .

                //top scroll position
                     "<input name=\"ifc_top\" " .
                            "type=\"hidden\" " .
                            "value=\"0\">");

                //constant parameters
                $content_offset = 5;

                if (is_array($param))
                {
                        if (! isset($param["ifc_param"])) $this->param("ifc_param");
                        $content_offset += $this->param($param);
                }
                elseif (nstre($param))
                {
                        $content_offset += $this->param("ifc_param", $param);
                }
                else
                {
                        $this->param("ifc_param");
                };

                if (is_array($ifc_page))
                {
                        //control
                        echo("<div id=\"ifc-control\">" .

                        //logo
                             "<a id=\"ifc-logo\" " .
                                "href=\"http://" . x(CMS_HOMEPAGE) . "/\" " .
                                "title=\"" . x(CMS) . "\">" .
                             image(CMS_IFC_EDITION . "/logo") .
                             "</a>" .

                        //control head
                             "<div id=\"ifc-control-head\">" .

                        //desktop
                             option(cms_permission("desktop", NULL, NULL),
                                    "<a href=\"" . x(CMS_MODULES_URL) . "desktop.php\">" .
                                    image(CMS_IFC_EDITION . "/icon_desktop") . " " . CMS_L_IFC_006 .
                                    "</a>") .

                        //web site
                             "<a href=\"" . x(CMS_ROOT_URL) . "\">" .
                             image(CMS_IFC_EDITION . "/icon_website") . " " . CMS_L_IFC_008 .
                             "</a>" .
                             "</div>" .

                        //control body
                             "<div id=\"ifc-control-body\">");

                        $flag = FALSE;

                        reset($ifc_page);
                        while (list($key, $value) = each($ifc_page))
                        {
                                //skip hidden interface pages (starting with '#')
                                if (preg_match("/^#/", $value)) continue;

                                //separator
                                if ($value === "-")
                                {
                                        $flag = TRUE;
                                        continue;
                                };

                                if ($flag)
                                {
                                        echo("<hr>");
                                        $flag = FALSE;
                                };

                                $image = image_exists("interface/icon_$key") ?
                                         "icon_$key" : "icon_unknown";
                                $value = x($value);

                                //select button
                                echo("<a href=\"" . x(CMS_ACTIVE_URL .
                                                      querystring(array("ifc_message" => "ifc_select_page",
                                                                        "ifc_param"   => $key))) . "\" " .
                                        "class=\"go\">" .
                                     image("interface/$image") . " " .
                                     (streq($key, CMS_IFC_PAGE) ? "<strong>$value</strong>" : $value) .
                                     "</a>");
                        };

                        echo("</div>" .

                        //control foot
                             "<div id=\"ifc-control-foot\">" .

                        //logout
                             "<a href=\"" . x(CMS_MODULES_URL) . "identification.php?cms_logout=1\">" .
                             image(CMS_IFC_EDITION . "/icon_logout") . " " . CMS_L_IFC_007 .
                             "</a>" .
                             "</div>");

                        //language
                        if (CMS_LANGUAGE_ENABLED)
                        {
                                echo("<div id=\"ifc-language\">");

                                $language = explode(",", CMS_LANGUAGE_ENABLED);

                                reset($language);
                                while (list($key, $value) = each($language))
                                {
                                        $url = CMS_ACTIVE_URL .
                                               querystring(array("ifc_page"            => CMS_IFC_PAGE,
                                                                 "cms_select_language" => $value));

                                        echo("<a href=\"" . x($url) . "\" " .
                                               "class=\"language-" . (streq($value, CMS_LANGUAGE) ? "on" : "off") . "\">" .
                                             image(is_file(CMS_DATA_PATH . "language/" . $value . "_small.gif") ?
                                                   CMS_DATA_URL . "language/" . r($value) . "_small.gif" :
                                                   "language/default_small",
                                                   16, 12, $value) .
                                             "</a>");
                                };

                                echo("</div>");
                        };

                        //date
                        echo("</div>");

                        if ($content_container_id === NULL) $content_container_id = "ifc-content";
                }
                else
                {
                        if ($content_container_id === NULL) $content_container_id = "ifc-content-full";
                };

                echo("<div id=\"" . x($content_container_id) . "\">");

                //default menu
                if ($menu && (! is_array($menu)))
                        $menu = array(CMS_L_COMMAND_CONFIRM . "|" . CMS_IFC_EDITION . "/command_confirm" => "javascript:ifc_submit();",
                                      CMS_L_COMMAND_CANCEL .  "|" . CMS_IFC_EDITION . "/command_cancel"  => NULL);

                //close external window
                if (substr_count(CMS_IFC_OPTION, "external"))
                        $menu[CMS_L_IFC_002 . "|" . CMS_IFC_EDITION . "/command_close"] = "javascript:top.close();";

                //menu selection
                if ($menu)
                {
                        echo("<div id=\"ifc-command\">");

                        //subpage
                        if ($subpage)
                        {
                                $subpage = x($subpage);
                                echo("<div id=\"ifc-subpage\" " .
                                          "title=\"$subpage\">" .
                                     $subpage .
                                     "</div>");
                        };

                        $array    = array(0 => array(), array());
                        $charlist = "";
                        $i        = 0;

                        reset($menu);
                        while (list($key, $value) = each($menu))
                        {
                                //command image
                                if (($_i = strpos($key, "|")) !== FALSE)
                                {
                                        $class = "";
                                        $image = substr($key, $_i + 1);
                                        $key   = substr($key, 0, $_i);
                                }
                                else
                                {
                                        $class = " class=\"ifc-no-image\"";
                                        $image = FALSE;
                                };

                                //security check
                                if (streq(substr($value, 0, 1), "#"))
                                {
                                        $array[0][$i] = q($key);
                                        $value        = substr($value, 1);
                                }
                                else
                                {
                                        $array[0][$i] = NULL;
                                };

                                //javascript
                                if (streq(substr($value, 0, 11), "javascript:"))
                                        $array[1][$i] = q(substr($value, 11));

                                //message
                                elseif ($value)
                                        $array[1][$i] = q("ifc_post('" . q($value) . "');");

                                //cancel
                                else
                                        $array[1][$i] = q("ifc_cancel($content_offset);");

                                //accesskey
                                $_key = $key;

                                while (TRUE)
                                {
                                        $_i   = strspn($_key, $charlist);
                                        $char = substr($_key, $_i, 1);
                                        $_key = substr($_key, $_i + 1);

                                        if (preg_match("/[a-z]/i", $char))
                                        {
                                                $_i        = strlen($key) - strlen($_key) - 1;
                                                $charlist .= strtolower($char) . strtoupper($char);
                                                $key       = x(substr($key, 0, $_i)) . "<strong>$char</strong>" . x($_key);
                                                break;
                                        };

                                        if ($_key === FALSE)
                                        {
                                                $key       = x($key);
                                                $char      = "";
                                                break;
                                        };
                                };

                                echo("<a href=\"javascript:ifc_select_menu($i);\" " .
                                        "accesskey=\"$char\"$class>" .
                                     option($image !== FALSE, image($image) . " ") . $key .
                                     "</a>");

                                $i++;
                        };

                        echo(jscript("var ifc_menu = new Array(new Array(\"" . implode("\", \"", $array[0]) . "\"), " .
                                                              "new Array(\"" . implode("\", \"", $array[1]) . "\"));") .

                             "</div>");
                };

                echo("<noscript>" .
                     "<div class=\"error\">" .
                     CMS_L_NOSCRIPT .
                     "</div>" .
                     "</noscript>" .

                //response
                     "<div>" .
                     "<div id=\"ifc-response\"" .
                           option(nstre($ifc_response), " class=\"ifc-response-fade\"") . ">" .
                     $ifc_response .
                     "</div>" .
                     "</div>");

                flush();
        }
//methods
//-----------------------------------------------------------------------------------------------------
        function param($param,
                       $value = NULL)
        {
                $count = 0;

                if (is_array($param))
                {
                        while (list($key, $value) = each($param))
                                $count += $this->param($key, $value);
                }
                elseif (is_array($value))
                {
                        reset($value);
                        while (list($key, $_value) = each($value))
                                $count += $this->param($param . "[" . $key . "]", $_value);
                }
                else
                {
                        echo("<input name=\"" . x($param) . "\" " .
                                    "type=\"hidden\" " .
                                    "value=\"" . x($value) . "\">");
                        $count++;
                };

                return $count;
        }
//-----------------------------------------------------------------------------------------------------
        function set($text     = NULL,
                     $type     = "new text 40 60 b",

                     /*
                      * : - inline
                      * b - linebreak after
                      * c - checked
                      * d - disabled
                      * i - image (texteditor option)
                      * l - language
                      * n - nowrap
                      * r - reference (texteditor option)
                      * t - token (texteditor option)
                      */

                     $value    = NULL,
                     $checked  = NULL,
                     $name     = NULL,
                     $language = NULL)
        {
                static $index        = 0,
                       $file_index   = 0,
                       $radio_index  = 0,
                       $_radio_index = 0,
                       $radio_name   = NULL,
                       $prev_type    = NULL,
                       $label        = NULL;

                $_break = NULL;
                $affix  = NULL;

                //settings
                //                1        2           3           4          5
                //                new      type        length      maximum    parameter
                //                radio                            length
                if (preg_match("/^(new)?\s*([a-z_]+)\s*([0-9x]*)\s*([0-9]*)\s*([:bcdilnrt]*)$/", $type, $array) !== 1) return;

                //radio reset
                $new  = (bool)$array[1];

                //type
                $type = $array[2];

                //length
                if ($array[3])
                {
                        //extract horizontal/vertical length
                        if (preg_match("/^([0-9]+)x([0-9]+)$/", $array[3], $_array))
                        {
                                $size   = (int)$_array[1];
                                $height = (int)$_array[2];
                        }
                        else
                        {
                                $size   = (int)$array[3];
                                $height = 1;
                        };
                }
                else
                {
                        $size    = 1;
                        $height  = 1;
                };

                //maximum length
                $maxlength = (int)$array[4];

                //break after label text
                $break = stristr($array[5], ":") ? " " : "<br>";

                //parameter
                $checked   = $checked || stristr($array[5], "c");
                $disabled  = (bool)stristr($array[5], "d");
                $image     = stristr($array[5], "i") &&
                             cms_available("image") &&
                             ifc_available("image") &&
                             cms_permission("interface.image", NULL, NULL);
                $_language = (bool)stristr($array[5], "l");
                $nowrap    = (bool)stristr($array[5], "n");
                $reference = stristr($array[5], "r") &&
                             cms_available("content") &&
                             ifc_available("content") &&
                             cms_permission("interface.content", NULL, NULL);
                $token     = stristr($array[5], "t") &&
                             cms_available("token") &&
                             ifc_available("token") &&
                             cms_permission("interface.token", NULL, NULL);

                //language
                if ($_language)
                {
                        //extract languages
                        if ($language === NULL)
                                $language = (CMS_LANGUAGE_ENABLED !== "") ?
                                            explode(",", "," . CMS_LANGUAGE_ENABLED) : array();
                        else
                                $language = explode(",", $language);

                        //check active language
                        $__language = in_array(CMS_LANGUAGE, $language) ?
                                      CMS_LANGUAGE : reset($language);
                };

                //output
                switch ($type)
                {
                case "button":
//.....................................................................................................
                        $_language = FALSE; //disable language

                        if (streq(substr($value, 0, 11), "javascript:"))
                                $value = substr($value, 11);
                        elseif ($value)
                                $value = "ifc_post('" . q($value) . "');";
                        else
                                $value = "ifc_submit();";

                        //element
                        echo("<input type=\"button\" " .
                                    "value=\"" . x($text) . "\" " .
                                    "onclick=\"" . x($value) . "\"" .
                                    option($disabled, " disabled") .
                                    option(in_array($text, array(CMS_L_COMMAND_CONFIRM, CMS_L_COMMAND_SAVE)),
                                           " style=\"FONT-WEIGHT:bold\"") . ">");
                        break;

                case "password":
//.....................................................................................................
                        $_language = FALSE; //disable language

                case "text":
//.....................................................................................................

                        //generate element name
                        if (stre($name))
                        {
                                $index++;
                                $name = "ifc_param$index";
                        };

                        //get active language value
                        if ($_language)
                        {
                                $_name  = "l_$name";
                                $_value = language_get($value, $__language, TRUE);

                                if (stre($_value) && in_array("", $language))
                                {
                                        $__language = "";
                                        $_value     = language_get($value, FALSE);
                                };
                        }
                        else
                        {
                                $_name  = $name;
                                $_value = $value;
                        };

                        if (nstre($text))
                        {
                                echo("<label class=\"element\">" .
                                     nl2br(x($text)) . $break);
                                $affix = "</label>";
                        };

                        echo("<input name=\"" . x($_name) . "\" " .
                                    "type=\"$type\" " .
                                    "value=\"" . x($_value) . "\" " .
                                    "size=\"$size\"" .
                                    option($maxlength, " maxlength=\"$maxlength\"") .
                                    option($disabled,  " disabled") .
                                    option($_language, " onblur=\"ifc_language_save(this,'" . qx($name) . "');\"") . ">");
                        break;

                case "file":
//.....................................................................................................

                        //generate element name
                        if (stre($name))
                        {
                                $file_index++;
                                $name = "ifc_file$file_index";
                        };

                        $_language = FALSE; //disable language

                        //label
                        if (nstre($text))
                        {
                                echo("<label class=\"element\">" .
                                     nl2br(x($text)) . $break);
                                $affix = "</label>";
                        };

                        //element
                        echo("<input name=\"" . x($name) . "\" " .
                                    "type=\"file\"" .
                                    option($disabled, " disabled") . ">");
                        break;

                case "multifile":
//.....................................................................................................

                        //generate element name
                        if (stre($name))
                        {
                                $file_index++;
                                $name = "ifc_file$file_index";
                        };

                        $_language = FALSE; //disable language

                        //label
                        if (nstre($text))
                        {
                                echo("<label class=\"element\">" .
                                     nl2br(x($text)) . $break);
                                $affix = "</label>";
                        };

                        //element
                        echo("<input name=\"" . x($name) . "[]\" " .
                                    "type=\"file\" " .
                                    "multiple" .
                                    option($disabled, " disabled") . ">");
                        break;

                case "texteditor":
//.....................................................................................................

                        //generate element name
                        if (stre($name))
                        {
                                $index++;
                                $name = "ifc_param$index";
                        };

                        //get active language value
                        if ($_language)
                        {
                                $_name  = "l_$name";
                                $_value = language_get($value, $__language, TRUE);

                                if (stre($_value) && in_array("", $language))
                                {
                                        $__language = "";
                                        $_value     = language_get($value, FALSE);
                                };
                        }
                        else
                        {
                                $_name  = $name;
                                $_value = $value;
                        };

                        //label
                        if (nstre($text))
                        {
                                echo("<label class=\"element\">" .
                                     nl2br(x($text)) . " " .

                                     "<a href=\"javascript:ifc_resize('" . qx($_name) . "');\">" .
                                     image(CMS_IFC_EDITION . "/icon_resize", NULL, NULL, CMS_L_IFC_004) .
                                     "</a>" .

                                     $break);

                                $affix = "</label>";
                        };

                        $extension = "";

                        //image selector
                        if ($image)
                        {
                                $buffer     = "javascript:load_page('" .
                                              q(CMS_ACTIVE_URL) .
                                              "?ifc_page=image" .
                                              "&ifc_option=external" .
                                              "&ifc_select=_object[]" .
                                              "&ifc_select_action=" .
                                              "javascript:textcontrol_set(" .
                                              "\'this.opener.document.ifc." . rq(q($_name)) . "\'," .
                                              "\'%2523image_left\'," .
                                              "\'%return%\');');";
                                $extension .= "<a href=\"" . x($buffer) . "\">" .
                                              image("textcontrol/button_image_external", NULL, NULL, "image") .
                                              "</a>";
                        };

                        //link selector
                        if ($reference)
                        {
                                $buffer     = "javascript:load_page('" .
                                              q(CMS_ACTIVE_URL) .
                                              "?ifc_page=content" .
                                              "&ifc_message=edit_href" .
                                              "&ifc_option=external" .
                                              "&ifc_select=_value" .
                                              "&ifc_select_action=" .
                                              "javascript:textcontrol_set(" .
                                              "\'this.opener.document.ifc." . rq(q($_name)) . "\'," .
                                              "\'%2523hyperlink\'," .
                                              "\'%return%\');');";
                                $extension .= "<a href=\"" . x($buffer) . "\">" .
                                              image("textcontrol/button_hyperlink_external", NULL, NULL, "hyperlink") .
                                              "</a>";
                        };

                        //token selector
                        if ($token)
                        {
                                $buffer     = "javascript:load_page('" .
                                              q(CMS_ACTIVE_URL) .
                                              "?ifc_page=token" .
                                              "&ifc_option=external" .
                                              "&ifc_select=token" .
                                              "&ifc_select_action=" .
                                              "javascript:textcontrol_set(" .
                                              "\'this.opener.document.ifc." . rq(q($_name)) . "\'," .
                                              "\'%2523token\'," .
                                              "\'%return%\');');";
                                $extension .= "<a href=\"" . x($buffer) . "\">" .
                                              image("textcontrol/button_token_external", NULL, NULL, "token") .
                                              "</a>";
                        };

                        //element
                        echo(jscript("textcontrol(" .
                                     "\"document.ifc." . q($_name) . "\", " .
                                     "\"" . q(CMS_IMAGES_URL) . "\"" .
                                     option(nstre($extension), ", \"" . q($extension) . "\"") . ");") .
                             "<textarea name=\"" . x($_name) . "\" " .
                                       "cols=\"$size\" " .
                                       "rows=\"$height\" " .
                                       option($maxlength, "onkeyup=\"ifc_limit(this,$maxlength);\" ") .
                                       option($disabled,  "disabled ") .
                                       option($_language, "onblur=\"ifc_language_save(this,'" . qx($name) . "');\" ") .
                                       option($nowrap,    "style=\"WHITE-SPACE:pre\" ") .
                                       "onchange=\"textcontrol_select(this);\" " .
                                       "onkeydown=\"return ifc_keydown(event);\" " .
                                       "onscroll=\"ifc_linenumber(this);\" " .
                                       "onselect=\"textcontrol_select(this);\">\n" .
                             x($_value) .
                             "</textarea>");
                        break;

                case "textarea":
//.....................................................................................................

                        //generate element name
                        if (stre($name))
                        {
                                $index++;
                                $name = "ifc_param$index";
                        };

                        //get active language value
                        if ($_language)
                        {
                                $_name  = "l_$name";
                                $_value = language_get($value, $__language, TRUE);

                                if (stre($_value) && in_array("", $language))
                                {
                                        $__language = "";
                                        $_value     = language_get($value, FALSE);
                                };
                        }
                        else
                        {
                                $_name  = $name;
                                $_value = $value;
                        };

                        //label
                        if (nstre($text))
                        {
                                echo("<label class=\"element\">" .
                                     nl2br(x($text)) . " " .

                                     "<a href=\"javascript:ifc_resize('" . qx($_name) . "');\">" .
                                     image(CMS_IFC_EDITION . "/icon_resize", NULL, NULL, CMS_L_IFC_004) .
                                     "</a>" .

                                     $break);

                                $affix = "</label>";
                        };

                        //element
                        echo("<textarea name=\"" . x($_name) . "\" " .
                                       "cols=\"$size\" " .
                                       "rows=\"$height\" " .
                                       option($maxlength, "onkeyup=\"ifc_limit(this,$maxlength);\" ") .
                                       option($disabled,  "disabled ") .
                                       option($_language, "onblur=\"ifc_language_save(this,'" . qx($name) . "');\" ") .
                                       option($nowrap,    "style=\"WHITE-SPACE:pre\" ") .
                                       "onkeydown=\"return ifc_keydown(event);\" " .
                                       "onscroll=\"ifc_linenumber(this);\">\n" .
                             x($_value) .
                             "</textarea>");
                        break;

                case "checkbox":
//.....................................................................................................

                        //generate element name
                        if (stre($name))
                        {
                                $index++;
                                $name = "ifc_param$index";
                        };

                        $_language = FALSE; //disable language

                        //label
                        if ($flag = nstre($text))
                                echo("<label class=\"element nobr\">");

                        //element
                        echo("<input name=\"" . x($name) . "\" " .
                                    "type=\"checkbox\" " .
                                    "value=\"" . x($value) . "\"" .
                                    option($checked,  " checked") .
                                    option($disabled, " disabled") . ">");

                        //label
                        if ($flag)
                                echo(nl2br(x($text)) .
                                     "</label>");
                        break;

                case "select":
                case "multiselect":
//.....................................................................................................

                        //generate element name
                        if (stre($name))
                        {
                                $index++;
                                $name = "ifc_param$index";
                        };

                        $_language = FALSE; //disable language
                        $flag      = TRUE;

                        //use previously set label
                        if ($label !== NULL)
                        {
                                echo("<label class=\"element\">" .
                                     nl2br(x($label)) . $break);
                                $affix = "</label>";
                                $label = NULL;
                        };

                        //element
                        if (streq($type, "multiselect"))
                                echo("<select name=\"" . x($name) . "[]\" " .
                                             "size=\"$height\" " .
                                             "multiple>");
                        else
                                echo("<select name=\"" . x($name) . "\" " .
                                             "size=\"$height\">");

                        if ((! is_array($text)) || (! count($text)))
                                $text = array(" " => NULL);

                        //add missing options
                        if ($value)
                        {
                                if (is_array($value))
                                {
                                        reset($value);
                                        while (list($key, $_value) = each($value))
                                        {
                                                if (nstre($_value) && (! in_array($_value, $text)))
                                                        $text[CMS_L_IFC_005 . ": $_value"] = $_value;
                                        };
                                }
                                elseif (! in_array($value, $text))
                                {
                                        $text[CMS_L_IFC_005 . ": $value"] = $value;
                                };
                        };

                        //display options
                        reset($text);
                        while (list($key, $_value) = each($text))
                        {
                                echo("<option value=\"" . x($_value) . "\"" .
                                              option((is_array($value) && in_array($_value, $value)) || streq($_value, $value),
                                                     " selected") . ">");
                                if ($size > 1)
                                {
                                        $_size = utf8_strlen($key);
                                        if ($_size > $size) $key  = strabridge($key, $size);
                                        elseif ($flag)      $key .= str_repeat(" ", $size - $_size);
                                        $flag  = FALSE;
                                };
                                echo(whitechartostring(x($key), "&nbsp;") .
                                     "</option>");
                        };

                        echo("</select>");
                        break;

                case "radio":
                case "image":
//.....................................................................................................

                        //generate element name
                        if (stre($name))
                        {
                                $name  = "ifc_param";

                                //new group
                                if (
                                     $new
                                     ||
                                     ($radio_index === 0)
                                     ||
                                     nstreq($name, $radio_name)
                                   )
                                {
                                        $index++;
                                        $radio_index  = $index;
                                        $_radio_index = 0;
                                        $radio_name   = $name;
                                };

                                $name .= $radio_index;
                        }
                        elseif (nstreq($name, $radio_name))
                        {
                                $_radio_index = 0;
                                $radio_name   = $name;
                        };

                        $_language = FALSE; //disable language

                        //label
                        if ($flag = nstre($text))
                                echo("<label class=\"element nobr\">");

                        //element
                        echo("<input name=\"" . x($name) . "\" " .
                                    "type=\"radio\" " .
                                    "value=\"" . x($value) . "\"" .
                                    option($checked,  " checked") .
                                    option($disabled, " disabled") . ">");

                        //label
                        if ($flag)
                                echo((streq($type, "radio") ? nl2br(x($text)) : image($text)) .
                                     "</label>");

                        $_radio_index++;
                        break;

                case "title":
//.....................................................................................................
                        $_language = FALSE; //disable language

                        //element
                        echo("<div class=\"title\">" .
                             $text .
                             "</div>");
                        break;

                case "label":
//.....................................................................................................

                        //memorize label
                        $label = $text;
                        return;

                case "description":
//.....................................................................................................
                        $_language = FALSE; //disable language

                        //element
                        echo("<div class=\"label\">" .
                             nl2br(x($text)) .
                             "</div>");
                        break;

                case "info":
//.....................................................................................................
                        $_language = FALSE; //disable language

                        //element
                        echo("<div class=\"info\">" .
                             $text .
                             "</div>");
                        break;

                case "alert":
//.....................................................................................................
                        $_language = FALSE; //disable language

                        //element
                        echo(image("icon_alert") . "&nbsp;$text");
                        break;

                case "dummy":
//.....................................................................................................
                        $index += $size; //skip one or more elements
                        return;

                case "default":
//.....................................................................................................
                        return; //invalid
                };

                //language selector
                if ($_language)
                {
                        echo("<input name=\"" . x($name) . "\" " .
                                    "type=\"hidden\" " .
                                    "value=\"" . x($value) . "\">");

                        if (count($language) > 0)
                        {
                                //memorize initial language
                                echo(jscript("ifc_get_object(\"" . q($_name) . "\").language = \"" . q($__language) . "\";") .
                                     "<div class=\"language\">");

                                //language buttons
                                reset($language);
                                while (list(, $value) = each($language))
                                {
                                        echo("<a id=\"" . x($name) . ":" . x($value) . "\" " .
                                                "href=\"#\" " .
                                                "onclick=\"ifc_language_select('" . rqx($name) . "','" . rqx($_name) . "','" . rqx($value) . "');return false;\" " .
                                                "class=\"language-" . (streq($__language, $value) ? "on" : "off") . "\">");
                                        if ($value === "")
                                                echo(image("language/default_small", 16, 12, CMS_L_LANGUAGE_DEFAULT));
                                        else
                                                echo(image(is_file(CMS_DATA_PATH . "language/" . $value . "_small.gif") ?
                                                           CMS_DATA_URL . "language/" . $value . "_small.gif" :
                                                           "language/default_small",
                                                           16, 12, $value));
                                        echo("</a>");
                                };

                                echo("</div>");
                        };
                };

                echo($affix);

                //line break
                if (stristr($array[5], "b")) echo("<br>");

                //memorize type
                $prev_type = $type;
        }
//-----------------------------------------------------------------------------------------------------
        function dummy($count = 1)
        {
                $this->set(NULL, "dummy $count");
        }
//-----------------------------------------------------------------------------------------------------
        function close()
        {
                echo("</div>" .
                     "</form>" .

                //set focus on first input area
                     jscript("ifc_autofocus();"));

                //permission
                if ($permission = ifc_permission())
                {
                        echo("<div id=\"ifc-permission\">");
                        permission($permission);
                        echo("</div>" .
                             jscript("fx_register_callback(ifc_permission_event);"));
                };

                //return value
                if (CMS_IFC_SELECT)
                {
?>
<script>
var ifc_return_opener_url = this.opener.document.URL;

function ifc_return() //arguments: 0 => value
{
        if (this.opener && (! this.opener.closed) && (this.opener.document.URL == ifc_return_opener_url))
        {
                var value  = (arguments.length > 0) ?
                             arguments[0] :
                             ifc_value("<?php echo(q(CMS_IFC_SELECT));?>");
<?php
                        if (streq(substr(CMS_IFC_SELECT_ACTION, 0, 11), "javascript:"))
                        {
?>
                var action = "<?php echo(substr(q(CMS_IFC_SELECT_ACTION), 11));?>";
                action     = action.replace(/%return%/, addslashes(value));
                action     = action.replace(/%left%/,   this.opener.fx_page_left());
                action     = action.replace(/%top%/,    this.opener.fx_page_top());
                eval(action);
<?php
                        }
                        else
                        {
?>
                var action = "<?php echo(q(CMS_IFC_SELECT_ACTION));?>";
                action     = action.replace(/%return%/, encodeURIComponent(value));
                action     = action.replace(/%left%/,   this.opener.fx_page_left());
                action     = action.replace(/%top%/,    this.opener.fx_page_top());
                this.opener.location.replace(action);
<?php
                        };
?>
        };

        if (this == top) this.close();
};
//................................
function ifc_return_close()
{
        if ((! this.opener) || this.opener.closed || (this.opener.document.URL != ifc_return_opener_url))
        {
                this.close();
                return;
        };
};
//................................
function ifc_return_clear()
{
        ifc_return("");
};

if (
     (ifc_get_object("<?php echo(q(CMS_IFC_SELECT));?>") == null)
     &&
     (this == top)
   )
        ifc_return_close();
</script>
<?php
                };

                //restore scroll position
                global $ifc_left, //left scroll position
                       $ifc_top;  //top scroll position

                if ($ifc_left || $ifc_top)
                        echo(jscript("window.scrollTo(" . (int)$ifc_left . ", " . (int)$ifc_top . ");"));

                echo("</body>" .
                     "</html>");
                exit();
        }
};
?>