<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

//-----------------------------------------------------------------------------------------------------
//===< MEDIA TYPE CLASS >==============================================================================
//-----------------------------------------------------------------------------------------------------

//permission
define("CMS_MEDIA_TYPE_PERMISSION_OPERATOR", "operator");

//related functions
//-----------------------------------------------------------------------------------------------------
function media_type_get_select()
{
        $data       = new data("#system/media.type");

        $array[CMS_L_MEDIA_TYPE_001] = NULL;

        $data->move("first");
        while ($key = $data->move("next"))
        {
                $name                       = $data->get($key, "name");
                $array[$name ? $name : " "] = $key;
        };

        ksort($array);

        return $array;
};

class media_type
{
//properties
//-----------------------------------------------------------------------------------------------------
        public $data     = NULL;
        public $operator = NULL;
        public $type     = NULL;
//constructor
//-----------------------------------------------------------------------------------------------------
        function __construct()
        {
                $this->data     = new data("#system/media.type");
                $this->operator = cms_permission(CMS_MEDIA_TYPE_PERMISSION_OPERATOR);

                mkpath(CMS_DATA_PATH . "media.type");
                if (! $this->type = cms_cache("media_type.type")) $this->update_type();
        }
//methods
//-----------------------------------------------------------------------------------------------------
        function add($name,
                     $type = NULL,
                     $code = NULL)
        {
                if (! $this->operator) return FALSE;

                $index = unique_id();
                return $this->set($index, $name, $type, $code) ? $index : FALSE;
        }
//-----------------------------------------------------------------------------------------------------
        function set($index,
                     $name,
                     $type = NULL,
                     $code = NULL)
        {
                if (! $this->operator) return FALSE;

                //write code
                if (! $hfile = fopen(CMS_DATA_PATH . "media.type/$index.htm", "wb")) return FALSE;
                fputs($hfile, $code);
                fclose($hfile);

                //write entry
                $this->data->set(array("name" => $name,
                                       "type" => $type),
                                 $index);

                //save
                return $this->save();
        }
//-----------------------------------------------------------------------------------------------------
        function get_index($type)
        {
                $type = utf8_strtolower($type);

                if (! isset($this->type[$type])) return FALSE;
                return $this->type[$type];
        }
//-----------------------------------------------------------------------------------------------------
        function get_code($index)
        {
                return ($code = read_file(CMS_DATA_PATH . "media.type/$index.htm")) ? $code : FALSE;
        }
//-----------------------------------------------------------------------------------------------------
        function delete($index)
        {
                if (! $this->operator) return FALSE;

                $this->data->del($index);

                if ($this->save())
                {
                        if (is_file(CMS_DATA_PATH . "media.type/$index.htm"))
                                unlink(CMS_DATA_PATH . "media.type/$index.htm");

                        return TRUE;
                };

                return FALSE;
        }
//-----------------------------------------------------------------------------------------------------
        function parse($index,
                       $url,
                       $id     = NULL,
                       $type   = NULL,
                       $width  = NULL,
                       $height = NULL,
                       $alt    = NULL,
                       $title  = NULL,
                       $class  = NULL,
                       $style  = NULL)
        {
                //code
                if (! $code = l($this->get_code($index)))
                {
                        //fallback type identification
                        $_type = substr(strrchr($url, "."), 1);

                        if (
                             (! $index = $this->get_index($_type))
                             ||
                             (! $code  = l($this->get_code($index)))
                           )
                                $code = "<object[ id=\"%id%\"] " .
                                                "data=\"%src%\" " .
                                                "type=\"%type%\"" .
                                              "[ width=\"%width%\"]" .
                                              "[ height=\"%height%\"]" .
                                              "[ standby=\"" . sprintf(CMS_L_MEDIA_TYPE_002, "%title%") . "\"]" .
                                              "[ class=\"%class%\"]" .
                                              "[ style=\"%style%\"]>" .
                                        sprintf(CMS_L_MEDIA_TYPE_003, "%src%", "%alt%") .
                                        "</object>";
                };

                //type
                if (! $type) $type = get_mime_type($url);

                //replace placeholder
                $array = array("id"     => $id,
                               "src"    => $url,
                               "type"   => $type,
                               "width"  => $width,
                               "height" => $height,
                               "alt"    => $alt,
                               "title"  => $title,
                               "class"  => $class,
                               "style"  => $style);

                reset($array);
                while (list($key, $value) = each($array))
                        $code = nstre($value) ?
                                preg_replace("/(\\[([^\\[\\]]*))?%$key%(([^\\[\\]]*)\\])?/",
                                             "\${2}$value\\4",
                                             $code) :
                                preg_replace("/(?:\\[[^\\[\\]]*)?%$key%(?:[^\\[\\]]*\\])?/",
                                             "",
                                             $code);

                return $code;
        }
//-----------------------------------------------------------------------------------------------------
        function update_type()
        {
                $this->type = array();

                data_sort($this->data, "name");

                $this->data->move("first");
                while ($key = $this->data->move("next"))
                {
                        $type = $this->data->get($key, "type");
                        $type = explode(",", $type);

                        reset($type);
                        while (list(, $value) = each($type))
                        {
                                $value = utf8_trim($value);
                                $value = utf8_strtolower($value);
                                $this->type[$value] = $key;
                        };
                };

                cms_cache("media_type.type", $this->type);
        }
//-----------------------------------------------------------------------------------------------------
        function save()
        {
                if (! $this->operator) return FALSE;

                $this->update_type();
                return $this->data->save();
        }
};
?>