<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

if (! defined("CMS_MENU_FILTER_NORMAL"))
{
        define("CMS_MENU_FILTER_NORMAL",      0);
        define("CMS_MENU_FILTER_OPEN",        1);
        define("CMS_MENU_FILTER_ACTIVE",      2);
        define("CMS_MENU_FILTER_ACTIVE_OPEN", 3);
};

//-----------------------------------------------------------------------------------------------------
//===< MENU CLASS >====================================================================================
//-----------------------------------------------------------------------------------------------------

class menu
{
//properties
//-----------------------------------------------------------------------------------------------------
        protected $flexview    = NULL;
        protected $level       = 0;
        protected $depth       = NULL;
        protected $filter      = CMS_MENU_FILTER_NORMAL;
        protected $show_images = NULL;
        protected $exclude     = NULL;
//constructor
//-----------------------------------------------------------------------------------------------------
        function __construct($index            = 0,
                             $base             = 0,
                             $level            = 0,
                             $depth            = NULL,
                             $filter           = CMS_MENU_FILTER_NORMAL,
                             $show_icons       = FALSE,
                             $show_images      = FALSE,
                             $show_description = FALSE,
                             $exclude          = NULL)
        {
                if (! cms_load("directory")) return; //library unavailable

                //get display object containing the directory tree structure
                $this->flexview = directory_get_flexview();

                //set currently selected directory entry (or nearest visible entry above)
                $index = directory_get_visible($index);
                $this->flexview->set_index($index);

                //set base entry of directory branch to be displayed
                //all elements outside that branch are ignored
                $this->flexview->set_base($base);

                //set action to be executed on selection, usually an url
                $this->flexview->set_action(CMS_ROOT_URL . "%index%" . cms_param());

                //set display string
                $this->flexview->display =
                        "<a[ href=\"%action%\"]" .
                          "[ class=\"%class%\"]>" .
                        option($show_icons,       "[%icon% ]") .
                        "%name%" .
                        option($show_description, "[ <span>%description%</span>]") .
                        "</a>";

                //set visibility parameters

                //level
                $this->level = preg_match("/^(?:\+|-)/", $level) ?
                               count($this->flexview->get_path()) + (int)$level : //relative
                               (int)$level;                                       //absolute
                if ($this->level < 0) $this->level = 0;

                //depth
                if ($depth !== NULL)
                {
                        $depth = (int)$depth;
                        if ($depth < 0) $depth = 0;
                };
                $this->depth = $depth;

                //filter
                switch (utf8_strtolower($filter))
                {
                case "open":        $this->filter = CMS_MENU_FILTER_OPEN;        break;
                case "active":      $this->filter = CMS_MENU_FILTER_ACTIVE;      break;
                case "active-open": $this->filter = CMS_MENU_FILTER_ACTIVE_OPEN; break;
                default:            $this->filter = CMS_MENU_FILTER_NORMAL;      break;
                };

                //show images
                $this->show_images = $show_images;

                //exclusion
                if ($exclude !== NULL)
                        $this->exclude = is_array($exclude) ?
                                         $exclude :
                                         preg_split("/\s+/", trim($exclude));

                //evoke display method
                $this->flexview->show_custom(array($this, "show"));
        }
//methods
//-----------------------------------------------------------------------------------------------------
        function show($flexview_entry)
        {
                static $indentation = -1;
                static $open        = TRUE;
                static $instance    = 0;

                //the passed variable flexview_entry is a structure containing
                //all necessary information about the current entry
                //each entry below the given base entry is processed
                //beginning with the top of the tree and following the branches

                switch ($flexview_entry->type) //the type of event that is passed
                {
                case CMS_FLEXVIEW_ENTRY_TYPE_BASE: //before first entry
//.....................................................................................................

                        //reset
                        $indentation  = -1;
                        $open         = array(-1 => FALSE);
                        $instance    += 1;
                        return;

                case CMS_FLEXVIEW_ENTRY_TYPE_ENTRY: //current entry
//.....................................................................................................

                        $open[$flexview_entry->indentation] = $flexview_entry->open;

                        //check if entry is to be displayed
                        if (($this->filter === CMS_MENU_FILTER_ACTIVE_OPEN) &&
                            (! $flexview_entry->open) &&
                            (! $open[$flexview_entry->indentation - 1]))
                                return TRUE; //skip subsequent entries

                        //check if entry is excluded
                        if (($this->exclude !== NULL) &&
                            in_array($flexview_entry->index, $this->exclude))
                                return TRUE; //skip subsequent entries

                        //check if entry is to be displayed
                        $visible = ($flexview_entry->indentation >=  $this->level) &&
                                   (($this->depth                === NULL) ||
                                   ($flexview_entry->indentation <   ($this->level + $this->depth))) &&
                                   (($this->filter !== CMS_MENU_FILTER_ACTIVE) || $open[0]);

                        if (! $visible) return;

                        //start sublist
                        if ($indentation < $flexview_entry->indentation)
                        {
                                $this->_ul_li($instance, $flexview_entry->index, $flexview_entry->open);
                                $indentation = $flexview_entry->indentation;
                        }

                        //end sublist
                        else
                        {
                                for (; $indentation > $flexview_entry->indentation; $indentation--)
                                        echo("</li></ul>");

                                //separate entry
                                echo("</li>");
                                $this->_li($instance, $flexview_entry->index, $flexview_entry->open);
                        };

                        //display entry
                        $this->flexview->display($flexview_entry->index,
                                                 $flexview_entry->open);

                        //check if subsequent entries are to be displayed
                        if (($this->filter === CMS_MENU_FILTER_OPEN) &&
                            (! $flexview_entry->open))
                                return TRUE; //skip subsequent entries

                        return; //display subsequent entries

                case CMS_FLEXVIEW_ENTRY_TYPE_END: //after last entry
//.....................................................................................................

                        //end sublist
                        for (; $indentation >= $this->level; $indentation--)
                                echo("</li></ul>");
                };
        }
//-----------------------------------------------------------------------------------------------------
        protected function _ul_li($instance, $index, $open)
        {
                $id = "menu" . x($instance) . "-" . x($index);
                $this->_style($id, $index, $open);
                echo("<ul><li id=\"" . x($id) . "\">");
        }
//-----------------------------------------------------------------------------------------------------
        protected function _li($instance, $index, $open)
        {
                $id = "menu" . x($instance) . "-" . x($index);
                $this->_style($id, $index, $open);
                echo("<li id=\"" . x($id) . "\">");
        }
//-----------------------------------------------------------------------------------------------------
        protected function _style($id, $index, $open)
        {
                if (! $this->show_images) return;

                $image_url   = $this->flexview->get_image($index, $open);
                $image_hover = isset($this->flexview->object[$index]["#data"][$this->flexview->image_hover_key]) ?
                               translate_url(l($this->flexview->object[$index]["#data"][$this->flexview->image_hover_key])) : NULL;
                $flag        = ($image_url !== NULL) || ($image_hover !== NULL);

                if ($flag)                 echo("<style scoped>");
                if ($image_url   !== NULL) echo("#" . x($id) .       "{BACKGROUND-IMAGE:url(\"" . x($image_url) .   "\");}");
                if ($image_hover !== NULL) echo("#" . x($id) . ":hover{BACKGROUND-IMAGE:url(\"" . x($image_hover) . "\");}");
                if ($flag)                 echo("</style>");
        }
};
?>
