<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

//-----------------------------------------------------------------------------------------------------
//===< RSS PARSER CLASS >==============================================================================
//-----------------------------------------------------------------------------------------------------
class rss_parser_attribute_list
{
//properties
//-----------------------------------------------------------------------------------------------------
//constructor
//-----------------------------------------------------------------------------------------------------
//methods
//-----------------------------------------------------------------------------------------------------
        function add($name,
                     $value)
        {
                $this->{$name} = new rss_parser_attribute($value);
        }
//-----------------------------------------------------------------------------------------------------
        function __get($name)
        {
                if (isset($this->{$name})) return $this->{$name};
                return NULL;
        }
//-----------------------------------------------------------------------------------------------------
        function __toString()
        {
                return "";
        }
};

class rss_parser_attribute
{
//properties
//-----------------------------------------------------------------------------------------------------
        public $value = "";
//constructor
//-----------------------------------------------------------------------------------------------------
        function __construct($value)
        {
                $this->value = $value;
        }
//methods
//-----------------------------------------------------------------------------------------------------
        function __get($name)
        {
                return $this->value;
        }
//-----------------------------------------------------------------------------------------------------
        function __toString()
        {
                return $this->value;
        }
};

class rss_parser_node
{
//properties
//-----------------------------------------------------------------------------------------------------
        public $_parent = NULL;
        public $_name   = "";
        public $_data   = "";
        public $_attrib = NULL;
//constructor
//-----------------------------------------------------------------------------------------------------
        function __construct($name = "")
        {
                $this->_name   = $name;
                $this->_attrib = new rss_parser_attribute_list();
        }
//-----------------------------------------------------------------------------------------------------
        function add_attribute($name,
                               $value)
        {
                $this->_attrib->add($name, $value);
        }
//-----------------------------------------------------------------------------------------------------
        function add_node($name)
        {
                $node          = new rss_parser_node($name);
                $node->_parent = $this;

                if (isset($this->{$name}))
                {
                        if (is_array($this->{$name}))
                                $this->{$name}[] = $node;
                        else
                                $this->{$name}   = array($this->{$name}, $node);

                        $buffer = end($this->{$name});
                        return $buffer;
                };

                $this->{$name} = $node;
                return $this->{$name};
        }
//-----------------------------------------------------------------------------------------------------
        function get_parent()
        {
                return $this->_parent;
        }
//-----------------------------------------------------------------------------------------------------
        function get_path()
        {
                $buffer = $this->_name;
                $object = $this;

                while ($object = $object->_parent)
                {
                        if ($object->_name !== "")
                                $buffer = $object->_name . ":$buffer";
                };

                return $buffer;
        }
//-----------------------------------------------------------------------------------------------------
        function __get($name)
        {
                if (isset($this->{$name})) return $this->{$name};
                return NULL;
        }
//-----------------------------------------------------------------------------------------------------
        function __toString()
        {
                return $this->_data;
                return is_array($this->_data) ? reset($this->_data) : $this->_data;
        }
};

class rss_parser
{
//properties
//-----------------------------------------------------------------------------------------------------
        public $show_channel                 = TRUE;
        public $show_channel_title           = TRUE;
        public $show_channel_link            = TRUE;
        public $show_channel_description     = TRUE;
        public $show_channel_copyright       = TRUE;
        public $show_channel_managing_editor = TRUE;
        public $show_channel_web_master      = TRUE;
        public $show_channel_pub_date        = TRUE;
        public $show_channel_last_build_date = TRUE;
        public $show_channel_category        = TRUE;
        public $show_channel_generator       = TRUE;
        public $show_channel_cloud           = TRUE;
        public $show_channel_image           = TRUE;
        public $show_channel_text_input      = TRUE;
        public $show_item                    = TRUE;
        public $show_item_title              = TRUE;
        public $show_item_link               = TRUE;
        public $show_item_description        = TRUE;
        public $show_item_author             = TRUE;
        public $show_item_category           = TRUE;
        public $show_item_comments           = TRUE;
        public $show_item_enclosure          = TRUE;
        public $show_item_guid               = TRUE;
        public $show_item_pub_date           = TRUE;
        public $show_item_source             = TRUE;

        public $channel_image_default_width  = 120;
        public $channel_image_default_height = NULL; //auto

        public $enable_html_filter           = FALSE;
        public $max_text_length              = 75;
        public $max_description_length       = 400;  //html filter only
        public $max_item_number              = 20;

        public $parser                       = NULL;
        public $data                         = NULL;
        public $node                         = NULL;
        public $stack                        = array();
        public $structure                    = array(
                "rss",
                "rss.version",
                "rss:channel",
                "rss:channel:title",
                "rss:channel:link",
                "rss:channel:description",
                "rss:channel:language",
                "rss:channel:copyright",
                "rss:channel:managingEditor",
                "rss:channel:webMaster",
                "rss:channel:pubDate",
                "rss:channel:lastBuildDate",
                "rss:channel:category",
                "rss:channel:generator",
                "rss:channel:docs",
                "rss:channel:cloud",
                "rss:channel:cloud.domain",
                "rss:channel:cloud.port",
                "rss:channel:cloud.path",
                "rss:channel:cloud.registerProcedure",
                "rss:channel:cloud.protocol",
                "rss:channel:ttl",
                "rss:channel:image",
                "rss:channel:image:url",
                "rss:channel:image:title",
                "rss:channel:image:link",
                "rss:channel:image:width",
                "rss:channel:image:height",
                "rss:channel:image:description",
                "rss:channel:rating",
                "rss:channel:textInput",
                "rss:channel:textInput:title",
                "rss:channel:textInput:description",
                "rss:channel:textInput:name",
                "rss:channel:textInput:link",
                "rss:channel:skipHours",
                "rss:channel:skipDays",
                "rss:channel:item",
                "rss:channel:item:title",
                "rss:channel:item:link",
                "rss:channel:item:description",
                "rss:channel:item:author",
                "rss:channel:item:category",
                "rss:channel:item:category.domain",
                "rss:channel:item:comments",
                "rss:channel:item:enclosure",
                "rss:channel:item:enclosure.url",
                "rss:channel:item:enclosure.length",
                "rss:channel:item:enclosure.type",
                "rss:channel:item:guid",
                "rss:channel:item:guid.isPermaLink",
                "rss:channel:item:pubDate",
                "rss:channel:item:source",
                "rss:channel:item:source.url",);
//constructor
//-----------------------------------------------------------------------------------------------------
        function __construct()
        {
                //create and setup xml parser
                $this->parser = xml_parser_create("UTF-8");
                xml_set_object($this->parser, $this);
                xml_set_element_handler($this->parser, "_start", "_end");
                xml_set_character_data_handler($this->parser, "_data");
                xml_parser_set_option($this->parser, XML_OPTION_CASE_FOLDING, FALSE);
        }
//methods
//-----------------------------------------------------------------------------------------------------
        function parse($source)
        {
                //caching
                $cache_key  = "rss_parser.$source";
                $cache_time = cms_cache_time($cache_key);
                $time       = time();

                if (($cache_time !== FALSE) && ($cache_time > ($time - 60))) //60 seconds delay
                {
                        $this->data  = unserialize(cms_cache($cache_key));
                        $this->node  = $this->data;
                        $this->stack = array();
                        return TRUE;
                };

                //initialize
                $this->data  = new rss_parser_node();
                $this->node  = $this->data;
                $this->stack = array();

                //open source
                if (! ($hfile = fopen($source, "rb"))) return FALSE;

                //process data
                while ($buffer = fread($hfile, 65536))
                {
                        if (! xml_parse($this->parser, $buffer, feof($hfile)))
                        {
                                //parse error
                                fclose($hfile);
                                return FALSE;
                        };
                };

                //close source
                fclose($hfile);

                //cache permanently
                cms_cache($cache_key, serialize($this->data), TRUE);

                return TRUE;
        }
//-----------------------------------------------------------------------------------------------------
        function display()
        {
                if ($this->show_channel)
                {
                        if (! isset($this->data->rss->channel)) return;

                        $channel = $this->data->rss->channel;
                        echo("<div class=\"rss-channel\">");

//.....................................................................................................

                        //image
                        if ($this->show_channel_image)
                        {
                                echo("<div class=\"rss-channel-image\">");

                                if ($buffer = $channel->image)
                                {


                                        $array = is_array($buffer) ? $buffer : array($buffer);

                                        reset($array);
                                        while (list(, $value) = each($array))
                                        {
                                                if ($value->url && $value->title && $value->link)
                                                {
                                                        //width
                                                        $width  = $this->channel_image_default_width;
                                                        if ($value->width)
                                                        {
                                                                $width  = (int)$value->width->__toString();
                                                                if ($width > 144)  $width  = 144;
                                                        };

                                                        //height
                                                        $height = $this->channel_image_default_height;
                                                        if ($value->height)
                                                        {
                                                                $height = (int)$value->height->__toString();
                                                                if ($height > 400) $height = 400;
                                                        };

                                                        echo("<a href=\"" . x($value->link) . "\"" .
                                                             ($value->description ? " title=\"" . x($value->description) . "\">" : ">") .
                                                             image($value->url, $width, $height, $value->title) .
                                                             "</a>");
                                                };
                                        };
                                };

                                echo("</div>");
                        };

//.....................................................................................................

                        //category
                        if ($this->show_channel_category)
                        {
                                echo("<div class=\"rss-channel-category\">");

                                if ($buffer = $channel->category)
                                {
                                        $array = is_array($buffer) ? $buffer : array($buffer);
                                        $i     = 1;

                                        reset($array);
                                        while (list(, $value) = each($array))
                                        {
                                                $flag = (bool)$value->_attrib->domain;

                                                echo(($i++ === 1) ? "" : " // ");
                                                if ($flag) echo("<a href=\"" . x($value->_attrib->domain) . "\">");
                                                echo($this->ft($value));
                                                if ($flag) echo("</a>");
                                        };
                                };

                                echo("</div>");
                        };

//.....................................................................................................

                        //title
                        if ($this->show_channel_title)
                        {
                                echo("<div class=\"rss-channel-title\">");

                                if ($buffer = $channel->title)
                                {
                                        $flag = $this->show_channel_link && $channel->link;

                                        echo("<h2>");
                                        if ($flag) echo("<a href=\"" . x($channel->link) . "\">");
                                        echo($this->ft($channel->title));
                                        if ($flag) echo("</a>");
                                        echo("</h2>");
                                };

                                echo("</div>");
                        };

//.....................................................................................................

                        //description
                        if ($this->show_channel_description)
                        {
                                echo("<div class=\"rss-channel-description\">");

                                if ($buffer = $channel->description)
                                        echo("<p>" .
                                             $this->fd($buffer) .
                                             "</p>");

                                echo("</div>");
                        };

//.....................................................................................................

                        echo("<div class=\"rss-channel-block-1\">");

//.....................................................................................................

                        //publication date
                        if ($this->show_channel_pub_date)
                        {
                                echo("<div class=\"rss-channel-pub-date\">");

                                if ($buffer = $channel->pubDate)
                                {
                                        $array = is_array($buffer) ? $buffer : array($buffer);
                                        $i     = 1;

                                        reset($array);
                                        while (list(, $value) = each($array))
                                        {
                                                $buffer = strtotime($value);
                                                echo((($i++ === 1) ? CMS_L_RSS_PARSER_001 . ": " : "; ") .
                                                     x(friendly_date($buffer)));
                                        };
                                };

                                echo("</div>");
                        };

//.....................................................................................................

                        //last build date
                        if ($this->show_channel_last_build_date)
                        {
                                echo("<div class=\"rss-channel-last-build-date\">");

                                if ($buffer = $channel->lastBuildDate)
                                {
                                        $array = is_array($buffer) ? $buffer : array($buffer);
                                        $i     = 1;

                                        reset($array);
                                        while (list(, $value) = each($array))
                                        {
                                                $buffer = strtotime($value);
                                                echo((($i++ === 1) ? CMS_L_RSS_PARSER_002 . ": " : "; ") .
                                                     x(friendly_date($buffer)));
                                        };
                                };

                                echo("</div>");
                        };

//.....................................................................................................

                        //generator
                        if ($this->show_channel_generator)
                        {
                                echo("<div class=\"rss-channel-generator\">");

                                if ($buffer = $channel->generator)
                                {
                                        $array = is_array($buffer) ? $buffer : array($buffer);
                                        $i     = 1;

                                        reset($array);
                                        while (list(, $value) = each($array))
                                                echo((($i++ === 1) ? CMS_L_RSS_PARSER_003 . ": " : ", ") .
                                                     $this->ft($value));
                                };

                                echo("</div>");
                        };

//.....................................................................................................

                        echo("</div>" .
                             "<div class=\"rss-channel-block-2\">");

//.....................................................................................................

                        //copyright
                        if ($this->show_channel_copyright)
                        {
                                echo("<div class=\"rss-channel-copyright\">");

                                if ($buffer = $channel->copyright)
                                {
                                        $array = is_array($buffer) ? $buffer : array($buffer);
                                        $i     = 1;

                                        reset($array);
                                        while (list(, $value) = each($array))
                                                echo((($i++ === 1) ? "\xC2\xA9 " : "; ") .
                                                     $this->ft($value));
                                };

                                echo("</div>");
                        };

//.....................................................................................................

                        //managing Editor
                        if ($this->show_channel_managing_editor)
                        {
                                echo("<div class=\"rss-channel-managing-editor\">");

                                if ($buffer = $channel->managingEditor)
                                {
                                        $array = is_array($buffer) ? $buffer : array($buffer);
                                        $i     = 1;

                                        reset($array);
                                        while (list(, $value) = each($array))
                                                echo((($i++ === 1) ? CMS_L_RSS_PARSER_004 . ": " : ", ") .
                                                     $this->ft($value));
                                };

                                echo("</div>");
                        };

//.....................................................................................................

                        //web master
                        if ($this->show_channel_web_master)
                        {
                                echo("<div class=\"rss-channel-web-master\">");

                                if ($buffer = $channel->webMaster)
                                {
                                        $array = is_array($buffer) ? $buffer : array($buffer);
                                        $i     = 1;

                                        reset($array);
                                        while (list(, $value) = each($array))
                                                echo((($i++ === 1) ? CMS_L_RSS_PARSER_005 . ": " : ", ") .
                                                     $this->ft($value));
                                };

                                echo("</div>");
                        };

//.....................................................................................................

                        echo("</div><br style=\"CLEAR:both\">" .
                             "</div>");
                };

                if (! $this->show_item)                       return;
                if (! isset($this->data->rss->channel->item)) return;

                $item = $this->data->rss->channel->item;
                $item = is_array($item) ? $item : array($item);
                $i    = 0;

                reset($item);
                while (($i++ < $this->max_item_number) && (list(, $value) = each($item)))
                {
                        echo("<div class=\"rss-item\">");

//.....................................................................................................

                        //category
                        if ($this->show_item_category)
                        {
                                echo("<div class=\"rss-item-category\">");

                                if ($buffer = $value->category)
                                {
                                        $array = is_array($buffer) ? $buffer : array($buffer);
                                        $_i    = 1;

                                        reset($array);
                                        while (list(, $_value) = each($array))
                                        {
                                                $flag = (bool)$_value->_attrib->domain;

                                                echo(($_i++ === 1) ? "" : " // ");
                                                if ($flag) echo("<a href=\"" . x($_value->_attrib->domain) . "\">");
                                                echo($this->ft($_value));
                                                if ($flag) echo("</a>");
                                        };
                                };

                                echo("</div>");
                        };

//.....................................................................................................

                        //publication date
                        if ($this->show_item_pub_date)
                        {
                                echo("<div class=\"rss-item-pub-date\">");

                                if ($buffer = $value->pubDate)
                                {
                                        $array = is_array($buffer) ? $buffer : array($buffer);
                                        $_i    = 1;

                                        reset($array);
                                        while (list(, $_value) = each($array))
                                        {
                                                $buffer = strtotime($_value);
                                                echo((($_i++ === 1) ? "" : "; ") .
                                                     x(friendly_date($buffer)));
                                        };
                                };

                                echo("</div>");
                        };

//.....................................................................................................

                        //title
                        if ($this->show_item_title)
                        {
                                echo("<div class=\"rss-item-title\">");

                                if ($value->title)
                                {
                                        $flag = $this->show_item_link && $value->link;

                                        echo("<h3>");
                                        if ($flag) echo("<a href=\"" . x($value->link) . "\">");
                                        echo($this->ft($value->title));
                                        if ($flag) echo("</a>");
                                        echo("</h3>");
                                };

                                echo("</div>");
                        };

//.....................................................................................................

                        //description
                        if ($this->show_item_description)
                        {
                                echo("<div class=\"rss-item-description\">");

                                if ($buffer = $value->description)
                                        echo("<p>" .
                                             $this->fd($buffer) .
                                             "</p>");

                                echo("</div>");
                        };

//.....................................................................................................

                        //enclosure
                        if ($this->show_item_enclosure)
                        {
                                echo("<div class=\"rss-item-enclosure\">");

                                if (
                                     ($buffer = $value->enclosure)
                                     &&
                                     cms_load("media_type")
                                   )
                                {
                                        $media_type = new media_type();
                                        $array      = is_array($buffer) ? $buffer : array($buffer);

                                        reset($array);
                                        while (list(, $_value) = each($array))
                                        {
                                                if (
                                                     $_value->_attrib->url
                                                     &&
                                                     $_value->_attrib->length
                                                     &&
                                                     $_value->_attrib->type
                                                   )
                                                        echo("<p>" .
                                                             $media_type->parse(NULL,                                //index
                                                                                x($value->enclosure->_attrib->url),  //url
                                                                                "media_" . unique_id(),              //id
                                                                                x($value->enclosure->_attrib->type), //type
                                                                                300,                                 //width
                                                                                225,                                 //height
                                                                                NULL,                                //alt
                                                                                NULL,                                //title
                                                                                NULL,                                //class
                                                                                NULL) .                              //style
                                                             "</p>");
                                        };
                                };

                                echo("</div>");
                        };

//.....................................................................................................

                        //comments
                        if ($this->show_item_comments)
                        {
                                echo("<div class=\"rss-item-comments\">");

                                if ($buffer = $value->comments)
                                {
                                        if (is_array($buffer))
                                        {
                                                echo(CMS_L_RSS_PARSER_006 . ":");

                                                $_i = 1;

                                                reset($buffer);
                                                while (list(, $_value) = each($buffer))
                                                        echo(" " .
                                                             "<a href=\"" . x($_value->comments) . "\">" .
                                                             "[" . ($_i++) . "]" .
                                                             "</a>");
                                        }
                                        else
                                        {
                                                echo("<a href=\"" . x($buffer) . "\">" .
                                                     CMS_L_RSS_PARSER_006 .
                                                     "</a>");
                                        };
                                };

                                echo("</div>");
                        };

//.....................................................................................................

                        echo("<div class=\"rss-item-block\">");

//.....................................................................................................

                        //author
                        if ($this->show_item_author)
                        {
                                echo("<div class=\"rss-item-author\">");

                                if ($buffer = $value->author)
                                {
                                        $array = is_array($buffer) ? $buffer : array($buffer);
                                        $_i    = 1;

                                        reset($array);
                                        while (list(, $_value) = each($array))
                                                echo((($_i++ === 1) ? CMS_L_RSS_PARSER_007 . " " : ", ") .
                                                     $this->ft($_value));
                                };

                                echo("</div>");
                        };

//.....................................................................................................

                        //source
                        if ($this->show_item_source)
                        {
                                echo("<div class=\"rss-item-source\">");

                                if ($buffer = $value->source)
                                {
                                        $array = is_array($buffer) ? $buffer : array($buffer);
                                        $_i    = 1;

                                        reset($array);
                                        while (list(, $_value) = each($array))
                                        {
                                                if ($buffer = $_value->_attrib->url)
                                                        echo((($_i === 1) ? CMS_L_RSS_PARSER_008 . ": " : ", ") .
                                                             "<a href=\"" . x($buffer) . "\">" .
                                                             "<sup>" .
                                                             "[" . ($_i++) . "]" .
                                                             "</sup>" .
                                                             $this->ft($_value) .
                                                             "</a>");
                                        };
                                };

                                echo("</div>");
                        };

//.....................................................................................................

                        //guid
                        if ($this->show_item_guid)
                        {
                                echo("<div class=\"rss-item-guid\">");

                                if ($buffer = $value->guid)
                                {
                                        if (is_array($buffer))
                                        {
                                                $_i = 1;

                                                reset($buffer);
                                                while (list(, $_value) = each($buffer))
                                                {
                                                        if (streq($_value->_attrib->isPermaLink, "true"))
                                                                echo((($_i === 1) ? CMS_L_RSS_PARSER_009 . ": " : " ") .
                                                                     "<a href=\"" . x($_value) . "\">" .
                                                                     "[" . ($_i++) . "]" .
                                                                     "</a>");
                                                };
                                        }
                                        else
                                        {
                                                if (streq($buffer->_attrib->isPermaLink, "true"))
                                                        echo("<a href=\"" . x($buffer) . "\">" .
                                                             CMS_L_RSS_PARSER_009 .
                                                             "</a>");
                                        };
                                };

                                echo("</div>");
                        };

//.....................................................................................................

                        echo("</div><br style=\"CLEAR:both\">" .
                             "</div>");
                };
        }
//-----------------------------------------------------------------------------------------------------
        function fd($value) //filter description
        {
                if (! $this->enable_html_filter) return $value; //filter disabled

                //filter html markup
                $value = htmltoplain($value);

                //limit string length
                if (strlen($value) > $this->max_description_length)
                        $value = utf8_substr($value, 0, $this->max_description_length - 1) . "\xE2\x80\xA6";

                //escape xml special characters
                $value = x($value);

                return $value;
        }
//-----------------------------------------------------------------------------------------------------
        function ft($value) //filter text
        {
                //limit string length
                if (strlen($value) > $this->max_text_length)
                        $value = utf8_substr($value, 0, $this->max_text_length - 1) . "\xE2\x80\xA6";

                //escape xml special characters
                $value = x($value);

                return $value;
        }
//-----------------------------------------------------------------------------------------------------
        function _start($parser,
                        $tag,
                        $attribute)
        {
                //current path
                $path  = $this->node->get_path();
                $path .= ($path === "") ? $tag : ":$tag";

                //ignore invalid or misplaced tag
                if (! in_array($path, $this->structure)) return;

                //create and select node
                $this->node = $this->node->add_node($tag);

                //add attributes
                reset($attribute);
                while (list($key, $value) = each($attribute))
                {
                        //validate attribute
                        if (in_array("$path.$key", $this->structure))
                                $this->node->add_attribute($key, $value);
                };

                //open tag
                array_push($this->stack, $tag);
        }
//-----------------------------------------------------------------------------------------------------
        function _data($parser,
                       $data)
        {
                //set value
                $this->node->_data .= $data;
        }
//-----------------------------------------------------------------------------------------------------
        function _end($parser,
                      $tag)
        {
                //ignore invalid or misplaced closing tag
                if (nstreq($tag, end($this->stack))) return;

                //select parent node
                $this->node = $this->node->get_parent();

                //close tag
                array_pop($this->stack);
        }
};
?>