<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

//-----------------------------------------------------------------------------------------------------
//===< SEARCH CLASS >==================================================================================
//-----------------------------------------------------------------------------------------------------

//permission
define("CMS_SEARCH_PERMISSION_SUBMIT",        "submit");

//database
define("CMS_DB_SEARCH_ENTRY",                 CMS_DB_PREFIX . "search_entry");
define("CMS_DB_SEARCH_ENTRY_INDEX",           "id");
define("CMS_DB_SEARCH_ENTRY_ADDRESS",         "address");
define("CMS_DB_SEARCH_ENTRY_ADDRESS_HASH",    "address_hash");
define("CMS_DB_SEARCH_ENTRY_TITLE",           "title");
define("CMS_DB_SEARCH_ENTRY_TEXT",            "text");
define("CMS_DB_SEARCH_ENTRY_TEXT_HASH",       "text_hash");
define("CMS_DB_SEARCH_ENTRY_TIME",            "time");
define("CMS_DB_SEARCH_ENTRY_UPDATE_INTERVAL", "update_interval");
define("CMS_DB_SEARCH_ENTRY_SCORE",           "score");
define("CMS_DB_SEARCH_ENTRY_LINK_COUNT",      "link_count");
define("CMS_DB_SEARCH_ENTRY_ERROR",           "error");

define("CMS_DB_SEARCH_WORD",                  CMS_DB_PREFIX . "search_word");
define("CMS_DB_SEARCH_WORD_INDEX",            "id");
define("CMS_DB_SEARCH_WORD_TEXT",             "text");
define("CMS_DB_SEARCH_WORD_LANGUAGE",         "language");

define("CMS_DB_SEARCH_WEIGHT",                CMS_DB_PREFIX . "search_weight");
define("CMS_DB_SEARCH_WEIGHT_WORD",           "word");
define("CMS_DB_SEARCH_WEIGHT_SOURCE",         "source");
define("CMS_DB_SEARCH_WEIGHT_TARGET",         "target");
define("CMS_DB_SEARCH_WEIGHT_VALUE",          "value");

define("CMS_DB_SEARCH_LINK",                  CMS_DB_PREFIX . "search_link");
define("CMS_DB_SEARCH_LINK_SOURCE",           "source");
define("CMS_DB_SEARCH_LINK_TARGET_HASH",      "target_hash");
define("CMS_DB_SEARCH_LINK_TARGET",           "target");
define("CMS_DB_SEARCH_LINK_TEXT",             "text");
define("CMS_DB_SEARCH_LINK_LEVEL",            "level");

define("CMS_DB_SEARCH_CLUSTER",               CMS_DB_PREFIX . "search_cluster");
define("CMS_DB_SEARCH_CLUSTER_SOURCE",        "source");
define("CMS_DB_SEARCH_CLUSTER_TARGET",        "target");

define("CMS_DB_SEARCH_QUEUE",                 CMS_DB_PREFIX . "search_queue");
define("CMS_DB_SEARCH_QUEUE_ADDRESS",         "address");
define("CMS_DB_SEARCH_QUEUE_TYPE",            "type");
define("CMS_DB_SEARCH_QUEUE_TIME",            "time");
define("CMS_DB_SEARCH_QUEUE_CODE",            "code");
define("CMS_DB_SEARCH_QUEUE_ERROR",           "error");

//scan result code
define("CMS_SEARCH_SCAN_UNKNOWN_ERROR",                  1);
define("CMS_SEARCH_SCAN_REDIRECTION_LIMIT_EXCEEDED",     2);
define("CMS_SEARCH_SCAN_INVALID_ADDRESS",                4);
define("CMS_SEARCH_SCAN_ADDRESS_REJECTED",               8);
define("CMS_SEARCH_SCAN_NO_CONNECTION",                 16);
define("CMS_SEARCH_SCAN_UNSUPPORTED_RESOURCE_FORMAT",   32);
define("CMS_SEARCH_SCAN_NO_MODIFICATION",               64);
define("CMS_SEARCH_SCAN_UPDATE_OMITTED",               128);
define("CMS_SEARCH_SCAN_DATA_FETCH_FAILED",            256);
define("CMS_SEARCH_SCAN_NO_CONTENT",                   512);
define("CMS_SEARCH_SCAN_INDEXING_UNDESIRED",          1024);
define("CMS_SEARCH_SCAN_INDEXED",                     2048);
define("CMS_SEARCH_SCAN_INDEXING_FAILED",             4096);
define("CMS_SEARCH_SCAN_FATAL_ERROR",                 CMS_SEARCH_SCAN_REDIRECTION_LIMIT_EXCEEDED  |
                                                      CMS_SEARCH_SCAN_INVALID_ADDRESS             |
                                                      CMS_SEARCH_SCAN_ADDRESS_REJECTED            |
                                                      CMS_SEARCH_SCAN_UNSUPPORTED_RESOURCE_FORMAT |
                                                      CMS_SEARCH_SCAN_NO_CONTENT);
define("CMS_SEARCH_SCAN_ERROR",                       CMS_SEARCH_SCAN_FATAL_ERROR                 |
                                                      CMS_SEARCH_SCAN_UNKNOWN_ERROR               |
                                                      CMS_SEARCH_SCAN_NO_CONNECTION               |
                                                      CMS_SEARCH_SCAN_DATA_FETCH_FAILED           |
                                                      CMS_SEARCH_SCAN_INDEXING_FAILED);

//queue type
define("CMS_SEARCH_QUEUE_TYPE_NONE",                     0);
define("CMS_SEARCH_QUEUE_TYPE_INTERNAL",                 1);
define("CMS_SEARCH_QUEUE_TYPE_SELECTION",                2);
define("CMS_SEARCH_QUEUE_TYPE_SUBMISSION",               4);
define("CMS_SEARCH_QUEUE_TYPE_REFERENCE",                8);
define("CMS_SEARCH_QUEUE_TYPE_UPDATE",                  16);
define("CMS_SEARCH_QUEUE_TYPE_ALL",                    255);

class search
{
//properties
//-----------------------------------------------------------------------------------------------------
        public $mysql                            = NULL;
        public $enabled                          = FALSE;

        //list of regular expressions used for address exclusion
        public $address_accepted_blacklist       = array();

        //list of regular expressions used for address inclusion
        //applied after blacklist check
        public $address_accepted_whitelist       = array();

        //maximum number of permitted subsequent address redirects
        public $scan_routing_limit               = 5;

        //time in seconds since last full scan after which another full scan is applied
        //if the server delivers a modification date that lies before the last full scan
        public $scan_force_update_interval       = 604800; //1 week

        //time in seconds since last full scan during which another full scan is omitted
        public $scan_omit_update_interval        = 3600;   //1 hour

        //maximum number of simhash bit differences for assuming similarity
        public $entry_set_maximum_bit_difference = 2;

        //maximum weight of indexed information types
        public $entry_set_weight_factor_title    = 0.1;
        public $entry_set_weight_factor_text     = 1.0;
        public $entry_set_weight_factor_address  = 0.1;
        public $entry_set_weight_factor_link     = 0.1;

        //number of subsequent remove calls before entry is removed permanently
        public $entry_remove_error_limit         = 3;

        //number of retries on failure
        public $queue_process_error_limit        = 3;

        //number of results to display per page
        public $find_results_per_page            = 10;

        //number of score refinement iterations
        public $score_compute_iteration_number   = 100;

        //inherited portion of score
        public $score_compute_dampening_factor   = 0.85;

//constructor
//-----------------------------------------------------------------------------------------------------
        function __construct()
        {
                $this->mysql = new mysql();

                //verify database
                if (
                     $this->mysql->verify_table(CMS_DB_SEARCH_ENTRY,
                                                array(CMS_DB_SEARCH_ENTRY_INDEX           => "INT UNSIGNED PRIMARY KEY AUTO_INCREMENT NOT NULL",
                                                      CMS_DB_SEARCH_ENTRY_ADDRESS         => "VARCHAR(255) UNIQUE NOT NULL DEFAULT ''",
                                                      CMS_DB_SEARCH_ENTRY_ADDRESS_HASH    => "VARBINARY(32) UNIQUE NOT NULL DEFAULT ''",
                                                      CMS_DB_SEARCH_ENTRY_TITLE           => "VARCHAR(255) NOT NULL DEFAULT ''",
                                                      CMS_DB_SEARCH_ENTRY_TEXT            => "TEXT NOT NULL DEFAULT ''",
                                                      CMS_DB_SEARCH_ENTRY_TEXT_HASH       => "BIT(64) NOT NULL DEFAULT b'" .
                                                                                             "00000000000000000000000000000000" .
                                                                                             "00000000000000000000000000000000'",
                                                      CMS_DB_SEARCH_ENTRY_TIME            => "INT UNSIGNED NOT NULL DEFAULT '0'",
                                                      CMS_DB_SEARCH_ENTRY_UPDATE_INTERVAL => "INT UNSIGNED NOT NULL DEFAULT '0'",
                                                      CMS_DB_SEARCH_ENTRY_SCORE           => "FLOAT UNSIGNED NOT NULL DEFAULT '1.0'",
                                                      CMS_DB_SEARCH_ENTRY_LINK_COUNT      => "INT UNSIGNED NOT NULL DEFAULT '0'",
                                                      CMS_DB_SEARCH_ENTRY_ERROR           => "TINYINT UNSIGNED NOT NULL DEFAULT '0'"))
                     &&
                     $this->mysql->verify_table(CMS_DB_SEARCH_WORD,
                                                array(CMS_DB_SEARCH_WORD_INDEX            => "INT UNSIGNED PRIMARY KEY AUTO_INCREMENT NOT NULL",
                                                      CMS_DB_SEARCH_WORD_TEXT             => "VARBINARY(255) NOT NULL DEFAULT ''",
                                                      CMS_DB_SEARCH_WORD_LANGUAGE         => "VARCHAR(40) CHARACTER SET ascii COLLATE ascii_general_ci NOT NULL DEFAULT '', " .
                                                                                             "ADD UNIQUE (" .
                                                                                             CMS_DB_SEARCH_WORD_TEXT . ", " .
                                                                                             CMS_DB_SEARCH_WORD_LANGUAGE . ")"))
                     &&
                     $this->mysql->verify_table(CMS_DB_SEARCH_WEIGHT,
                                                array(CMS_DB_SEARCH_WEIGHT_WORD           => "INT UNSIGNED NOT NULL DEFAULT '0'",
                                                      CMS_DB_SEARCH_WEIGHT_SOURCE         => "INT UNSIGNED NOT NULL DEFAULT '0'",
                                                      CMS_DB_SEARCH_WEIGHT_TARGET         => "INT UNSIGNED NOT NULL DEFAULT '0'",
                                                      CMS_DB_SEARCH_WEIGHT_VALUE          => "FLOAT UNSIGNED NOT NULL DEFAULT '0.0', " .
                                                                                             "ADD UNIQUE (" .
                                                                                             CMS_DB_SEARCH_WEIGHT_WORD . ", " .
                                                                                             CMS_DB_SEARCH_WEIGHT_SOURCE . ", " .
                                                                                             CMS_DB_SEARCH_WEIGHT_TARGET . ")"))
                     &&
                     $this->mysql->verify_table(CMS_DB_SEARCH_LINK,
                                                array(CMS_DB_SEARCH_LINK_SOURCE           => "INT UNSIGNED NOT NULL DEFAULT '0'",
                                                      CMS_DB_SEARCH_LINK_TARGET_HASH      => "VARBINARY(32) NOT NULL DEFAULT ''",
                                                      CMS_DB_SEARCH_LINK_TARGET           => "INT UNSIGNED NOT NULL DEFAULT '0'",
                                                      CMS_DB_SEARCH_LINK_TEXT             => "VARBINARY(255) NOT NULL DEFAULT ''",
                                                      CMS_DB_SEARCH_LINK_LEVEL            => "TINYINT UNSIGNED NOT NULL DEFAULT '0'"))
                     &&
                     $this->mysql->verify_table(CMS_DB_SEARCH_CLUSTER,
                                                array(CMS_DB_SEARCH_CLUSTER_SOURCE        => "INT UNSIGNED NOT NULL DEFAULT '0'",
                                                      CMS_DB_SEARCH_CLUSTER_TARGET        => "INT UNSIGNED NOT NULL DEFAULT '0', " .
                                                                                             "ADD UNIQUE (" .
                                                                                             CMS_DB_SEARCH_CLUSTER_SOURCE . ", " .
                                                                                             CMS_DB_SEARCH_CLUSTER_TARGET . ")"))
                     &&
                     $this->mysql->verify_table(CMS_DB_SEARCH_QUEUE,
                                                array(CMS_DB_SEARCH_QUEUE_ADDRESS         => "VARCHAR(255) UNIQUE NOT NULL DEFAULT ''",
                                                      CMS_DB_SEARCH_QUEUE_TYPE            => "TINYINT UNSIGNED NOT NULL DEFAULT '0'",
                                                      CMS_DB_SEARCH_QUEUE_TIME            => "INT UNSIGNED NOT NULL DEFAULT '0'",
                                                      CMS_DB_SEARCH_QUEUE_CODE            => "VARBINARY(8) NOT NULL DEFAULT ''",
                                                      CMS_DB_SEARCH_QUEUE_ERROR           => "TINYINT UNSIGNED NOT NULL DEFAULT '0'"))
                   )
                {
                        //load settings
                        $system = new system();

                        $this->address_accepted_blacklist = preg_split("/\r?\n/", $system->getval("search", "blacklist"),
                                                                       -1, PREG_SPLIT_NO_EMPTY);
                        $this->address_accepted_whitelist = preg_split("/\r?\n/", $system->getval("search", "whitelist"),
                                                                       -1, PREG_SPLIT_NO_EMPTY);

                        $this->enabled = TRUE;
                };
        }
//methods
//-----------------------------------------------------------------------------------------------------
        function scan($address,
                      $follow_links = TRUE,
                      $force_update = FALSE)
        {
                if (! $this->enabled)   return CMS_SEARCH_SCAN_UNKNOWN_ERROR;
                if (! cms_load("html")) return CMS_SEARCH_SCAN_UNKNOWN_ERROR;

                //standardize address
                if (($address = $this->address_standardize($address)) === FALSE)
                        return CMS_SEARCH_SCAN_INVALID_ADDRESS;

                //check if entry is updated
                if (
                     ($result = mysql_query("SELECT " . CMS_DB_SEARCH_ENTRY_INDEX . ", " .
                                                        CMS_DB_SEARCH_ENTRY_TIME . " " .
                                            "FROM " .   CMS_DB_SEARCH_ENTRY . " " .
                                            "WHERE " .  CMS_DB_SEARCH_ENTRY_ADDRESS_HASH .
                                            "='" .      sqlesc(md5($address)) . "' " .
                                            "LIMIT 1"))
                     &&
                     ($resultrow = mysql_fetch_row($result))
                   )
                {
                        //scan
                        $value = $this->_scan($address,                           //address
                                              $follow_links,                      //follow links
                                              $force_update ? 0 : $resultrow[1]); //entry time

                        //remove on error
                        if (CMS_SEARCH_SCAN_ERROR & $value)
                                $this->entry_remove($resultrow[0],
                                                    (bool)(CMS_SEARCH_SCAN_FATAL_ERROR & $value));

                        return $value;
                };

                //scan
                return $this->_scan($address, $follow_links);
        }
//-----------------------------------------------------------------------------------------------------
        private function _scan($address,
                               $follow_links,
                               $entry_time = 0)
        {
                //connect and retrieve header
                $_address = $address;
                $i        = 0;

                while (TRUE)
                {
                        //check address admission
                        if ($this->address_accepted($_address) === FALSE)
                                return CMS_SEARCH_SCAN_ADDRESS_REJECTED;

                        //retrieve http header
                        if ((! $header = html_header($_address)) || is_string($header))
                                return CMS_SEARCH_SCAN_NO_CONNECTION;

                        //rerouting
                        if (isset($header["location"]))
                        {
                                //exit multiple routings
                                if (++$i > $this->scan_routing_limit)
                                        return CMS_SEARCH_SCAN_REDIRECTION_LIMIT_EXCEEDED;

                                //standardize address
                                if (($_address = $this->address_standardize($header["location"])) === FALSE)
                                        return CMS_SEARCH_SCAN_INVALID_ADDRESS;

                                continue;
                        };

                        break;
                };

                //verify resource type
                if (isset($header["content-type"]) && cms_load("mime"))
                {
                        $content_type = mime_extract_rfc2822_header($header["content-type"]);
                        $content_type = strtolower($content_type["#value"]);

                        switch ($content_type)
                        {
                        case "text/html":
                        case "application/xhtml+xml":
                        case "application/xml":
                                break;

                        default:
                                return CMS_SEARCH_SCAN_UNSUPPORTED_RESOURCE_FORMAT;
                        };
                };

                //verify modification date
                $current_time = time();

                if (
                     (! isset($header["last-modified"]))
                     ||
                     (! $header["last-modified"] = strtotime($header["last-modified"]))
                   )
                        $header["last-modified"] = $current_time;

                //check if modified or update force interval exceeded
                if (
                     ($entry_time >= $header["last-modified"])
                     &&
                     ($current_time <= $entry_time + $this->scan_force_update_interval)
                   )
                        return CMS_SEARCH_SCAN_NO_MODIFICATION;

                //check if omission interval exceeded
                if ($current_time <= $entry_time + $this->scan_omit_update_interval)
                        return CMS_SEARCH_SCAN_UPDATE_OMITTED;

                //read content
                if (! $data = html_page_info($_address))
                        return CMS_SEARCH_SCAN_DATA_FETCH_FAILED;

                //verify content
                if (stre($data["body"]))
                        return CMS_SEARCH_SCAN_NO_CONTENT;

                //add links to queue
                if (
                     $follow_links
                     &&
                     (! (isset($data["robots"]) && preg_match("/\bnofollow\b/i", $data["robots"])))
                   )
                {
                        reset($data["links"]);
                        while (list(, $value) = each($data["links"]))
                                $this->queue_add($value["url"], CMS_SEARCH_QUEUE_TYPE_REFERENCE);
                };

                //check robots directive
                if (isset($data["robots"]) && preg_match("/\bnoindex\b/i", $data["robots"]))
                        return CMS_SEARCH_SCAN_INDEXING_UNDESIRED;

                //title
                if (! isset($data["title"])) $data["title"] = $address;

                //language
                if (isset($data["content-language"]))       $language = $data["content-language"];
                elseif (isset($header["content-language"])) $language = $header["content-language"];
                else                                        $language = language_detect($data["title"] . " " . $data["body"]);

                $data_language = new data("#system/language");
                $language_list = implode(",", array_keys($data_language->data));
                $language      = cms_language_extract($language, $language_list);

                //index
                return $this->entry_set($address,
                                        $data["title"],
                                        $data["body"],
                                        $language,
                                        $data["links"]) ?
                       CMS_SEARCH_SCAN_INDEXED :
                       CMS_SEARCH_SCAN_INDEXING_FAILED;
        }
//-----------------------------------------------------------------------------------------------------
        function entry_set($address,
                           $title,
                           $text,
                           $language,
                           $links = NULL)
        {
                if (! $this->enabled) return FALSE;

                //standardize and check address admission
                if (($address = $this->address_standardize($address)) === FALSE) return FALSE; //invalid address
                if ($this->address_accepted($address) === FALSE)                 return FALSE; //address rejected

                //prepare values
                $address_hash = md5($address);
                $title        = preg_match("/^\s*(.{0,254}\S)(?:$|\s)/", $title, $match) ?
                                $match[1] : "";
                $text         = preg_match("/^\s*(.{0,65534}\S)(?:$|\s)/", $text, $match) ?
                                $match[1] : "";
                $text_hash    = simhash($text);
                $time         = time();
                if (! is_array($links)) $links = array();
                $link_count   = 0;

                //preprocess outbound links
                $_links = array();

                reset($links);
                while (list(, $value) = each($links))
                {
                        //standardize link address
                        $link_url = $this->address_standardize($value["url"]);
                        if ($link_url === FALSE) continue; //invalid address

                        //hash link address
                        $link_url = md5($link_url);
                        if (streq($link_url, $address_hash)) continue; //link to self

                        //cleanse link text
                        $link_text = htmltoplain($value["text"]);
                        $link_text = preg_replace("/[\p{C}\p{Ps}\p{Pe}\p{Pi}\p{Pf}\p{Po}\p{Z}]+/u",
                                                  " ", $link_text);
                        $link_text = preg_match("/^\s*(.{0,254}\S)(?:$|\s)/", $link_text, $match) ?
                                     $match[1] : "";

                        //keep link
                        $_links[] = array(0 => $link_url,
                                          1 => $link_text);
                        $link_count++;
                };

                //check if entry is already indexed
                $query = "SELECT " . CMS_DB_SEARCH_ENTRY_INDEX . ", " .
                                     CMS_DB_SEARCH_ENTRY_TITLE . ", " .
                                     CMS_DB_SEARCH_ENTRY_TEXT . " " .
                         "FROM " .   CMS_DB_SEARCH_ENTRY . " " .
                         "WHERE " .  CMS_DB_SEARCH_ENTRY_ADDRESS .
                         "='" .      sqlesc($address) . "' " .
                         "LIMIT 1";

                //execute
                $result = mysql_query($query);
                if (mysql_errno() !== 0) return mysql_error(); //error check

                //process existing entry
                if ($resultrow = mysql_fetch_row($result))
                {
                        //check if entry has changed
                        if (
                             streq($title, $resultrow[1])
                             &&
                             streq($text, $resultrow[2])
                           )
                                return TRUE; //no change, skip update

                        $entry_index  = $resultrow[0];
                        $_entry_index = sqlesc($entry_index);

                        //remove previous word weights, inbound and outbound
                        mysql_query("DELETE FROM " . CMS_DB_SEARCH_WEIGHT . " " .
                                    "WHERE " .       CMS_DB_SEARCH_WEIGHT_SOURCE .
                                    "='" .           $_entry_index . "' " .
                                    "OR " .          CMS_DB_SEARCH_WEIGHT_TARGET .
                                    "='" .           $_entry_index . "'");
                        if (mysql_errno() !== 0) return mysql_error(); //error check

                        //remove previous outbound links
                        mysql_query("DELETE FROM " . CMS_DB_SEARCH_LINK . " " .
                                    "WHERE " .       CMS_DB_SEARCH_LINK_SOURCE .
                                    "='" .           $_entry_index . "'");
                        if (mysql_errno() !== 0) return mysql_error(); //error check

                        //remove previous cluster associations
                        mysql_query("DELETE FROM " . CMS_DB_SEARCH_CLUSTER . " " .
                                    "WHERE " .       CMS_DB_SEARCH_CLUSTER_SOURCE .
                                    "='" .           $_entry_index . "' " .
                                    "OR " .          CMS_DB_SEARCH_CLUSTER_TARGET .
                                    "='" .           $_entry_index . "'");
                        if (mysql_errno() !== 0) return mysql_error(); //error check

                        //update entry
                        $query = "UPDATE " . CMS_DB_SEARCH_ENTRY . " " .
                                 "SET " .    CMS_DB_SEARCH_ENTRY_TITLE .
                                 "='" .      sqlesc($title) . "', " .
                                             CMS_DB_SEARCH_ENTRY_TEXT .
                                 "='" .      sqlesc($text) . "', " .
                                             CMS_DB_SEARCH_ENTRY_TEXT_HASH .
                                 "=b'" .     sqlesc($text_hash) . "', " .
                                             CMS_DB_SEARCH_ENTRY_TIME .
                                 "='" .      sqlesc($time) . "', " .
                                             CMS_DB_SEARCH_ENTRY_LINK_COUNT .
                                 "='" .      sqlesc($link_count) . "', " .
                                             CMS_DB_SEARCH_ENTRY_ERROR .
                                 "='0' " .
                                 "WHERE " .  CMS_DB_SEARCH_ENTRY_INDEX .
                                 "='" .      $_entry_index . "' " .
                                 "LIMIT 1";

                        //execute
                        mysql_query($query);
                        if (mysql_errno() !== 0) return mysql_error(); //error check
                }

                //new entry
                else
                {
                        //check for identical and nearly identical content
                        $query = "SELECT " . CMS_DB_SEARCH_ENTRY_INDEX . " " .
                                 "FROM " .   CMS_DB_SEARCH_ENTRY . " " .
                                 "WHERE " .  CMS_DB_SEARCH_ENTRY_TEXT_HASH .
                                 "='" .      sqlesc($text_hash) . "' " .
                                 "LIMIT 1";

                        //execute
                        $result = mysql_query($query);
                        if (mysql_errno() !== 0)         return mysql_error(); //error check
                        if (mysql_num_rows($result) > 0) return FALSE;         //duplicate found

                        //add entry
                        $query = "INSERT " .  CMS_DB_SEARCH_ENTRY . " " .
                                        "(" . CMS_DB_SEARCH_ENTRY_ADDRESS . ", " .
                                              CMS_DB_SEARCH_ENTRY_ADDRESS_HASH . ", " .
                                              CMS_DB_SEARCH_ENTRY_TITLE . ", " .
                                              CMS_DB_SEARCH_ENTRY_TEXT . ", " .
                                              CMS_DB_SEARCH_ENTRY_TEXT_HASH . ", " .
                                              CMS_DB_SEARCH_ENTRY_TIME . ", " .
                                              CMS_DB_SEARCH_ENTRY_LINK_COUNT . ") " .
                                 "VALUES (" .
                                       "'" .  sqlesc($address) . "', " .
                                       "'" .  sqlesc($address_hash) . "', " .
                                       "'" .  sqlesc($title) . "', " .
                                       "'" .  sqlesc($text) . "', " .
                                       "b'" . sqlesc($text_hash) . "', " .
                                       "'" .  sqlesc($time) . "', " .
                                       "'" .  sqlesc($link_count) . "')";

                        //execute
                        mysql_query($query);
                        if (mysql_errno() !== 0) return mysql_error(); //error check

                        $entry_index  = mysql_insert_id();
                        $_entry_index = sqlesc($entry_index);
                };

                //add title to search index
                $result  = $this->_entry_set($entry_index,                            //source index
                                             $entry_index,                            //target index
                                             $title,                                  //text
                                             $language,                               //language
                                             $this->entry_set_weight_factor_title);   //maximum weight
                if ($result !== TRUE) return $result;

                //add text to search index
                $result  = $this->_entry_set($entry_index,                            //source index
                                             $entry_index,                            //target index
                                             $text,                                   //text
                                             $language,                               //language
                                             $this->entry_set_weight_factor_text);    //maximum weight
                if ($result !== TRUE) return $result;

                //add address to search index
                $address = analyze_url($address);
                $host    = preg_replace(array("/^www\./i", "/\.[^\.]+$/", "/[\p{C}\p{P}\p{Z}]+/u"),
                                        array("",          "",            " "),
                                        $address["host"]);
                $path    = preg_replace(array("/\.[^\.]+$/", "/[\p{C}\p{P}\p{Z}]+/u"),
                                        array("",            " "),
                                        $address["path"]);
                $result  = $this->_entry_set($entry_index,                            //source index
                                             $entry_index,                            //target index
                                             "$host $path",                           //text
                                             $language,                               //language
                                             $this->entry_set_weight_factor_address); //maximum weight
                if ($result !== TRUE) return $result;

                if ($link_count > 0)
                {
                        //process outbound links
                        $string        = "";
                        $weight_factor = $this->entry_set_weight_factor_link / $link_count;

                        reset($_links);
                        while (list(, $value) = each($_links))
                        {
                                $link_url = sqlesc($value[0]);

                                //append link data to sql insert string
                                $string .= "('$_entry_index', '$link_url', '" . sqlesc($value[1]) . "'), ";

                                //retrieve index of link target entry
                                $query = "SELECT " . CMS_DB_SEARCH_ENTRY_INDEX . " " .
                                         "FROM " .   CMS_DB_SEARCH_ENTRY . " " .
                                         "WHERE " .  CMS_DB_SEARCH_ENTRY_ADDRESS_HASH .
                                         "='" .      $link_url . "' " .
                                         "LIMIT 1";

                                //execute
                                $result = mysql_query($query);
                                if (mysql_errno() !== 0) return mysql_error(); //error check

                                $target_index = mysql_result($result, 0);

                                //add link text to search index
                                $result = $this->_entry_set($entry_index,    //source index
                                                            $target_index,   //target index
                                                            $value[1],       //text
                                                            $language,       //language
                                                            $weight_factor); //maximum weight
                                if ($result !== TRUE) return $result;
                        };

                        //store link data
                        $query = "INSERT IGNORE INTO " . CMS_DB_SEARCH_LINK . " " .
                                                   "(" . CMS_DB_SEARCH_LINK_SOURCE . ", " .
                                                         CMS_DB_SEARCH_LINK_TARGET_HASH . ", " .
                                                         CMS_DB_SEARCH_LINK_TEXT . ") " .
                                 "VALUES " .             substr($string, 0, -2);

                        //execute
                        mysql_query($query);
                        if (mysql_errno() !== 0) return mysql_error(); //error check
                };

                //process inbound links
                $query = "SELECT " . CMS_DB_SEARCH_LINK_SOURCE . ", " .
                                     CMS_DB_SEARCH_LINK_TEXT . " " .
                         "FROM " .   CMS_DB_SEARCH_LINK . " " .
                         "WHERE " .  CMS_DB_SEARCH_LINK_TARGET_HASH .
                         "='" .      sqlesc($address_hash) . "'";

                //execute
                $result = mysql_query($query);
                if (mysql_errno() !== 0) return mysql_error(); //error check

                if (($link_count = mysql_num_rows($result)) > 0)
                {
                        $weight_factor = $this->entry_set_weight_factor_link / $link_count;

                        while ($resultrow = mysql_fetch_row($result))
                        {
                                //add link text to search index
                                $_result = $this->_entry_set($resultrow[0],   //source index
                                                             $entry_index,    //target index
                                                             $resultrow[1],   //text
                                                             $language,       //language
                                                             $weight_factor); //maximum weight
                                if ($_result !== TRUE) return $_result;
                        };
                };

                //find similar entries
                $query = "SELECT " .          CMS_DB_SEARCH_ENTRY_INDEX . " " .
                         "FROM " .            CMS_DB_SEARCH_ENTRY . " " .
                         "WHERE BIT_COUNT(" . CMS_DB_SEARCH_ENTRY_TEXT_HASH .
                         "^b'" .              sqlesc($text_hash) . "')" .
                         "<='" .              sqlesc($this->entry_set_maximum_bit_difference) . "' " .
                         "AND " .             CMS_DB_SEARCH_ENTRY_INDEX .
                         "!='" .              $_entry_index . "'";

                //execute
                $result = mysql_query($query);
                if (mysql_errno() !== 0) return mysql_error(); //error check

                //cluster similar entries
                if (mysql_num_rows($result) > 0)
                {
                        $list = "";
                        while ($resultrow = mysql_fetch_row($result))
                        {
                                $value  = sqlesc($resultrow[0]);
                                $list  .= "('$entry_index', '$value'), " .
                                          "('$value', '$entry_index'), ";
                        };

                        //connect similar entries
                        $query = "INSERT IGNORE INTO " . CMS_DB_SEARCH_CLUSTER . " " .
                                                   "(" . CMS_DB_SEARCH_CLUSTER_SOURCE . ", " .
                                                         CMS_DB_SEARCH_CLUSTER_TARGET . ") " .
                                 "VALUES " .             substr($list, 0, -2);

                        //execute
                        mysql_query($query);
                        if (mysql_errno() !== 0) return mysql_error(); //error check
                };

                return TRUE;
        }
//-----------------------------------------------------------------------------------------------------
        private function _entry_set($source_index,
                                    $target_index,
                                    $text,
                                    $language,
                                    $weight_factor = 1.0)
        {
                //strip stop words
                $text = language_strip_stopword($text, $language);

                //generate word list
                $list = preg_split("/[\p{C}\p{Ps}\p{Pe}\p{Pi}\p{Pf}\p{Po}\p{Z}]+/u",
                                   $text, -1, PREG_SPLIT_NO_EMPTY);
                if (count($list) === 0) return TRUE; //no content

                //count word occurences
                $_list  = array();
                $__list = array();
                $total  = 0;

                while (list(, $value) = each($list))
                {
                        $value = utf8_strtolower($value);

                        //already listed
                        if (isset($_list[$value]))
                        {
                                $_list[$value] += 1;
                        }

                        //new word
                        else
                        {
                                $_list[$value]  = 1;
                                $__list[]       = sqlesc($value);
                        };

                        $total++;
                };

                //insert words into word table
                $query = "INSERT IGNORE INTO " . CMS_DB_SEARCH_WORD . " " .
                                           "(" . CMS_DB_SEARCH_WORD_TEXT . ", " .
                                                 CMS_DB_SEARCH_WORD_LANGUAGE . ") " .
                         "VALUES ('" .           implode("', '" . sqlesc($language) . "'), ('", $__list) . "', " .
                                           "'" . sqlesc($language) . "')";

                //execute
                mysql_query($query);
                if (mysql_errno() !== 0) return mysql_error(); //error check

                //retrieve word indices
                $query = "SELECT " . CMS_DB_SEARCH_WORD_INDEX . ", " .
                                     CMS_DB_SEARCH_WORD_TEXT . " " .
                         "FROM " .   CMS_DB_SEARCH_WORD . " " .
                         "WHERE " .  CMS_DB_SEARCH_WORD_LANGUAGE .
                         "='" .      sqlesc($language) . "' " .
                         "AND " .    CMS_DB_SEARCH_WORD_TEXT . " " .
                         "IN ('" .   implode("', '", $__list) . "')";

                //execute
                $result = mysql_query($query);
                if (mysql_errno() !== 0) return mysql_error(); //error check

                //compute word ratings
                $source_index = sqlesc($source_index);
                $target_index = sqlesc($target_index);
                $factor       = 1.0 / $total * $weight_factor;
                $__list       = array();

                while ($resultrow = mysql_fetch_row($result))
                {
                        $word_index = sqlesc($resultrow[0]);
                        $text       = $resultrow[1];
                        $value      = sqlesc($factor * $_list[$text]);

                        $__list[]   = "'$word_index', '$source_index', '$target_index', '$value'";
                };

                //insert word ratings into weight table
                $query = "INSERT " .                  CMS_DB_SEARCH_WEIGHT . " " .
                                                "(" . CMS_DB_SEARCH_WEIGHT_WORD . ", " .
                                                      CMS_DB_SEARCH_WEIGHT_SOURCE . ", " .
                                                      CMS_DB_SEARCH_WEIGHT_TARGET . ", " .
                                                      CMS_DB_SEARCH_WEIGHT_VALUE . ") " .
                         "VALUES (" .                 implode("), (", $__list) . ") " .
                         "ON DUPLICATE KEY UPDATE " . CMS_DB_SEARCH_WEIGHT_VALUE .
                         "=" .                        CMS_DB_SEARCH_WEIGHT_VALUE .
                         "+VALUES(" .                 CMS_DB_SEARCH_WEIGHT_VALUE . ")";

                //execute
                mysql_query($query);
                if (mysql_errno() !== 0) return mysql_error(); //error check

                return TRUE;
        }
//-----------------------------------------------------------------------------------------------------
        function entry_remove($index,
                              $force = TRUE)
        {
                if (! $this->enabled) return FALSE;

                $index = sqlesc($index);

                if (
                     $force
                     ||
                     (
                       ($result = mysql_query("SELECT " . CMS_DB_SEARCH_ENTRY_ERROR . " " .
                                              "FROM " .   CMS_DB_SEARCH_ENTRY . " " .
                                              "WHERE " .  CMS_DB_SEARCH_ENTRY_INDEX . "='$index' " .
                                              "LIMIT 1"))
                       &&
                       (mysql_result($result, 0) >= $this->entry_remove_error_limit)
                     )
                   )
                        //remove permanently
                        mysql_query("DELETE FROM " . CMS_DB_SEARCH_ENTRY . " " .
                                    "WHERE " .       CMS_DB_SEARCH_ENTRY_INDEX . "='$index' " .
                                    "LIMIT 1");
                else
                        //increase error count
                        mysql_query("UPDATE " . CMS_DB_SEARCH_ENTRY . " " .
                                    "SET " .    CMS_DB_SEARCH_ENTRY_ERROR .
                                    "=" .       CMS_DB_SEARCH_ENTRY_ERROR . "+1 " .
                                    "WHERE " .  CMS_DB_SEARCH_ENTRY_INDEX . "='$index' " .
                                    "LIMIT 1");

                //remove word weights, inbound and outbound
                mysql_query("DELETE FROM " . CMS_DB_SEARCH_WEIGHT . " " .
                            "WHERE " .       CMS_DB_SEARCH_WEIGHT_SOURCE .  "='$index' " .
                            "OR " .          CMS_DB_SEARCH_WEIGHT_TARGET .  "='$index'");

                //remove outbound links
                mysql_query("DELETE FROM " . CMS_DB_SEARCH_LINK . " " .
                            "WHERE " .       CMS_DB_SEARCH_LINK_SOURCE .    "='$index'");

                //remove cluster associations
                mysql_query("DELETE FROM " . CMS_DB_SEARCH_CLUSTER . " " .
                            "WHERE " .       CMS_DB_SEARCH_CLUSTER_SOURCE . "='$index' " .
                            "OR " .          CMS_DB_SEARCH_CLUSTER_TARGET . "='$index'");

                return TRUE;
        }
//-----------------------------------------------------------------------------------------------------
        function queue_add($address,
                           $type = CMS_SEARCH_QUEUE_TYPE_NONE)
        {
                if (! $this->enabled) return FALSE;

                //standardize and check address
                if (($address = $this->address_standardize($address)) === FALSE) return FALSE;
                if ($this->address_accepted($address) === FALSE)                 return FALSE;

                //submitted and discovered addresses are omitted if an entry already exists
                if (
                     ((CMS_SEARCH_QUEUE_TYPE_SUBMISSION | CMS_SEARCH_QUEUE_TYPE_REFERENCE) & $type)
                     &&
                     ($result = mysql_query("SELECT " . CMS_DB_SEARCH_ENTRY_INDEX . " " .
                                            "FROM " .   CMS_DB_SEARCH_ENTRY . " " .
                                            "WHERE " .  CMS_DB_SEARCH_ENTRY_ADDRESS .
                                            "='" .      sqlesc($address) . "' " .
                                            "LIMIT 1"))
                     &&
                     (mysql_num_rows($result) > 0)
                   )
                        return FALSE;

                //append task to queue
                mysql_query("INSERT IGNORE INTO " . CMS_DB_SEARCH_QUEUE . " " .
                                              "(" . CMS_DB_SEARCH_QUEUE_ADDRESS . ", " .
                                                    CMS_DB_SEARCH_QUEUE_TYPE . ", " .
                                                    CMS_DB_SEARCH_QUEUE_TIME . ") " .
                            "VALUES (" .
                                              "'" . sqlesc($address) . "', " .
                                              "'" . sqlesc($type) . "', " .
                                              "'" . sqlesc(time()) . "')");

                return TRUE;
        }
//-----------------------------------------------------------------------------------------------------
        function queue_add_all()
        {
                if (! $this->enabled) return FALSE;

                return (bool)mysql_query("INSERT IGNORE INTO " . CMS_DB_SEARCH_QUEUE . " " .
                                                           "(" . CMS_DB_SEARCH_QUEUE_ADDRESS . ", " .
                                                                 CMS_DB_SEARCH_QUEUE_TYPE . ", " .
                                                                 CMS_DB_SEARCH_QUEUE_TIME . ") " .
                                         "SELECT " .             CMS_DB_SEARCH_ENTRY_ADDRESS . ", " .
                                                           "'" . sqlesc(CMS_SEARCH_QUEUE_TYPE_UPDATE) . "', " .
                                                                 CMS_DB_SEARCH_ENTRY_TIME . " " .
                                         "FROM " .               CMS_DB_SEARCH_ENTRY);
        }
//-----------------------------------------------------------------------------------------------------
        function queue_remove($type = CMS_SEARCH_QUEUE_TYPE_ALL)
        {
                if (! $this->enabled) return FALSE;

                if ($type == CMS_SEARCH_QUEUE_TYPE_ALL)
                        return (bool)mysql_query("DELETE FROM " . CMS_DB_SEARCH_QUEUE);

                else
                        return (bool)mysql_query("DELETE FROM " . CMS_DB_SEARCH_QUEUE . " " .
                                                 "WHERE " .       CMS_DB_SEARCH_QUEUE_TYPE .
                                                 "&" .            (int)$type);
        }
//-----------------------------------------------------------------------------------------------------
        function queue_length($type = CMS_SEARCH_QUEUE_TYPE_ALL)
        {
                if (! $this->enabled) return FALSE;

                if ($type == CMS_SEARCH_QUEUE_TYPE_ALL)
                        $result = mysql_query("SELECT COUNT(*) " .
                                              "FROM " .  CMS_DB_SEARCH_QUEUE);

                else
                        $result = mysql_query("SELECT COUNT(*) " .
                                              "FROM " .  CMS_DB_SEARCH_QUEUE . " " .
                                              "WHERE " . CMS_DB_SEARCH_QUEUE_TYPE .
                                              "&" .      (int)$type);

                if ($resultrow = mysql_fetch_row($result)) return $resultrow[0];
                return FALSE;
        }
//-----------------------------------------------------------------------------------------------------
        function queue_process($type         = CMS_SEARCH_QUEUE_TYPE_ALL,
                               $follow_links = TRUE,
                               $force_update = FALSE)
        {
                if (! $this->enabled) return FALSE;

                $code = sqlesc(unique_id());
                $time = time();

                if (

                //mark entry to process
                     mysql_query("UPDATE " .   CMS_DB_SEARCH_QUEUE . " " .
                                 "SET " .      CMS_DB_SEARCH_QUEUE_TIME .
                                 "='" .        sqlesc($time + 3600) . "', " .
                                               CMS_DB_SEARCH_QUEUE_CODE .
                                 "='" .        $code . "' " .

                                 (($type == CMS_SEARCH_QUEUE_TYPE_ALL) ?

                                 "WHERE " .    CMS_DB_SEARCH_QUEUE_CODE .
                                 "='' " .
                                 "OR " .       CMS_DB_SEARCH_QUEUE_TIME .
                                 "<'" .        sqlesc($time) . "' " :

                                 "WHERE (" .   CMS_DB_SEARCH_QUEUE_TYPE .
                                 "&" .         (int)$type . ") " .
                                 "AND (" .     CMS_DB_SEARCH_QUEUE_CODE .
                                 "='' " .
                                 "OR " .       CMS_DB_SEARCH_QUEUE_TIME .
                                 "<'" .        sqlesc($time) . "') ") .

                                 "ORDER BY " . CMS_DB_SEARCH_QUEUE_TIME . " " .
                                 "LIMIT 1")
                     &&

                //fetch data
                     ($result = mysql_query("SELECT " . CMS_DB_SEARCH_QUEUE_ADDRESS . ", " .
                                                        CMS_DB_SEARCH_QUEUE_ERROR . " " .
                                            "FROM " .   CMS_DB_SEARCH_QUEUE . " " .
                                            "WHERE " .  CMS_DB_SEARCH_QUEUE_CODE . "='$code' " .
                                            "LIMIT 1"))
                     &&
                     ($resultrow = mysql_fetch_row($result))
                   )
                {
                        //scan address
                        $value = $this->scan($resultrow[0], $follow_links, $force_update);

                        if (

                        //non fatal error
                             (CMS_SEARCH_SCAN_ERROR & ~CMS_SEARCH_SCAN_FATAL_ERROR & $value)
                             &&

                        //error count below limit
                             ($resultrow[1] < $this->queue_process_error_limit)
                             &&

                        //increase error count
                             mysql_query("UPDATE " . CMS_DB_SEARCH_QUEUE . " " .
                                         "SET " .    CMS_DB_SEARCH_QUEUE_ERROR .
                                         "=" .       CMS_DB_SEARCH_QUEUE_ERROR . "+1 " .
                                         "WHERE " .  CMS_DB_SEARCH_QUEUE_CODE .
                                         "='" .      $code . "' " .
                                         "LIMIT 1")
                           )
                                return $value;

                        //remove queue entry
                        mysql_query("DELETE FROM " . CMS_DB_SEARCH_QUEUE . " " .
                                    "WHERE " .       CMS_DB_SEARCH_QUEUE_CODE . "='$code' " .
                                    "LIMIT 1");
                        return $value;
                };

                return FALSE;
        }
//-----------------------------------------------------------------------------------------------------
        function find($term,
                      $page     = 0,
                      $language = NULL)
        {
                if (! $this->enabled) return FALSE;

                //start time
                $time = microtime(TRUE);

                //tokenize search term
                $list  = preg_split("/[\p{C}\p{Ps}\p{Pe}\p{Pi}\p{Pf}\p{Po}\p{Z}]+/u",
                                    $term, -1, PREG_SPLIT_NO_EMPTY);
                $_list = array();

                //preprocess search tokens
                reset($list);
                while (list(, $value) = each($list))
                {
                        $value         = sqlesc(utf8_strtolower($value));
                        $_list[$value] = TRUE;
                };

                //retrieve word indices matching search tokens
                $subquery = "SELECT " . CMS_DB_SEARCH_WORD_INDEX . " " .
                            "FROM " .   CMS_DB_SEARCH_WORD . " " .
                            (($language === NULL) ?
                            "WHERE " .  CMS_DB_SEARCH_WORD_TEXT . " " :
                            "WHERE " .  CMS_DB_SEARCH_WORD_LANGUAGE .
                            "='" .      sqlesc($language) . "' " .
                            "AND " .    CMS_DB_SEARCH_WORD_TEXT . " ") .
                            "IN ('" .   implode("', '", array_keys($_list)) . "')";

                //retrieve result entries
                $query    = "SELECT " .   "`#a`." . CMS_DB_SEARCH_WEIGHT_TARGET . " " .
                            "FROM " .               CMS_DB_SEARCH_WEIGHT . " AS `#a`, " .
                                                    CMS_DB_SEARCH_ENTRY .  " AS `#b` " .
                            "WHERE " .    "`#a`." . CMS_DB_SEARCH_WEIGHT_WORD . " " .
                            "IN (" .                $subquery . ") " .
                            "AND " .      "`#a`." . CMS_DB_SEARCH_WEIGHT_TARGET .
                            "=" .         "`#b`." . CMS_DB_SEARCH_ENTRY_INDEX . " " .
                            "GROUP BY " . "`#a`." . CMS_DB_SEARCH_WEIGHT_TARGET . " " .

                //order by number of search tokens matched
                            "ORDER BY " .          "COUNT(*) DESC, " .

                //order by total weight of matched search tokens
                            "SUM(" .      "`#a`." . CMS_DB_SEARCH_WEIGHT_VALUE . ")" .

                //add page score
                            "*" .         "`#b`." . CMS_DB_SEARCH_ENTRY_SCORE . " DESC";

                //execute
                if (
                     ($result = mysql_query($query))
                     &&
                     (mysql_num_rows($result) > 0)
                   )
                {
                        //retrieve result entries
                        $list         = array();
                        $supplemental = array();
                        $offset       = $page * $this->find_results_per_page;
                        $i            = 0;

                        while ($resultrow = mysql_fetch_row($result))
                        {
                                $index = $resultrow[0];
                                if (isset($supplemental[$index])) continue; //skip similar entry
                                $index = sqlesc($index);

                                //retrieve similar entries
                                $query = "SELECT " . CMS_DB_SEARCH_CLUSTER_TARGET . " " .
                                         "FROM " .   CMS_DB_SEARCH_CLUSTER . " " .
                                         "WHERE " .  CMS_DB_SEARCH_CLUSTER_SOURCE .
                                         "='" .      $index . "'";

                                //execute
                                if ($_result = mysql_query($query))
                                {
                                        //store similar entries
                                        while ($_resultrow = mysql_fetch_row($_result))
                                                $supplemental[$_resultrow[0]] = TRUE;
                                };

                                if (++$i > $offset)
                                {
                                        $list[$index] = mysql_num_rows($_result);
                                        if (count($list) === $this->find_results_per_page) break;
                                };
                        };

                        //extract requested result page
                        $string = "'" . implode("', '", array_keys($list)) . "'";

                        //retrieve entry data
                        if (
                             ($result = mysql_query("SELECT " .         CMS_DB_SEARCH_ENTRY_INDEX . ", " .
                                                                        CMS_DB_SEARCH_ENTRY_ADDRESS . ", " .
                                                                        CMS_DB_SEARCH_ENTRY_TITLE . ", " .
                                                                        CMS_DB_SEARCH_ENTRY_TEXT . ", " .
                                                                        CMS_DB_SEARCH_ENTRY_TIME . ", " .
                                                                        CMS_DB_SEARCH_ENTRY_SCORE . " " .
                                                    "FROM " .           CMS_DB_SEARCH_ENTRY . " " .
                                                    "WHERE " .          CMS_DB_SEARCH_ENTRY_INDEX . " " .
                                                    "IN (" .            $string . ")" .
                                                    "ORDER BY FIELD(" . CMS_DB_SEARCH_ENTRY_INDEX . ", " . $string . ")"))
                             &&
                             (mysql_num_rows($result) > 0)
                           )
                        {
                                //collect entry data
                                $_list = array();

                                while ($resultrow = mysql_fetch_row($result))
                                {
                                        //extract quotation
                                        $buffer = quote_text($resultrow[3], $term, 1);

                                        //alternatively display beginning of text
                                        if ($buffer === "")
                                        {
                                                preg_match("/^(.{0,225}[^\p{C}\p{P}\p{Z}]*)/u",
                                                           $resultrow[3],
                                                           $buffer);
                                                $buffer = x($buffer[1]) . "\xE2\x80\xA6";
                                        };

                                        //add data to list
                                        $_list[$resultrow[0]] =
                                                array("address"      => $resultrow[1],
                                                      "title"        => $resultrow[2],
                                                      "text"         => $buffer,
                                                      "time"         => $resultrow[4],
                                                      "score"        => $resultrow[5],
                                                      "supplemental" => $list[sqlesc($resultrow[0])]);
                                };

                                return $_list;
                        };
                };

                return FALSE;
        }
//-----------------------------------------------------------------------------------------------------
        function tag($entry_index = NULL,
                     $limit       = 10)
        {
                if (! $this->enabled) return FALSE;

                //retrieve most frequent words of all entries
                if ($entry_index === NULL)
                {
                        $subquery_total_weight =
                                "(SELECT SUM(" .    CMS_DB_SEARCH_WEIGHT_VALUE . ") " .
                                "FROM " .           CMS_DB_SEARCH_WEIGHT . ")";

                        $subquery_tag_list =
                                "(SELECT " .        CMS_DB_SEARCH_WEIGHT_WORD . ", " .
                                "(100/" .           $subquery_total_weight . "*" .
                                "SUM(" .            CMS_DB_SEARCH_WEIGHT_VALUE . ")) " .
                                "AS " .            "`#score` " .
                                "FROM " .           CMS_DB_SEARCH_WEIGHT . " " .
                                "GROUP BY " .       CMS_DB_SEARCH_WEIGHT_WORD . " " .
                                "ORDER BY " .      "`#score` DESC)";

                        $query =
                                "SELECT " .         CMS_DB_SEARCH_WORD_TEXT . ", " .
                                                   "`#score` " .
                                "FROM " .           $subquery_tag_list . " AS `#tag_list`, " .
                                                    CMS_DB_SEARCH_WORD . " " .
                                "WHERE " .          CMS_DB_SEARCH_WORD . "." . CMS_DB_SEARCH_WORD_INDEX .
                                "=" .              "`#tag_list`" .       "." . CMS_DB_SEARCH_WEIGHT_WORD . " " .
                                "AND BIT_LENGTH(" . CMS_DB_SEARCH_WORD_TEXT . ")" .
                                ">'" .             "8' " .
                                "LIMIT " .          sqlesc($limit);
                }

                //retrieve most frequent words of selected entry
                else
                {
                        $subquery_tag_list =
                                "(SELECT " .        CMS_DB_SEARCH_WEIGHT_WORD . ", " .
                                                    CMS_DB_SEARCH_WEIGHT_VALUE . " " .
                                "FROM " .           CMS_DB_SEARCH_WEIGHT . " " .
                                "WHERE " .          CMS_DB_SEARCH_WEIGHT_TARGET . " " .
                                "='" .              sqlesc($entry_index) . "' " .
                                "GROUP BY " .       CMS_DB_SEARCH_WEIGHT_WORD . " " .
                                "ORDER BY " .       CMS_DB_SEARCH_WEIGHT_VALUE . " DESC)";

                        $query =
                                "SELECT " .         CMS_DB_SEARCH_WORD_TEXT . ", " .
                                                    CMS_DB_SEARCH_WEIGHT_VALUE . " " .
                                "FROM " .           $subquery_tag_list . " AS `#tag_list`, " .
                                                    CMS_DB_SEARCH_WORD . " " .
                                "WHERE " .          CMS_DB_SEARCH_WORD . "." . CMS_DB_SEARCH_WORD_INDEX .
                                "=" .              "`#tag_list`" .       "." . CMS_DB_SEARCH_WEIGHT_WORD . " " .
                                "AND BIT_LENGTH(" . CMS_DB_SEARCH_WORD_TEXT . ")" .
                                ">'" .             "8' " .
                                "LIMIT " .          sqlesc($limit);
                };

                //execute and return
                if ($result = mysql_query($query))
                {
                        $list = array();

                        while ($resultrow = mysql_fetch_row($result))
                                $list[$resultrow[0]] = $resultrow[1];

                        return $list;
                };

                return FALSE;
        }
//-----------------------------------------------------------------------------------------------------
        function score_compute()
        {
                if (! $this->enabled) return FALSE;

                //initialize
                $result = $this->score_compute_initialize();
                if ($result !== TRUE) return $result;

                //iterate
                for ($i = 0; $i < $this->score_compute_iteration_number; $i++)
                {
                        $result = $this->score_compute_iterate();
                        if ($result !== TRUE) return $result;
                };

                //finalize
                $result = $this->score_compute_finalize();
                if ($result !== TRUE) return $result;

                //retrieve dangling link level
                $link_level = $this->score_compute_link_level();
                if ($link_level === FALSE) return FALSE;

                //iterate dangling link scores
                for ($i = $link_level; $i > 0; $i--)
                {
                        $result = $this->score_compute_iterate($i);
                        if ($result !== TRUE) return $result;
                };

                return TRUE;
        }
//-----------------------------------------------------------------------------------------------------
        function score_compute_initialize()
        {
                if (! $this->enabled) return FALSE;

                //preprocess links

                //reset link targets
                mysql_query("UPDATE " .      CMS_DB_SEARCH_LINK . " " .
                            "SET " .         CMS_DB_SEARCH_LINK_TARGET .
                            "='" .          "0', " .
                                             CMS_DB_SEARCH_LINK_LEVEL .
                            "='" .          "1'");
                if (mysql_errno() !== 0) return mysql_error(); //error check

                //seek and store link targets
                mysql_query("UPDATE " .      CMS_DB_SEARCH_LINK . ", " .
                                             CMS_DB_SEARCH_ENTRY . " " .
                            "SET " .         CMS_DB_SEARCH_LINK .  "." .  CMS_DB_SEARCH_LINK_TARGET .
                            "=" .            CMS_DB_SEARCH_ENTRY . "." .  CMS_DB_SEARCH_ENTRY_INDEX . ", " .
                                             CMS_DB_SEARCH_LINK .  "." .  CMS_DB_SEARCH_LINK_LEVEL .
                            "='" .          "0' " .
                            "WHERE " .       CMS_DB_SEARCH_LINK .  "." .  CMS_DB_SEARCH_LINK_TARGET_HASH .
                            "=" .            CMS_DB_SEARCH_ENTRY . "." .  CMS_DB_SEARCH_ENTRY_ADDRESS_HASH);
                if (mysql_errno() !== 0) return mysql_error(); //error check

                //remove self links
                mysql_query("DELETE FROM " . CMS_DB_SEARCH_LINK . " " .
                            "WHERE " .       CMS_DB_SEARCH_LINK_TARGET .
                            "=" .            CMS_DB_SEARCH_LINK_SOURCE);
                if (mysql_errno() !== 0) return mysql_error(); //error check

                //recursively deactivate dangling links
                $link_level = 1;

                do
                {
                        //retrieve dangling links
                        $result = mysql_query("SELECT " .     CMS_DB_SEARCH_LINK_SOURCE . " " .
                                              "FROM " .       CMS_DB_SEARCH_LINK . " " .
                                              "GROUP BY " .   CMS_DB_SEARCH_LINK_SOURCE . " " .
                                              "HAVING MIN(" . CMS_DB_SEARCH_LINK_LEVEL . ")" .
                                              "!='" .        "0'");
                        if (mysql_errno() !== 0) return mysql_error(); //error check

                        $num_rows      = mysql_num_rows($result);
                        $current_row   = 0;
                        $affected_rows = 0;

                        while ($current_row < $num_rows)
                        {
                                //collect link block
                                $string = "";
                                for ($i = 0; $i < 1000; $i++)
                                {
                                        if (! $resultrow = mysql_fetch_row($result)) break;
                                        $string .= "'" . sqlesc($resultrow[0]) . "', ";
                                };

                                //deactivate dangling links
                                mysql_query("UPDATE " . CMS_DB_SEARCH_LINK . " " .
                                            "SET " .    CMS_DB_SEARCH_LINK . "." . CMS_DB_SEARCH_LINK_LEVEL .
                                            "='" .      sqlesc(++$link_level) . "' " .
                                            "WHERE " .  CMS_DB_SEARCH_LINK . "." . CMS_DB_SEARCH_LINK_LEVEL .
                                            "='" .     "0' " .
                                            "AND " .    CMS_DB_SEARCH_LINK . "." . CMS_DB_SEARCH_LINK_TARGET . " " .
                                            "IN (" .    substr($string, 0, -2) . ")");
                                if (mysql_errno() !== 0) return mysql_error(); //error check

                                $current_row   += $i;
                                $affected_rows += mysql_affected_rows();
                        };
                }
                while ($affected_rows > 0);

                //preprocess entries

                //reset scores and outbound link counts
                mysql_query("UPDATE " .      CMS_DB_SEARCH_ENTRY . " " .
                            "SET " .         CMS_DB_SEARCH_ENTRY_SCORE .
                            "='" .          "1.0', " .
                                             CMS_DB_SEARCH_ENTRY_LINK_COUNT .
                            "='" .          "0'");
                if (mysql_errno() !== 0) return mysql_error(); //error check

                //count links per source omitting dangling links (source => count)
                $subquery =  "(SELECT " .     CMS_DB_SEARCH_LINK_SOURCE . ", " .
                                             "COUNT(*) AS `#count` " .
                             "FROM " .        CMS_DB_SEARCH_LINK . " " .
                             "WHERE " .       CMS_DB_SEARCH_LINK_LEVEL .
                             "='" .          "0' " .
                             "GROUP BY " .    CMS_DB_SEARCH_LINK_SOURCE . ")";
                if (mysql_errno() !== 0) return mysql_error(); //error check

                //store links per source
                mysql_query("UPDATE " .      CMS_DB_SEARCH_ENTRY . ", " .
                                             $subquery . " AS `#link` " .
                            "SET " .         CMS_DB_SEARCH_ENTRY . "." .  CMS_DB_SEARCH_ENTRY_LINK_COUNT .
                            "=" .           "`#link`" .            "." . "`#count` " .
                            "WHERE " .       CMS_DB_SEARCH_ENTRY . "." .  CMS_DB_SEARCH_ENTRY_INDEX .
                            "=" .           "`#link`" .            "." .  CMS_DB_SEARCH_LINK_SOURCE);
                if (mysql_errno() !== 0) return mysql_error(); //error check

                return TRUE;
        }
//-----------------------------------------------------------------------------------------------------
        function score_compute_iterate($link_level = 0)
        {
                if (! $this->enabled) return FALSE;

                $dampening_factor_inverted = 1.0 - $this->score_compute_dampening_factor;

                //retrieve inherited score portion per source (target => score portion)
                $subquery_inherited_score_portion =
                             "(SELECT " .         CMS_DB_SEARCH_LINK .  "." .  CMS_DB_SEARCH_LINK_TARGET . ", " .
                                                  CMS_DB_SEARCH_ENTRY . "." .  CMS_DB_SEARCH_ENTRY_SCORE .
                             "/" .                CMS_DB_SEARCH_ENTRY . "." .  CMS_DB_SEARCH_ENTRY_LINK_COUNT . " " .
                             "AS " .             "`#score_portion` " .
                             "FROM " .            CMS_DB_SEARCH_LINK . ", " .
                                                  CMS_DB_SEARCH_ENTRY . " " .
                             "WHERE " .           CMS_DB_SEARCH_LINK .  "." .  CMS_DB_SEARCH_LINK_LEVEL .
                             "='" .               sqlesc($link_level) . "' " .
                             "AND " .             CMS_DB_SEARCH_LINK .  "." .  CMS_DB_SEARCH_LINK_SOURCE .
                             "=" .                CMS_DB_SEARCH_ENTRY . "." .  CMS_DB_SEARCH_ENTRY_INDEX . ")";

                //compute total score per target (entry => score)
                $subquery_entry_score =
                             "(SELECT " .         CMS_DB_SEARCH_ENTRY . "." .  CMS_DB_SEARCH_ENTRY_INDEX . ", " .
                             "'" .                sqlesc($dampening_factor_inverted) . "'" .
                             "+'" .               sqlesc($this->score_compute_dampening_factor) . "'" .
                             "*SUM(" .           "`#link`" .            "." . "`#score_portion`) AS `#score` " .
                             "FROM " .            CMS_DB_SEARCH_ENTRY . ", " .
                                                  $subquery_inherited_score_portion . " AS `#link` " .
                             "WHERE " .           CMS_DB_SEARCH_ENTRY . "." .  CMS_DB_SEARCH_ENTRY_INDEX .
                             "=" .               "`#link`" .            "." .  CMS_DB_SEARCH_LINK_TARGET . " " .
                             "GROUP BY " .        CMS_DB_SEARCH_ENTRY . "." .  CMS_DB_SEARCH_ENTRY_INDEX . ")";

                //store score
                $query_score =
                             "UPDATE " .          CMS_DB_SEARCH_ENTRY . ", " .
                                                  $subquery_entry_score . " AS `#entry` " .
                             "SET " .             CMS_DB_SEARCH_ENTRY . "." .  CMS_DB_SEARCH_ENTRY_SCORE .
                             "=" .               "`#entry`" .           "." . "`#score` " .
                             "WHERE " .           CMS_DB_SEARCH_ENTRY . "." .  CMS_DB_SEARCH_ENTRY_INDEX .
                             "=" .               "`#entry`" .           "." .  CMS_DB_SEARCH_ENTRY_INDEX;

                //execute
                mysql_query($query_score);
                if (mysql_errno() !== 0) return mysql_error(); //error check

                return TRUE;
        }
//-----------------------------------------------------------------------------------------------------
        function score_compute_finalize()
        {
                if (! $this->enabled) return FALSE;

                //count links per source (source => count)
                $subquery =  "(SELECT " .     CMS_DB_SEARCH_LINK_SOURCE . ", " .
                                             "COUNT(*) AS `#count` " .
                             "FROM " .        CMS_DB_SEARCH_LINK . " " .
                             "GROUP BY " .    CMS_DB_SEARCH_LINK_SOURCE . ")";
                if (mysql_errno() !== 0) return mysql_error(); //error check

                //store links per source
                mysql_query("UPDATE " .      CMS_DB_SEARCH_ENTRY . ", " .
                                             $subquery . " AS `#link` " .
                            "SET " .         CMS_DB_SEARCH_ENTRY . "." .  CMS_DB_SEARCH_ENTRY_LINK_COUNT .
                            "=" .           "`#link`" .            "." . "`#count` " .
                            "WHERE " .       CMS_DB_SEARCH_ENTRY . "." .  CMS_DB_SEARCH_ENTRY_INDEX .
                            "=" .           "`#link`" .            "." .  CMS_DB_SEARCH_LINK_SOURCE);
                if (mysql_errno() !== 0) return mysql_error(); //error check

                return TRUE;
        }
//-----------------------------------------------------------------------------------------------------
        function score_compute_link_level()
        {
                if (! $this->enabled) return FALSE;

                if (
                     ($result = mysql_query("SELECT MAX(" . CMS_DB_SEARCH_LINK_LEVEL . ") " .
                                            "FROM " .       CMS_DB_SEARCH_LINK))
                     &&
                     ($resultrow = mysql_fetch_row($result))
                   )
                        return $resultrow[0];

                return FALSE;
        }
//-----------------------------------------------------------------------------------------------------
        function clean()
        {
                if (! $this->enabled) return FALSE;

                //clean up weight table
                mysql_query("DELETE " .    "a " .
                            "FROM " .             CMS_DB_SEARCH_WEIGHT . " a " .
                            "LEFT JOIN " .        CMS_DB_SEARCH_ENTRY .  " b " .
                            "ON " .        "a." . CMS_DB_SEARCH_WEIGHT_TARGET .
                            "=" .          "b." . CMS_DB_SEARCH_ENTRY_INDEX . " " .
                            "OR " .        "a." . CMS_DB_SEARCH_WEIGHT_SOURCE .
                            "=" .          "b." . CMS_DB_SEARCH_ENTRY_INDEX . " " .
                            "WHERE " .     "b." . CMS_DB_SEARCH_ENTRY_INDEX . " IS NULL");

                //clean up word table, not really necessary
                mysql_query("DELETE " .    "a " .
                            "FROM " .             CMS_DB_SEARCH_WORD .   " a " .
                            "LEFT JOIN " .        CMS_DB_SEARCH_WEIGHT . " b " .
                            "ON " .        "a." . CMS_DB_SEARCH_WORD_INDEX .
                            "=" .          "b." . CMS_DB_SEARCH_WEIGHT_WORD . " " .
                            "WHERE " .     "b." . CMS_DB_SEARCH_WEIGHT_WORD . " IS NULL");

                //clean up link table
                mysql_query("DELETE " .    "a " .
                            "FROM " .             CMS_DB_SEARCH_LINK .  " a " .
                            "LEFT JOIN " .        CMS_DB_SEARCH_ENTRY . " b " .
                            "ON " .        "a." . CMS_DB_SEARCH_LINK_SOURCE .
                            "=" .          "b." . CMS_DB_SEARCH_ENTRY_INDEX . " " .
                            "WHERE " .     "b." . CMS_DB_SEARCH_ENTRY_INDEX . " IS NULL");

                //clean up cluster table
                mysql_query("DELETE " .    "a " .
                            "FROM " .             CMS_DB_SEARCH_CLUSTER . " a " .
                            "LEFT JOIN " .        CMS_DB_SEARCH_ENTRY .   " b " .
                            "ON " .        "a." . CMS_DB_SEARCH_CLUSTER_SOURCE .
                            "=" .          "b." . CMS_DB_SEARCH_ENTRY_INDEX . " " .
                            "OR " .        "a." . CMS_DB_SEARCH_CLUSTER_TARGET .
                            "=" .          "b." . CMS_DB_SEARCH_ENTRY_INDEX . " " .
                            "WHERE " .     "b." . CMS_DB_SEARCH_ENTRY_INDEX . " IS NULL");

                $table = array(CMS_DB_SEARCH_ENTRY,
                               CMS_DB_SEARCH_WORD,
                               CMS_DB_SEARCH_WEIGHT,
                               CMS_DB_SEARCH_LINK,
                               CMS_DB_SEARCH_CLUSTER,
                               CMS_DB_SEARCH_QUEUE);
                $error = FALSE;

                //repair tables
                reset($table);
                while (list(, $value) = each($table))
                        $error |= ! mysql_query("REPAIR TABLE `" . sqlesc($value) . "`");

                //optimize tables
                reset($table);
                while (list(, $value) = each($table))
                        $error |= ! mysql_query("OPTIMIZE TABLE `" . sqlesc($value) . "`");

                return ! $error;
        }
//-----------------------------------------------------------------------------------------------------
        function address_standardize($address)
        {
                if (! $this->enabled) return FALSE;

                //verify address
                if (($address = analyze_url($address)) === FALSE) return FALSE; //invalid url
                $address["scheme"] = strtolower($address["scheme"]);

                switch ($address["scheme"])
                {
                case "":
                case "http":
                case "https":
                        break;

                default:
                        return FALSE;
                };

                //clean up querystring
                parse_str($address["query"], $array);
                $address["query"] = "";

                ksort($array); //order by key

                reset($array);
                while (list($key, $value) = each($array))
                        $address["query"] .= "$key=$value&";

                $address["query"] = substr($address["query"], 0, -1);

                //reassemble address
                $address = url($address["scheme"], //scheme
                               NULL,               //user
                               NULL,               //pass
                               $address["host"],   //host
                               NULL,               //port
                               $address["path"],   //path
                               $address["query"]); //query

                return $address;
        }
//-----------------------------------------------------------------------------------------------------
        function address_accepted($address)
        {
                if (! $this->enabled) return FALSE;

                //check for address exclusion
                reset($this->address_accepted_blacklist);
                while (list(, $value) = each($this->address_accepted_blacklist))
                {
                        if ($value === "") continue; //empty expression

                        //seek inversion marker
                        if ($value[0] === "!")
                        {
                                $value = substr($value, 1);
                                $flag  = FALSE;
                        }
                        else
                        {
                                $flag = TRUE;
                        };

                        //exclusion match
                        if ((bool)preg_match($value, $address) === $flag)
                        {
                                //check for address inclusion overriding exclusion
                                reset($this->address_accepted_whitelist);
                                while (list(, $value) = each($this->address_accepted_whitelist))
                                {
                                        if ($value === "") continue; //empty expression

                                        //seek inversion marker
                                        if ($value[0] === "!")
                                        {
                                                $value = substr($value, 1);
                                                $flag  = FALSE;
                                        }
                                        else
                                        {
                                                $flag = TRUE;
                                        };

                                        //inclusion match
                                        if ((bool)preg_match($value, $address) === $flag)
                                                return TRUE; //accepted
                                };

                                return FALSE; //rejected
                        };
                };

                return TRUE; //accepted
        }
};
?>