<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

//-----------------------------------------------------------------------------------------------------
//===< TEMPLATE CLASS >================================================================================
//-----------------------------------------------------------------------------------------------------

//permission
define("CMS_TEMPLATE_PERMISSION_OPERATOR", "operator");

//constants
define("CMS_TEMPLATE_CACHE_SEPARATOR",     "\x1C"); //ascii file separator

//option flag
define("CMS_TEMPLATE_OPTION_NONE",                  0);
define("CMS_TEMPLATE_OPTION_HREF",                  1);
define("CMS_TEMPLATE_OPTION_PLUGIN",                2);
define("CMS_TEMPLATE_OPTION_TEXT",                  4);
define("CMS_TEMPLATE_OPTION_VALUE",                 8);
define("CMS_TEMPLATE_OPTION_DOWNLOAD",             16);
define("CMS_TEMPLATE_OPTION_IMAGE",                32);
define("CMS_TEMPLATE_OPTION_THUMBNAIL",            64);
define("CMS_TEMPLATE_OPTION_MEDIA",               128);
define("CMS_TEMPLATE_OPTION_TEMPLATE",            256);
define("CMS_TEMPLATE_OPTION_GROUP",               512);
define("CMS_TEMPLATE_OPTION_REPEAT",             1024);
define("CMS_TEMPLATE_OPTION_SHIFT",              2048);
define("CMS_TEMPLATE_OPTION_CALT",               4096);
define("CMS_TEMPLATE_OPTION_CBLOCK",             8192);
define("CMS_TEMPLATE_OPTION_DEBUG",             16384);
define("CMS_TEMPLATE_OPTION_LAYOUT",       CMS_TEMPLATE_OPTION_TEMPLATE  |
                                           CMS_TEMPLATE_OPTION_VALUE     |
                                           CMS_TEMPLATE_OPTION_REPEAT    |
                                           CMS_TEMPLATE_OPTION_SHIFT     |
                                           CMS_TEMPLATE_OPTION_CBLOCK);
define("CMS_TEMPLATE_OPTION_EDIT",         CMS_TEMPLATE_OPTION_HREF      |
                                           CMS_TEMPLATE_OPTION_PLUGIN    |
                                           CMS_TEMPLATE_OPTION_TEXT      |
                                           CMS_TEMPLATE_OPTION_VALUE     |
                                           CMS_TEMPLATE_OPTION_DOWNLOAD  |
                                           CMS_TEMPLATE_OPTION_IMAGE     |
                                           CMS_TEMPLATE_OPTION_THUMBNAIL |
                                           CMS_TEMPLATE_OPTION_MEDIA     |
                                           CMS_TEMPLATE_OPTION_GROUP     |
                                           CMS_TEMPLATE_OPTION_CBLOCK);
define("CMS_TEMPLATE_OPTION_ALL",          4294967295                &
                                           ~CMS_TEMPLATE_OPTION_CALT &
                                           ~CMS_TEMPLATE_OPTION_DEBUG);

define("CMS_TEMPLATE_ACTION",                       0);
define("CMS_TEMPLATE_CONTROL",                      1);
define("CMS_TEMPLATE_LOCK",                         2);
define("CMS_TEMPLATE_CODE",                         3);
define("CMS_TEMPLATE_IMAGE",                        4);
define("CMS_TEMPLATE_COMMAND",                      5);
define("CMS_TEMPLATE_SWITCH",                       6);

//type
define("CMS_TEMPLATE_TYPE_NONE",                    0);
define("CMS_TEMPLATE_TYPE_HEAD",                    1);
define("CMS_TEMPLATE_TYPE_HREF",                    2);
define("CMS_TEMPLATE_TYPE_PLUGIN",                  4);
define("CMS_TEMPLATE_TYPE_TEXT",                    8);
define("CMS_TEMPLATE_TYPE_VALUE",                  16);
define("CMS_TEMPLATE_TYPE_DOWNLOAD",               32);
define("CMS_TEMPLATE_TYPE_IMAGE",                  64);
define("CMS_TEMPLATE_TYPE_THUMBNAIL",             128);
define("CMS_TEMPLATE_TYPE_MEDIA",                 256);
define("CMS_TEMPLATE_TYPE_TEMPLATE",              512);
define("CMS_TEMPLATE_TYPE_GROUP",                1024);
define("CMS_TEMPLATE_TYPE_REPEAT",               2048);
define("CMS_TEMPLATE_TYPE_SHIFT",                4096);
define("CMS_TEMPLATE_TYPE_MENU",                 8192);
define("CMS_TEMPLATE_TYPE_CBLOCK",              16384);
define("CMS_TEMPLATE_TYPE_CALT",                32768);
define("CMS_TEMPLATE_TYPE_BASE",                65536);
define("CMS_TEMPLATE_TYPE_NAMESPACE",          131072);
define("CMS_TEMPLATE_TYPE_NOCACHE",            262144);
define("CMS_TEMPLATE_TYPE_CONTROL",            524288);
define("CMS_TEMPLATE_TYPE_BACKLINK",          1048576);
define("CMS_TEMPLATE_TYPE_DEBUG",             2097152);
define("CMS_TEMPLATE_TYPE_ALL",            4294967295);

//type filter
define("CMS_TEMPLATE_TYPE_EDIT",           CMS_TEMPLATE_TYPE_HREF      |
                                           CMS_TEMPLATE_TYPE_PLUGIN    |
                                           CMS_TEMPLATE_TYPE_TEXT      |
                                           CMS_TEMPLATE_TYPE_VALUE     |
                                           CMS_TEMPLATE_TYPE_DOWNLOAD  |
                                           CMS_TEMPLATE_TYPE_IMAGE     |
                                           CMS_TEMPLATE_TYPE_THUMBNAIL |
                                           CMS_TEMPLATE_TYPE_MEDIA     |
                                           CMS_TEMPLATE_TYPE_TEMPLATE  |
                                           CMS_TEMPLATE_TYPE_GROUP     |
                                           CMS_TEMPLATE_TYPE_REPEAT    |
                                           CMS_TEMPLATE_TYPE_SHIFT);
define("CMS_TEMPLATE_TYPE_SPAN",           CMS_TEMPLATE_TYPE_HREF      |
                                           CMS_TEMPLATE_TYPE_DOWNLOAD  |
                                           CMS_TEMPLATE_TYPE_GROUP     |
                                           CMS_TEMPLATE_TYPE_REPEAT    |
                                           CMS_TEMPLATE_TYPE_SHIFT     |
                                           CMS_TEMPLATE_TYPE_CBLOCK    |
                                           CMS_TEMPLATE_TYPE_CALT      |
                                           CMS_TEMPLATE_TYPE_BASE      |
                                           CMS_TEMPLATE_TYPE_NAMESPACE |
                                           CMS_TEMPLATE_TYPE_NOCACHE);
define("CMS_TEMPLATE_TYPE_PATH",           CMS_TEMPLATE_TYPE_TEMPLATE  |
                                           CMS_TEMPLATE_TYPE_GROUP     |
                                           CMS_TEMPLATE_TYPE_REPEAT    |
                                           CMS_TEMPLATE_TYPE_SHIFT     |
                                           CMS_TEMPLATE_TYPE_BASE      |
                                           CMS_TEMPLATE_TYPE_NAMESPACE);

//command
define("CMS_TEMPLATE_COMMAND_NONE",                 0);
define("CMS_TEMPLATE_COMMAND_BUFFER",               1);
define("CMS_TEMPLATE_COMMAND_PASTE",                2);
define("CMS_TEMPLATE_COMMAND_SWAP",                 4);
define("CMS_TEMPLATE_COMMAND_KICK1",                8);
define("CMS_TEMPLATE_COMMAND_KICK2",               16);
define("CMS_TEMPLATE_COMMAND_DROP1",               32);
define("CMS_TEMPLATE_COMMAND_DROP2",               64);
define("CMS_TEMPLATE_COMMAND_RELEASE",            128);
define("CMS_TEMPLATE_COMMAND_REFERENCE",          256);
define("CMS_TEMPLATE_COMMAND_EXPORT",             512);
define("CMS_TEMPLATE_COMMAND_CLEAR",             1024);
define("CMS_TEMPLATE_COMMAND_DRAGDROP1",         2048);
define("CMS_TEMPLATE_COMMAND_DRAGDROP2",         4096);
define("CMS_TEMPLATE_COMMAND_ALL",         4294967295);

//structure
define("CMS_TEMPLATE_STRUCTURE_INDEX",              0);
define("CMS_TEMPLATE_STRUCTURE_PATH",               1);
define("CMS_TEMPLATE_STRUCTURE_PARENT",             2);
define("CMS_TEMPLATE_STRUCTURE_TYPE",               3);

//related functions
//-----------------------------------------------------------------------------------------------------
function template_get_array($page = NULL)
{
        $data  = new data("#system/template");
        $array = array("" => array());

        $data->move("first");
        while ($key = $data->move("next"))
        {
                if ($page !== NULL)
                {
                        if ($data->get($key, "page"))
                        {
                                if ($page === FALSE) continue;
                        }
                        else
                        {
                                if ($page === TRUE)  continue;
                        };
                };

                $category                  = $data->get($key, "category");
                if (! $category) $category = "";
                $name                      = $data->get($key, "name");
                if (! $name)     $name     = $key;

                $_name                     = $name;
                $i                         = 1;
                while (isset($array[$category][$_name]))
                        $_name = $name . " (" . $i++ . ")";

                $array[$category][$_name]  = $key;
        };

        ksort($array);
        reset($array);
        while (list($key) = each($array))
                if (is_array($array[$key])) ksort($array[$key]);

        return $array;
};
//-----------------------------------------------------------------------------------------------------
function template_get_select($page = NULL)
{
        $data  = new data("#system/template");
        $array = array("" => "");

        $data->move("first");
        while ($key = $data->move("next"))
        {
                if ($page !== NULL)
                {
                        if ($data->get($key, "page"))
                        {
                                if ($page === FALSE) continue;
                        }
                        else
                        {
                                if ($page === TRUE)  continue;
                        };
                };

                $category                          = $data->get($key, "category");
                $array[$category ? $category : ""] = $category;
        };

        ksort($array);

        return $array;
};
//-----------------------------------------------------------------------------------------------------
function template_read_plugin($url)
{
        return (cms_load("html")) ? html_page_data($url) : FALSE;
};
//-----------------------------------------------------------------------------------------------------
function template_preview($index)
{
        if (! cms_load("document")) return;

        $document = new document();
        $template = new template();
        $page     = $template->data->get($index, "page");

        $text     = "<CMS:template id=\"preview\" " .
                                  "edit=\"off\"/>";

        $document->set("preview", "template", $index);

        _template_preview($document, $text, $page);
};
//-----------------------------------------------------------------------------------------------------
function _template_preview($document,
                           $text,
                           $page = FALSE)
{
        if (! $page)
                $text = CMS_DOCTYPE_HTML .
                        "<html>" .
                        "<head>" .
                        "<CMS:head/>" .
                        stylesheet(CMS_URL . "stylesheet/template.css") .
                        "</head>" .
                        "<body>" .
                        template_lock() .
                        "<div>" .
                        $text .
                        "</div>" .
                        "</body>" .
                        "</html>" .
                        jscript("document.documentElement.style.display = \"block\";");

        $template = new template();

        echo($template->parse_code(
                $document,
                $text,
                CMS_L_TEMPLATE_002,
                NULL,
                NULL,
                array(CMS_TEMPLATE_ACTION =>
                      array(CMS_TEMPLATE_TYPE_HREF       => array(CMS_TEMPLATE_CODE  => "javascript:return false;",
                                                                  CMS_TEMPLATE_IMAGE => CMS_IMAGES_URL . "content/button_href.png"),
                            CMS_TEMPLATE_TYPE_PLUGIN     => array(CMS_TEMPLATE_CODE  => "javascript:return false;",
                                                                  CMS_TEMPLATE_IMAGE => CMS_IMAGES_URL . "content/button_plugin.png"),
                            CMS_TEMPLATE_TYPE_TEXT       => array(CMS_TEMPLATE_CODE  => "javascript:return false;",
                                                                  CMS_TEMPLATE_IMAGE => CMS_IMAGES_URL . "content/button_text.png"),
                            CMS_TEMPLATE_TYPE_VALUE      => array(CMS_TEMPLATE_CODE  => "javascript:return false;",
                                                                  CMS_TEMPLATE_IMAGE => CMS_IMAGES_URL . "content/button_value.png"),
                            CMS_TEMPLATE_TYPE_DOWNLOAD   => array(CMS_TEMPLATE_CODE  => "javascript:return false;",
                                                                  CMS_TEMPLATE_IMAGE => CMS_IMAGES_URL . "content/button_download.png"),
                            CMS_TEMPLATE_TYPE_IMAGE      => array(CMS_TEMPLATE_CODE  => "javascript:return false;",
                                                                  CMS_TEMPLATE_IMAGE => CMS_IMAGES_URL . "content/button_image.png"),
                            CMS_TEMPLATE_TYPE_THUMBNAIL  => array(CMS_TEMPLATE_CODE  => "javascript:return false;",
                                                                  CMS_TEMPLATE_IMAGE => CMS_IMAGES_URL . "content/button_thumbnail.png"),
                            CMS_TEMPLATE_TYPE_MEDIA      => array(CMS_TEMPLATE_CODE  => "javascript:return false;",
                                                                  CMS_TEMPLATE_IMAGE => CMS_IMAGES_URL . "content/button_media.png"),
                            CMS_TEMPLATE_TYPE_TEMPLATE   => array(CMS_TEMPLATE_CODE  => "javascript:return false;",
                                                                  CMS_TEMPLATE_IMAGE => CMS_IMAGES_URL . "content/button_template.png"),
                            CMS_TEMPLATE_TYPE_GROUP      => array(CMS_TEMPLATE_CODE  => "javascript:return false;",
                                                                  CMS_TEMPLATE_IMAGE => CMS_IMAGES_URL . "content/button_group.png"),
                            CMS_TEMPLATE_TYPE_REPEAT     => array(CMS_TEMPLATE_CODE  => "javascript:return false;",
                                                                  CMS_TEMPLATE_IMAGE => CMS_IMAGES_URL . "content/button_repeat.png"),
                            CMS_TEMPLATE_TYPE_SHIFT      => array(CMS_TEMPLATE_CODE  => "javascript:return false;",
                                                                  CMS_TEMPLATE_IMAGE => CMS_IMAGES_URL . "content/button_shift.png"),
                            CMS_TEMPLATE_TYPE_CBLOCK     => TRUE),
                      CMS_TEMPLATE_LOCK   => TRUE)));
};
//-----------------------------------------------------------------------------------------------------
function template_lock()
{
        return "<a href=\"#\" " .
                  "style=\"BACKGROUND:transparent;" .
                          "BORDER:none;" .
                          "BOTTOM:0;" .
                          "CURSOR:not-allowed;" .
                          "LEFT:0;" .
                          "MARGIN:0;" .
                          "PADDING:0;" .
                          "POSITION:fixed;" .
                          "RIGHT:0;" .
                          "TOP:0;" .
                          "Z-INDEX:99999\" " .
                   "onclick=\"return false;\" " .
                   "onfocus=\"this.blur();\">" .
               "</a>";
};
//-----------------------------------------------------------------------------------------------------
function template_error($code,
                        $message,
                        $path = NULL,
                        $line = NULL)
{
        return cms_error($code,
                         $message,
                         "embedded PHP script",
                         $GLOBALS["template_error_line"] + $line,
                         (bool)CMS_TEMPLATE_EDIT);
};

class template
{
//properties
//-----------------------------------------------------------------------------------------------------
        public $data         = NULL;
        public $operator     = NULL;

        public $tlist        = array(""                           => CMS_TEMPLATE_TYPE_NONE,
                                     "head"                       => CMS_TEMPLATE_TYPE_HEAD,
                                     "href"                       => CMS_TEMPLATE_TYPE_HREF,
                                     "plugin"                     => CMS_TEMPLATE_TYPE_PLUGIN,
                                     "text"                       => CMS_TEMPLATE_TYPE_TEXT,
                                     "value"                      => CMS_TEMPLATE_TYPE_VALUE,
                                     "download"                   => CMS_TEMPLATE_TYPE_DOWNLOAD,
                                     "image"                      => CMS_TEMPLATE_TYPE_IMAGE,
                                     "thumbnail"                  => CMS_TEMPLATE_TYPE_THUMBNAIL,
                                     "media"                      => CMS_TEMPLATE_TYPE_MEDIA,
                                     "template"                   => CMS_TEMPLATE_TYPE_TEMPLATE,
                                     "group"                      => CMS_TEMPLATE_TYPE_GROUP,
                                     "repeat"                     => CMS_TEMPLATE_TYPE_REPEAT,
                                     "shift"                      => CMS_TEMPLATE_TYPE_SHIFT,
                                     "cblock"                     => CMS_TEMPLATE_TYPE_CBLOCK,
                                     "calt"                       => CMS_TEMPLATE_TYPE_CALT,
                                     "base"                       => CMS_TEMPLATE_TYPE_BASE,
                                     "namespace"                  => CMS_TEMPLATE_TYPE_NAMESPACE,
                                     "nocache"                    => CMS_TEMPLATE_TYPE_NOCACHE,
                                     "control"                    => CMS_TEMPLATE_TYPE_CONTROL,
                                     "backlink"                   => CMS_TEMPLATE_TYPE_BACKLINK,
                                     "menu"                       => CMS_TEMPLATE_TYPE_MENU);
        public $tname        = array(CMS_TEMPLATE_TYPE_NONE       => CMS_L_UNKNOWN,
                                     CMS_TEMPLATE_TYPE_HREF       => CMS_L_TEMPLATE_039,
                                     CMS_TEMPLATE_TYPE_PLUGIN     => CMS_L_TEMPLATE_003,
                                     CMS_TEMPLATE_TYPE_TEXT       => CMS_L_TEMPLATE_029,
                                     CMS_TEMPLATE_TYPE_VALUE      => CMS_L_TEMPLATE_043,
                                     CMS_TEMPLATE_TYPE_DOWNLOAD   => CMS_L_TEMPLATE_030,
                                     CMS_TEMPLATE_TYPE_IMAGE      => CMS_L_TEMPLATE_031,
                                     CMS_TEMPLATE_TYPE_THUMBNAIL  => CMS_L_TEMPLATE_032,
                                     CMS_TEMPLATE_TYPE_MEDIA      => CMS_L_TEMPLATE_033,
                                     CMS_TEMPLATE_TYPE_TEMPLATE   => CMS_L_TEMPLATE_034,
                                     CMS_TEMPLATE_TYPE_GROUP      => CMS_L_TEMPLATE_035,
                                     CMS_TEMPLATE_TYPE_REPEAT     => CMS_L_TEMPLATE_036,
                                     CMS_TEMPLATE_TYPE_SHIFT      => CMS_L_TEMPLATE_037);
        public $action       = NULL;
        public $image        = NULL;
        public $media        = NULL;
        public $download     = NULL;
        public $editable     = NULL;
        public $structure_id = NULL;
        public $parent_id    = NULL;
        public $execute_vars = array();
//constructor
//-----------------------------------------------------------------------------------------------------
        function __construct()
        {
                $this->data     = new data("#system/template");
                $this->operator = cms_permission(CMS_TEMPLATE_PERMISSION_OPERATOR);

                mkpath(CMS_DATA_PATH . "#template");
                mkpath(CMS_DATA_PATH . "template/stylesheet");
        }
//methods
//-----------------------------------------------------------------------------------------------------
        function add($name,
                     $category   = NULL,
                     $page       = NULL,
                     $code       = NULL,
                     $stylesheet = NULL)
        {
                if (! $this->operator) return FALSE;

                $name     = utf8_trim($name);
                $category = utf8_trim($category);
                $page     = (bool)$page;

                if (! $name) $name = CMS_L_TEMPLATE_001;

                $this->data->set_buffer(array(array("name"     => $name,
                                                    "category" => $category,
                                                    "page"     => $page)));

                if (
                     ($return = $this->data->insert())
                     &&
                     $this->set_code($return, $code)
                     &&
                     $this->set_stylesheet($return, $stylesheet)
                     &&
                     $this->data->save()
                   )
                        return $return;

                return FALSE;
        }
//-----------------------------------------------------------------------------------------------------
        function set($index,
                     $name     = NULL,
                     $category = NULL,
                     $page     = NULL)
        {
                if (! $this->operator) return FALSE;

                $name     = utf8_trim($name);
                $category = utf8_trim($category);
                $page     = (bool)$page;

                if (! $name) $name = CMS_L_TEMPLATE_001;

                $this->data->set(array("name"     => $name,
                                       "category" => $category,
                                       "page"     => $page),
                                 $index);

                return $this->data->save();
        }
//-----------------------------------------------------------------------------------------------------
        function get($index)
        {
                return $this->data->get($index);
        }
//-----------------------------------------------------------------------------------------------------
        function set_code($index,
                          $text)
        {
                if (! $this->operator) return FALSE;

                $array = language_get_array($text);
                $error = FALSE;

                reset($array);
                while (list($key, $value) = each($array))
                {
                        if (nstre($key)) $key .= ".";

                        //delete file
                        if (! $value)
                        {
                                if (is_file(CMS_DATA_PATH . "#template/$key$index.htm"))
                                        $error = ! unlink(CMS_DATA_PATH . "#template/$key$index.htm");
                                continue;
                        };

                        //write file
                        if ($hfile = fopen(CMS_DATA_PATH . "#template/$key$index.htm", "wb"))
                        {
                                fputs($hfile, $value);
                                fclose($hfile);

                                cms_cache("template.$index", $text);
                        }
                        else
                        {
                                $error = TRUE;
                        };
                };

                touch(CMS_DATA_PATH . "#system/template.dat");

                return ! $error;
        }
//-----------------------------------------------------------------------------------------------------
        function get_code($index,
                          $language = NULL)
        {
                //multiple language string
                if ($language === FALSE)
                {
                        $return   = NULL;
                        $language = CMS_LANGUAGE_ENABLED ?
                                    array_merge(array(""), explode(",", CMS_LANGUAGE_ENABLED)) :
                                    array("");

                        reset($language);
                        while (list($key, $value) = each($language))
                        {
                                $key    = $value ? "$value." : "";

                                //retrieve from temporary cache
                                if (! $text = cms_cache("template.$key$index"))
                                {
                                        //retrieve from file
                                        $text = read_file(CMS_DATA_PATH . "#template/$key$index.htm");

                                        //temporary caching
                                        cms_cache("template.$key$index", $text);
                                };

                                $return = language_set($return, $text, $value);
                        };

                        return $return;
                };

                //active language string
                if ($language === NULL) $language  = CMS_LANGUAGE;
                if (nstre($language))   $language .= ".";

                //retrieve from temporary cache
                if (! $text = cms_cache("template.$language$index"))
                {
                        //retrieve from file
                        $text = read_file(CMS_DATA_PATH . "#template/$language$index.htm");

                        //temporary caching
                        cms_cache("template.$language$index", $text);

                        if ($language && ! $text)
                        {
                                if (! $text = cms_cache("template.$index"))
                                {
                                        //retrieve from file
                                        $text = read_file(CMS_DATA_PATH . "#template/$index.htm");

                                        //temporary caching
                                        cms_cache("template.$index", $text);
                                };
                        };
                };

                return $text;
        }
//-----------------------------------------------------------------------------------------------------
        function set_stylesheet($index,
                                $text)
        {
                if (! $this->operator) return FALSE;

                $array = language_get_array($text);
                $error = FALSE;

                reset($array);
                while (list($key, $value) = each($array))
                {
                        if (nstre($key)) $key .= ".";

                        //delete file
                        if (! $value)
                        {
                                if (is_file(CMS_DATA_PATH . "template/stylesheet/$key$index.css"))
                                        $error = ! unlink(CMS_DATA_PATH . "template/stylesheet/$key$index.css");
                                continue;
                        };

                        //write file
                        if ($hfile = fopen(CMS_DATA_PATH . "template/stylesheet/$key$index.css", "wb"))
                        {
                                fputs($hfile, $value);
                                fclose($hfile);
                        }
                        else
                        {
                                $error = TRUE;
                        };
                };

                touch(CMS_DATA_PATH . "#system/template.dat");

                return ! $error;
        }
//-----------------------------------------------------------------------------------------------------
        function get_stylesheet($index,
                                $language = NULL,
                                $check    = FALSE)
        {
                //multiple language string
                if ($language === FALSE)
                {
                        $return   = NULL;
                        $language = CMS_LANGUAGE_ENABLED ?
                                    array_merge(array(""), explode(",", CMS_LANGUAGE_ENABLED)) :
                                    array("");

                        reset($language);
                        while (list($key, $value) = each($language))
                        {
                                //retrieve from file
                                $key    = $value ? "$value." : "";
                                $text   = read_file(CMS_DATA_PATH . "template/stylesheet/$key$index.css");
                                $return = language_set($return, $text, $value);
                        };

                        return $return;
                };

                //active language string
                if ($language === NULL) $language  = CMS_LANGUAGE;
                if (nstre($language))   $language .= ".";

                //retrieve from file
                $file = "template/stylesheet/$language$index.css";
                $text = $check ? is_file(CMS_DATA_PATH . $file) : read_file(CMS_DATA_PATH . $file);

                if ($language && ! $text)
                {
                        //retrieve from file
                        $file = "template/stylesheet/$index.css";
                        $text = $check ? is_file(CMS_DATA_PATH . $file) : read_file(CMS_DATA_PATH . $file);
                };

                return $check ? ($text ? $file : FALSE) : $text;
        }
//-----------------------------------------------------------------------------------------------------
        function delete($index)
        {
                if (! $this->operator) return FALSE;
                if (! $index)          return FALSE;

                $language = CMS_LANGUAGE_ENABLED ?
                            array_merge(array(""), explode(",", CMS_LANGUAGE_ENABLED)) :
                            array("");

                reset($language);
                while (list($key, $value) = each($language))
                {
                        $key = $value ? "$value." : "";

                        if (is_file(CMS_DATA_PATH . "#template/$key$index.htm"))
                        {
                                if (! unlink(CMS_DATA_PATH . "#template/$key$index.htm")) return FALSE;

                                //remove from temporary cache
                                cms_cache("template.$key$index", "");
                        };

                        if (is_file(CMS_DATA_PATH . "template/stylesheet/$key$index.css"))
                                unlink(CMS_DATA_PATH . "template/stylesheet/$key$index.css");
                };

                $this->data->del($index);

                return $this->data->save();
        }
//-----------------------------------------------------------------------------------------------------
        function parse(&$document,
                       $index,
                       $title       = NULL,
                       $description = NULL,
                       $keyword     = NULL,
                       $action      = NULL,
                       $header      = NULL,
                       $base_id     = NULL) //do not change for basic template
        {
                //link stylesheet of employed template
                if (($file = $this->get_stylesheet($index, NULL, TRUE)) !== FALSE)
                        $header = stylesheet(CMS_DATA_URL . $file, FALSE) .
                                  (stre($header) ? "" : "\n") . $header;

                return $this->_parse($document,
                                     $this->get_code($index),
                                     $title,
                                     $description,
                                     $keyword,
                                     $action,
                                     $header,
                                     $base_id);
        }
//-----------------------------------------------------------------------------------------------------
        function parse_code(&$document,
                            $template,
                            $title       = NULL,
                            $description = NULL,
                            $keyword     = NULL,
                            $action      = NULL,
                            $header      = NULL,
                            $base_id     = NULL) //do not change for basic template
        {
                return $this->_parse($document,
                                     $template,
                                     $title,
                                     $description,
                                     $keyword,
                                     $action,
                                     $header,
                                     $base_id);
        }
//-----------------------------------------------------------------------------------------------------
        private function _parse(&$document,
                                $template,
                                $title       = NULL,
                                $description = NULL,
                                $keyword     = NULL,
                                $action      = NULL,
                                $header      = NULL,
                                $base_id     = NULL, //do not change for basic template
                                $cache       = FALSE)
        {
                static $template_list  = array(); //list of employed templates
                static $template_depth = -1;      //template nesting counter
                static $group_depth    =  0;      //group nesting counter, used for block color
                static $index          = -1;      //dataset index
                static $lock           = FALSE;   //preview mode
                static $debug          = FALSE;   //debug mode

                if (stre($template)) return FALSE; //template is empty
                if (nstre($base_id)) $base_id .= ".";

                //increase template nesting counter
                $template_depth++;

                //replace tokens in template code
                $template = parse_token($template);

                //load image extension
                if ((! $this->image) && cms_load("image"))
                        $this->image = new image();

                //load media extension
                if ((! $this->media) && cms_load("media"))
                        $this->media = new media();

                //load download extension
                if ((! $this->download) && cms_load("download"))
                        $this->download = new download();

                //initialize
                if (! $template_depth) //zero on first pass
                {
                        //reset list of employed templates
                        $template_list  = array();

                        //reset dataset index
                        $index          = -1;

                        //set preview mode
                        $lock           = isset($action[CMS_TEMPLATE_LOCK]);

                        //set debug mode
                        $debug          = (! $cache) && isset($action[CMS_TEMPLATE_ACTION][CMS_TEMPLATE_TYPE_DEBUG]);

                        //buffer action definitions to save stack memory
                        $this->action   = $action; //copy to class variable
                        $action         = NULL;    //and set to null value

                        //scripting constants
                        if (! defined("CMS_TEMPLATE"))        define("CMS_TEMPLATE",      TRUE);
                        if (! defined("CMS_TEMPLATE_EDIT"))   define("CMS_TEMPLATE_EDIT", (! $cache) && isset($this->action[CMS_TEMPLATE_ACTION]));
                        if (! defined("CMS_TEMPLATE_OPTION")) define("CMS_TEMPLATE_OPTION",
                                (isset($this->action[CMS_TEMPLATE_ACTION][CMS_TEMPLATE_TYPE_TEXT])      * CMS_TEMPLATE_OPTION_TEXT)      |
                                (isset($this->action[CMS_TEMPLATE_ACTION][CMS_TEMPLATE_TYPE_VALUE])     * CMS_TEMPLATE_OPTION_VALUE)     |
                                (isset($this->action[CMS_TEMPLATE_ACTION][CMS_TEMPLATE_TYPE_IMAGE])     * CMS_TEMPLATE_OPTION_IMAGE)     |
                                (isset($this->action[CMS_TEMPLATE_ACTION][CMS_TEMPLATE_TYPE_THUMBNAIL]) * CMS_TEMPLATE_OPTION_THUMBNAIL) |
                                (isset($this->action[CMS_TEMPLATE_ACTION][CMS_TEMPLATE_TYPE_MEDIA])     * CMS_TEMPLATE_OPTION_MEDIA)     |
                                (isset($this->action[CMS_TEMPLATE_ACTION][CMS_TEMPLATE_TYPE_HREF])      * CMS_TEMPLATE_OPTION_HREF)      |
                                (isset($this->action[CMS_TEMPLATE_ACTION][CMS_TEMPLATE_TYPE_DOWNLOAD])  * CMS_TEMPLATE_OPTION_DOWNLOAD)  |
                                (isset($this->action[CMS_TEMPLATE_ACTION][CMS_TEMPLATE_TYPE_PLUGIN])    * CMS_TEMPLATE_OPTION_PLUGIN)    |
                                (isset($this->action[CMS_TEMPLATE_ACTION][CMS_TEMPLATE_TYPE_TEMPLATE])  * CMS_TEMPLATE_OPTION_TEMPLATE)  |
                                (isset($this->action[CMS_TEMPLATE_ACTION][CMS_TEMPLATE_TYPE_GROUP])     * CMS_TEMPLATE_OPTION_GROUP)     |
                                (isset($this->action[CMS_TEMPLATE_ACTION][CMS_TEMPLATE_TYPE_REPEAT])    * CMS_TEMPLATE_OPTION_REPEAT)    |
                                (isset($this->action[CMS_TEMPLATE_ACTION][CMS_TEMPLATE_TYPE_SHIFT])     * CMS_TEMPLATE_OPTION_SHIFT)     |
                                (isset($this->action[CMS_TEMPLATE_ACTION][CMS_TEMPLATE_TYPE_CALT])      * CMS_TEMPLATE_OPTION_CALT)      |
                                (isset($this->action[CMS_TEMPLATE_ACTION][CMS_TEMPLATE_TYPE_CBLOCK])    * CMS_TEMPLATE_OPTION_CBLOCK)    |
                                ($debug                                                                 * CMS_TEMPLATE_OPTION_DEBUG));
                };

                $offset           = 0;                 //primary text position
                $_offset          = 0;                 //secondary text position
                $length           = strlen($template); //template code length

                $auto_id          = 0;                 //automatic element index
                $_base_id         = $base_id;          //base index working copy

                $index_stack      = array();           //dataset index stack
                $span_stack       = array();           //open span stack
                $group_stack      = array();           //group data stack
                $base_stack       = array();           //base data stack
                $namespace_stack  = array();           //namespace data stack
                $repeat_stack     = array();           //repeat data stack
                $shift_stack      = array();           //shift data stack
                $temp             = sprintf("%x", crc32($base_id . $index));
                $temp_stack       = array($temp);      //temporary random index stack
                $debug_stack      = array();           //debug information stack

                $cblock_depth     = 0;                 //conditional block depth
                $cblock_limit     = 0;                 //conditional block limit
                $calt_depth       = 0;                 //conditional alternative depth
                $calt_limit       = 0;                 //conditional alternative limit
                $nocache_depth    = 0;                 //nocache depth

                $output           = NULL;              //output buffer

                //process template code
                while (
                        ($offset < $length) //not at end of template code
                        &&
                        (($offset = strpos($template, "<", $offset)) !== FALSE) //seek next opening tag
                      )
                {
                        //append preliminary unparsed data to output if not suppressed or within nocache block
                        if (! ($cblock_limit || $calt_limit))
                                $output .= substr($template, $_offset, $offset - $_offset);

//.....................................................................................................

                        //php code
                        if (
                             ($template[$offset + 1] === "?") //fast check of second character
                             &&
                             preg_match("/^(<\?(php\s|=|)?\s*)/i", substr($template, $offset, 6), $string) //extensive check
                           )
                        {
                                $_length = strlen($string[1]); //length of php tag including trailing whitespace
                                $flag    = $string[2] === "="; //set output flag
                                $_offset = $offset + $_length; //set offset to start of php code
                                $char    = NULL;
                                $string  = NULL;
                                $result  = TRUE;

                                //seek end tag position
                                while ($_offset < $length)
                                {
                                        $_char = $char;
                                        $char  = $template[$_offset++];

                                        //literal / comment
                                        if ($string)
                                        {
                                                //single line comment
                                                if ($string === "\n")
                                                {
                                                        switch ($char)
                                                        {

                                                        //single line comment end
                                                        case "\n":
                                                                $string = NULL;
                                                                break;

                                                        //php code end
                                                        case ">":
                                                                if ($_char === "?")  break 2;
                                                                break;
                                                        };
                                                }
                                                elseif ($char === $string)
                                                {
                                                        switch ($char)
                                                        {

                                                        //literal end
                                                        case "'":
                                                        case "\"":
                                                                if ($_char !== "\\") $string = NULL;
                                                                break;

                                                        //multi line comment end
                                                        case "/":
                                                                if ($_char === "*")  $string = NULL;
                                                                break;
                                                        };
                                                };
                                        }

                                        //php code
                                        else
                                        {
                                                switch ($char)
                                                {

                                                //literal start
                                                case "'":
                                                case "\"":
                                                        $string = $char;
                                                        break;

                                                //single line comment start
                                                case "#":
                                                        $string = "\n";
                                                        break;

                                                case "/":
                                                        if ($_char === "/") $string = "\n";
                                                        break;

                                                //multi line comment start
                                                case "*":
                                                        if ($_char === "/") $string = "/";
                                                        break;

                                                //php code end
                                                case ">":
                                                        if ($_char === "?") break 2;
                                                        break;
                                                };
                                        };
                                };

                                //not suppressed
                                if (! ($cblock_limit || $calt_limit))
                                {
                                        //append unparsed source code if within nocache block
                                        if ($nocache_depth)
                                        {
                                                $output .= substr($template, $offset, $_offset - $offset);
                                        }

                                        //parse script
                                        else
                                        {
                                                //get php source code
                                                $__offset = $offset + $_length;
                                                $buffer   = substr($template, $__offset, $_offset - $__offset - 2);

                                                //apply short output
                                                if ($flag) $buffer = "echo $buffer;";

                                                //set number of lines preceding script block
                                                $GLOBALS["template_error_line"] = ($offset > 0) ? substr_count($template, "\n", 0, $offset) : 0;

                                                //start new output buffer
                                                ob_start();

                                                //check for syntax errors
                                                if (@eval("return TRUE; $buffer") === FALSE)
                                                {
                                                        //get error information
                                                        $error = error_get_last();

                                                        //display error
                                                        template_error($error["type"], $error["message"], NULL, $error["line"]);
                                                }

                                                //execute script
                                                else
                                                {
                                                        //execute script
                                                        $result = $this->execute($buffer, $document, $base_id, $_base_id, end($temp_stack));
                                                };

                                                //append function output
                                                $output .= ob_get_clean();
                                        };
                                };

                                //continue after php code
                                $offset = $_offset;

                                //skip processing of remaining template if script returns false explicitly
                                if ($result === FALSE) $offset = $_offset = $length;
                        }

//.....................................................................................................

                        //start tag
                        elseif (
                                 ($template[$offset + 1] === "C") //fast check of initial tag characters
                                 &&
                                 ($template[$offset + 2] === "M")
                                 &&
                                 ($template[$offset + 3] === "S")
                                 &&
                                 ($template[$offset + 4] === ":")
                               )
                        {
                                if (($_offset = strpos($template, ">", $offset + 5)) !== FALSE) //seek tag end
                                {
                                        $_offset++; //advance position after tag
                                        $string = substr($template, $offset, $_offset - $offset); //get tag code

                                        //verify tag
                                        if (
                                             preg_match("/^<CMS:([a-z]+)(?:\s|\/|>)/", $string, $_string) //extensive check
                                             &&
                                             isset($this->tlist[$_string[1]]) //check if element is listed
                                           )
                                        {
                                                //skip processing within nocache block
                                                if ($nocache_depth)
                                                {
                                                        $output .= $string;  //append unparsed element code to output
                                                        $offset  = $_offset; //continue after tag
                                                        continue;            //skip
                                                };

                                                $buffer = NULL; //output buffer

                                                //template type
                                                $_ttype = $_string[1];           //literal
                                                $ttype  = $this->tlist[$_ttype]; //numeric

                                                //thumbnail equals image
                                                if ($ttype === CMS_TEMPLATE_TYPE_THUMBNAIL) $_ttype = "image";

                                                //extract attributes
                                                if (preg_match_all("/([^\s\"'>\/=]+)\s*(?:=\s*(?:\"([^\"]*)|\'([^']*)|([^\s>]*)))?/", $string, $_string))
                                                {
                                                        $attribute = array();
                                                        for ($i = 1, $c = count($_string[0]); $i < $c; $i++)
                                                                $attribute[utf8_strtolower($_string[1][$i])] =
                                                                        htmlentities_decode($_string[2][$i] . $_string[3][$i] . $_string[4][$i]);
                                                };

                                                //id

                                                /*
                                                $id  //source index (pointing to content)
                                                $_id //inherited index (new base for embedded elements)
                                                */

                                                while (TRUE)
                                                {
                                                        //extract id
                                                        while (isset($attribute["id"]))
                                                        {
                                                                //process relative automatic indexes
                                                                $_string = preg_replace_callback(
                                                                        "/(?<=^|\.)(\+|-)([0-9]+)(?=$|\.)/",
                                                                        function($a) use (&$auto_id) { return $auto_id + (($a[1] === "+") ? $a[2] : -$a[2]); },
                                                                        $attribute["id"]);

                                                                //verify id
                                                                if (! preg_match("/^(\.{0,2})([0-9a-zA-Z_\.]+?)(\.?)$/", $_string, $_string)) break;

                                                                $id = $_string[2];

                                                                //process modifiers
                                                                switch ($_string[1])
                                                                {
                                                                case ".": //relative to base
                                                                        if ($_string[3] === ".")
                                                                        {
                                                                                $id  = $base_id . $id;
                                                                                $_id = $id;
                                                                        }
                                                                        else
                                                                        {
                                                                                $_id = $_base_id . $id;
                                                                                $id  = $base_id . $id;
                                                                        };
                                                                        break;

                                                                case "..": //absolute
                                                                        $_id = ($_string[3] === ".") ?
                                                                               $id : $_base_id . $id;
                                                                        break;

                                                                default: //relative
                                                                        $id  = $_base_id . $id;
                                                                        $_id = $id;
                                                                        break;
                                                                };

                                                                break 2;
                                                        };

                                                        //relative using automatic index
                                                        $id  = $_base_id . $auto_id;
                                                        $_id = $id;
                                                        break;
                                                };

                                                if (CMS_TEMPLATE_TYPE_EDIT & $ttype) //element is editable
                                                {
                                                        //edit option
                                                        $edit    = (! isset($attribute["edit"])) ||
                                                                   (! preg_match("/^(?:off|no|false)$/i", $attribute["edit"]));

                                                        //element title
                                                        $_title  = isset($attribute["title"]) ?
                                                                   $attribute["title"] : $this->tname[$ttype];

                                                        //content value
                                                        $_cvalue = $document->get($id, $_ttype);
                                                        $cvalue  = stre($_cvalue) ? NULL : l($_cvalue);

                                                        if (stre($cvalue) && isset($attribute["default"]))
                                                                $cvalue = $_cvalue = $attribute["default"];
                                                }

                                                else //not an editable element
                                                {
                                                        $edit    = FALSE; //edit option
                                                        $_title  = NULL;  //element title
                                                        $_cvalue = NULL;  //raw content value
                                                        $cvalue  = NULL;  //content value
                                                };

                                                //close open tags and display editing functions when in edit mode
                                                if (

                                                //edit mode
                                                     CMS_TEMPLATE_EDIT
                                                     &&

                                                //element is editable
                                                     (CMS_TEMPLATE_TYPE_EDIT & $ttype)
                                                     &&

                                                //element is not suppressed
                                                     (! ($cblock_limit || $calt_limit))
                                                     &&

                                                //there is an action defined for the current element type
                                                     isset($this->action[CMS_TEMPLATE_ACTION][$ttype][CMS_TEMPLATE_CODE])
                                                   )
                                                {
                                                        //close open href tags
                                                        if (isset($span_stack[CMS_TEMPLATE_TYPE_HREF]))
                                                        {
                                                                while (array_pop($span_stack[CMS_TEMPLATE_TYPE_HREF]))

                                                                        //display button to activate link
                                                                        $output .= "<img src=\"" . x(CMS_IMAGES_URL) . "content/button_open.png\" " .
                                                                                        "alt=\"*\" " .
                                                                                        "title=\"" . CMS_L_TEMPLATE_019 . "\">" .
                                                                                   "</a>";
                                                        };

                                                        //close open download tags
                                                        if (isset($span_stack[CMS_TEMPLATE_TYPE_DOWNLOAD]))
                                                        {
                                                                while (array_pop($span_stack[CMS_TEMPLATE_TYPE_DOWNLOAD]))

                                                                        //display button to activate link
                                                                        $output .= "<img src=\"" . x(CMS_IMAGES_URL) . "content/button_open.png\" " .
                                                                                        "alt=\"*\" " .
                                                                                        "title=\"" . CMS_L_TEMPLATE_019 . "\">" .
                                                                                   "</a>";
                                                        };

                                                        if ($edit) $index++; //advance dataset index
                                                };

                                                //memorize spannable element as open
                                                if (CMS_TEMPLATE_TYPE_SPAN & $ttype)
                                                {
                                                        if (! isset($span_stack[$ttype])) $span_stack[$ttype] = array();
                                                        array_push($span_stack[$ttype], TRUE); //add to span stack
                                                };

                                                //process element by type
                                                switch ($ttype)
                                                {
                                                case CMS_TEMPLATE_TYPE_HEAD:
//.....................................................................................................

                                                        /*
                                                                <CMS:head [base="{url}"]
                                                                          [stylesheet="on|yes|true|off|no|false|{url}"]
                                                                          [title="{text}"]/>
                                                        */

                                                        //capture title
                                                        $_title       = isset($attribute["title"]) ?

                                                        //if given use preset title that can embed the document title
                                                                        sprintf($attribute["title"], l($title)) :

                                                        //else use document title only
                                                                        l($title);

                                                        //description
                                                        $_description = l($description);
                                                        $_description = remove_format($_description);
                                                        $_description = stripspaces($_description);

                                                        //keyword
                                                        $_keyword     = l($keyword);
                                                        $_keyword     = preg_split("/[\p{Po}\p{Z}]+/u", $_keyword, -1, PREG_SPLIT_NO_EMPTY);
                                                        $_keyword     = array_unique($_keyword);
                                                        $_keyword     = implode(", ", $_keyword);

                                                        //html meta data

                                                        //document title
                                                        $buffer  .= "<title>" .
                                                                    x($_title) .
                                                                    "</title>" .

                                                        //short summary
                                                                    "\n<meta name=\"description\" " .
                                                                            "content=\"" . x($_description) . "\">" .

                                                        //related keywords
                                                                    "\n<meta name=\"keywords\" " .
                                                                            "content=\"" . x($_keyword) . "\">" .

                                                        //generating software
                                                                    "\n<meta name=\"generator\" " .
                                                                            "content=\"" . x(CMS) . "\">" .

                                                        //character set
                                                                    "\n<meta http-equiv=\"content-type\" " .
                                                                            "content=\"text/html; charset=utf-8\">";

                                                        //base url
                                                        if (
                                                             isset($attribute["base"])
                                                             &&
                                                             (! preg_match("/^(?:on|yes|true)$/i", $attribute["base"]))
                                                           )
                                                        {
                                                                //use custom base url if given
                                                                if (! preg_match("/^(?:off|no|false)$/i", $attribute["base"]))
                                                                        $buffer .= "\n<base href=\"" . x($attribute["base"]) . "\">";
                                                        }

                                                        //use default base url
                                                        else
                                                        {
                                                                $buffer .= "\n<base href=\"" . x(CMS_ROOT_URL) . "\">";
                                                        };

                                                        //directory entry
                                                        if (defined("CMS_CONTENT_DIRECTORY_INDEX") && (CMS_CONTENT_DIRECTORY_INDEX !== 0))
                                                        {
                                                                //canonical
                                                                $language = (CMS_LANGUAGE != "") ? CMS_LANGUAGE . "." : "";
                                                                $map      = new map("#system/" . $language . "directory.canonical");
                                                                $key      = CMS_CONTENT_DIRECTORY_INDEX;

                                                                if (($url = $map->get_value($key)) !== NULL)
                                                                        $buffer .= "\n<link rel=\"canonical\" " .
                                                                                           "href=\"" . x($url) . "\">";

                                                                //alternate
                                                                $language = explode(",", CMS_LANGUAGE_ENABLED);

                                                                reset($language);
                                                                while (list(, $value) = each($language))
                                                                {
                                                                        $map = new map("#system/$value.directory");

                                                                        if (($url = $map->get_value(CMS_CONTENT_DIRECTORY_INDEX)) !== NULL)
                                                                        {
                                                                                $url     = CMS_ROOT_URL . cms_path_urlencode($url);
                                                                                $buffer .= "\n<link rel=\"alternate\" " .
                                                                                                   "href=\"" . x($url) . "\" " .
                                                                                                   "hreflang=\"" . x($value) . "\">";
                                                                        };
                                                                };
                                                        }

                                                        //no directory entry
                                                        elseif (defined("CMS_CONTENT_INDEX"))
                                                        {
                                                                //canonical
                                                                if (cms_load("directory"))
                                                                {
                                                                        $language  = (strpos(CMS_LANGUAGE_ENABLED, ",") === FALSE) ? NULL : CMS_LANGUAGE;
                                                                        $url       = substr(CMS_MODULES_URL, strlen(CMS_ROOT_URL)) . "content.php";
                                                                        $_url      = absolute_path(directory_get_canonical(), $url) .
                                                                                     querystring(array("content_index"       => CMS_CONTENT_INDEX,
                                                                                                       "cms_select_language" => $language));
                                                                        $buffer   .= "\n<link rel=\"canonical\" " .
                                                                                             "href=\"" . x($_url) . "\">";
                                                                };

                                                                //alternate
                                                                $language = explode(",", CMS_LANGUAGE_ENABLED);

                                                                reset($language);
                                                                while (list(, $value) = each($language))
                                                                {
                                                                        $url     = CMS_MODULES_URL . "content.php" .
                                                                                   querystring(array("content_index"       => CMS_CONTENT_INDEX,
                                                                                                     "cms_select_language" => $value));
                                                                        $buffer .= "\n<link rel=\"alternate\" " .
                                                                                           "href=\"" . x($url) . "\" " .
                                                                                           "hreflang=\"" . x($value) . "\">";
                                                                };
                                                        };

                                                        //stylesheet
                                                        if (
                                                             isset($attribute["stylesheet"])
                                                             &&
                                                             (! preg_match("/^(?:on|yes|true)$/i", $attribute["stylesheet"]))
                                                           )
                                                        {
                                                                //use custom stylesheet if given
                                                                if (! preg_match("/^(?:off|no|false)$/i", $attribute["stylesheet"]))
                                                                        $buffer .= "\n" . stylesheet($attribute["stylesheet"], FALSE);
                                                        }

                                                        //use default stylesheet
                                                        else
                                                        {
                                                                $buffer .= "\n" . CMS_STYLESHEET;
                                                        };

                                                        //special template stylesheet
                                                        if (CMS_TEMPLATE_EDIT)
                                                                $buffer .= "\n" . stylesheet(CMS_URL . "stylesheet/template.css");

                                                        //additional header information
                                                        if ($header) $buffer .= "\n$header";

                                                        //default javascript libraries
                                                        $_buffer  = x(CMS_JAVASCRIPT_URL);
                                                        $buffer  .= "\n<script src=\"" . $_buffer . "common.js\">" .
                                                                    "</script>" .
                                                                    "\n<script src=\"" . $_buffer . "fx.js\">" .
                                                                    "</script>" .
                                                                    "\n<script>" .
                                                                    "load_script(\"" . q(CMS_JAVASCRIPT_URL) . "defer.js\", " .
                                                                    "function() { defer_init(); });" .
                                                                    "</script>" .
                                                                    "<noscript>" .
                                                                    "<style>" .
                                                                    "IMG[data-defer-src]" .
                                                                    "{" .
                                                                    "DISPLAY: none !important;" .
                                                                    "}" .
                                                                    "</style>" .
                                                                    "</noscript>";

                                                        //javascript libraries required for editing
                                                        if (CMS_TEMPLATE_EDIT && (! $lock))
                                                        {
                                                                $buffer .= "\n<script src=\"" . $_buffer . "dragdrop.js\">" .
                                                                           "</script>" .
                                                                           "\n<script src=\"" . $_buffer . "template.js\">" .
                                                                           "</script>" .

                                                                //dragdrop action
                                                                           "<script>" .

                                                                           "function tp_dd_event(event, source, target)" .
                                                                           "{" .
                                                                           "switch (event)" .
                                                                           "{" .

                                                                //element is doubleclicked
                                                                           "case \"dblclick\":" .
                                                                           "var id     = source.getAttribute(\"data-tp-id\");" .
                                                                           "var object = document.getElementById(\"tp-edt-a-\" + id);" .
                                                                           "if (object == null) break;" .
                                                                           "location.replace(object.href);" .
                                                                           "break;" .

                                                                //element is about to be dragged
                                                                           "case \"beforedragstart\":" .
                                                                           "tp_beforedragstart();" .
                                                                           "break;";

                                                                //element is dropped on target
                                                                if (isset($this->action[CMS_TEMPLATE_COMMAND][CMS_TEMPLATE_COMMAND_DRAGDROP1]))
                                                                {
                                                                        $buffer .=
                                                                           "case \"dropon\":" .
                                                                           "var code = \"" . q($this->action[CMS_TEMPLATE_COMMAND][CMS_TEMPLATE_COMMAND_DRAGDROP1]) . "\";";
                                                                        if (isset($this->action[CMS_TEMPLATE_COMMAND][CMS_TEMPLATE_COMMAND_DRAGDROP2])) $buffer .=
                                                                           "if (dd_flag) code = \"" . q($this->action[CMS_TEMPLATE_COMMAND][CMS_TEMPLATE_COMMAND_DRAGDROP2]) . "\";";
                                                                        $buffer .=
                                                                           "code     = code.replace(/%index%/g,     encodeURIComponent(source.getAttribute(\"data-tp-index\")))" .
                                                                                          ".replace(/%path%/g,      encodeURIComponent(source.getAttribute(\"data-tp-path\")))" .
                                                                                          ".replace(/%value%/g,     encodeURIComponent(target.getAttribute(\"data-tp-path\")))" .
                                                                                          ".replace(/%type%/g,      encodeURIComponent(source.getAttribute(\"data-tp-type\")))" .
                                                                                          ".replace(/%reference%/g, encodeURIComponent(source.getAttribute(\"data-tp-reference\")))" .
                                                                                          ".replace(/%id%/g,        encodeURIComponent(target.getAttribute(\"data-tp-id\")));" .
                                                                           "location.replace(code);" .
                                                                           "break;";
                                                                };
                                                                $buffer .= "};" .
                                                                           "};" .

                                                                //register event callback function
                                                                            "dd_set_callback(tp_dd_event);";

                                                                $buffer .= "</script>";
                                                        };

                                                        break;

                                                case CMS_TEMPLATE_TYPE_CONTROL:
//.....................................................................................................

                                                        /*
                                                                <CMS:control/>
                                                        */

                                                        //document is to be displayed as read only
                                                        if ($lock)
                                                        {
                                                                $buffer .= template_lock();
                                                                break;
                                                        };

                                                        //skip if not in edit mode
                                                        if (! CMS_TEMPLATE_EDIT) break;

                                                        //no control functions defined
                                                        if (! isset($this->action[CMS_TEMPLATE_CONTROL])) break;

                                                        $i = 0;

                                                        //display document index
                                                        if (defined("CMS_CONTENT_INDEX"))
                                                        {
                                                                $query     = "SELECT " . CMS_DB_CONTENT_WRITER_TIME . ", " .
                                                                                         CMS_DB_CONTENT_TIME . " " .
                                                                             "FROM " .   CMS_DB_CONTENT . " " .
                                                                             "WHERE " .  CMS_DB_CONTENT_INDEX . "='" . sqlesc(CMS_CONTENT_INDEX) . "' " .
                                                                             "LIMIT 1";
                                                                $result    = mysql_query($query);
                                                                $resultrow = mysql_fetch_row($result);

                                                                if ($resultrow[0] === $resultrow[1])
                                                                    $buffer .= "<div id=\"tp-marker\">" .
                                                                               CMS_L_TEMPLATE_014 . ": " . x(CMS_CONTENT_INDEX) .
                                                                               "</div>";
                                                                else
                                                                    $buffer .= "<div id=\"tp-marker\" " .
                                                                                    "class=\"tp-mod\">" .
                                                                               CMS_L_TEMPLATE_014 . ": " . x(CMS_CONTENT_INDEX) . " " .
                                                                               "(" . CMS_L_TEMPLATE_049 . " " . x(friendly_date($resultrow[0])) . ")" .
                                                                               "</div>";
                                                        }
                                                        else
                                                        {
                                                                //output marker
                                                                $buffer .= "<div id=\"tp-marker\">";
                                                                           CMS_L_TEMPLATE_048 .
                                                                            "</div>";
                                                        };

                                                        //output control pad
                                                        $buffer .= "<div id=\"tp-ctrl\">";

                                                        //display element display/hide selector
                                                        if (isset($this->action[CMS_TEMPLATE_CONTROL][CMS_TEMPLATE_SWITCH][CMS_TEMPLATE_CODE]))
                                                        {
                                                                $buffer .= "<form name=\"tp_ctrl_opt\" " .
                                                                                 "action=\"#\">" .
                                                                           "<div id=\"tp-ctrl-opt-switch\">" .

                                                                //text
                                                                           "<a href=\"javascript:tp_ctrl_opt_switch(" . CMS_TEMPLATE_OPTION_TEXT . ");\">" .
                                                                           "<img id=\"tp-ctrl-opt-img-0\" " .
                                                                                "src=\"" . x(CMS_IMAGES_URL) . "content/button_text" .
                                                                                ((CMS_TEMPLATE_OPTION & CMS_TEMPLATE_OPTION_TEXT)       ? "" : "_disabled") . ".png\" " .
                                                                                "alt=\"*\" " .
                                                                                "title=\"" . CMS_L_TEMPLATE_007 . "\">" .
                                                                           "</a>" .

                                                                //value
                                                                           "<a href=\"javascript:tp_ctrl_opt_switch(" . CMS_TEMPLATE_OPTION_VALUE . ");\">" .
                                                                           "<img id=\"tp-ctrl-opt-img-1\" " .
                                                                                "src=\"" . x(CMS_IMAGES_URL) . "content/button_value" .
                                                                                ((CMS_TEMPLATE_OPTION & CMS_TEMPLATE_OPTION_VALUE)      ? "" : "_disabled") . ".png\" " .
                                                                                "alt=\"*\" " .
                                                                                "title=\"" . CMS_L_TEMPLATE_044 . "\">" .
                                                                           "</a>" .

                                                                //image
                                                                           "<a href=\"javascript:tp_ctrl_opt_switch(" . CMS_TEMPLATE_OPTION_IMAGE . ");\">" .
                                                                           "<img id=\"tp-ctrl-opt-img-2\" " .
                                                                                "src=\"" . x(CMS_IMAGES_URL) . "content/button_image" .
                                                                                ((CMS_TEMPLATE_OPTION & CMS_TEMPLATE_OPTION_IMAGE)      ? "" : "_disabled") . ".png\" " .
                                                                                "alt=\"*\" " .
                                                                                "title=\"" . CMS_L_TEMPLATE_009 . "\">" .
                                                                           "</a>" .

                                                                //thumbnail
                                                                           "<a href=\"javascript:tp_ctrl_opt_switch(" . CMS_TEMPLATE_OPTION_THUMBNAIL . ");\">" .
                                                                           "<img id=\"tp-ctrl-opt-img-3\" " .
                                                                                "src=\"" . x(CMS_IMAGES_URL) . "content/button_thumbnail" .
                                                                                ((CMS_TEMPLATE_OPTION & CMS_TEMPLATE_OPTION_THUMBNAIL)  ? "" : "_disabled") . ".png\" " .
                                                                                "alt=\"*\" " .
                                                                                "title=\"" . CMS_L_TEMPLATE_010 . "\">" .
                                                                           "</a>" .

                                                                //media
                                                                           "<a href=\"javascript:tp_ctrl_opt_switch(" . CMS_TEMPLATE_OPTION_MEDIA . ");\">" .
                                                                           "<img id=\"tp-ctrl-opt-img-4\" " .
                                                                                "src=\"" . x(CMS_IMAGES_URL) . "content/button_media" .
                                                                                ((CMS_TEMPLATE_OPTION & CMS_TEMPLATE_OPTION_MEDIA)      ? "" : "_disabled") . ".png\" " .
                                                                                "alt=\"*\" " .
                                                                                "title=\"" . CMS_L_TEMPLATE_021 . "\">" .
                                                                           "</a>" .

                                                                //href
                                                                           "<a href=\"javascript:tp_ctrl_opt_switch(" . CMS_TEMPLATE_OPTION_HREF . ");\">" .
                                                                           "<img id=\"tp-ctrl-opt-img-5\" " .
                                                                                "src=\"" . x(CMS_IMAGES_URL) . "content/button_href" .
                                                                                ((CMS_TEMPLATE_OPTION & CMS_TEMPLATE_OPTION_HREF)       ? "" : "_disabled") . ".png\" " .
                                                                                "alt=\"*\" " .
                                                                                "title=\"" . CMS_L_TEMPLATE_005 . "\">" .
                                                                           "</a>" .

                                                                //download
                                                                           "<a href=\"javascript:tp_ctrl_opt_switch(" . CMS_TEMPLATE_OPTION_DOWNLOAD . ");\">" .
                                                                           "<img id=\"tp-ctrl-opt-img-6\" " .
                                                                                "src=\"" . x(CMS_IMAGES_URL) . "content/button_download" .
                                                                                ((CMS_TEMPLATE_OPTION & CMS_TEMPLATE_OPTION_DOWNLOAD)   ? "" : "_disabled") . ".png\" " .
                                                                                "alt=\"*\" " .
                                                                                "title=\"" . CMS_L_TEMPLATE_008 . "\">" .
                                                                           "</a>" .

                                                                //plugin
                                                                           "<a href=\"javascript:tp_ctrl_opt_switch(" . CMS_TEMPLATE_OPTION_PLUGIN . ");\">" .
                                                                           "<img id=\"tp-ctrl-opt-img-7\" " .
                                                                                "src=\"" . x(CMS_IMAGES_URL) . "content/button_plugin" .
                                                                                ((CMS_TEMPLATE_OPTION & CMS_TEMPLATE_OPTION_PLUGIN)     ? "" : "_disabled") . ".png\" " .
                                                                                "alt=\"*\" " .
                                                                                "title=\"" . CMS_L_TEMPLATE_006 . "\">" .
                                                                           "</a>" .

                                                                //template
                                                                           "<a href=\"javascript:tp_ctrl_opt_switch(" . CMS_TEMPLATE_OPTION_TEMPLATE . ");\">" .
                                                                           "<img id=\"tp-ctrl-opt-img-8\" " .
                                                                                "src=\"" . x(CMS_IMAGES_URL) . "content/button_template" .
                                                                                ((CMS_TEMPLATE_OPTION & CMS_TEMPLATE_OPTION_TEMPLATE)   ? "" : "_disabled") . ".png\" " .
                                                                                "alt=\"*\" " .
                                                                                "title=\"" . CMS_L_TEMPLATE_011 . "\">" .
                                                                           "</a>" .

                                                                //group
                                                                           "<a href=\"javascript:tp_ctrl_opt_switch(" . CMS_TEMPLATE_OPTION_GROUP . ");\">" .
                                                                           "<img id=\"tp-ctrl-opt-img-9\" " .
                                                                                "src=\"" . x(CMS_IMAGES_URL) . "content/button_group" .
                                                                                ((CMS_TEMPLATE_OPTION & CMS_TEMPLATE_OPTION_GROUP)      ? "" : "_disabled") . ".png\" " .
                                                                                "alt=\"*\" " .
                                                                                "title=\"" . CMS_L_TEMPLATE_028 . "\">" .
                                                                           "</a>" .

                                                                //repeat
                                                                           "<a href=\"javascript:tp_ctrl_opt_switch(" . CMS_TEMPLATE_OPTION_REPEAT . ");\">" .
                                                                           "<img id=\"tp-ctrl-opt-img-10\" " .
                                                                                "src=\"" . x(CMS_IMAGES_URL) . "content/button_repeat" .
                                                                                ((CMS_TEMPLATE_OPTION & CMS_TEMPLATE_OPTION_REPEAT)     ? "" : "_disabled") . ".png\" " .
                                                                                "alt=\"*\" " .
                                                                                "title=\"" . CMS_L_TEMPLATE_020 . "\">" .
                                                                           "</a>" .

                                                                //shift
                                                                           "<a href=\"javascript:tp_ctrl_opt_switch(" . CMS_TEMPLATE_OPTION_SHIFT . ");\">" .
                                                                           "<img id=\"tp-ctrl-opt-img-11\" " .
                                                                                "src=\"" . x(CMS_IMAGES_URL) . "content/button_shift" .
                                                                                ((CMS_TEMPLATE_OPTION & CMS_TEMPLATE_OPTION_SHIFT)      ? "" : "_disabled") . ".png\" " .
                                                                                "alt=\"*\" " .
                                                                                "title=\"" . CMS_L_TEMPLATE_022 . "\">" .
                                                                           "</a>" .

                                                                //cblock
                                                                           "<a href=\"javascript:tp_ctrl_opt_switch(" . CMS_TEMPLATE_OPTION_CBLOCK . ");\">" .
                                                                           "<img id=\"tp-ctrl-opt-img-13\" " .
                                                                                "src=\"" . x(CMS_IMAGES_URL) . "content/button_cblock" .
                                                                                ((CMS_TEMPLATE_OPTION & CMS_TEMPLATE_OPTION_CBLOCK)     ? "" : "_disabled") . ".png\" " .
                                                                                "alt=\"*\" " .
                                                                                "title=\"" . CMS_L_TEMPLATE_004 . "\">" .
                                                                           "</a>" .

                                                                //calt
                                                                           "<a href=\"javascript:tp_ctrl_opt_switch(" . CMS_TEMPLATE_OPTION_CALT . ");\">" .
                                                                           "<img id=\"tp-ctrl-opt-img-12\" " .
                                                                                "src=\"" . x(CMS_IMAGES_URL) . "content/button_calt" .
                                                                                ((CMS_TEMPLATE_OPTION & CMS_TEMPLATE_OPTION_CALT) ? "" : "_disabled") . ".png\" " .
                                                                                "alt=\"*\" " .
                                                                                "title=\"" . CMS_L_TEMPLATE_023 . "\">" .
                                                                           "</a>" .

                                                                //debug
                                                                           "<a href=\"javascript:tp_ctrl_opt_switch(" . CMS_TEMPLATE_OPTION_DEBUG . ");\">" .
                                                                           "<img id=\"tp-ctrl-opt-img-14\" " .
                                                                                "src=\"" . x(CMS_IMAGES_URL) . "content/button_debug" .
                                                                                ($debug                                                 ? "" : "_disabled") . ".png\" " .
                                                                                "alt=\"*\" " .
                                                                                "title=\"" . CMS_L_TEMPLATE_047 . "\">" .
                                                                           "</a>" .

                                                                //apply
                                                                           "<a id=\"tp-ctrl-opt-apply\" " .
                                                                              "href=\"" . x($this->action[CMS_TEMPLATE_CONTROL][CMS_TEMPLATE_SWITCH][CMS_TEMPLATE_CODE]) . "\">" .
                                                                           CMS_L_COMMAND_CONFIRM .
                                                                           "</a>" .
                                                                           "</div>" .

                                                                //presets
                                                                           "<div id=\"tp-ctrl-opt-set\">" .

                                                                //all
                                                                           "<a href=\"javascript:tp_ctrl_opt_set(" . CMS_TEMPLATE_OPTION_ALL . ");\" " .
                                                                              "accesskey=\"a\">" .
                                                                           image("template/command_view_full") . " " . CMS_L_TEMPLATE_012 . " [A]" .
                                                                           "</a>" .

                                                                //layout
                                                                           "<a href=\"javascript:tp_ctrl_opt_set(" . CMS_TEMPLATE_OPTION_LAYOUT . ");\" " .
                                                                              "accesskey=\"l\">" .
                                                                           image("template/command_view_layout") . " " . CMS_L_TEMPLATE_015 . " [L]" .
                                                                           "</a>" .

                                                                //edit
                                                                           "<a href=\"javascript:tp_ctrl_opt_set(" . CMS_TEMPLATE_OPTION_EDIT . ");\" " .
                                                                              "accesskey=\"c\">" .
                                                                           image("template/command_view_edit") . " " . CMS_L_TEMPLATE_016 . " [C]" .
                                                                           "</a>" .

                                                                //none
                                                                           "<a href=\"javascript:tp_ctrl_opt_set(" . CMS_TEMPLATE_OPTION_NONE . ");\" " .
                                                                              "accesskey=\"v\">" .
                                                                           image("template/command_view") . " " .CMS_L_TEMPLATE_013 . " [V]" .
                                                                           "</a>" .
                                                                           "</div>" .
                                                                           "</form>" .

                                                                //set current selection
                                                                           jscript("tp_ctrl_opt_value = " . CMS_TEMPLATE_OPTION . ";" .

                                                                //set path to images
                                                                                   "tp_ctrl_opt_img_url = \"" . q(CMS_IMAGES_URL) . "content/\";");
                                                        };

                                                        //display extended commands
                                                        if (
                                                             isset($this->action[CMS_TEMPLATE_CONTROL][CMS_TEMPLATE_COMMAND])
                                                             &&
                                                             is_array($this->action[CMS_TEMPLATE_CONTROL][CMS_TEMPLATE_COMMAND])
                                                           )
                                                        {
                                                                $buffer .= "<div id=\"tp-ctrl-cmd\">";
                                                                $array   = $this->action[CMS_TEMPLATE_CONTROL][CMS_TEMPLATE_COMMAND];

                                                                //process commands and display as link
                                                                reset($array);
                                                                while (list($key, $value) = each($array))
                                                                        $buffer .= "<a href=\"" . x($value) . "\">" .
                                                                                   x($key) .
                                                                                   "</a>";

                                                                $buffer .= "</div>";
                                                        };

                                                        //display language selection
                                                        if (CMS_LANGUAGE_ENABLED)
                                                        {
                                                                $buffer   .= "<div id=\"tp-ctrl-language\">";
                                                                $language  = explode(",", CMS_LANGUAGE_ENABLED);

                                                                reset($language);
                                                                while (list(, $value) = each($language))
                                                                {
                                                                        $url     = CMS_ACTIVE_URL . cms_param(array("cms_select_language" => $value));
                                                                        $buffer .= "<a href=\"" . x($url) . "\"" .
                                                                                       (streq($value, CMS_LANGUAGE) ?
                                                                                     " class=\"tp-ctrl-language-active\">" : ">") .
                                                                                   image(is_file(CMS_DATA_PATH . "language/" . $value . "_small.gif") ?
                                                                                         CMS_DATA_URL . "language/" . r($value) . "_small.gif" :
                                                                                         "language/default_small",
                                                                                         16, 12, $value) .
                                                                                   "</a>";
                                                                };

                                                                $buffer .= "</div>";
                                                        };

                                                        //display permission editor
                                                        if (defined("CMS_CONTENT_INDEX"))
                                                                $buffer .= "<div id=\"tp-ctrl-permission\">" .
                                                                           permission(array(CMS_CONTENT_PERMISSION_READER . "." . CMS_CONTENT_INDEX => CMS_L_READ,
                                                                                            CMS_CONTENT_PERMISSION_WRITER . "." . CMS_CONTENT_INDEX => CMS_L_WRITE),
                                                                                            TRUE) .
                                                                           "</div>";
                                                        $buffer .= "</div>";

                                                        break;

                                                case CMS_TEMPLATE_TYPE_MENU:
//.....................................................................................................

                                                        /*
                                                                <CMS:menu [base="{id}"]
                                                                          [level="{number}|+{number}|-{number}"]
                                                                          [depth="off|no|false|{number}"]
                                                                          [filter="normal|open|active|active-open"]
                                                                          [show-icons="on|yes|true|off|no|false"]
                                                                          [show-images="on|yes|true|off|no|false"]
                                                                          [show-description="on|yes|true|off|no|false"]
                                                                          [exclude="{id}[ {id}]"]/>
                                                        */

                                                        if ($cblock_limit)      break; //suppressed by conditional block
                                                        if ($calt_limit)        break; //suppressed by alternative block
                                                        if (! cms_load("menu")) break; //library unavailable


                                                        $base             = isset($attribute["base"])    ? $attribute["base"]    : 0;
                                                        $level            = isset($attribute["level"])   ? $attribute["level"]   : 0;
                                                        $depth            = (isset($attribute["depth"])      &&
                                                                            (! preg_match("/^(?:off|no|false)$/i", $attribute["depth"])))
                                                                                                         ? $attribute["depth"]   : NULL;
                                                        $filter           = isset($attribute["filter"])  ? $attribute["filter"]  : NULL;
                                                        $show_icons       = isset($attribute["show-icons"])  &&
                                                                            preg_match("/^(?:on|yes|true)$/i", $attribute["show-icons"]);
                                                        $show_images      = isset($attribute["show-images"]) &&
                                                                            preg_match("/^(?:on|yes|true)$/i", $attribute["show-images"]);
                                                        $show_description = isset($attribute["show-description"]) &&
                                                                            preg_match("/^(?:on|yes|true)$/i", $attribute["show-description"]);
                                                        $exclude          = isset($attribute["exclude"]) ? $attribute["exclude"] : 0;

                                                        ob_start();
                                                        new menu(defined("CMS_CONTENT_DIRECTORY_INDEX") ? CMS_CONTENT_DIRECTORY_INDEX : 0,
                                                                 $base,
                                                                 $level,
                                                                 $depth,
                                                                 $filter,
                                                                 $show_icons,
                                                                 $show_images,
                                                                 $show_description,
                                                                 $exclude);
                                                        $buffer = ob_get_clean();

                                                        break;


                                                case CMS_TEMPLATE_TYPE_BACKLINK:
//.....................................................................................................

                                                        /*
                                                                <CMS:backlink/>
                                                        */

                                                        if ($cblock_limit)   break; //suppressed by conditional block
                                                        if ($calt_limit)     break; //suppressed by alternative block
                                                        if ($template_depth) break; //not the base template

                                                        //replace with backlink code
                                                        $buffer .= "<!-- backlink -->" .
                                                                   "<a href=\"http://" . x(CMS_HOMEPAGE) . "/\">" .
                                                                   x(CMS_SOFTWARE) .
                                                                   "</a>";

                                                        break;

                                                case CMS_TEMPLATE_TYPE_CBLOCK:
//.....................................................................................................

                                                        /*
                                                                <CMS:cblock [idref="+{number}|-{number}|{alphanumeric_.}"]>

                                                                ...

                                                                </CMS:cblock>
                                                        */

                                                        //do not apply if conditional blocks are flagged to be displayed
                                                        //debug mode overrides in order to retrieve idref
                                                        $flag = CMS_TEMPLATE_OPTION & CMS_TEMPLATE_OPTION_CBLOCK;
                                                        if ($flag && (! $debug)) break;

                                                        //idref

                                                        /*
                                                        $idref //source index (pointing to content)
                                                        */

                                                        while (TRUE)
                                                        {
                                                                //extract idref
                                                                while (isset($attribute["idref"]))
                                                                {
                                                                        //process relative automatic indexes
                                                                        $_string = preg_replace_callback(
                                                                                "/(?<=^|\.)(\+|-)([0-9]+)(?=$|\.)/",
                                                                                function($a) use (&$auto_id) { return $auto_id + (($a[1] === "+") ? $a[2] : -$a[2]); },
                                                                                $attribute["idref"]);

                                                                        //verify idref
                                                                        if (! preg_match("/^(\.{0,2})([0-9a-zA-Z_\.]+)$/", $_string, $_string)) break;

                                                                        //process modifiers
                                                                        switch ($_string[1])
                                                                        {
                                                                        case ".": //relative to base
                                                                                $idref = $base_id . $_string[2];
                                                                                break;

                                                                        case "..": //absolute
                                                                                $idref = $_string[2];
                                                                                break;

                                                                        default: //relative
                                                                                $idref = $_base_id . $_string[2];
                                                                                break;
                                                                        };

                                                                        break 2;
                                                                };

                                                                //relative using automatic index
                                                                $idref = $_base_id . $auto_id;
                                                                break;
                                                        };

                                                        //do not apply if conditional blocks are flagged to be displayed
                                                        if ($flag) break;

                                                        //increase block nesting count
                                                        $cblock_depth++;

                                                        //set suppression
                                                        if (

                                                        //not suppressed already
                                                             (! $cblock_limit)
                                                             &&

                                                        //referenced element has no value
                                                             stre(l($document->get($idref)))
                                                           )
                                                                $cblock_limit = $cblock_depth;

                                                        break;

                                                case CMS_TEMPLATE_TYPE_CALT:
//.....................................................................................................

                                                        /*
                                                                <CMS:calt [idref="+{number}|-{number}|{alphanumeric_.}"]>

                                                                ...

                                                                </CMS:calt>
                                                        */

                                                        //do not apply if alternative blocks are flagged to be displayed
                                                        //debug mode overrides in order to retrieve idref
                                                        $flag = CMS_TEMPLATE_OPTION & CMS_TEMPLATE_OPTION_CALT;
                                                        if ($flag && (! $debug)) break;

                                                        //idref

                                                        /*
                                                        $idref  //source index (pointing to content)
                                                        */

                                                        while (TRUE)
                                                        {
                                                                //extract idref
                                                                while (isset($attribute["idref"]))
                                                                {
                                                                        //process relative automatic indexes
                                                                        $_string = preg_replace_callback(
                                                                                "/(?<=^|\.)(\+|-)([0-9]+)(?=$|\.)/",
                                                                                function($a) use (&$auto_id) { return $auto_id + (($a[1] === "+") ? $a[2] : -$a[2]); },
                                                                                $attribute["idref"]);

                                                                        //verify idref
                                                                        if (! preg_match("/^(\.{0,2})([0-9a-zA-Z_\.]+)$/", $_string, $_string)) break;

                                                                        //process modifiers
                                                                        switch ($_string[1])
                                                                        {
                                                                        case ".": //relative to base
                                                                                $idref = $base_id . $_string[2];
                                                                                break;

                                                                        case "..": //absolute
                                                                                $idref = $_string[2];
                                                                                break;

                                                                        default: //relative
                                                                                $idref = $_base_id . $_string[2];
                                                                                break;
                                                                        };

                                                                        break 2;
                                                                };

                                                                //relative using automatic index
                                                                $idref = $_base_id . $auto_id;
                                                                break;
                                                        };

                                                        //do not apply if alternative blocks are flagged to be displayed
                                                        if ($flag) break;

                                                        //increase block nesting count
                                                        $calt_depth++;

                                                        //set suppression
                                                        if (

                                                        //not suppressed already
                                                             (! $calt_limit)
                                                             &&
                                                             (

                                                        //conditional blocks are flagged to be displayed
                                                               isset($this->action[CMS_TEMPLATE_ACTION][CMS_TEMPLATE_TYPE_CBLOCK])
                                                               ||

                                                        //or referenced element has value
                                                               nstre(l($document->get($idref)))
                                                             )
                                                           )
                                                                $calt_limit = $calt_depth;

                                                        break;

                                                case CMS_TEMPLATE_TYPE_GROUP:
//.....................................................................................................

                                                        /*
                                                                <CMS:group [id="+{number}|-{number}|{alphanumeric_.}"]
                                                                           [edit="on|yes|true|off|no|false"]
                                                                           [title="{text}"]>

                                                                ...

                                                                </CMS:group>
                                                        */

                                                        if ($cblock_limit) break; //suppressed by conditional block
                                                        if ($calt_limit)   break; //suppressed by alternative block

                                                        //put data on group stack
                                                        array_push($group_stack,
                                                                   array("base_id" => $_base_id, //backup base index
                                                                         "edit"    => $edit));   //backup edit option

                                                        $_base_id = "$_id."; //set base index

                                                        //determine if editing functions are used
                                                        $flag = CMS_TEMPLATE_EDIT &&
                                                                $edit &&
                                                                isset($this->action[CMS_TEMPLATE_ACTION][$ttype]);

                                                        if ($flag) //editing
                                                        {
                                                                //color map
                                                                //$color   = $this->depth_color[$group_depth % 6];
                                                                $buffer .= "<div id=\"tp-grp-$index\" " .
                                                                                "class=\"tp-grp\" " .
                                                                //                "style=\"BORDER-COLOR:" . x($color) . "\" " .
                                                                                "onmouseenter=\"tp_grp_dsp(this);\">";

                                                                //increase group nesting counter
                                                                $group_depth++;
                                                        };

                                                        break;

                                                case CMS_TEMPLATE_TYPE_BASE:
//.....................................................................................................

                                                        /*
                                                                <CMS:base [id="+{number}|-{number}|{alphanumeric_.}"]>

                                                                ...

                                                                </CMS:base>
                                                        */

                                                        if ($cblock_limit) break; //suppressed by conditional block
                                                        if ($calt_limit)   break; //suppressed by alternative block

                                                        //put data on base stack
                                                        array_push($base_stack,
                                                                   array("auto_id"  => $auto_id,    //backup automatic index
                                                                         "base_id"  => $base_id,    //backup base index
                                                                         "_base_id" => $_base_id)); //backup base index

                                                        $auto_id  = 0;       //set automatic index
                                                        $base_id  = "$_id."; //set base index
                                                        $_base_id = "$_id."; //set base index

                                                        //put data on temporary random index stack
                                                        $temp     = sprintf("%x", crc32($base_id . $index));
                                                        array_push($temp_stack, $temp);

                                                        break;

                                                case CMS_TEMPLATE_TYPE_NAMESPACE:
//.....................................................................................................

                                                        /*
                                                                <CMS:namespace [id="+{number}|-{number}|{alphanumeric_.}"]>

                                                                ...

                                                                </CMS:namespace>
                                                        */

                                                        if ($cblock_limit) break; //suppressed by conditional block
                                                        if ($calt_limit)   break; //suppressed by alternative block

                                                        //put data on namespace stack
                                                        array_push($namespace_stack, $_base_id); //backup base index

                                                        $_base_id = "$_id."; //set base index

                                                        break;

                                                case CMS_TEMPLATE_TYPE_REPEAT:
//.....................................................................................................

                                                        /*
                                                                <CMS:repeat [id="+{number}|-{number}|{alphanumeric_.}"]
                                                                            [default="{number}"]
                                                                            [edit="on|yes|true|off|no|false"]
                                                                            [title="{text}"]>

                                                                ...

                                                                </CMS:repeat>
                                                        */

                                                        //no suppression in order to provide automatic index consistency

                                                        //determine repeat count
                                                        $cvalue = (int)$cvalue;
                                                        if ($cvalue < 1) $cvalue = 1; //at least one repetition

                                                        //put data on repeat stack
                                                        array_push($repeat_stack,
                                                                   array("count"   => $cvalue,     //repeat count
                                                                         "offset"  => $_offset,    //repeat offset
                                                                         "base_id" => $_base_id)); //backup base index

                                                        $_base_id = "$_id."; //set base index

                                                        if (CMS_TEMPLATE_EDIT && CMS_TEMPLATE_OPTION && $edit)
                                                                $buffer .= "<div class=\"tp-value\">" .
                                                                           "<span>" .
                                                                           x(strabridge($_title, 10)) .
                                                                           "</span>" .
                                                                           (stre($cvalue) ? CMS_L_TEMPLATE_050 : x(strabridge($cvalue, 20))) .
                                                                           "</div>";

                                                        break;

                                                case CMS_TEMPLATE_TYPE_SHIFT:
//.....................................................................................................

                                                        /*
                                                                <CMS:shift [id="+{number}|-{number}|{alphanumeric_.}"]
                                                                           [default="{number}"]
                                                                           [edit="on|yes|true|off|no|false"]
                                                                           [title="{text}"]>

                                                                ...

                                                                </CMS:shift>
                                                        */

                                                        if ($cblock_limit) break; //suppressed by conditional block
                                                        if ($calt_limit)   break; //suppressed by alternative block

                                                        //determine shift count
                                                        $cvalue = (int)$cvalue;
                                                        if (! $cvalue) $cvalue = 1; //fallback value is one

                                                        //put data on shift stack
                                                        array_push($shift_stack, $_base_id); //backup base index

                                                        $_base_id = "$_id."; //set base index

                                                        break;

                                                case CMS_TEMPLATE_TYPE_TEMPLATE:
//.....................................................................................................

                                                        /*
                                                                <CMS:template [id="+{number}|-{number}|{alphanumeric_.}"]
                                                                              [default="{identifier}"]
                                                                              [edit="on|yes|true|off|no|false"]
                                                                              [title="{text}"]/>
                                                        */

                                                        //backup current dataset index
                                                        $_index = $index;

                                                        if (streq("$id.", $base_id)) break; //prevent infinite loop
                                                        if ($cblock_limit)           break; //suppressed by conditional block
                                                        if ($calt_limit)             break; //suppressed by alternative block
                                                        if (! $cvalue)               break; //no value

                                                        //determine if editing functions are used
                                                        $flag = CMS_TEMPLATE_EDIT &&
                                                                $edit &&
                                                                isset($this->action[CMS_TEMPLATE_ACTION][$ttype]);

                                                        if ($flag) //editing
                                                        {
                                                                //color map
                                                                //$color   = $this->depth_color[$group_depth % 6];
                                                                $buffer .= "<div id=\"tp-grp-$index\" " .
                                                                                "class=\"tp-grp\" " .
                                                                //                "style=\"BORDER-COLOR:" . x($color) . "\" " .
                                                                                "onmouseenter=\"tp_grp_dsp(this);\">";

                                                                //increase group nesting counter
                                                                $group_depth++;
                                                        };

                                                        if (! isset($template_list[$cvalue]))
                                                        {
                                                                //add to list of employed templates
                                                                $template_list[$cvalue] = TRUE;

                                                                //link stylesheet of employed template
                                                                if (($file = $this->get_stylesheet($cvalue, NULL, TRUE)) !== FALSE)
                                                                        $buffer .= stylesheet(CMS_DATA_URL . $file);
                                                        };

                                                        //recursively call parser function to process subtemplate
                                                        $buffer .= $this->_parse(
                                                                $document,                //document
                                                                $this->get_code($cvalue), //template
                                                                NULL,                     //title
                                                                NULL,                     //description
                                                                NULL,                     //keyword
                                                                NULL,                     //action
                                                                NULL,                     //header
                                                                $_id,                     //base id
                                                                $cache);                  //cache

                                                        if ($flag) //editing
                                                        {
                                                                $buffer .= "</div>";

                                                                //decrease group nesting counter
                                                                $group_depth--;
                                                        };

                                                        break;

                                                case CMS_TEMPLATE_TYPE_TEXT:
//.....................................................................................................

                                                        /*
                                                                <CMS:text [id="+{number}|-{number}|{alphanumeric_.}"]
                                                                          [default="{text}"]
                                                                          [edit="on|yes|true|off|no|false"]
                                                                          [title="{text}"]/>
                                                        */

                                                        if ($cblock_limit) break; //suppressed by conditional block
                                                        if ($calt_limit)   break; //suppressed by alternative block

                                                        //parse text if given
                                                        if (nstre($cvalue))        $buffer .= parse_text($cvalue, TRUE);

                                                        //or output element title when in edit mode
                                                        elseif (CMS_TEMPLATE_EDIT) $buffer .= x($_title);

                                                        break;

                                                case CMS_TEMPLATE_TYPE_VALUE:
//.....................................................................................................

                                                        /*
                                                                <CMS:value [id="+{number}|-{number}|{alphanumeric_.}"]
                                                                           [default="{text}"]
                                                                           [regex="{text}"]
                                                                           [edit="on|yes|true|off|no|false"]
                                                                           [title="{text}"]/>
                                                        */

                                                        if ($cblock_limit) break; //suppressed by conditional block
                                                        if ($calt_limit)   break; //suppressed by alternative block
                                                        if (! $edit)       break; //edit option disabled

                                                        if (CMS_TEMPLATE_EDIT && CMS_TEMPLATE_OPTION && $edit)
                                                                $buffer .= "<div class=\"tp-value\">" .
                                                                           "<span>" .
                                                                           x(strabridge($_title, 10)) .
                                                                           "</span>" .
                                                                           (stre($cvalue) ? CMS_L_TEMPLATE_050 : x(strabridge($cvalue, 20))) .
                                                                           "</div>";

                                                        break;

                                                case CMS_TEMPLATE_TYPE_HREF:
//.....................................................................................................

                                                        /*
                                                                <CMS:href [id="+{number}|-{number}|{alphanumeric_.}"]
                                                                          [_id="{text}"]
                                                                          [default="{url}"]
                                                                          [target="{identifier}"]
                                                                          [rel="{text}"]
                                                                          [edit="on|yes|true|off|no|false"]
                                                                          [title="{text}"]
                                                                          [class="{text}"]
                                                                          [style="{text}"]>

                                                                ...

                                                                </CMS:href>
                                                        */

                                                        if ($cblock_limit) break; //suppressed by conditional block
                                                        if ($calt_limit)   break; //suppressed by alternative block

                                                        //parse document value
                                                        if (preg_match("/^([^:]+):(.+)$/s", $cvalue, $_string))
                                                        {
                                                                $type  = $_string[1]; //type of reference
                                                                $value = $_string[2]; //address of reference

                                                                //process reference
                                                                switch ($type)
                                                                {
                                                                case "directory":
                                                                case "content":
                                                                        $value = translate_url("$type://$value");
                                                                        break;

                                                                default:
                                                                case "address":
                                                                        $value = translate_url($value);
                                                                        break;
                                                                };

                                                                //html hyperlink
                                                                $buffer .= "<a" . $this->attribute($attribute, array("href" => $value)) . ">";
                                                        }

                                                        //no or invalid value
                                                        else
                                                        {
                                                                //close element span
                                                                array_pop($span_stack[$ttype]);
                                                        };

                                                        break;

                                                case CMS_TEMPLATE_TYPE_IMAGE:
//.....................................................................................................

                                                        /*
                                                                <CMS:image [id="+{number}|-{number}|{alphanumeric_.}"]
                                                                           [_id="{text}"]
                                                                           [default="{url}"]
                                                                           [width="{width}[%]"]
                                                                           [height="{height}[%]"]
                                                                           [alt="{text}"]
                                                                           [edit="on|yes|true|off|no|false"]
                                                                           [title="{text}"]
                                                                           [class="{text}"]
                                                                           [style="{text}"]/>
                                                        */

                                                        if ($cblock_limit) break; //suppressed by conditional block
                                                        if ($calt_limit)   break; //suppressed by alternative block

                                                        if ($cvalue)
                                                        {
                                                                //process image address
                                                                $location         = translate_url($cvalue);

                                                                //prepare image (scaling, caching, ...)
                                                                $width            = isset($attribute["width"])  ? $attribute["width"]  : NULL;
                                                                $height           = isset($attribute["height"]) ? $attribute["height"] : NULL;
                                                                $_location        = image_process($location, $width, $height);
                                                                $attribute["src"] = $_location;

                                                                //generate alternative resolution sources
                                                                $resolution       = array(320, 640, 960, 1280); //resolution list
                                                                list($_width)     = getimagesize($location);    //original size
                                                                list($__width)    = getimagesize($_location);   //processed size
                                                                $srcset           = $_location . " " . $__width . "w,";

                                                                reset($resolution);
                                                                while (list(, $value) = each($resolution))
                                                                {
                                                                        if ($_width < $value)                                       break;    //original is too small
                                                                        if ($__width < $value)                                      break;    //exceeds target size
                                                                        if (($__width !== FALSE) && (abs($__width - $value) < 100)) continue; //already in range

                                                                        //prepare alternative resolution image (scaling, caching, ...)
                                                                        $__location  = image_process($location, $value);
                                                                        $srcset     .= x($__location . " " . $value . "w,");
                                                                };

                                                                //set srcset/sizes attribute
                                                                $attribute["srcset"] = substr($srcset, 0, -1);
                                                                if (! isset($attribute["sizes"]))  $attribute["sizes"]  = "(max-width:" . $__width . "px) 100vw," . $__width . "px";

                                                                //alt text
                                                                if (! isset($attribute["alt"]))    $attribute["alt"]    = l($this->image->data->get($cvalue, "name"));
                                                                if (! isset($attribute["_title"])) $attribute["_title"] = $attribute["alt"];

                                                                //html image reference fallback
                                                                $buffer .= "<noscript>" .
                                                                           "<img" . $this->attribute($attribute) . ">" .
                                                                           "</noscript>";

                                                                //html image reference lazy load
                                                                $attribute["data-defer-src"]    = $attribute["src"];
                                                                $attribute["src"]               = CMS_IMAGES_URL . "blank.png";
                                                                $attribute["data-defer-srcset"] = $attribute["srcset"];
                                                                unset($attribute["srcset"]);
                                                                $attribute["data-defer-sizes"]  = $attribute["sizes"];
                                                                unset($attribute["sizes"]);

                                                                $buffer .= "<img" . $this->attribute($attribute) . ">";
                                                        }

                                                        //no value
                                                        else
                                                        {
                                                                if (! CMS_TEMPLATE_OPTION) break; //output mode, no display

                                                                //prepare dummy display
                                                                $attribute["src"]   = CMS_IMAGES_URL . "blank.png";
                                                                if (! isset($attribute["width"]))  $attribute["width"]  = 150;
                                                                if (! isset($attribute["height"])) $attribute["height"] = 125;
                                                                $attribute["alt"]   = "";
                                                                $attribute["style"] = "BACKGROUND:#808080 url(" . qx(CMS_IMAGES_URL) . "template/no_image.png) center no-repeat";

                                                                //html image reference
                                                                $buffer .= "<img" . $this->attribute($attribute) . ">";
                                                        };

                                                        break;

                                                case CMS_TEMPLATE_TYPE_THUMBNAIL:
//.....................................................................................................

                                                        /*
                                                                <CMS:thumbnail [id="+{number}|-{number}|{alphanumeric_.}"]
                                                                               [_id="{text}"]
                                                                               [default="{url}"]
                                                                               [twidth="{width}[%]"]
                                                                               [theight="{height}[%]"]
                                                                               [width="{width}[%]"]
                                                                               [height="{height}[%]"]
                                                                               [alt="{text}"]
                                                                               [edit="on|yes|true|off|no|false"]
                                                                               [title="{text}"]
                                                                               [style="{text}"]/>
                                                        */

                                                        if ($cblock_limit) break; //suppressed by conditional block
                                                        if ($calt_limit)   break; //suppressed by alternative block

                                                        if ($cvalue)
                                                        {
                                                                //process image address
                                                                $location      = translate_url($cvalue);

                                                                //prepare thumbnail image
                                                                $twidth        = isset($attribute["twidth"])  ? $attribute["twidth"]  : NULL;
                                                                $theight       = isset($attribute["theight"]) ? $attribute["theight"] : NULL;
                                                                $tlocation     = image_process($location, $twidth, $theight);

                                                                //generate alternative resolution sources
                                                                $resolution    = array(320, 640, 960, 1280);      //resolution list
                                                                list($_width)  = $size = getimagesize($location); //original size
                                                                list($__width) = getimagesize($tlocation);        //processed size
                                                                $srcset        = $tlocation . " " . $__width . "w,";
                                                                $flag          = ! isset($attribute["data-size"]);

                                                                reset($resolution);
                                                                while (list(, $value) = each($resolution))
                                                                {
                                                                        if ($_width < $value)               break;    //original is too small
                                                                        if ($__width < $value)              break;    //exceeds target size
                                                                        if (($__width !== FALSE) &&
                                                                            (abs($__width - $value) < 100)) continue; //already in range

                                                                        //prepare alternative resolution image (scaling, caching, ...)
                                                                        $_location  = image_process($location, $value);
                                                                        $srcset    .= x($_location . " " . $value . "w,");
                                                                };

                                                                //set sizes attribute
                                                                $sizes         = isset($attribute["sizes"]) ?
                                                                                 $attribute["sizes"] :
                                                                                 "(max-width:" . $__width . "px) 100vw," . $__width . "px";

                                                                //prepare zoom image
                                                                $width         = isset($attribute["width"])   ? $attribute["width"]   : NULL;
                                                                $height        = isset($attribute["height"])  ? $attribute["height"]  : NULL;

                                                                if (

                                                                //width, height or both are given
                                                                     (((! $width) && $height) || preg_match("/^\d+$/", $width))
                                                                     &&
                                                                     (((! $height) && $width) || preg_match("/^\d+$/", $height))
                                                                   )
                                                                {
                                                                        //prepare zoom image (scaling, caching, ...)
                                                                        $location = image_process($location, $width, $height);
                                                                        if ($flag) $size = getimagesize($location);
                                                                };

                                                                //image size
                                                                if ($flag) $attribute["data-size"] = implode("x", array_slice($size, 0, 2));

                                                                //alt text
                                                                $alt           = isset($attribute["alt"]) ?
                                                                                 $attribute["alt"] :
                                                                                 l($this->image->data->get($cvalue, "name"));
                                                                if (! isset($attribute["_title"])) $attribute["_title"] = $alt;

                                                                //prepare thumbnail image attributes
                                                                $_attribute = array("src"    => $tlocation,
                                                                                    "srcset" => substr($srcset, 0, -1),
                                                                                    "sizes"  => $sizes,
                                                                                    "width"  => $twidth,
                                                                                    "height" => $theight,
                                                                                    "alt"    => $alt);

                                                                //html hyperlink to zoom image
                                                                $buffer .= "<a" . $this->attribute($attribute,
                                                                                                   array("href" => $location),
                                                                                                   array("twidth", "theight", "width", "height", "alt")) . ">" .

                                                                //html thumbnail image reference fallback
                                                                           "<noscript>" .
                                                                           "<img" . $this->attribute($_attribute) . ">" .
                                                                           "</noscript>";

                                                                //html thumbnail image reference lazy load
                                                                $_attribute["data-defer-src"]    = $_attribute["src"];
                                                                $_attribute["src"]               = CMS_IMAGES_URL . "blank.png";
                                                                $_attribute["data-defer-srcset"] = $_attribute["srcset"];
                                                                unset($_attribute["srcset"]);
                                                                $_attribute["data-defer-sizes"]  = $_attribute["sizes"];
                                                                unset($_attribute["sizes"]);

                                                                $buffer .= "<img" . $this->attribute($_attribute) . ">" .
                                                                           "</a>";
                                                        }

                                                        //no value
                                                        else
                                                        {
                                                                if (! CMS_TEMPLATE_OPTION) break; //output mode, no display

                                                                //prepare dummy display
                                                                $location   = NULL;
                                                                $tlocation  = CMS_IMAGES_URL . "blank.png";
                                                                $twidth     = isset($attribute["twidth"])  ? $attribute["twidth"]  : 150;
                                                                $theight    = isset($attribute["theight"]) ? $attribute["theight"] : 125;
                                                                $alt        = "";
                                                                $style      = "BACKGROUND:#808080 url(" . qx(CMS_IMAGES_URL) . "template/no_image.png) center no-repeat";

                                                                $_attribute = array("src"    => $tlocation,
                                                                                    "width"  => $twidth,
                                                                                    "height" => $theight,
                                                                                    "alt"    => $alt,
                                                                                    "style"  => $style);

                                                                //html hyperlink to zoom image
                                                                $buffer .= "<a" . $this->attribute($attribute,
                                                                                                   array("href" => $location),
                                                                                                   array("twidth", "theight", "width", "height", "alt")) . ">" .

                                                                //html thumbnail image reference
                                                                           "<img" . $this->attribute($_attribute) . ">" .
                                                                           "</a>";
                                                        };

                                                        break;

                                                case CMS_TEMPLATE_TYPE_MEDIA:
//.....................................................................................................

                                                        /*
                                                                <CMS:media [id="+{number}|-{number}|{alphanumeric_.}"]
                                                                           [_id="{text}"]
                                                                           [default="{url}"]
                                                                           [width="{width}[%]"]
                                                                           [height="{height}[%]"]
                                                                           [alt="{text}"]
                                                                           [edit="on|yes|true|off|no|false"]
                                                                           [title="{text}"]
                                                                           [class="{text}"]
                                                                           [style="{text}"]/>
                                                        */

                                                        if ($cblock_limit) break; //suppressed by conditional block
                                                        if ($calt_limit)   break; //suppressed by alternative block

                                                        //attributes
                                                        $__id   = isset($attribute["_id"])    ? x($attribute["_id"])    : NULL;
                                                        $width  = isset($attribute["width"])  ? x($attribute["width"])  : NULL;
                                                        $height = isset($attribute["height"]) ? x($attribute["height"]) : NULL;
                                                        $alt    = isset($attribute["alt"])    ? x($attribute["alt"])    : NULL;
                                                        $class  = isset($attribute["class"])  ? x($attribute["class"])  : NULL;
                                                        $style  = isset($attribute["style"])  ? x($attribute["style"])  : NULL;

                                                        if ($cvalue)
                                                        {
                                                                $location = $cvalue;

                                                                //alt text
                                                                if (! $alt) $alt = x(l($this->media->data->get($cvalue, "name")));
                                                        }

                                                        //no value
                                                        else
                                                        {
                                                                if (! CMS_TEMPLATE_OPTION) break; //output mode, no display

                                                                //prepare dummy display
                                                                $location = CMS_IMAGES_URL . "blank.png";
                                                                if (! $width)  $width  = 150;
                                                                if (! $height) $height = 125;
                                                                $alt      = $_title;
                                                                $style    = "BACKGROUND:#808080 url(" . qx(CMS_IMAGES_URL) . "template/no_media.png) center no-repeat";
                                                        };

                                                        //add linebreak if element action button is to be displayed
                                                        if (isset($this->action[CMS_TEMPLATE_ACTION][$ttype]))
                                                                $buffer .= "<br>";

                                                        //generate media display code
                                                        $buffer .= $this->media->parse(
                                                                $location, //address
                                                                $__id,     //id
                                                                $width,    //width
                                                                $height,   //height
                                                                $alt,      //alt
                                                                $alt,      //title
                                                                $class,    //class
                                                                $style);   //style

                                                        break;

                                                case CMS_TEMPLATE_TYPE_DOWNLOAD:
//.....................................................................................................

                                                        /*
                                                                <CMS:download [id="+{number}|-{number}|{alphanumeric_.}"]
                                                                              [_id="{text}"]
                                                                              [default="{identifier}"]
                                                                              [edit="on|yes|true|off|no|false"]
                                                                              [title="{text}"]
                                                                              [class="{text}"]
                                                                              [style="{text}"]>

                                                                ...

                                                                </CMS:download>
                                                        */

                                                        if ($cblock_limit) break; //suppressed by conditional block
                                                        if ($calt_limit)   break; //suppressed by alternative block

                                                        if ($cvalue)
                                                        {
                                                                //address of download module
                                                                $location  = CMS_MODULES_URL . "download.php" .
                                                                             "?download_index=" . r($cvalue) .
                                                                             "&download_start=1";

                                                                //alt text
                                                                if (! isset($attribute["_title"]))
                                                                        $attribute["_title"] = l($this->download->data->get($cvalue, "name"));

                                                                //html hyperlink to download
                                                                $buffer .= "<a" . $this->attribute($attribute, array("href" => $location)) . ">";
                                                        }

                                                        //no value
                                                        else
                                                        {
                                                                //close element span
                                                                array_pop($span_stack[$ttype]);
                                                        };

                                                        break;

                                                case CMS_TEMPLATE_TYPE_PLUGIN:
//.....................................................................................................

                                                        /*
                                                                <CMS:plugin [id="+{number}|-{number}|{alphanumeric_.}"]
                                                                            [default="{url}"]
                                                                            [edit="on|yes|true|off|no|false"]
                                                                            [title="{text}"]/>
                                                        */

                                                        if ($cblock_limit) break; //suppressed by conditional block
                                                        if ($calt_limit)   break; //suppressed by alternative block

                                                        //retrieve data from given address
                                                        if (nstre($cvalue))
                                                                $buffer .= template_read_plugin(translate_url($cvalue));

                                                        break;

                                                case CMS_TEMPLATE_TYPE_NOCACHE:
//.....................................................................................................

                                                        /*
                                                                <CMS:nocache>

                                                                ...

                                                                </CMS:nocache>
                                                        */

                                                        if (! $cache) break; //do not generate cache version

                                                        //if not within uncached span
                                                        if (! $nocache_depth++)

                                                                //output cache separator
                                                                $buffer .= CMS_TEMPLATE_CACHE_SEPARATOR .
                                                                           substr($base_id, 0, -1) .
                                                                           CMS_TEMPLATE_CACHE_SEPARATOR;

                                                        break;
                                                };

                                                //editing
                                                if (CMS_TEMPLATE_TYPE_EDIT & $ttype) //element is editable
                                                {
                                                        if (

                                                        //editing prepared, implies edit mode
                                                             CMS_TEMPLATE_EDIT
                                                             &&

                                                        //editing is enabled for current element
                                                             $edit
                                                             &&

                                                        //not suppressed
                                                             (! ($cblock_limit || $calt_limit))
                                                             &&

                                                        //there is an action defined for the current element type
                                                             isset($this->action[CMS_TEMPLATE_ACTION][$ttype][CMS_TEMPLATE_CODE])
                                                           )
                                                        {
                                                                $href = NULL;

                                                                //close open tags
                                                                if (

                                                                //element is href or download
                                                                     ((CMS_TEMPLATE_TYPE_HREF | CMS_TEMPLATE_TYPE_DOWNLOAD) & $ttype)
                                                                     &&

                                                                //element display code has been generated
                                                                     $buffer
                                                                   )
                                                                {
                                                                        $href   = $buffer;
                                                                        $buffer = NULL;

                                                                        //close element span
                                                                        array_pop($span_stack[$ttype]);
                                                                };

                                                                //set edit options
                                                                $edit_flag = CMS_TEMPLATE_COMMAND_BUFFER  |
                                                                             CMS_TEMPLATE_COMMAND_PASTE   |
                                                                             CMS_TEMPLATE_COMMAND_SWAP    |
                                                                             CMS_TEMPLATE_COMMAND_KICK1   |
                                                                             CMS_TEMPLATE_COMMAND_KICK2   |
                                                                             CMS_TEMPLATE_COMMAND_DROP1   |
                                                                             CMS_TEMPLATE_COMMAND_DROP2   |
                                                                             CMS_TEMPLATE_COMMAND_RELEASE |
                                                                             CMS_TEMPLATE_COMMAND_REFERENCE;

                                                                //value is set
                                                                if (nstre($_cvalue))
                                                                {
                                                                        //export
                                                                        if (

                                                                        //element is template
                                                                             ($ttype === CMS_TEMPLATE_TYPE_TEMPLATE)
                                                                             &&

                                                                        //value is set for current language
                                                                             nstre($cvalue)
                                                                           )
                                                                                $edit_flag |= CMS_TEMPLATE_COMMAND_EXPORT;

                                                                        //clear
                                                                        $edit_flag |= CMS_TEMPLATE_COMMAND_CLEAR;
                                                                };

                                                                //use backup index on template
                                                                if ($ttype !== CMS_TEMPLATE_TYPE_TEMPLATE)
                                                                        $_index = $index;

                                                                //put index on dataset index stack
                                                                $index_stack[] = $_index;

                                                                //collect url parameters
                                                                $_action = $this->action[CMS_TEMPLATE_ACTION][$ttype][CMS_TEMPLATE_CODE];
                                                                $replace = array("%index%"     => $id,
                                                                                 "%path%"      => $_id,
                                                                                 "%value%"     => (strpos($_action, "%value%") !== FALSE) ? $_cvalue : NULL,
                                                                                 "%type%"      => $_ttype,
                                                                                 "%reference%" => $document->get_reference($_id, $_ttype),
                                                                                 "%id%"        => $_index);
                                                                $__ttype = ((CMS_TEMPLATE_TYPE_IMAGE | CMS_TEMPLATE_TYPE_THUMBNAIL) & $ttype) ?
                                                                           CMS_TEMPLATE_TYPE_IMAGE | CMS_TEMPLATE_TYPE_THUMBNAIL : $ttype;
                                                                $image   = $this->action[CMS_TEMPLATE_ACTION][$ttype][CMS_TEMPLATE_IMAGE];
                                                                if (nstre($replace["%reference%"]))
                                                                {
                                                                        $image = preg_replace("/(\.[^\.]+)$/", "_reference$1", $image, 1);
                                                                        if ($replace["%reference%"] === TRUE) $replace["%reference%"] = "";
                                                                };

                                                                //create editing container and functions
                                                                $_buffer = $lock ?

                                                                //open dummy container
                                                                           "<div class=\"tp_edt\">" :

                                                                //open dragdrop container
                                                                           "<div id=\"tp-dd-" . x($_index) . "\" " .
                                                                                "title=\"" . CMS_L_TEMPLATE_045 . "\" " .
                                                                                "class=\"tp-dd" . (((CMS_TEMPLATE_TYPE_TEMPLATE | CMS_TEMPLATE_TYPE_GROUP) & $ttype) ? "100" : "") . "\" " .
                                                                                "data-tp-index=\"" . x($replace["%index%"]) . "\" " .
                                                                                "data-tp-path=\"" . x($replace["%path%"]) . "\" " .
                                                                                "data-tp-value=\"" . x($replace["%value%"]) . "\" " .
                                                                                "data-tp-type=\"" . x($replace["%type%"]) . "\" " .
                                                                                "data-tp-reference=\"" . x($replace["%reference%"]) . "\" " .
                                                                                "data-tp-id=\"" . x($replace["%id%"]) . "\" " .
                                                                                "data-tp-dd-type=\"" . x($ttype) . "\" " .
                                                                                "data-tp-dd-accept=\"" . x($__ttype) . "\">" .

                                                                //open extended editing functions container
                                                                           "<div class=\"tp-edt\" " .
                                                                                "onmouseleave=\"tp_edt_dsp('" . qx($_index) . "');\">";

                                                                //flip switch
                                                                if ((CMS_TEMPLATE_TYPE_TEMPLATE | CMS_TEMPLATE_TYPE_GROUP) & $ttype)
                                                                        $_buffer .= "<a id=\"tp-flp-" . x($_index) . "\" " .
                                                                                       "href=\"#\" " .
                                                                                       "title=\"" . CMS_L_TEMPLATE_046 . "\" " .
                                                                                       "onclick=\"return tp_flp('" . qx($_index) . "');\" " .
                                                                                       "class=\"tp-flp\">" .
                                                                                    "</a>";

                                                                //code for primary button that invokes element action
                                                                $replace  = array_map(__NAMESPACE__ . "\\r", $replace);
                                                                $search   = array_keys($replace);
                                                                $_buffer .= "<a id=\"tp-edt-a-" . x($_index) . "\" " .
                                                                               "href=\"" . x(str_replace($search, $replace, $_action)) . "\" " .
                                                                               "title=\"" . x($_id) . "\" " .
                                                                               "data-title=\"" . x($_title) . "\">" .
                                                                            "<img src=\"" . x($image) . "\" " .
                                                                                 "alt=\"*\">" .
                                                                            "</a>";

                                                                if (! $lock)
                                                                {
                                                                        //extended editing functions that invoke additional actions
                                                                        //displayed as a panel that expands when touching primary button
                                                                        $_buffer .= "<div id=\"tp-edt-" . x($_index) . "\" " .
                                                                                         "class=\"tp-edt-dsp-none\">";

                                                                        if (nstre($href))
                                                                                $_buffer .= $href .
                                                                                            "<img src=\"" . x(CMS_IMAGES_URL) . "content/button_open.png\" " .
                                                                                                 "alt=\"*\" " .
                                                                                                 "title=\"" . CMS_L_TEMPLATE_019 . "\">" .
                                                                                            "</a>";

                                                                        //copy action
                                                                        if (($edit_flag & CMS_TEMPLATE_COMMAND_BUFFER) && isset($this->action[CMS_TEMPLATE_COMMAND][CMS_TEMPLATE_COMMAND_BUFFER]))
                                                                                $_buffer .= "<a href=\"" . x(str_replace($search, $replace, $this->action[CMS_TEMPLATE_COMMAND][CMS_TEMPLATE_COMMAND_BUFFER])) . "\">" .
                                                                                            "<img src=\"" . x(CMS_IMAGES_URL) . "content/button_buffer.png\" " .
                                                                                                 "alt=\"*\" " .
                                                                                                 "title=\"" . CMS_L_COMMAND_BUFFER . "\">" .
                                                                                            "</a>";

                                                                        //paste action
                                                                        if (($edit_flag & CMS_TEMPLATE_COMMAND_PASTE) && isset($this->action[CMS_TEMPLATE_COMMAND][CMS_TEMPLATE_COMMAND_PASTE]))
                                                                                $_buffer .= "<a href=\"" . x(str_replace($search, $replace, $this->action[CMS_TEMPLATE_COMMAND][CMS_TEMPLATE_COMMAND_PASTE])) . "\">" .
                                                                                            "<img src=\"" . x(CMS_IMAGES_URL) . "content/button_paste.png\" " .
                                                                                                 "alt=\"*\" " .
                                                                                                 "title=\"" . CMS_L_COMMAND_PASTE . "\">" .
                                                                                            "</a>";

                                                                        //swap action
                                                                        if (($edit_flag & CMS_TEMPLATE_COMMAND_SWAP) && isset($this->action[CMS_TEMPLATE_COMMAND][CMS_TEMPLATE_COMMAND_SWAP]))
                                                                                $_buffer .= "<a href=\"" . x(str_replace($search, $replace, $this->action[CMS_TEMPLATE_COMMAND][CMS_TEMPLATE_COMMAND_SWAP])) . "\">" .
                                                                                            "<img src=\"" . x(CMS_IMAGES_URL) . "content/button_swap.png\" " .
                                                                                                 "alt=\"*\" " .
                                                                                                 "title=\"" . CMS_L_TEMPLATE_018 . "\">" .
                                                                                            "</a>";

                                                                        $_buffer .= "<br>";

                                                                        //kick down action
                                                                        if (($edit_flag & CMS_TEMPLATE_COMMAND_KICK1) && isset($this->action[CMS_TEMPLATE_COMMAND][CMS_TEMPLATE_COMMAND_KICK1]))
                                                                                $_buffer .= "<a href=\"" . x(str_replace($search, $replace, $this->action[CMS_TEMPLATE_COMMAND][CMS_TEMPLATE_COMMAND_KICK1])) . "\">" .
                                                                                            "<img src=\"" . x(CMS_IMAGES_URL) . "content/button_kick1.png\" " .
                                                                                                 "alt=\"*\" " .
                                                                                                 "title=\"" . CMS_L_TEMPLATE_024 . "\">" .
                                                                                            "</a>";

                                                                        //kick up action
                                                                        if (($edit_flag & CMS_TEMPLATE_COMMAND_KICK2) && isset($this->action[CMS_TEMPLATE_COMMAND][CMS_TEMPLATE_COMMAND_KICK2]))
                                                                                $_buffer .= "<a href=\"" . x(str_replace($search, $replace, $this->action[CMS_TEMPLATE_COMMAND][CMS_TEMPLATE_COMMAND_KICK2])) . "\">" .
                                                                                            "<img src=\"" . x(CMS_IMAGES_URL) . "content/button_kick2.png\" " .
                                                                                                 "alt=\"*\" " .
                                                                                                 "title=\"" . CMS_L_TEMPLATE_025 . "\">" .
                                                                                            "</a>";

                                                                        //drop up action
                                                                        if (($edit_flag & CMS_TEMPLATE_COMMAND_DROP1) && isset($this->action[CMS_TEMPLATE_COMMAND][CMS_TEMPLATE_COMMAND_DROP1]))
                                                                                $_buffer .= "<a href=\"" . x(str_replace($search, $replace, $this->action[CMS_TEMPLATE_COMMAND][CMS_TEMPLATE_COMMAND_DROP1])) . "\">" .
                                                                                            "<img src=\"" . x(CMS_IMAGES_URL) . "content/button_drop1.png\" " .
                                                                                                 "alt=\"*\" " .
                                                                                                 "title=\"" . CMS_L_TEMPLATE_026 . "\">" .
                                                                                            "</a>";

                                                                        //drop down action
                                                                        if (($edit_flag & CMS_TEMPLATE_COMMAND_DROP2) && isset($this->action[CMS_TEMPLATE_COMMAND][CMS_TEMPLATE_COMMAND_DROP2]))
                                                                                $_buffer .= "<a href=\"" . x(str_replace($search, $replace, $this->action[CMS_TEMPLATE_COMMAND][CMS_TEMPLATE_COMMAND_DROP2])) . "\">" .
                                                                                            "<img src=\"" . x(CMS_IMAGES_URL) . "content/button_drop2.png\" " .
                                                                                                 "alt=\"*\" " .
                                                                                                 "title=\"" . CMS_L_TEMPLATE_027 . "\">" .
                                                                                            "</a>";

                                                                        $_buffer .= "<br>";

                                                                        //release pool action
                                                                        if (($edit_flag & CMS_TEMPLATE_COMMAND_RELEASE) && isset($this->action[CMS_TEMPLATE_COMMAND][CMS_TEMPLATE_COMMAND_RELEASE]))
                                                                                $_buffer .= "<a href=\"" . x(str_replace($search, $replace, $this->action[CMS_TEMPLATE_COMMAND][CMS_TEMPLATE_COMMAND_RELEASE])) . "\">" .
                                                                                            "<img src=\"" . x(CMS_IMAGES_URL) . "content/button_release.png\" " .
                                                                                                 "alt=\"*\" " .
                                                                                                 "title=\"" . CMS_L_TEMPLATE_040 . "\">" .
                                                                                            "</a>";

                                                                        //reference pool action
                                                                        if (($edit_flag & CMS_TEMPLATE_COMMAND_REFERENCE) && isset($this->action[CMS_TEMPLATE_COMMAND][CMS_TEMPLATE_COMMAND_REFERENCE]))
                                                                                $_buffer .= "<a href=\"" . x(str_replace($search, $replace, $this->action[CMS_TEMPLATE_COMMAND][CMS_TEMPLATE_COMMAND_REFERENCE])) . "\">" .
                                                                                            "<img src=\"" . x(CMS_IMAGES_URL) . "content/button_reference.png\" " .
                                                                                                 "alt=\"*\" " .
                                                                                                 "title=\"" . CMS_L_TEMPLATE_041 . "\">" .
                                                                                            "</a>";

                                                                        //export template action
                                                                        if (($edit_flag & CMS_TEMPLATE_COMMAND_EXPORT) && isset($this->action[CMS_TEMPLATE_COMMAND][CMS_TEMPLATE_COMMAND_EXPORT]))
                                                                                $_buffer .= "<a href=\"" . x(str_replace($search, $replace, $this->action[CMS_TEMPLATE_COMMAND][CMS_TEMPLATE_COMMAND_EXPORT])) . "\">" .
                                                                                            "<img src=\"" . x(CMS_IMAGES_URL) . "content/button_export.png\" " .
                                                                                                 "alt=\"*\" " .
                                                                                                 "title=\"" . CMS_L_COMMAND_EXPORT . "\">" .
                                                                                            "</a>";

                                                                        $_buffer .= "<br>";

                                                                        //clear action
                                                                        if (($edit_flag & CMS_TEMPLATE_COMMAND_CLEAR) && isset($this->action[CMS_TEMPLATE_COMMAND][CMS_TEMPLATE_COMMAND_CLEAR]))
                                                                                $_buffer .= "<a href=\"" . x(str_replace($search, $replace, $this->action[CMS_TEMPLATE_COMMAND][CMS_TEMPLATE_COMMAND_CLEAR])) . "\">" .
                                                                                            "<img src=\"" . x(CMS_IMAGES_URL) . "content/button_clear.png\" " .
                                                                                                 "alt=\"*\" " .
                                                                                                 "title=\"" . CMS_L_COMMAND_DELETE . "\">" .
                                                                                            "</a>";

                                                                        $_buffer .= "</div>";
                                                                };

                                                                $_buffer .= "</div>";
                                                                $buffer   = $_buffer . $buffer;

                                                                //current element is not spannable
                                                                if ((CMS_TEMPLATE_TYPE_HREF      |
                                                                     CMS_TEMPLATE_TYPE_PLUGIN    |
                                                                     CMS_TEMPLATE_TYPE_TEXT      |
                                                                     CMS_TEMPLATE_TYPE_VALUE     |
                                                                     CMS_TEMPLATE_TYPE_DOWNLOAD  |
                                                                     CMS_TEMPLATE_TYPE_IMAGE     |
                                                                     CMS_TEMPLATE_TYPE_THUMBNAIL |
                                                                     CMS_TEMPLATE_TYPE_MEDIA     |
                                                                     CMS_TEMPLATE_TYPE_TEMPLATE  |
                                                                     CMS_TEMPLATE_TYPE_REPEAT    |
                                                                     CMS_TEMPLATE_TYPE_SHIFT     ) & $ttype)
                                                                {
                                                                        //retrieve index from dataset index stack
                                                                        $_index = array_pop($index_stack);

                                                                        //no preview
                                                                        if (! $lock)

                                                                                //close dragdrop container
                                                                                $buffer  .= "</div>";
                                                                };
                                                        };

                                                        $auto_id++; //advance automatic index
                                                };

                                                //debug information
                                                if ($debug)
                                                {
                                                        //element name
                                                        $info = $_ttype;

                                                        //add id
                                                        if (CMS_TEMPLATE_TYPE_EDIT & $ttype)
                                                                $info .= " id:$id";

                                                        //add idref
                                                        elseif (
                                                                 ((CMS_TEMPLATE_TYPE_CBLOCK | CMS_TEMPLATE_TYPE_CALT) & $ttype)
                                                                 &&
                                                                 isset($idref)
                                                               )
                                                                $info .= " idref:$idref";

                                                        //add element start marker
                                                        $buffer = "<!-- #debug# $info -->$buffer";

                                                        //closing tag expected later
                                                        if (CMS_TEMPLATE_TYPE_SPAN & $ttype)
                                                        {
                                                                //memorize spannable element as open
                                                                if (! isset($debug_stack[$ttype])) $debug_stack[$ttype] = array();
                                                                array_push($debug_stack[$ttype], $info); //add to debug stack
                                                        }

                                                        //implicit closing tag
                                                        else
                                                        {
                                                                //add element end marker
                                                                $buffer .= "<!-- #debug# /$info -->";
                                                        };
                                                };

                                                //append parsed element code to output
                                                $output .= $buffer;
                                        };

                                        //continue after tag
                                        $offset = $_offset;
                                };
                        }

//.....................................................................................................

                        //closing tag
                        elseif (
                                 ($template[$offset + 1] === "/") //fast check of initial tag characters
                                 &&
                                 ($template[$offset + 2] === "C")
                                 &&
                                 ($template[$offset + 3] === "M")
                                 &&
                                 ($template[$offset + 4] === "S")
                                 &&
                                 ($template[$offset + 5] === ":")
                               )
                        {
                                if (($_offset = strpos($template, ">", $offset + 6)) !== FALSE) //seek tag end
                                {
                                        $_offset++; //advance position after tag
                                        $string = substr($template, $offset, $_offset - $offset); //get closing tag code

                                        //verify tag
                                        if (
                                             preg_match("/^<\/CMS:([a-z]+)\s*>$/", $string, $_string) //extensive check
                                             &&
                                             isset($this->tlist[$_string[1]]) //check if element is listed
                                           )
                                        {
                                                //template type
                                                $_ttype = $_string[1];           //literal
                                                $ttype  = $this->tlist[$_ttype]; //numeric

                                                //skip processing within nocache block
                                                if ($nocache_depth && ($ttype !== CMS_TEMPLATE_TYPE_NOCACHE))
                                                {
                                                        $output .= $string;  //append unparsed element code to output
                                                        $offset  = $_offset; //continue after tag
                                                        continue;            //skip
                                                };

                                                if (
                                                     (CMS_TEMPLATE_TYPE_SPAN & $ttype) //spannable element
                                                     &&
                                                     isset($span_stack[$ttype])        //element is open
                                                     &&
                                                     array_pop($span_stack[$ttype])    //close element
                                                   )
                                                {
                                                        $buffer = NULL;

                                                        //process element by type
                                                        switch ($ttype)
                                                        {
                                                        case CMS_TEMPLATE_TYPE_CBLOCK:
//.....................................................................................................
                                                                if (

                                                                //within conditional block
                                                                     $cblock_depth
                                                                     &&

                                                                //decrease block nesting count
                                                                //to fall below limit
                                                                     (--$cblock_depth < $cblock_limit)
                                                                   )

                                                                        //abolish suppression
                                                                        $cblock_limit = 0;

                                                                break;

                                                        case CMS_TEMPLATE_TYPE_CALT:
//.....................................................................................................
                                                                if (

                                                                //within alternative block
                                                                     $calt_depth
                                                                     &&

                                                                //decrease block nesting count
                                                                //to fall below limit
                                                                     (--$calt_depth < $calt_limit)
                                                                   )

                                                                        //abolish suppression
                                                                        $calt_limit = 0;

                                                                break;

                                                        case CMS_TEMPLATE_TYPE_GROUP:
//.....................................................................................................
                                                                if ($cblock_limit) break; //suppressed by conditional block
                                                                if ($calt_limit)   break; //suppressed by alternative block

                                                                //retrieve data from group stack
                                                                if (count($group_stack) === 0) break; //no group data
                                                                $array    = array_pop($group_stack);
                                                                $_base_id = $array["base_id"]; //restore base index

                                                                //determine if editing functions are used
                                                                $flag = CMS_TEMPLATE_EDIT &&
                                                                        $array["edit"] &&
                                                                        isset($this->action[CMS_TEMPLATE_ACTION][$ttype]);

                                                                if ($flag) //editing
                                                                {
                                                                        //retrieve index from dataset index stack
                                                                        $_index  = array_pop($index_stack);

                                                                        $buffer .= "</div>";

                                                                        //no preview
                                                                        if (! $lock)

                                                                                //close dragdrop container
                                                                                $buffer .= "</div>";

                                                                        //decrease group nexting counter
                                                                        $group_depth--;
                                                                };

                                                                break;

                                                        case CMS_TEMPLATE_TYPE_BASE:
//.....................................................................................................
                                                                if ($cblock_limit) break; //suppressed by conditional block
                                                                if ($calt_limit)   break; //suppressed by alternative block

                                                                //retrieve data from base stack
                                                                if (count($base_stack) === 0) break; //no base data
                                                                $array    = array_pop($base_stack);
                                                                $auto_id  = $array["auto_id"];  //restore automatic index
                                                                $base_id  = $array["base_id"];  //restore base index
                                                                $_base_id = $array["_base_id"]; //restore base index

                                                                //remove data from temporary random index stack
                                                                array_pop($temp_stack);

                                                                break;

                                                        case CMS_TEMPLATE_TYPE_NAMESPACE:
//.....................................................................................................
                                                                if ($cblock_limit) break; //suppressed by conditional block
                                                                if ($calt_limit)   break; //suppressed by alternative block

                                                                //retrieve data from namespace stack
                                                                if (count($namespace_stack) === 0) break; //no namespace data
                                                                $_base_id = array_pop($namespace_stack);  //restore base index

                                                                break;

                                                        case CMS_TEMPLATE_TYPE_REPEAT:
//.....................................................................................................

                                                                //no suppression in order to provide automatic index consistency

                                                                //retrieve data from repeat stack
                                                                if (count($repeat_stack) === 0) break; //no repeat task
                                                                $array = array_pop($repeat_stack);
                                                                $count = --$array["count"];

                                                                //decrease repeat count
                                                                if ($count === 0) //end repeat task
                                                                {
                                                                        $_base_id = $array["base_id"]; //restore base index
                                                                        break;
                                                                };

                                                                //jump back
                                                                $_offset = $array["offset"];

                                                                //put updated repeat task on stack
                                                                array_push($repeat_stack, $array);

                                                                //memorize spannable element as open
                                                                array_push($span_stack[CMS_TEMPLATE_TYPE_REPEAT], TRUE);

                                                                break;

                                                        case CMS_TEMPLATE_TYPE_SHIFT:
//.....................................................................................................
                                                                if ($cblock_limit) break; //suppressed by conditional block
                                                                if ($calt_limit)   break; //suppressed by alternative block

                                                                //retrieve data from shift stack
                                                                if (count($shift_stack) === 0) break; //no shift data
                                                                $_base_id = array_pop($shift_stack);  //restore base index

                                                                break;

                                                        case CMS_TEMPLATE_TYPE_HREF:
                                                        case CMS_TEMPLATE_TYPE_DOWNLOAD:
//.....................................................................................................
                                                                if ($cblock_limit) break; //suppressed by conditional block
                                                                if ($calt_limit)   break; //suppressed by alternative block

                                                                $buffer .= "</a>";

                                                                break;

                                                        case CMS_TEMPLATE_TYPE_NOCACHE:
//.....................................................................................................
                                                                if (! $cache) break; //do not generate cache version

                                                                //not within uncached span
                                                                if ($nocache_depth && (! --$nocache_depth))

                                                                        //output cache separator
                                                                        $buffer .= CMS_TEMPLATE_CACHE_SEPARATOR;

                                                                break;
                                                        };

                                                        //append parsed element code to output
                                                        $output .= $buffer;
                                                };

                                                //debug information
                                                if ($debug)
                                                {
                                                        //no open tag
                                                        if (
                                                             (! isset($debug_stack[$ttype]))
                                                             ||
                                                             (count($debug_stack[$ttype]) === 0)
                                                           )
                                                        {
                                                                //error
                                                                $output .= "<!-- #debug# error: redundant closing tag /$_ttype -->";
                                                        }

                                                        //repeat
                                                        elseif (CMS_TEMPLATE_TYPE_REPEAT & $ttype)
                                                        {
                                                                //repetitions remaining
                                                                if ($count > 0)
                                                                {
                                                                        //retrieve data from debug stack
                                                                        $info    = end($debug_stack[$ttype]);

                                                                        //add element end marker
                                                                        $output .= "<!-- #debug# /$info (left:$count) -->" .

                                                                        //add element start marker
                                                                                   "<!-- #debug# $info -->";
                                                                }

                                                                //final repetition
                                                                else
                                                                {
                                                                        //retrieve data from debug stack
                                                                        $info    = array_pop($debug_stack[$ttype]);

                                                                        //add element end marker
                                                                        $output .= "<!-- #debug# /$info (final) -->";
                                                                };
                                                        }

                                                        //other
                                                        else
                                                        {
                                                                //retrieve data from debug stack
                                                                $info    = array_pop($debug_stack[$ttype]);

                                                                //add element end marker
                                                                $output .= "<!-- #debug# /$info -->";
                                                        };
                                                };
                                        };

                                        //continue after closing tag
                                        $offset = $_offset;
                                };
                        }

                        //unparsed data
                        else
                        {
                                //proceed after current position
                                $_offset = $offset;
                                $offset++;
                        };
                };

                //append offcut to output
                $output .= substr($template, $_offset);

                //decrease template nesting counter
                $template_depth--;

                //base level
                if ($template_depth === -1)
                {
                        //instant transmission when editing to optimize speed
                        if (CMS_TEMPLATE_EDIT)
                        {
                                echo($output);
                                flush();

                                //return empty string
                                return "";
                        };
                };

                //return parsed data
                return $output;
        }
//-----------------------------------------------------------------------------------------------------
        function export(&$document,
                        $index,
                        $base_id           = NULL, //do not change for basic template
                        $return_stylesheet = FALSE)
        {
                static $template_list  = array(); //list of employed templates
                static $template_depth = -1;      //template nesting counter
                static $nocache_depth  =  0;      //nocache depth

                //get template code
                if (
                     (
                       (! $index)
                       ||
                       (! $template = $this->get_code($index))
                     )
                     &&
                     ($index = $document->get($base_id, "template"))
                   )
                        $template = $this->get_code($index);

                if (stre($template)) return FALSE; //no template code
                if (nstre($base_id)) $base_id .= ".";

                //increase template nesting counter
                $template_depth++;

                //initialize
                if (! $template_depth) //zero on first pass
                {
                        //reset list of employed templates
                        $template_list = ($base_id === NULL) ? array($index => TRUE) : array();

                        //reset nocache depth
                        $nocache_depth = 0;
                };

                $offset          = 0;                 //primary text position
                $_offset         = 0;                 //secondary text position
                $length          = strlen($template); //template code length

                $auto_id         = 0;                 //automatic element index
                $_base_id        = $base_id;          //base index working copy

                $group_stack     = array();           //group data stack
                $base_stack      = array();           //base data stack
                $namespace_stack = array();           //namespace data stack
                $repeat_stack    = array();           //repeat data stack
                $shift_stack     = array();           //shift data stack

                $cblock_stack    = array();           //conditional block stack
                $calt_depth      = 0;                 //conditional alternative depth
                $calt_limit      = 0;                 //conditional alternative limit

                $output          = NULL;              //output buffer

                if (! isset($template_list[$index]))
                {
                        //add to list of employed templates
                        $template_list[$index] = TRUE;

                        //link stylesheet of employed template
                        if ((! $return_stylesheet) && (($file = $this->get_stylesheet($index, NULL, TRUE)) !== FALSE))
                                $output .= stylesheet(CMS_DATA_URL . $file);
                };

                //process template code
                while (
                        ($offset < $length) //not at end of template code
                        &&
                        (($offset = strpos($template, "<", $offset)) !== FALSE) //seek next opening tag
                      )
                {
                        //append preliminary unparsed data to output if not suppressed
                        if (! $calt_limit) $output .= substr($template, $_offset, $offset - $_offset);

                        //start tag
                        if (
                             ($template[$offset + 1] === "C") //fast check of initial tag characters
                             &&
                             ($template[$offset + 2] === "M")
                             &&
                             ($template[$offset + 3] === "S")
                             &&
                             ($template[$offset + 4] === ":")
                           )
                        {
                                if (($_offset = strpos($template, ">", $offset + 5)) !== FALSE) //seek tag end
                                {
                                        $_offset++; //advance position after tag
                                        $string = substr($template, $offset, $_offset - $offset); //get tag code

                                        //verify tag
                                        if (
                                             preg_match("/^<CMS:([a-z]+)(?:\s|\/|>)/", $string, $_string) //extensive check
                                             &&
                                             isset($this->tlist[$_string[1]]) //check if element is listed
                                           )
                                        {
                                                //template type
                                                $_ttype = $_string[1];           //literal
                                                $ttype  = $this->tlist[$_ttype]; //numeric

                                                //id

                                                /*
                                                $id   //original index (as given in code)
                                                $_id  //source index (pointing to content)
                                                $__id //inherited index (new base for embedded elements)
                                                */

                                                while (TRUE)
                                                {
                                                        //extract id
                                                        while (preg_match("/\s+id\s*=\s*\"([^\"]+)\"/", $string, $_string))
                                                        {
                                                                //process relative automatic indexes
                                                                $id = preg_replace_callback(
                                                                        "/(?<=^|\.)(\+|-)([0-9]+)(?=$|\.)/",
                                                                        function($a) use (&$auto_id) { return $auto_id + (($a[1] === "+") ? $a[2] : -$a[2]); },
                                                                        $_string[1]);

                                                                //verify id
                                                                if (! preg_match("/^(\.{0,2})([0-9a-zA-Z_\.]+?)(\.?)$/", $id, $_string)) break;

                                                                $_id = $_string[2];

                                                                //process modifiers
                                                                switch ($_string[1])
                                                                {
                                                                case ".": //relative to base
                                                                        if ($_string[3] === ".")
                                                                        {
                                                                                $_id  = $base_id . $_id;
                                                                                $__id = $_id;
                                                                        }
                                                                        else
                                                                        {
                                                                                $__id = $_base_id . $_id;
                                                                                $_id  = $base_id . $_id;
                                                                        };
                                                                        break;

                                                                case "..": //absolute
                                                                        $__id = ($_string[3] === ".") ?
                                                                                $_id : $_base_id . $_id;
                                                                        break;

                                                                default: //relative
                                                                        $_id  = $_base_id . $_id;
                                                                        $__id = $_id;
                                                                        break;
                                                                };

                                                                break 2;
                                                        };

                                                        //relative using automatic index
                                                        $id   = $auto_id;
                                                        $_id  = $_base_id . $auto_id;
                                                        $__id = $_id;
                                                        break;
                                                };

                                                //process element by type
                                                switch ($ttype)
                                                {
                                                case CMS_TEMPLATE_TYPE_HEAD:
//.....................................................................................................

                                                        //pass through
                                                        $output .= $string;

                                                        //link stylesheet of employed template
                                                        if ((! $return_stylesheet) && (($file = $this->get_stylesheet($index, NULL, TRUE)) !== FALSE))
                                                                $output .= "\n" . stylesheet(CMS_DATA_URL . $file, FALSE);

                                                        break;

                                                case CMS_TEMPLATE_TYPE_CBLOCK:
//.....................................................................................................
                                                        if ($calt_limit) break; //suppressed by alternative block

                                                        //idref

                                                        /*
                                                        $idref  //source index (pointing to content)
                                                        $_idref //inherited index (new base for embedded elements)
                                                        */

                                                        while (TRUE)
                                                        {
                                                                //extract idref
                                                                while (preg_match("/\s+idref\s*=\s*\"([^\"]+)\"/", $string, $_string))
                                                                {
                                                                        //process relative automatic indexes
                                                                        $idref = preg_replace_callback(
                                                                                "/(?<=^|\.)(\+|-)([0-9]+)(?=$|\.)/",
                                                                                function($a) use (&$auto_id) { return $auto_id + (($a[1] === "+") ? $a[2] : -$a[2]); },
                                                                                $_string[1]);

                                                                        //verify idref
                                                                        if (! preg_match("/^(\.{0,2})([0-9a-zA-Z_\.]+)$/", $idref, $_string)) break;

                                                                        //process modifiers
                                                                        switch ($_string[1])
                                                                        {
                                                                        case ".": //relative to base
                                                                                $_idref = $base_id . $_string[2];
                                                                                break;

                                                                        case "..": //absolute
                                                                                $_idref = $_string[2];
                                                                                break;

                                                                        default: //relative
                                                                                $_idref = $_base_id . $_string[2];
                                                                                break;
                                                                        };

                                                                        break 2;
                                                                };

                                                                //relative using automatic index
                                                                $idref  = $auto_id;
                                                                $_idref = $_base_id . $auto_id;
                                                                break;
                                                        };

                                                        //referenced element is a template and value is set
                                                        if (nstre(l($document->get($_idref, "template"))))
                                                        {
                                                                //mark associated closing tag to be discarded
                                                                array_push($cblock_stack, FALSE);
                                                        }

                                                        //referenced element is not a template or no value is set
                                                        else
                                                        {
                                                                //set reference index and adopt conditional block
                                                                $string  = preg_replace("/\s+idref\s*=\s*\"[^\"]*\"/", "", $string);
                                                                $output .= preg_replace("/^(<CMS:[a-z]+)/", "\\1 idref=\"$idref\"", $string);

                                                                //mark associated closing tag to be applied
                                                                array_push($cblock_stack, TRUE);
                                                        };

                                                        break;

                                                case CMS_TEMPLATE_TYPE_CALT:
//.....................................................................................................

                                                        //idref

                                                        /*
                                                        $idref  //source index (pointing to content)
                                                        $_idref //inherited index (new base for embedded elements)
                                                        */

                                                        while (TRUE)
                                                        {
                                                                //extract idref
                                                                while (preg_match("/\s+idref\s*=\s*\"([^\"]+)\"/", $string, $_string))
                                                                {
                                                                        //process relative automatic indexes
                                                                        $idref = preg_replace_callback(
                                                                                "/(?<=^|\.)(\+|-)([0-9]+)(?=$|\.)/",
                                                                                function($a) use (&$auto_id) { return $auto_id + (($a[1] === "+") ? $a[2] : -$a[2]); },
                                                                                $_string[1]);

                                                                        //verify idref
                                                                        if (! preg_match("/^(\.{0,2})([0-9a-zA-Z_\.]+)$/", $idref, $_string)) break;

                                                                        //process modifiers
                                                                        switch ($_string[1])
                                                                        {
                                                                        case ".": //relative to base
                                                                                $_idref = $base_id . $_string[2];
                                                                                break;

                                                                        case "..": //absolute
                                                                                $_idref = $_string[2];
                                                                                break;

                                                                        default: //relative
                                                                                $_idref = $_base_id . $_string[2];
                                                                                break;
                                                                        };

                                                                        break 2;
                                                                };

                                                                //relative using automatic index
                                                                $idref  = $auto_id;
                                                                $_idref = $_base_id . $auto_id;
                                                                break;
                                                        };

                                                        //increase block nesting count
                                                        $calt_depth++;

                                                        if (! $calt_limit) //not suppressed already
                                                        {
                                                                //referenced element is a template and value is set
                                                                if (nstre(l($document->get($_idref, "template"))))
                                                                {
                                                                        //set suppression
                                                                        $calt_limit = $calt_depth;
                                                                }

                                                                //referenced element is not a template or no value is set
                                                                else
                                                                {
                                                                        //set reference index and adopt alternative block
                                                                        $string  = preg_replace("/\s+idref\s*=\s*\"[^\"]*\"/", "", $string);
                                                                        $output .= preg_replace("/^(<CMS:[a-z]+)/", "\\1 idref=\"$idref\"", $string);
                                                                };
                                                        };

                                                        break;

                                                case CMS_TEMPLATE_TYPE_GROUP:
//.....................................................................................................
                                                        if ($calt_limit) break; //suppressed by alternative block

                                                        //put data on group stack
                                                        array_push($group_stack, $_base_id); //backup base index

                                                        $_base_id  = "$__id."; //set base index

                                                        //set index and adopt element
                                                        $string    = preg_replace("/\s+id\s*=\s*\"[^\"]*\"/", "", $string);
                                                        $output   .= preg_replace("/^(<CMS:[a-z]+)/", "\\1 id=\"$id\"", $string);

                                                        break;

                                                case CMS_TEMPLATE_TYPE_BASE:
//.....................................................................................................
                                                        if ($calt_limit) break; //suppressed by alternative block

                                                        //put data on base stack
                                                        array_push($base_stack,
                                                                   array("auto_id"  => $auto_id,    //backup automatic index
                                                                         "base_id"  => $base_id,    //backup base index
                                                                         "_base_id" => $_base_id)); //backup base index

                                                        $auto_id   = 0;        //set automatic index
                                                        $base_id   = "$__id."; //set base index
                                                        $_base_id  = "$__id."; //set base index

                                                        //set index and adopt element
                                                        $string    = preg_replace("/\s+id\s*=\s*\"[^\"]*\"/", "", $string);
                                                        $output   .= preg_replace("/^(<CMS:[a-z]+)/", "\\1 id=\"$id\"", $string);

                                                        break;

                                                case CMS_TEMPLATE_TYPE_NAMESPACE:
//.....................................................................................................
                                                        if ($calt_limit) break; //suppressed by alternative block

                                                        //put data on namespace stack
                                                        array_push($namespace_stack, $_base_id); //backup base index

                                                        $_base_id  = "$__id."; //set base index

                                                        //set index and adopt element
                                                        $string    = preg_replace("/\s+id\s*=\s*\"[^\"]*\"/", "", $string);
                                                        $output   .= preg_replace("/^(<CMS:[a-z]+)/", "\\1 id=\"$id\"", $string);

                                                        break;

                                                case CMS_TEMPLATE_TYPE_REPEAT:
//.....................................................................................................

                                                        //no suppression in order to provide automatic index consistency

                                                        //determine repeat count
                                                        if (! $cvalue = (int)$document->get($_id, $_ttype))
                                                                $cvalue = preg_match("/\s+default\s*=\s*\"([^\"]*)\"/",  $string, $_string) ?
                                                                          (int)$_string[1] : 1;
                                                        if ($cvalue < 1) $cvalue = 1; //at least one repetition

                                                        //put data on repeat stack
                                                        array_push($repeat_stack,
                                                                   array("count"   => $cvalue,     //repeat count
                                                                         "offset"  => $_offset,    //repeat offset
                                                                         "base_id" => $_base_id)); //backup base index

                                                        $_base_id  = "$__id."; //set base index

                                                        //replace with namespace
                                                        $output   .= "<CMS:namespace id=\"$id\">";

                                                        break;

                                                case CMS_TEMPLATE_TYPE_SHIFT:
//.....................................................................................................
                                                        if ($calt_limit) break; //suppressed by alternative block

                                                        //put data on shift stack
                                                        array_push($shift_stack, $_base_id); //backup base index

                                                        $_base_id  = "$__id."; //set base index

                                                        //replace with namespace
                                                        $output   .= "<CMS:namespace id=\"$id\">";

                                                        break;

                                                case CMS_TEMPLATE_TYPE_TEMPLATE:
//.....................................................................................................
                                                        if ($calt_limit)              break; //suppressed by alternative block
                                                        if (streq("$_id.", $base_id)) break; //prevent infinite loop

                                                        //get template index
                                                        if (! $cvalue = ($_string = $document->get($_id, $_ttype)) ? l($_string) : NULL)
                                                                $cvalue = preg_match("/\s+default\s*=\s*\"([^\"]*)\"/",  $string, $_string) ?
                                                                          $_string[1] : NULL;

                                                        //subtemplate is given
                                                        if ($cvalue)
                                                        {
                                                                //add template start marker
                                                                $name    = x(l($this->data->get($cvalue, "name")));
                                                                $name    = preg_replace("/-{2,}/", "-", $name);
                                                                $output .= "<!-- template $name -->" .

                                                                //recursively call export function to process subtemplate
                                                                           "<CMS:base id=\"$id\">" .
                                                                           $this->export($document,            //document
                                                                                         $cvalue,              //index
                                                                                         $__id,                //base_id
                                                                                         $return_stylesheet) . //return_stylesheet
                                                                           "</CMS:base>" .

                                                                //add template end marker
                                                                           "<!-- /template $name -->";
                                                                break;
                                                        };

                                                        //set index and adopt element
                                                        $string  = preg_replace("/\s+id\s*=\s*\"[^\"]*\"/", "", $string);
                                                        $output .= preg_replace("/^(<CMS:[a-z]+)/", "\\1 id=\"$id\"", $string);

                                                        break;

                                                case CMS_TEMPLATE_TYPE_NOCACHE:
//.....................................................................................................

                                                        //if not within uncached span
                                                        if (! $nocache_depth++)

                                                                //nocache block is adopted
                                                                $output .= "<CMS:nocache>";

                                                        break;

                                                default:
//.....................................................................................................
                                                        if ($calt_limit) break; //suppressed by alternative block

                                                        //set index and adopt element
                                                        if (CMS_TEMPLATE_TYPE_EDIT & $ttype)
                                                        {
                                                                $string  = preg_replace("/\s+id\s*=\s*\"[^\"]*\"/", "", $string);
                                                                $output .= preg_replace("/^(<CMS:[a-z]+)/", "\\1 id=\"$id\"", $string);
                                                        }

                                                        //just pass through
                                                        else
                                                        {
                                                                $output .= $string;
                                                        };

                                                        break;
                                                };

                                                //advance automatic index if element is editable
                                                if (CMS_TEMPLATE_TYPE_EDIT & $ttype) $auto_id++;
                                        };

                                        //continue after tag
                                        $offset = $_offset;
                                };
                        }

                        //closing tag
                        elseif (
                                 ($template[$offset + 1] === "/") //fast check of initial tag characters
                                 &&
                                 ($template[$offset + 2] === "C")
                                 &&
                                 ($template[$offset + 3] === "M")
                                 &&
                                 ($template[$offset + 4] === "S")
                                 &&
                                 ($template[$offset + 5] === ":")
                               )
                        {
                                if (($_offset = strpos($template, ">", $offset + 7)) !== FALSE) //seek tag end
                                {
                                        $_offset++; //advance position after tag
                                        $string = substr($template, $offset, $_offset - $offset); //get closing tag code

                                        //verify tag
                                        if (
                                             preg_match("/^<\/CMS:([a-z]+)\s*>$/", $string, $_string) //extensive check
                                             &&
                                             isset($this->tlist[$_string[1]]) //check if element is listed
                                           )
                                        {
                                                //template type
                                                $ttype = $this->tlist[$_string[1]];

                                                if (CMS_TEMPLATE_TYPE_SPAN & $ttype) //spannable element
                                                {
                                                        //process element by type
                                                        switch ($ttype)
                                                        {
                                                        case CMS_TEMPLATE_TYPE_CBLOCK:
//.....................................................................................................
                                                                if ($calt_limit) break; //suppressed by alternative block

                                                                //pass through closing tag if marked to be adopted
                                                                if (array_pop($cblock_stack)) $output .= $string;

                                                                break;

                                                        case CMS_TEMPLATE_TYPE_CALT:
//.....................................................................................................

                                                                //pass through closing tag if not suppressed
                                                                if (! $calt_limit) $output .= $string;

                                                                if (

                                                                //within alternative block
                                                                     $calt_depth
                                                                     &&

                                                                //decrease block nesting count
                                                                //to fall below limit
                                                                     (--$calt_depth < $calt_limit)
                                                                   )

                                                                        //abolish suppression
                                                                        $calt_limit = 0;

                                                                break;

                                                        case CMS_TEMPLATE_TYPE_GROUP:
//.....................................................................................................
                                                                if ($calt_limit) break; //suppressed by alternative block

                                                                //retrieve data from group stack
                                                                if (count($group_stack) === 0) break; //no group data
                                                                $_base_id  = array_pop($group_stack); //restore base index

                                                                //pass through closing tag
                                                                $output   .= $string;

                                                                break;

                                                        case CMS_TEMPLATE_TYPE_BASE:
//.....................................................................................................
                                                                if ($calt_limit) break; //suppressed by alternative block

                                                                //retrieve data from base stack
                                                                if (count($base_stack) === 0) break; //no base data
                                                                $array     = array_pop($base_stack);
                                                                $auto_id   = $array["auto_id"];  //restore automatic index
                                                                $base_id   = $array["base_id"];  //restore base index
                                                                $_base_id  = $array["_base_id"]; //restore base index

                                                                //just pass through
                                                                $output   .= $string;

                                                                break;

                                                        case CMS_TEMPLATE_TYPE_NAMESPACE:
//.....................................................................................................
                                                                if ($calt_limit) break; //suppressed by alternative block

                                                                //retrieve data from namespace stack
                                                                if (count($namespace_stack) === 0) break; //no namespace data
                                                                $_base_id  = array_pop($namespace_stack); //restore base index

                                                                //just pass through
                                                                $output   .= $string;

                                                                break;

                                                        case CMS_TEMPLATE_TYPE_REPEAT:
//.....................................................................................................

                                                                //no suppression in order to provide automatic index consistency

                                                                //retrieve data from repeat stack
                                                                if (count($repeat_stack) === 0) break; //no repeat task
                                                                $array = array_pop($repeat_stack);

                                                                //decrease repeat count
                                                                if (--$array["count"] === 0) //end repeat task
                                                                {
                                                                        $_base_id  = $array["base_id"]; //restore base index

                                                                        //replace with namespace
                                                                        $output   .= "</CMS:namespace>";
                                                                        break;
                                                                };

                                                                //jump back
                                                                $_offset = $array["offset"];

                                                                //put updated repeat task on stack
                                                                array_push($repeat_stack, $array);

                                                                break;

                                                        case CMS_TEMPLATE_TYPE_SHIFT:
//.....................................................................................................
                                                                if ($calt_limit) break; //suppressed by alternative block

                                                                //retrieve data from shift stack
                                                                if (count($shift_stack) === 0) break; //no shift data
                                                                $_base_id  = array_pop($shift_stack); //restore base index

                                                                //replace with namespace
                                                                $output   .= "</CMS:namespace>";

                                                                break;

                                                        case CMS_TEMPLATE_TYPE_NOCACHE:
//.....................................................................................................

                                                                //not within uncached span
                                                                if ($nocache_depth && (! --$nocache_depth))

                                                                        //adopt closing tag
                                                                        $output .= "</CMS:nocache>";

                                                                break;

                                                        default:
//.....................................................................................................
                                                                if ($calt_limit) break; //suppressed by alternative block

                                                                //just pass through
                                                                $output .= $string;

                                                                break;
                                                        };
                                                };
                                        };

                                        //continue after closing tag
                                        $offset = $_offset;
                                };
                        }

                        //unparsed data
                        else
                        {
                                //proceed after current position
                                $_offset = $offset;
                                $offset++;
                        };
                };

                //append offcut to output
                $output .= substr($template, $_offset);

                //decrease template nesting counter
                $template_depth--;

                //base level
                if ($return_stylesheet && ($template_depth === -1))
                {
                        //combine stylesheets of employed templates
                        $buffer = "";

                        reset($template_list);
                        while (list($key) = each($template_list))
                        {
                                if (stre($file = $this->get_stylesheet($key, NULL))) continue;

                                $name    = str_replace("*/", "", l($this->data->get($key, "name")));
                                $_buffer = str_repeat("*", utf8_strlen($name));
                                $buffer .= "/***" . $_buffer . "****\n" .
                                           "*   " . $name .    "   *\n" .
                                           "****" . $_buffer . "***/\n" .
                                           "\n" .
                                           "$file\n" .
                                           "\n";
                        };

                        $buffer = substr($buffer, 0, -4);

                        return array("code" => $output, "stylesheet" => $buffer);
                };

                //return parsed data
                return $output;
        }
//-----------------------------------------------------------------------------------------------------
        function structure(&$document,
                           $index,
                           $base_id = NULL) //do not change for basic template
        {
                static $template_depth = -1; //template nesting counter

                //get template code
                $template = "";

                if (
                     (
                       (! $index)
                       ||
                       (! $template = $this->get_code($index))
                     )
                     &&
                     ($index = $document->get($base_id, "template"))
                   )
                        $template = $this->get_code($index);

                if (stre($template)) return FALSE; //no template code
                if (nstre($base_id)) $base_id .= ".";

                //increase template nesting counter
                $template_depth++;

                //initialize
                if (! $template_depth) //zero on first pass
                {
                        $this->structure_id = 1; //structure index
                        $this->parent_id    = 0; //parent index
                };

                $offset          = 0;                 //primary text position
                $_offset         = 0;                 //secondary text position
                $length          = strlen($template); //template code length

                $auto_id         = 0;                 //automatic element index
                $_base_id        = $base_id;          //base index working copy

                $parent_stack    = array();           //parent stack
                $group_stack     = array();           //group data stack
                $base_stack      = array();           //base data stack
                $namespace_stack = array();           //namespace data stack
                $repeat_stack    = array();           //repeat data stack
                $shift_stack     = array();           //shift data stack

                $calt_depth      = 0;                 //conditional alternative depth
                $calt_limit      = 0;                 //conditional alternative limit

                $structure       = array();           //structure

                //process template code
                while (
                        ($offset < $length) //not at end of template code
                        &&
                        (($offset = strpos($template, "<", $offset)) !== FALSE) //seek next opening tag
                      )
                {
                        //start tag
                        if (
                             ($template[$offset + 1] === "C") //fast check of initial tag characters
                             &&
                             ($template[$offset + 2] === "M")
                             &&
                             ($template[$offset + 3] === "S")
                             &&
                             ($template[$offset + 4] === ":")
                           )
                        {
                                if (($_offset = strpos($template, ">", $offset + 5)) !== FALSE) //seek tag end
                                {
                                        $_offset++; //advance position after tag
                                        $string = substr($template, $offset, $_offset - $offset); //get tag code

                                        //verify tag
                                        if (
                                             preg_match("/^<CMS:([a-z]+)(?:\s|\/|>)/", $string, $_string) //extensive check
                                             &&
                                             isset($this->tlist[$_string[1]]) //check if element is listed
                                           )
                                        {
                                                //template type
                                                $_ttype = $_string[1];           //literal
                                                $ttype  = $this->tlist[$_ttype]; //numeric

                                                //thumbnail equals image
                                                if ($ttype === CMS_TEMPLATE_TYPE_THUMBNAIL) $_ttype = "image";

                                                if (
                                                     (! $calt_limit) //not suppressed by alternative block
                                                     &&
                                                     ($ttype & (CMS_TEMPLATE_TYPE_EDIT | //editable element, base or namespace
                                                                CMS_TEMPLATE_TYPE_BASE |
                                                                CMS_TEMPLATE_TYPE_NAMESPACE))
                                                   )
                                                {
                                                        //id

                                                        /*
                                                        $id   //original index (as given in code)
                                                        $_id  //source index (pointing to content)
                                                        */

                                                        while (TRUE)
                                                        {
                                                                //extract id
                                                                while (preg_match("/\s+id\s*=\s*\"([^\"]+)\"/", $string, $_string))
                                                                {
                                                                        //process relative automatic indexes
                                                                        $_string = preg_replace_callback(
                                                                                "/(?<=^|\.)(\+|-)([0-9]+)(?=$|\.)/",
                                                                                function($a) use (&$auto_id) { return $auto_id + (($a[1] === "+") ? $a[2] : -$a[2]); },
                                                                                $_string[1]);

                                                                        //verify id
                                                                        if (! preg_match("/^(\.{0,2})([0-9a-zA-Z_\.]+?)(\.?)$/", $_string, $_string)) break;

                                                                        $id = $_string[2];

                                                                        //process modifiers
                                                                        switch ($_string[1])
                                                                        {
                                                                        case ".": //relative to base
                                                                                if ($_string[3] === ".")
                                                                                {
                                                                                        $id  = $base_id . $id;
                                                                                        $_id = $id;
                                                                                }
                                                                                else
                                                                                {
                                                                                        $_id = $_base_id . $id;
                                                                                        $id  = $base_id . $id;
                                                                                };
                                                                                break;

                                                                        case "..": //absolute
                                                                                $_id = ($_string[3] === ".") ?
                                                                                       $id : $_base_id . $id;
                                                                                break;

                                                                        default: //relative
                                                                                $id  = $_base_id . $id;
                                                                                $_id = $id;
                                                                                break;
                                                                        };

                                                                        break 2;
                                                                };

                                                                //relative using automatic index
                                                                $id  = $_base_id . $auto_id;
                                                                $_id = $id;
                                                                break;
                                                        };

                                                        //add element to structure
                                                        $structure[$this->structure_id] =
                                                                array(CMS_TEMPLATE_STRUCTURE_INDEX  => $id,
                                                                      CMS_TEMPLATE_STRUCTURE_PATH   => $_id,
                                                                      CMS_TEMPLATE_STRUCTURE_PARENT => $this->parent_id,
                                                                      CMS_TEMPLATE_STRUCTURE_TYPE   => $_ttype);

                                                        if (CMS_TEMPLATE_TYPE_PATH & $ttype) //path extending element
                                                        {
                                                                //put current parent on stack
                                                                array_push($parent_stack, $this->parent_id);

                                                                //set current element as parent
                                                                $this->parent_id = $this->structure_id;
                                                        };

                                                        //increase structure index
                                                        $this->structure_id++;
                                                };

                                                //process element by type
                                                switch ($ttype)
                                                {
                                                case CMS_TEMPLATE_TYPE_CALT:
//.....................................................................................................
break; //TEMPORARY; need to check if CALT is necessary at all
                                                        //increase block nesting count
                                                        $calt_depth++;

                                                        //set suppression if not active already
                                                        if (! $calt_limit) $calt_limit = $calt_depth;

                                                        break;

                                                case CMS_TEMPLATE_TYPE_GROUP:
//.....................................................................................................
                                                        if ($calt_limit) break; //suppressed by alternative block

                                                        //put data on group stack
                                                        array_push($group_stack, $_base_id); //backup base index

                                                        $_base_id = "$_id."; //set base index

                                                        break;

                                                case CMS_TEMPLATE_TYPE_BASE:
//.....................................................................................................
                                                        if ($calt_limit) break; //suppressed by alternative block

                                                        //put data on base stack
                                                        array_push($base_stack,
                                                                   array("auto_id"  => $auto_id,    //backup automatic index
                                                                         "base_id"  => $base_id,    //backup base index
                                                                         "_base_id" => $_base_id)); //backup base index

                                                        $auto_id  = 0;       //set automatic index
                                                        $base_id  = "$_id."; //set base index
                                                        $_base_id = "$_id."; //set base index

                                                        break;

                                                case CMS_TEMPLATE_TYPE_NAMESPACE:
//.....................................................................................................
                                                        if ($calt_limit) break; //suppressed by alternative block

                                                        //put data on namespace stack
                                                        array_push($namespace_stack, $_base_id); //backup base index

                                                        $_base_id = "$_id."; //set base index

                                                        break;

                                                case CMS_TEMPLATE_TYPE_REPEAT:
//.....................................................................................................

                                                        //no suppression in order to provide automatic index consistency

                                                        //determine repeat count
                                                        if (! $cvalue = $document->get($id, $_ttype))
                                                                $cvalue = preg_match("/\s+default\s*=\s*\"([^\"]*)\"/", $string, $_string) ?
                                                                          (int)$_string[1] : 1;
                                                        if ($cvalue < 0) $cvalue = 1; //at least one repetition

                                                        //put data on repeat stack
                                                        array_push($repeat_stack,
                                                                   array("count"   => $cvalue,     //repeat count
                                                                         "offset"  => $_offset,    //repeat offset
                                                                         "base_id" => $_base_id)); //backup base index

                                                        $_base_id = "$_id."; //set base index

                                                        break;

                                                case CMS_TEMPLATE_TYPE_SHIFT:
//.....................................................................................................
                                                        if ($calt_limit) break; //suppressed by alternative block

                                                        //put data on shift stack
                                                        array_push($shift_stack, $_base_id); //backup base index

                                                        $_base_id = "$_id."; //set base index

                                                        break;

                                                case CMS_TEMPLATE_TYPE_TEMPLATE:
//.....................................................................................................
                                                        if ($calt_limit)             break; //suppressed by alternative block
                                                        if (streq("$id.", $base_id)) break; //prevent infinite loop

                                                        //get template index
                                                        if (! $cvalue = ($_string = $document->get($id, $_ttype)) ? l($_string) : NULL)
                                                                $cvalue = preg_match("/\s+default\s*=\s*\"([^\"]*)\"/",  $string, $_string) ?
                                                                          $_string[1] : NULL;

                                                        //recursively call structure function to process subtemplate
                                                        if ($array = $this->structure($document, $cvalue, $_id))

                                                                //integrate substructure
                                                                $structure += $array;

                                                        //retrieve previous parent index from stack
                                                        //to be done here because there is no closing tag
                                                        $this->parent_id = array_pop($parent_stack);

                                                        break;
                                                };

                                                //advance automatic index if element is editable
                                                if (CMS_TEMPLATE_TYPE_EDIT & $ttype) $auto_id++;
                                        };

                                        //continue after tag
                                        $offset = $_offset;
                                };
                        }

                        //closing tag
                        elseif (
                                 ($template[$offset + 1] === "/") //fast check of initial tag characters
                                 &&
                                 ($template[$offset + 2] === "C")
                                 &&
                                 ($template[$offset + 3] === "M")
                                 &&
                                 ($template[$offset + 4] === "S")
                                 &&
                                 ($template[$offset + 5] === ":")
                               )
                        {
                                if (($_offset = strpos($template, ">", $offset + 7)) !== FALSE) //seek tag end
                                {
                                        $_offset++; //advance position after tag
                                        $string = substr($template, $offset, $_offset - $offset); //get closing tag code

                                        //verify tag
                                        if (
                                             preg_match("/^<\/CMS:([a-z]+)\s*>$/", $string, $_string) //extensive check
                                             &&
                                             isset($this->tlist[$_string[1]]) //check if element is listed
                                           )
                                        {
                                                //template type
                                                $ttype = $this->tlist[$_string[1]]; //numeric

                                                if (CMS_TEMPLATE_TYPE_SPAN & $ttype) //spannable element
                                                {
                                                        //process element by type
                                                        switch ($ttype)
                                                        {
                                                        case CMS_TEMPLATE_TYPE_CALT:
//.....................................................................................................
                                                                if (

                                                                //within alternative block
                                                                     $calt_depth
                                                                     &&

                                                                //decrease block nesting count
                                                                //to fall below limit
                                                                     (--$calt_depth < $calt_limit)
                                                                   )

                                                                        //abolish suppression
                                                                        $calt_limit = 0;

                                                                break;

                                                        case CMS_TEMPLATE_TYPE_GROUP:
//.....................................................................................................
                                                                if ($calt_limit) break; //suppressed by alternative block

                                                                //retrieve data from group stack
                                                                if (count($group_stack) === 0) break; //no group data
                                                                $_base_id = array_pop($group_stack);  //restore base index

                                                                //retrieve previous parent index from stack
                                                                $this->parent_id = array_pop($parent_stack);

                                                                break;

                                                        case CMS_TEMPLATE_TYPE_BASE:
//.....................................................................................................
                                                                if ($calt_limit) break; //suppressed by alternative block

                                                                //retrieve data from base stack
                                                                if (count($base_stack) === 0) break; //no base data
                                                                $array    = array_pop($base_stack);
                                                                $auto_id  = $array["auto_id"];  //restore automatic index
                                                                $base_id  = $array["base_id"];  //restore base index
                                                                $_base_id = $array["_base_id"]; //restore base index

                                                                break;

                                                        case CMS_TEMPLATE_TYPE_NAMESPACE:
//.....................................................................................................
                                                                if ($calt_limit) break; //suppressed by alternative block

                                                                //retrieve data from namespace stack
                                                                if (count($namespace_stack) === 0) break; //no namespace data
                                                                $_base_id = array_pop($namespace_stack);  //restore base index

                                                                break;

                                                        case CMS_TEMPLATE_TYPE_REPEAT:
//.....................................................................................................

                                                                //no suppression in order to provide automatic index consistency

                                                                //retrieve data from repeat stack
                                                                if (count($repeat_stack) === 0) break; //no repeat task
                                                                $array = array_pop($repeat_stack);

                                                                //decrease repeat count
                                                                if (--$array["count"] === 0) //end repeat task
                                                                {
                                                                        $_base_id = $array["base_id"]; //restore base index

                                                                        //retrieve previous parent index from stack
                                                                        $this->parent_id = array_pop($parent_stack);
                                                                        break;
                                                                };

                                                                //jump back
                                                                $_offset = $array["offset"];

                                                                //put updated repeat task on stack
                                                                array_push($repeat_stack, $array);

                                                                break;

                                                        case CMS_TEMPLATE_TYPE_SHIFT:
//.....................................................................................................
                                                                if ($calt_limit) break; //suppressed by alternative block

                                                                //retrieve data from shift stack
                                                                if (count($shift_stack) === 0) break; //no shift data
                                                                $_base_id = array_pop($shift_stack);  //restore base index

                                                                //retrieve previous parent index from stack
                                                                $this->parent_id = array_pop($parent_stack);

                                                                break;

                                                        default:
//.....................................................................................................
                                                                if ($calt_limit) break; //suppressed by alternative block

                                                                //no path extending element
                                                                if (! (CMS_TEMPLATE_TYPE_PATH & $ttype)) break;

                                                                //retrieve previous parent index from stack
                                                                $this->parent_id = array_pop($parent_stack);

                                                                break;
                                                        };
                                                };
                                        };

                                        //continue after closing tag
                                        $offset = $_offset;
                                };
                        }

                        //unparsed data
                        else
                        {
                                //proceed after current position
                                $_offset = $offset;
                                $offset++;
                        };
                };

                //decrease template nesting counter
                $template_depth--;

                return $structure;
        }
//-----------------------------------------------------------------------------------------------------
        function create_cache(&$document,
                              $index,
                              $title       = NULL,
                              $description = NULL,
                              $keyword     = NULL,
                              $action      = NULL,
                              $header      = NULL)
        {
                //merge template structure
                $template = $this->export($document, $index);

                $output   = $this->_parse($document,    //document
                                          $template,    //template
                                          $title,       //title
                                          $description, //description
                                          $keyword,     //keyword
                                          $action,      //action
                                          $header,      //header
                                          NULL,         //base id
                                          TRUE);        //cache

                return $output;
        }
//-----------------------------------------------------------------------------------------------------
        function process_cache(&$document,
                               $template,
                               $title       = NULL,
                               $description = NULL,
                               $keyword     = NULL,
                               $action      = NULL,
                               $header      = NULL)
        {
                $template = explode(CMS_TEMPLATE_CACHE_SEPARATOR, $template);
                $flag     = FALSE;
                $output   = NULL;

                reset($template);
                while (list(, $value) = each($template))
                {
                        //nocache span
                        if ($flag)
                        {
                                list(, $_value) = each($template);
                                $output .= $this->_parse($document,    //document
                                                         $_value,      //template
                                                         $title,       //title
                                                         $description, //description
                                                         $keyword,     //keyword
                                                         $action,      //action
                                                         $header,      //header
                                                         $value);      //base id
                        }

                        //cache span
                        else
                        {
                                $output .= $value;
                        };

                        $flag = ! $flag;
                };

                return $output;
        }
//-----------------------------------------------------------------------------------------------------
        function execute($cms_template_code,
                         $cms_template_document,
                         $cms_template_base_id,
                         $cms_template_path_id,
                         $cms_template_temp_id)
        {
                //temporarily replace error handler
                set_error_handler(__NAMESPACE__ . "\\template_error");

                //import variables
                if (
                     isset($this->execute_vars[$cms_template_temp_id])
                     &&
                     is_array($this->execute_vars[$cms_template_temp_id])
                   )
                        extract($this->execute_vars[$cms_template_temp_id], EXTR_SKIP);

                //execute code
                $cms_template_return = @eval("namespace " . __NAMESPACE__ . ";$cms_template_code");

                //store variables
                $this->execute_vars[$cms_template_temp_id] = get_defined_vars();

                //restore error handler
                restore_error_handler();

                return $cms_template_return;
        }
//-----------------------------------------------------------------------------------------------------
        function attribute($param,
                           $alter = NULL,
                           $skip  = NULL)
        {
                if (! is_array($param)) return FALSE;
                if (is_array($alter))   $param = array_merge($param, $alter);
                if (! is_array($skip))  $skip  = array($skip);

                $_skip  = array("id", "idref", "default", "edit", "title");
                $return = "";

                ksort($param);
                reset($param);
                while (list($key, $value) = each($param))
                {
                        if (stre($key))                     continue;
                        if ($value === NULL)                continue;
                        if (isset($param["_$key"]))         continue;
                        if (in_array($key, $skip))          continue;
                        if (in_array($key, $_skip))         continue;
                        if (streq(substr($key, 0, 1), "_")) $key = substr($key, 1);
                        $return .= " $key=\"" . x($value) . "\"";
                };

                return $return;
        }
};
?>