<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

//-----------------------------------------------------------------------------------------------------
//===< DATA CLASS >====================================================================================
//-----------------------------------------------------------------------------------------------------
//related functions
//.....................................................................................................
function data_sort(&$data,
                   $property,
                   $key = NULL)
{
        $array = array();
        $stack = array(0);
        $count = 0;

        $data->move("first");
        while (
                ($_key = $data->move("next"))
                &&
                ($count >= 0)
              )
        {
                switch ($data->get($_key, "#type"))
                {
                case "container":
//.....................................................................................................
                        $array[$_key]["#!"]           = $data->get($_key, $property);
                        $array[$stack[$count]][$_key] = &$array[$_key];
                        $stack[++$count]              = $_key;
                        break;

                default:
//.....................................................................................................
                        $array[$_key]["#!"]           = $data->get($_key, $property);
                        $array[$stack[$count]][$_key] = &$array[$_key];
                        break;

                case "/container":
//.....................................................................................................
                        $array[$stack[$count]]["#/"]  = $_key;
                        unset($stack[$count]);
                        $count--;
                };
        };

        $array[0]["#/"] = TRUE;

        $stack = array();
        $_key  = stre($key) ? 0 : $key;
        $count = array($_key => 0);

        reset($array[$_key]);
        while (TRUE)
        {
                while (list($__key) = each($array[$_key]))
                {
                        if ($__key === "#!") continue;
                        if ($__key === "#/")
                        {
                                if ($count[$_key]) uasort($array[$_key], __NAMESPACE__ . "\\_data_sort");
                                break;
                        };

                        $count[$_key]++;

                        if (isset($array[$__key]["#/"]))
                        {
                                array_push($stack, $_key);
                                $_key = $__key;
                                $count[$_key] = 0;
                                reset($array[$_key]);
                        };
                };

                unset($count[$_key]);
                $_key = array_pop($stack);
                if ($_key === NULL) break;
        };

        unset($array[0]["#/"]);

        $stack = array();
        $_key  = 0;
        $_data = NULL;

        reset($array[0]);
        while (TRUE)
        {
                while (list($__key) = each($array[$_key]))
                {
                        if ($__key === "#!") continue;
                        if ($__key === "#/") continue;

                        $_data[$__key] = $data->get($__key);

                        if (isset($array[$__key]["#/"]))
                        {
                                array_push($stack, $_key);
                                $_key = $__key;
                                reset($array[$_key]);
                        };
                };

                if (isset($array[$_key]["#/"]))
                {
                        $__key         = $array[$_key]["#/"];
                        $_data[$__key] = $data->get($__key);
                };

                $_key = array_pop($stack);
                if ($_key === NULL) break;
        };

        $data->data = $_data;
};
//-----------------------------------------------------------------------------------------------------
function _data_sort($value1, $value2)
{
        return utf8_strnatcasecmp($value1["#!"], $value2["#!"]);
};

class data
{
//properties
//-----------------------------------------------------------------------------------------------------
        public $file     = NULL;
        public $data     = NULL;
        public $buffer   = NULL;
        public $password = NULL;
//constructor
//-----------------------------------------------------------------------------------------------------
        function __construct($name     = NULL,
                             $password = NULL)
        {
                if ($name !== NULL) $this->open($name, $password);
        }
//methods
//-----------------------------------------------------------------------------------------------------
        function open($name,
                      $password = NULL)
        {
                $this->password = $password;
                $this->file     = streq(substr($name, -4), ".dat") ?
                                  $name : CMS_DATA_PATH . "$name.dat";

                //retrieve from temporary cache
                if ($this->data = cms_cache("data." . $this->file)) return TRUE;

                //retrieve from file
                $this->data = NULL;

                if (
                     is_file($this->file)
                     &&
                     (($hfile = fopen($this->file, "rb")) !== FALSE)
                   )
                {
                        $time = time();
                        $i    = 0;

                        while (! feof($hfile))
                        {
                                //retrieve dataset
                                $value = fgets($hfile, 8192);

                                //index
                                $offset = strpos($value, ":=");
                                if ($offset === FALSE) continue;
                                $key    = decchr(substr($value, 0, $offset));
                                if (stre($key))        continue;

                                //data
                                $value = substr($value, $offset + 2);
                                $value = rtrim($value, "\n\r");
                                $value = explode(";", $value);

                                reset($value);
                                while (list(, $_value) = each($value))
                                {
                                        $offset = strpos($_value, "=");
                                        $_key   = decchr(substr($_value, 0, $offset));
                                        $_value = decchr(substr($_value, $offset + 1));
                                        if ($password !== NULL) $_value = rc4decrypt($_value, $password);

                                        $this->data[$key][$_key] = $_value;
                                };

                                //type
                                $type = isset($this->data[$key]["#type"]) ?
                                        $this->data[$key]["#type"] : "";

                                //automatic expiration
                                if ($i > 0)
                                {
                                        unset($this->data[$key]);

                                        switch ($type)
                                        {
                                        case "container":  $i++; break;
                                        case "/container": $i--; break;
                                        };

                                        continue;
                                };

                                if (
                                     isset($this->data[$key]["#expire"])
                                     &&
                                     ($this->data[$key]["#expire"] < $time)
                                   )
                                {
                                        unset($this->data[$key]);

                                        //erase user data
                                        if (
                                             streq($this->file, CMS_DATA_PATH . "#system/permission.dat")
                                             &&
                                             permission_is_user($key)
                                           )
                                                permission_delete(substr($key, 5));

                                        if (streq($type, "container")) $i = 1;
                                };
                        };

                        fclose($hfile);
                };

                //temporary caching
                cms_cache("data." . $this->file, $this->data);

                return TRUE;
        }
//-----------------------------------------------------------------------------------------------------
        function save($name = NULL)
        {
                //set file name
                if ($name !== NULL)
                        $this->file = streq(substr($name, -4), ".dat") ?
                                      $name : CMS_DATA_PATH . "$name.dat";

                //path
                $path = dirname($this->file);
                if (! mkpath($path))  return FALSE;

                //file
                $hfile = fopen($this->file, "wb");
                if ($hfile === FALSE) return FALSE;

                //process data
                if (is_array($this->data))
                {
                        reset($this->data);
                        while (list($key, $value) = each($this->data))
                        {
                                if (stre($key))         continue; //no key
                                if (! is_array($value)) continue; //no values

                                //key
                                $buffer = encchr($key) . ":=";

                                //values
                                reset($value);
                                while (list($_key, $_value) = each($value))
                                {
                                        if (stre($_value)) continue; //empty value

                                        //encryption
                                        if ($this->password !== NULL)
                                                $_value = rc4encrypt((string)$_value, $this->password);

                                        //property / value pair
                                        $buffer .= encchr($_key) . "=" . encchr($_value) . ";";
                                };

                                fputs($hfile, substr($buffer, 0, -1) . "\n");
                        };
                };

                fclose($hfile);

                //temporary caching
                cms_cache("data." . $this->file, $this->data);

                return TRUE;
        }
//-----------------------------------------------------------------------------------------------------
        function set($value    = NULL,
                     $key      = NULL,
                     $property = NULL)
        {
                if ((! is_array($value)) && stre($value))
                {
                        $this->del($key, $property);
                        return;
                };

                if (stre($key) && stre($property)) $this->data                  = (array)$value;
                elseif (stre($property))           $this->data[$key]            = (array)$value;
                else                               $this->data[$key][$property] = (string)$value;
        }
//-----------------------------------------------------------------------------------------------------
        function get($key      = NULL,
                     $property = NULL)
        {
                if (stre($key) && stre($property))                   return $this->data;
                elseif (stre($property) && isset($this->data[$key])) return $this->data[$key];
                elseif (isset($this->data[$key][$property]))         return $this->data[$key][$property];
                return NULL;
        }
//-----------------------------------------------------------------------------------------------------
        function del($key       = NULL,
                     $property  = NULL,
                     $recursive = TRUE)
        {
                if ($this->data === NULL) return FALSE;

                if (stre($property))
                {
                        if (stre($key))
                        {
                                $this->data = array();
                        }
                        elseif (isset($this->data[$key]))
                        {
                                if ($recursive) return $this->copy($key, "del");

                                $this->move("to", $key);
                                unset($this->data[$key]);

                                $depth = 1;
                                while (
                                        ($depth > 0)
                                        &&
                                        (($key = $this->move("next")) !== NULL)
                                      )
                                {
                                        if (isset($this->data[$key]["#type"]))
                                        {
                                                switch ($this->data[$key]["#type"])
                                                {
                                                case "container":  $depth++; break;
                                                case "/container": $depth--; break;
                                                };
                                        };
                                };

                                unset($this->data[$key]);
                        };
                }
                elseif (isset($this->data[$key][$property]))
                {
                        unset($this->data[$key][$property]);
                };

                return TRUE;
        }
//-----------------------------------------------------------------------------------------------------
        function insert($key = NULL)
        {
                if ($this->buffer === NULL) return FALSE;

                if (stre($key)) $key = 0;
                $buffer = array();
                $_key   = 0;

                $this->move("first");
                do
                {
                        $buffer[$_key] = $this->get($_key);
                        if (streq($_key, $key))
                        {
                                reset($this->buffer);
                                while (list($_key, $_value) = each($this->buffer))
                                {
                                        $buffer[$_key] = $_value;
                                        if (empty($return))
                                        {
                                                end($buffer);
                                                $return = key($buffer);
                                        };
                                };
                        };
                }
                while ($_key = $this->move("next"));

                unset($buffer[0]);
                $this->data = $buffer;

                return empty($return) ? TRUE : $return;
        }
//-----------------------------------------------------------------------------------------------------
        function set_buffer($array)
        {
                $this->buffer = NULL;

                if (! is_array($array)) return FALSE;

                reset($array);
                while (list($key, $value) = each($array))
                        $this->buffer[unique_id()] = $value;

                return TRUE;
        }
//-----------------------------------------------------------------------------------------------------
        function cut($key)
        {
                return $this->copy($key, "cut");
        }
//-----------------------------------------------------------------------------------------------------
        function append($key = NULL)
        {
                return $this->copy($key, "append");
        }
//-----------------------------------------------------------------------------------------------------
        function copy($key,
                      $action = NULL)
        {
                if (streq($action, "append"))
                {
                        if ($this->buffer === NULL) return FALSE;
                        elseif (stre($key))         return $this->insert(0);
                };

                if (! preg_match("/^(append|del)$/", $action)) $this->buffer = NULL;
                if (! isset($this->data[$key]))                return FALSE;

                $depth = 0;
                $key   = $this->move("to", $key);

                do
                {
                        if (isset($this->data[$key]["#type"]))
                        {
                                switch ($this->data[$key]["#type"])
                                {
                                case "container":  $depth++; break;
                                case "/container": $depth--; break;
                                };
                        };

                        switch ($action)
                        {
                        default:
//.....................................................................................................
                                $this->buffer[unique_id()] = $this->data[$key];
                                break;

                        case "cut":
//.....................................................................................................
                                $this->buffer[$key]        = $this->data[$key];

                        case "del":
//.....................................................................................................
                                unset($this->data[$key]);

                        case "append":
//.....................................................................................................
                        };
                }
                while (($depth > 0) && ($key = $this->move("next")));

                if (streq($action, "append")) return $this->insert($key);
                return TRUE;
        }
//-----------------------------------------------------------------------------------------------------
        function seek($condition)
        {
                $this->move("first");
                while (($key = $this->move("next")) !== NULL)
                {
                        reset($condition);
                        while (list($_key, $value) = each($condition))
                        {
                                if (nstreq($this->get($key, $_key), $value))
                                {
                                        $key = NULL;
                                        break;
                                };
                        };

                        if ($key !== NULL) return $key;
                };

                return FALSE;
        }
//-----------------------------------------------------------------------------------------------------
        function move($target = "current", //"current", "first", "last", "prev", "next", "to", "parent"
                      $key    = NULL)
        {
                if ($this->data === NULL) return NULL;

                $_key = key($this->data);

                switch ($target)
                {
                case "first":
//.....................................................................................................
                        reset($this->data);
                        return key($this->data);

                case "last":
//.....................................................................................................
                        end($this->data);
                        return key($this->data);

                case "prev":
//.....................................................................................................
                        prev($this->data);
                        break;

                case "next":
//.....................................................................................................
                        next($this->data);
                        break;

                case "to":
//.....................................................................................................
                        reset($this->data);
                        while ((list($_key) = each($this->data)) && nstreq($key, $_key));
                        break;

                case "parent":
//.....................................................................................................
                        $container = array();

                        reset($this->data);
                        while ((list($_key) = each($this->data)) && nstreq($key, $_key))
                        {
                                switch ($this->get($_key, "#type"))
                                {
                                case "container":
//.....................................................................................................
                                        array_push($container, $_key);
                                        break;

                                case "/container":
//.....................................................................................................
                                        array_pop($container);
                                };
                        };

                        return end($container);
                };

                return $_key;
        }
//-----------------------------------------------------------------------------------------------------
        function is_container($key)
        {
                if (stre($key)) return FALSE;
                return streq($this->get($key, "#type"), "container");
        }
//-----------------------------------------------------------------------------------------------------
        function is_child($key,
                          $parent)
        {
                if (stre($key))                                        return FALSE;
                if (streq($key, $parent))                              return TRUE;
                if (nstreq($this->get($parent, "#type"), "container")) return FALSE;

                $depth = 0;

                reset($this->data);
                while (list($_key, $value) = each($this->data))
                {
                        if (streq($_key, $key)) return (bool)$depth;

                        if (streq($_key, $parent) || ($depth > 0))
                        {
                                switch ($this->get($_key, "#type"))
                                {
                                case "container":
//.....................................................................................................
                                        $depth++;
                                        break;

                                case "/container":
//.....................................................................................................
                                        if (--$depth === 0) return FALSE;
                                        break;
                                };
                        };
                };

                return FALSE;
        }
//-----------------------------------------------------------------------------------------------------
        function has_children($key)
        {
                if (stre($key))                                     return FALSE;
                if (nstreq($this->get($key, "#type"), "container")) return FALSE;

                $this->move("to", $key);
                $_key = $this->move("next");
                if ($_key === NULL) return FALSE;
                $type = $this->get($_key, "#type");

                return nstreq($type, "/container");
        }
};
?>