<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

//-----------------------------------------------------------------------------------------------------
//===< MAP CLASS >=====================================================================================
//-----------------------------------------------------------------------------------------------------

//constants
define("CMS_MAP_INDEX_KEY",   0);
define("CMS_MAP_INDEX_VALUE", 1);
define("CMS_MAP_DATA_KEY",    2);
define("CMS_MAP_DATA_VALUE",  3);

class map
{
//properties
//-----------------------------------------------------------------------------------------------------
        public $file = NULL;
        public $data = array(CMS_MAP_INDEX_KEY   => array(),
                             CMS_MAP_INDEX_VALUE => array(),
                             CMS_MAP_DATA_KEY    => array(),
                             CMS_MAP_DATA_VALUE  => array());
//constructor
//-----------------------------------------------------------------------------------------------------
        function __construct($name = NULL)
        {
                if ($name) $this->open($name);
        }
//methods
//-----------------------------------------------------------------------------------------------------
        function open($name)
        {
                $this->file = CMS_DATA_PATH . "$name.map";

                //retrieve from temporary cache
                if ($this->data = cms_cache("map." . $this->file)) return TRUE;

                //retrieve from file
                $this->data = array(CMS_MAP_INDEX_KEY   => array(),
                                    CMS_MAP_INDEX_VALUE => array(),
                                    CMS_MAP_DATA_KEY    => array(),
                                    CMS_MAP_DATA_VALUE  => array());

                if (
                     is_file($this->file)
                     &&
                     ($hfile = fopen($this->file, "rb"))
                   )
                {
                        $i = 1;

                        while (
                                (! feof($hfile))
                                &&
                                (($key = rtrim(fgets($hfile, 1024))) !== FALSE)
                                &&
                                (($value = rtrim(fgets($hfile, 1024))) !== FALSE)
                              )
                        {
                                $this->data[CMS_MAP_INDEX_KEY][crc32($key)]     = $i;
                                $this->data[CMS_MAP_INDEX_VALUE][crc32($value)] = $i;
                                $this->data[CMS_MAP_DATA_KEY][$i]               = $key;
                                $this->data[CMS_MAP_DATA_VALUE][$i++]           = $value;
                        };

                        fclose($hfile);
                };

                //temporary caching
                cms_cache("map." . $this->file, $this->data);

                return TRUE;
        }
//-----------------------------------------------------------------------------------------------------
        function save($name = NULL)
        {
                if ($name) $this->file = CMS_DATA_PATH . "$name.map";

                $path = dirname($this->file);

                if (! mkpath($path))                     return FALSE;
                if (! $hfile = fopen($this->file, "wb")) return FALSE;

                reset($this->data[CMS_MAP_DATA_KEY]);
                while (list($key, $value) = each($this->data[CMS_MAP_DATA_KEY]))
                        fputs($hfile, "$value\n" . $this->data[CMS_MAP_DATA_VALUE][$key] . "\n");

                fclose($hfile);

                //temporary caching
                cms_cache("map." . $this->file, $this->data);

                return TRUE;
        }
//-----------------------------------------------------------------------------------------------------
        function set($key,
                     $value)
        {
                $_key   = crc32($key);
                $_value = crc32($value);

                if (isset($this->data[CMS_MAP_INDEX_KEY][$_key]))
                {
                        $index = $this->data[CMS_MAP_INDEX_KEY][$_key];
                }
                else
                {
                        end($this->data[CMS_MAP_DATA_KEY]);
                        $index = key($this->data[CMS_MAP_DATA_KEY]) + 1;
                };

                $this->data[CMS_MAP_INDEX_KEY][$_key]     = $index;
                $this->data[CMS_MAP_INDEX_VALUE][$_value] = $index;
                $this->data[CMS_MAP_DATA_KEY][$index]     = $key;
                $this->data[CMS_MAP_DATA_VALUE][$index]   = $value;

                return TRUE;
        }
//-----------------------------------------------------------------------------------------------------
        function get_value($key)
        {
                $key = crc32($key);

                return isset($this->data[CMS_MAP_INDEX_KEY][$key]) ?
                       $this->data[CMS_MAP_DATA_VALUE][$this->data[CMS_MAP_INDEX_KEY][$key]] :
                       NULL;
        }
//-----------------------------------------------------------------------------------------------------
        function get_value_list()
        {
                return array_values($this->data[CMS_MAP_DATA_VALUE]);
        }
//-----------------------------------------------------------------------------------------------------
        function get_key($value)
        {
                $value = crc32($value);

                return isset($this->data[CMS_MAP_INDEX_VALUE][$value]) ?
                       $this->data[CMS_MAP_DATA_KEY][$this->data[CMS_MAP_INDEX_VALUE][$value]] :
                       NULL;
        }
//-----------------------------------------------------------------------------------------------------
        function del_key($key)
        {
                $key = crc32($key);

                if (isset($this->data[CMS_MAP_INDEX_KEY][$key]))
                {
                        $index = $this->data[CMS_MAP_INDEX_KEY][$key];
                        $value = crc32($this->data[CMS_MAP_DATA_VALUE][$index]);

                        unset($this->data[CMS_MAP_INDEX_KEY][$key]);
                        unset($this->data[CMS_MAP_INDEX_VALUE][$value]);
                        unset($this->data[CMS_MAP_DATA_KEY][$index]);
                        unset($this->data[CMS_MAP_DATA_VALUE][$index]);
                };
        }
//-----------------------------------------------------------------------------------------------------
        function del_value($value)
        {
                $value = crc32($value);

                if (isset($this->data[CMS_MAP_INDEX_VALUE][$value]))
                {
                        $index = $this->data[CMS_MAP_INDEX_VALUE][$value];
                        $key   = crc32($this->data[CMS_MAP_DATA_KEY][$index]);

                        unset($this->data[CMS_MAP_INDEX_KEY][$key]);
                        unset($this->data[CMS_MAP_INDEX_VALUE][$value]);
                        unset($this->data[CMS_MAP_DATA_KEY][$index]);
                        unset($this->data[CMS_MAP_DATA_VALUE][$index]);
                };
        }
};
?>