/*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

var defer_list = new Array();

function defer_init()
{
        load_css("nuos/stylesheet/defer.css");

        switch (document.readyState)
        {
        case "interactive":
                _defer_scan();
                break;

        case "complete":
                defer_scan();
                defer_finish();
                break;

        default:
        case "loading":
                fx_event_listen(document, "DOMContentLoaded", defer_event);
        };
};

function defer_scan()
{
        var object = null;
        var list   = document.getElementsByTagName("img");

        for (var i = 0, c = list.length; i < c; i++)
        {
                object = list.item(i);
                if (object.hasAttribute("data-defer-src")) defer_list.push(object);
        };
};

function _defer_scan()
{
        defer_scan();
        if (defer_list.length === 0) return;
        fx_event_listen(window, "load", defer_event);
        fx_event_listen(window, "scroll", defer_event);
        fx_event_listen(window, "resize", defer_event);
};

function defer_finish()
{
        for (var i = 0, c = defer_list.length; i < c; i++) defer_load(defer_list[i]);
        defer_list = new Array();
};

function defer_load(object)
{
        fx_event_listen(object, "load", _defer_load);

        if (object.hasAttribute("data-defer-srcset"))
        {
                object.setAttribute("srcset", object.getAttribute("data-defer-srcset"));
                object.removeAttribute("data-defer-srcset");
        };

        if (object.hasAttribute("data-defer-sizes"))
        {
                object.setAttribute("sizes", object.getAttribute("data-defer-sizes"));
                object.removeAttribute("data-defer-sizes");
        };

        object.setAttribute("src", object.getAttribute("data-defer-src"));
};

function _defer_load()
{
        this.removeAttribute("data-defer-src");
        fx_event_remove(this, "load", _defer_load);
};

function defer_event(event)
{
        if ((event.currentTarget === window) && (event.type === "load"))
        {
                defer_finish();
        }
        else
        {
                if ((event.currentTarget === document) && (event.type === "DOMContentLoaded"))
                {
                        fx_event_remove(document, "DOMContentLoaded", defer_event);
                        _defer_scan();
                };

                var object    = null;
                var object_x1 = 0;
                var object_y1 = 0;
                var object_x2 = 0;
                var object_y2 = 0;
                var page_x1   = 0;
                var page_y1   = 0;
                var page_x2   = 0;
                var page_y1   = 0;

                var i = 0;
                while (i < defer_list.length)
                {
                        object    = defer_list[i];
                        object_x1 = fx_offset_left(object);
                        page_x1   = fx_page_left();
                        page_x2   = page_x1 + fx_window_width;

                        if (object_x1 < page_x2)
                        {
                                object_y1 = fx_offset_top(object);
                                page_y1   = fx_page_top();
                                page_y2   = page_y1 + fx_window_height;

                                if (object_y1 < page_y2)
                                {
                                        object_x2 = object_x1 + fx_width(object);

                                        if (object_x2 > page_x1)
                                        {
                                                object_y2 = object_y1 + fx_height(object);

                                                if (object_y2 > page_y1)
                                                {
                                                        defer_load(object);
                                                        defer_list.splice(i, 1);
                                                        continue;
                                                };
                                        };
                                };
                        };

                        i++;
                };
        };

        if (defer_list.length === 0)
        {
                fx_event_remove(window, "load", defer_event);
                fx_event_remove(window, "scroll", defer_event);
                fx_event_remove(window, "resize", defer_event);
        };
};