/*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

//--------------------------------
//    ANIMATION
//--------------------------------
function fx_check_speed(callback)
{
        var _this  = this;
        this.flag  = false;
        this.count = 0;

        this.loop = function()
        {
                if (_this.flag) return;
                _this.count++;
                requestAnimationFrame(function() { _this.loop(); });
        };

        requestAnimationFrame(this.loop);
        setTimeout(function() { this.flag = true; callback(this.count); }, 1000);
};
//................................
function fx_animation_frame(callback) //arguments: 1 => delay
{
        if (typeof callback == "string") callback = new Function(callback);

        if (arguments.length > 1)
        {
                var delay = arguments[1];
                return setTimeout(function() { requestAnimationFrame(callback); }, delay);
        };

        return requestAnimationFrame(callback);
};
//................................
function fx_create_image(object,
                         image_url)
{
        var _object = document.createElement("img");

        with (_object)
        {
                id             = object;
                src            = image_url;
                style.position = "absolute";
        };

        document.getElementsByTagName("body")[0].appendChild(_object);
};
//................................
function fx_change_image(object,
                         image_url)
{
        if (typeof object != "object") object = document.getElementById(object);
        if (object == null)            return;

        if (object.tagName == "IMG")   object.src = image_url;
};
//................................
function fx_move(object,
                 _x,
                 _y)
{
        if (typeof object != "object") object = document.getElementById(object);
        if (object == null)            return;

        _x      = Math.floor(_x);
        _y      = Math.floor(_y);
        visible = fx_visible(object);

        with (object.style)
        {
                if ((parseInt(left) != _x) || (parseInt(top) != _y))
                {
                        if (visible) visibility = "hidden";
                        left = _x + "px";
                        top  = _y + "px";
                        if (visible) visibility = "visible";
                };
        };
};
//................................
function fx_scrollto(object)
{
        if (typeof object != "object") object = document.getElementById(object);
        if (object == null)            return;

        var _object  = object;
        while (_object.tagName != "BODY")
        {
                //if (fx_style(_object, "position") == "fixed") return;
                if ((_object = _object.offsetParent) == null) return;
        };

        var frames     = 200;
        var interval   = 5;
        var damping    = interval / 2;
        var pifraction = Math.PI / frames;
        var time       = 0;

        var _x         = fx_page_left();
        var _y         = fx_page_top();
        var xtarget    = fx_left(object);
        var ytarget    = fx_top(object);
        var xleap      = (xtarget - _x) / frames;
        var yleap      = (ytarget - _y) / frames;

        for (var frame = 0; frame < frames; frame++)
        {
                time += interval + damping * Math.sin(Math.PI + pifraction * frame);
                _x   += xleap;
                _y   += yleap;

                fx_animation_frame("window.scrollTo(" + _x + ", " + _y + ");", time);
        };
};
//................................
function fx_visible(object) //arguments: 1 => set
{
        if (typeof object != "object") object = document.getElementById(object);
        if (object == null)            return false;

        if (arguments.length > 1)
        {
                object.style.visibility = arguments[1] ? "visible" : "hidden";
                return arguments[1];
        };

        return fx_style(object, "visibility") == "visible";
};
//................................
function fx_transparency(object,
                         value)
{
        if (typeof object != "object") object = document.getElementById(object);
        if (object == null)            return false;

        value = Math.floor(value);
        if (value < 0)        value =   0;
        else if (value > 100) value = 100;

        object.style.opacity = value / 100;
        object.style.filter  = "Alpha(opacity=" + value + ")";
};
//................................
function fx_style(object,
                  property) //arguments: 2 => value
{
        if (typeof object != "object") object = document.getElementById(object);
        if (object == null)            return false;

        property = property.toLowerCase();

        //set value
        if (arguments.length > 2)
        {
                property = property.replace(/\-(\w)/g, function(a, b) { return b.toUpperCase(); });
                eval("object.style." + property + " = \"" + arguments[2] + "\";");
                return true;
        };

        //retrieve value (ie)
        if (object.currentStyle)
        {
                property = property.replace(/\-(\w)/g, function(a, b) { return b.toUpperCase(); });
                return object.currentStyle[property];
        };

        //retrieve value (dom)
        if (window.getComputedStyle)
        {
                return document.defaultView.getComputedStyle(object).getPropertyValue(property);
        };

        //retrieve value (fallback)
        property = property.replace(/\-(\w)/g, function(a, b) { return b.toUpperCase(); });
        return eval("object.style." + property);
};
//................................
function fx_left(object) //arguments: 1 => relative
{
        if (typeof object != "object") object = document.getElementById(object);
        if (object == null)            return 0;

        var relative = (arguments.length > 1) && arguments[1];
        if (relative) return object.offsetLeft;

        var value = 0;

        do
        {
                value += object.offsetLeft;
        }
        while (object = object.offsetParent);

        return value;
}
//................................
function fx_top(object) //arguments: 1 => relative
{
        if (typeof object != "object") object = document.getElementById(object);
        if (object == null)            return 0;

        var relative = (arguments.length > 1) && arguments[1];
        if (relative) return object.offsetTop;

        var value = 0;

        do
        {
                value += object.offsetTop;
        }
        while (object = object.offsetParent);

        return value;
}
//................................
function fx_offset_left(object) //arguments: 1 => no cropping
{
        if (typeof object != "object") object = document.getElementById(object);
        if (object == null)            return 0;

        var no_cropping = (arguments.length > 1) && arguments[1];
        var parent      = object.offsetParent;
        var value       = object.offsetLeft;
        var _value      = 0;
        var body        = document.getElementsByTagName("body")[0];
        var root        = document.documentElement;

        while (object = object.parentNode)
        {
                if ((object == body) || (object == root)) break;

                if (object.scrollLeft)
                {
                        value  -= object.scrollLeft;
                        _value  = object.offsetLeft;
                };

                if (object == parent)
                {
                        value  += object.offsetLeft;
                        _value += object.offsetLeft;
                        parent  = object.offsetParent;
                };
        };

        if ((! no_cropping) && (value < _value)) value = _value;
        return value;
};
//................................
function fx_offset_top(object) //arguments: 1 => no cropping
{
        if (typeof object != "object") object = document.getElementById(object);
        if (object == null)            return 0;

        var no_cropping = (arguments.length > 1) && arguments[1];
        var parent      = object.offsetParent;
        var value       = object.offsetTop;
        var _value      = 0;
        var body        = document.getElementsByTagName("body")[0];
        var root        = document.documentElement;

        while (object = object.parentNode)
        {
                if ((object == body) || (object == root)) break;

                if (object.scrollTop)
                {
                        value  -= object.scrollTop;
                        _value  = object.offsetTop;
                };

                if (object == parent)
                {
                        value  += object.offsetTop;
                        _value += object.offsetTop;
                        parent  = object.offsetParent;
                };
        };

        if ((! no_cropping) && (value < _value)) value = _value;
        return value;
};
//................................
function fx_width(object) //arguments: 1 => no cropping
{
        if (typeof object != "object") object = document.getElementById(object);
        if (object == null)            return 0;

        var no_cropping = (arguments.length > 1) && arguments[1];
        if (no_cropping) return object.offsetWidth;

        var parent      = object;
        var value       = object.offsetWidth;
        var _value      = 0;
        var offset      = fx_offset_left(object);
        var _offset     = 0;
        var body        = document.getElementsByTagName("body")[0];
        var root        = document.documentElement;

        while (parent = parent.parentNode)
        {
                if ((parent == body) || (parent == root)) break;

                _offset = fx_offset_left(parent, true);
                _value  = parent.offsetWidth - offset + _offset;

                if (_value < value) value = _value;
        };

        return value;
};
//................................
function fx_height(object) //arguments: 1 => no cropping
{
        if (typeof object != "object") object = document.getElementById(object);
        if (object == null)            return 0;

        var no_cropping = (arguments.length > 1) && arguments[1];
        if (no_cropping) return object.offsetHeight;

        var parent  = object;
        var value   = object.offsetHeight;
        var _value  = 0;
        var offset  = fx_offset_top(object);
        var _offset = 0;
        var body    = document.getElementsByTagName("body")[0];
        var root    = document.documentElement;

        while (parent = parent.parentNode)
        {
                if ((parent == body) || (parent == root)) break;

                _offset = fx_offset_top(parent, true);
                _value  = parent.offsetHeight - offset + _offset;

                if (_value < value) value = _value;
        };

        return value;
};
//................................
function fx_page_left()
{
        if (typeof window.pageYOffset == "number")
                return window.pageXOffset;

        if (document.documentElement && document.documentElement.scrollLeft)
                return document.documentElement.scrollLeft;

        if (document.body)
                return document.body.scrollLeft;

        return 0;
};
//................................
function fx_page_top()
{
        if (typeof window.pageYOffset == "number")
                return window.pageYOffset;

        if (document.documentElement && document.documentElement.scrollTop)
                return document.documentElement.scrollTop;

        if (document.body)
                return document.body.scrollTop;

        return 0;
};
//................................
var fx_window_width  = 0;
var fx_window_height = 0;

function fx_update_page_size()
{
        if (typeof window.innerWidth == "number")
        {
                fx_window_width  = window.innerWidth;
                fx_window_height = window.innerHeight;
        }
        else if (document.documentElement && document.documentElement.clientWidth)
        {
                fx_window_width  = document.documentElement.clientWidth;
                fx_window_height = document.documentElement.clientHeight;
        }
        else if (document.body)
        {
                fx_window_width  = document.body.clientWidth;
                fx_window_height = document.body.clientHeight;
        };
};
//................................
function fx_adjust_window()
{
        var object = (arguments.length > 0) ? arguments[0] : window;

        if (! object instanceof Window) return;
        if (! object.opener)            return;

        var width   = object.outerWidth;
        var height  = object.outerHeight;
        var root    = object.document.documentElement;
        var _width  = root.scrollWidth  + width  - object.innerWidth;
        var _height = root.scrollHeight + height - object.innerHeight;

        if (_width  < width)              _width  = width;
        if (_width  > screen.availWidth)  _width  = screen.availWidth;
        if (_height < height)             _height = height;
        if (_height > screen.availHeight) _height = screen.availHeight;

        if (width - _width + height - _height)
        {
                var left  = (typeof object.screenLeft == "number") ? object.screenLeft : object.screenX;
                var top   = (typeof object.screenTop  == "number") ? object.screenTop  : object.screenY;
                var _left = screen.availWidth  - _width;
                var _top  = screen.availHeight - _height;

                if (_left > left) _left = left;
                if (_top  > top)  _top  = top;

                if (left - _left + top - _top) object.moveTo(_left, _top);

                object.resizeTo(_width, _height);
        };

        object.focus();
};
//--------------------------------
//    EVENTS
//--------------------------------
/*
 * register callback function
 *
 * example 1:
 * function my_callback_function(my_event) { ... };
 * fx_register_callback(my_callback_function);
 *
 * example 2:
 * function my_callback_function(my_event) { ... };
 * function my_class() { ... this.callback = my_callback_function; ... };
 * var my_instance = new my_class();
 * fx_register_callback(my_instance);
 *
 * possible values for my_event:
 * "window_load", "window_scroll", "window_resize", "window_unload",
 * "mousedown",   "mousemove",     "mouseup",
 * "keydown",     "keypress",      "keyup"
 * "dblclick"
 */

var fx_callback   = new Array();
var fx_event      = new Array(10);
var fx_event_slot = -1;

function fx_register_callback(callback)
{
        fx_callback.push(callback);
};
//................................
function fx_unregister_callback(callback)
{
        for (var i = 0; i < fx_callback.length; i++)
        {
                if (fx_callback[i] == callback) fx_callback[i] = false;
        };
};
//................................
function fx_event_raise(event)
{
        requestAnimationFrame(function() { _fx_event_raise(event); });
};
//................................
function _fx_event_raise(event)
{
        if (fx_event_slot >= 9) return;

        var list = new Array();
        var slot = ++fx_event_slot;
        var callback;
        var data;

        fx_event[slot] = new Array(
                event,
                fx_window_width, fx_window_height,
                fx_mouse_key,
                fx_mouse_x, fx_mouse_y,
                fx_keyboard_key);

        for (var i = 0; i < fx_callback.length; i++)
        {
                callback = fx_callback[i];
                if (callback == false) continue;

                list.push(callback);

                switch (typeof callback)
                {
                case "object":
                        callback.callback(event);
                        break;

                case "function":
                        callback(event);
                };

                data = fx_event[slot];
                if (data == false) break;

                fx_window_width  = data[1];
                fx_window_height = data[2];
                fx_mouse_key     = data[3];
                fx_mouse_x       = data[4];
                fx_mouse_y       = data[5];
                fx_keyboard_key  = data[6];
        };

        fx_callback = list;
        fx_event_consume();
};
//................................
function fx_event_consume()
{
        if (fx_event_slot < 0) return;
        fx_event[fx_event_slot--] = false;
};
//................................
function fx_event_listen(object,
                         event,
                         _function)
{
        if (object.addEventListener) object.addEventListener(event, _function);
        else if (object.attachEvent) object.attachEvent("on" + event, _function);
};
//................................
function fx_event_remove(object,
                         event,
                         _function)
{
        if (object.removeEventListener) object.removeEventListener(event, _function);
        else if (object.detachEvent)    object.attachEvent("on" + event, _function);
};
//................................
var fx_event_debounce_list = new Object();

function fx_event_debounce(event)
{
        if (! fx_event_debounce_list[event]) fx_event_debounce_list[event] = new Array(0, false);

        var flag = fx_event_debounce_list[event][1];
        if (flag) return;

        var fps      = 15;
        var time     = fx_event_debounce_list[event][0];
        var interval = (1000 / fps) - (Date.now() - time);

        if (interval <= 0) { _fx_event_debounce(event); return; };

        fx_event_debounce_list[event][1] = true; //set flag
        setTimeout("_fx_event_debounce(\"" + event + "\");", interval);
};
//................................
function _fx_event_debounce(event)
{
        fx_event_debounce_list[event] = new Array(Date.now(), false);
        fx_event_raise(event);
};
//--------------------------------
//    VARIOUS
//--------------------------------
function fx_event_window_onload(e)
{
        fx_update_page_size();
        fx_event_raise("window_load");
};
//................................
function fx_event_window_onscroll(e)
{
        fx_update_mouse_position(e);
        fx_event_debounce("window_scroll");
};
//................................
function fx_event_window_onresize(e)
{
        fx_update_page_size();
        fx_event_debounce("window_resize");
};
//................................
function fx_event_window_onunload(e)
{
        fx_update_page_size();
        fx_event_raise("window_unload");
};
//................................
function fx_event_disabled() { return false };
//................................
//    MOUSE
//................................
var fx_mouse_key = false;

function fx_event_onmousedown(e)
{
        if (window.event)
        {
                fx_mouse_key = window.event.button;
                if (fx_mouse_key == 2)      fx_mouse_key = 3;
                else if (fx_mouse_key == 4) fx_mouse_key = 2;
        }
        else
        {
                fx_mouse_key = e.which;
        };

        fx_update_mouse_position(e);
        fx_event_raise("mousedown");
};
//................................
function fx_event_onmousemove(e)
{
        fx_update_mouse_position(e);
        fx_event_debounce("mousemove");
};
//................................
function fx_event_onmouseup(e)
{
        fx_mouse_key = false;
        fx_update_mouse_position(e);
        fx_event_raise("mouseup");
};
//................................
function fx_event_ondblclick(e)
{
        fx_event_raise("dblclick");
};
//................................
var fx_mouse_x = 0;
var fx_mouse_y = 0;

function fx_update_mouse_position(e)
{
        if (window.event)
        {
                fx_mouse_x = fx_page_left() + window.event.clientX;
                fx_mouse_y = fx_page_top()  + window.event.clientY;
        }
        else
        {
                fx_mouse_x = e.pageX;
                fx_mouse_y = e.pageY;
        };
}
//................................
//    KEYBOARD
//................................
var fx_keyboard_key = false;

function fx_event_onkeydown(e)
{
        if (window.event) fx_keyboard_key = window.event.keyCode;
        else              fx_keyboard_key = e.which
        fx_event_raise("keydown");
};
//................................
function fx_event_onkeypress()
{
        fx_event_raise("keypress");
};
//................................
function fx_event_onkeyup()
{
        fx_keyboard_key = false;
        fx_event_raise("keyup");
};

fx_event_listen(window,   "load",         fx_event_window_onload);
fx_event_listen(window,   "scroll",       fx_event_window_onscroll);
fx_event_listen(window,   "resize",       fx_event_window_onresize);
fx_event_listen(window,   "beforeunload", fx_event_window_onunload);
fx_event_listen(document, "keydown",      fx_event_onkeydown);
fx_event_listen(document, "keypress",     fx_event_onkeypress);
fx_event_listen(document, "keyup",        fx_event_onkeyup);
fx_event_listen(document, "mousedown",    fx_event_onmousedown);
fx_event_listen(document, "mousemove",    fx_event_onmousemove);
fx_event_listen(document, "mouseup",      fx_event_onmouseup);
fx_event_listen(document, "dblclick",     fx_event_ondblclick);

function fx_swipe(object,
                  callback)
{
        if (typeof object != "object") object = document.getElementById(object);
        if (object == null)            return;

        var active = false;
        var ignore = false;
        var x1     = 0, y1 = 0, x2 = 0, y2 = 0, x3 = 0, y3 = 0;

        var execute_swipe = function()
        {
                active = false;

                var diff_x = x2 - x1;
                var diff_y = y2 - y1;
                var abs_x  = Math.abs(diff_x);
                var abs_y  = Math.abs(diff_y);
                var dir    = "";

                if ((abs_x > abs_y) && (abs_x > 20)) dir = (diff_x > 0) ? "r" : "l";
                else if (abs_y > 20)                 dir = (diff_y > 0) ? "d" : "u";

                if ((dir !== "") && (typeof callback === "function"))
                {
                        ignore = true;
                        callback(object, dir);
                };
        };

        var correct_direction = function()
        {
                var diff_x = x2 - x1;
                var diff_y = y2 - y1;
                var abs_x  = Math.abs(diff_x);
                var abs_y  = Math.abs(diff_y);

                if (abs_x < x3) x1 = x2;
                if (abs_y < y3) y1 = y2;

                x3 = abs_x;
                y3 = abs_y;
        };

        fx_event_listen(object, "mousedown", function(e)
        {
                active  = true;
                ignore  = false;
                x1 = x2 = window.event ? fx_page_left() + window.event.clientX : e.pageX;
                y1 = y2 = window.event ? fx_page_top()  + window.event.clientY : e.pageY;
        });

        fx_event_listen(object, "mousemove", function(e)
        {
                if (! active) return;
                x2 = window.event ? fx_page_left() + window.event.clientX : e.pageX;
                y2 = window.event ? fx_page_top()  + window.event.clientY : e.pageY;
                correct_direction();
        });


        fx_event_listen(object, "dragstart", function(e)
        {
                e.preventDefault();
        });

        fx_event_listen(object, "mouseup", function(e)
        {
                execute_swipe();
        });

        fx_event_listen(object, "click", function(e)
        {
                if (ignore) e.preventDefault();
        });

        fx_event_listen(object, "touchstart", function(e)
        {
                var touch = e.touches[0];
                x1 = x2   = touch.screenX;
                y1 = y2   = touch.screenY;
        });

        fx_event_listen(object, "touchmove", function(e)
        {
                var touch = e.touches[0];
                x2        = touch.screenX;
                y2        = touch.screenY;
                correct_direction();
        });

        fx_event_listen(object, "touchend", function(e)
        {
                execute_swipe();
        });
};