/*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

function md5_hex(num)
{
        var characters = "0123456789abcdef";
        var ret        = "";

        for(i = 0; i < 4; i++)
                ret += characters.charAt((num >> (i * 8 + 4)) & 0x0F) +
                       characters.charAt((num >> (i * 8)) & 0x0F);

        return ret;
};
//................................
function md5_convert(str)
{
        var c   = ((str.length + 8) >> 6) + 1;
        var ret = new Array(c * 16);

        for(i = 0; i < c* 16; i++)      ret[i] = 0;
        for(i = 0; i < str.length; i++) ret[i >> 2] |= str.charCodeAt(i) << ((i % 4) * 8);

        ret[i >> 2] |= 0x80 << ((i % 4) * 8);
        ret[c * 16 - 2] = str.length * 8;

        return ret;
};
//................................
function md5_add(a, b)
{
        var lsw = (a & 0xFFFF) + (b & 0xFFFF);
        var msw = (a >> 16) + (b >> 16) + (lsw >> 16);

        return (msw << 16) | (lsw & 0xFFFF);
};
//................................
function md5_shift_bit(num, cnt)
{
        return (num << cnt) | (num >>> (32 - cnt));
};
//................................
function md5_cmn(q, a, b, x, s, t)   { return md5_add(md5_shift_bit(md5_add(md5_add(a, q), md5_add(x, t)), s), b); };
function md5_ff(a, b, c, d, x, s, t) { return md5_cmn((b & c) | ((~b) & d), a, b, x, s, t); };
function md5_gg(a, b, c, d, x, s, t) { return md5_cmn((b & d) | (c & (~d)), a, b, x, s, t); };
function md5_hh(a, b, c, d, x, s, t) { return md5_cmn(b ^ c ^ d, a, b, x, s, t); };
function md5_ii(a, b, c, d, x, s, t) { return md5_cmn(c ^ (b | (~d)), a, b, x, s, t); };
//................................
function md5(str)
{
        var x = md5_convert(str);
        var a =  1732584193;
        var b = -271733879;
        var c = -1732584194;
        var d =  271733878;

        for(i = 0; i < x.length; i += 16)
        {
                var prev_a = a;
                var prev_b = b;
                var prev_c = c;
                var prev_d = d;

                a = md5_ff(a, b, c, d, x[i +  0], 7 , -680876936);
                d = md5_ff(d, a, b, c, x[i +  1], 12, -389564586);
                c = md5_ff(c, d, a, b, x[i +  2], 17,  606105819);
                b = md5_ff(b, c, d, a, x[i +  3], 22, -1044525330);
                a = md5_ff(a, b, c, d, x[i +  4], 7 , -176418897);
                d = md5_ff(d, a, b, c, x[i +  5], 12,  1200080426);
                c = md5_ff(c, d, a, b, x[i +  6], 17, -1473231341);
                b = md5_ff(b, c, d, a, x[i +  7], 22, -45705983);
                a = md5_ff(a, b, c, d, x[i +  8], 7 ,  1770035416);
                d = md5_ff(d, a, b, c, x[i +  9], 12, -1958414417);
                c = md5_ff(c, d, a, b, x[i + 10], 17, -42063);
                b = md5_ff(b, c, d, a, x[i + 11], 22, -1990404162);
                a = md5_ff(a, b, c, d, x[i + 12], 7 ,  1804603682);
                d = md5_ff(d, a, b, c, x[i + 13], 12, -40341101);
                c = md5_ff(c, d, a, b, x[i + 14], 17, -1502002290);
                b = md5_ff(b, c, d, a, x[i + 15], 22,  1236535329);

                a = md5_gg(a, b, c, d, x[i +  1], 5 , -165796510);
                d = md5_gg(d, a, b, c, x[i +  6], 9 , -1069501632);
                c = md5_gg(c, d, a, b, x[i + 11], 14,  643717713);
                b = md5_gg(b, c, d, a, x[i +  0], 20, -373897302);
                a = md5_gg(a, b, c, d, x[i +  5], 5 , -701558691);
                d = md5_gg(d, a, b, c, x[i + 10], 9 ,  38016083);
                c = md5_gg(c, d, a, b, x[i + 15], 14, -660478335);
                b = md5_gg(b, c, d, a, x[i +  4], 20, -405537848);
                a = md5_gg(a, b, c, d, x[i +  9], 5 ,  568446438);
                d = md5_gg(d, a, b, c, x[i + 14], 9 , -1019803690);
                c = md5_gg(c, d, a, b, x[i +  3], 14, -187363961);
                b = md5_gg(b, c, d, a, x[i +  8], 20,  1163531501);
                a = md5_gg(a, b, c, d, x[i + 13], 5 , -1444681467);
                d = md5_gg(d, a, b, c, x[i +  2], 9 , -51403784);
                c = md5_gg(c, d, a, b, x[i +  7], 14,  1735328473);
                b = md5_gg(b, c, d, a, x[i + 12], 20, -1926607734);

                a = md5_hh(a, b, c, d, x[i +  5], 4 , -378558);
                d = md5_hh(d, a, b, c, x[i +  8], 11, -2022574463);
                c = md5_hh(c, d, a, b, x[i + 11], 16,  1839030562);
                b = md5_hh(b, c, d, a, x[i + 14], 23, -35309556);
                a = md5_hh(a, b, c, d, x[i +  1], 4 , -1530992060);
                d = md5_hh(d, a, b, c, x[i +  4], 11,  1272893353);
                c = md5_hh(c, d, a, b, x[i +  7], 16, -155497632);
                b = md5_hh(b, c, d, a, x[i + 10], 23, -1094730640);
                a = md5_hh(a, b, c, d, x[i + 13], 4 ,  681279174);
                d = md5_hh(d, a, b, c, x[i +  0], 11, -358537222);
                c = md5_hh(c, d, a, b, x[i +  3], 16, -722521979);
                b = md5_hh(b, c, d, a, x[i +  6], 23,  76029189);
                a = md5_hh(a, b, c, d, x[i +  9], 4 , -640364487);
                d = md5_hh(d, a, b, c, x[i + 12], 11, -421815835);
                c = md5_hh(c, d, a, b, x[i + 15], 16,  530742520);
                b = md5_hh(b, c, d, a, x[i +  2], 23, -995338651);

                a = md5_ii(a, b, c, d, x[i +  0], 6 , -198630844);
                d = md5_ii(d, a, b, c, x[i +  7], 10,  1126891415);
                c = md5_ii(c, d, a, b, x[i + 14], 15, -1416354905);
                b = md5_ii(b, c, d, a, x[i +  5], 21, -57434055);
                a = md5_ii(a, b, c, d, x[i + 12], 6 ,  1700485571);
                d = md5_ii(d, a, b, c, x[i +  3], 10, -1894986606);
                c = md5_ii(c, d, a, b, x[i + 10], 15, -1051523);
                b = md5_ii(b, c, d, a, x[i +  1], 21, -2054922799);
                a = md5_ii(a, b, c, d, x[i +  8], 6 ,  1873313359);
                d = md5_ii(d, a, b, c, x[i + 15], 10, -30611744);
                c = md5_ii(c, d, a, b, x[i +  6], 15, -1560198380);
                b = md5_ii(b, c, d, a, x[i + 13], 21,  1309151649);
                a = md5_ii(a, b, c, d, x[i +  4], 6 , -145523070);
                d = md5_ii(d, a, b, c, x[i + 11], 10, -1120210379);
                c = md5_ii(c, d, a, b, x[i +  2], 15,  718787259);
                b = md5_ii(b, c, d, a, x[i +  9], 21, -343485551);

                a = md5_add(a, prev_a);
                b = md5_add(b, prev_b);
                c = md5_add(c, prev_c);
                d = md5_add(d, prev_d);
        };

        return md5_hex(a) + md5_hex(b) + md5_hex(c) + md5_hex(d);
};