/*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

var mv_registry      = new Array();
var mv_object        = null;
var mv_onselectstart = null;
var mv_left          = 0;
var mv_top           = 0;
var mv_offset_left   = 0;
var mv_offset_top    = 0;
var mv_fixed         = false;
var mv_flag          = false;
var mv_no_cropping   = true;
var mv_scroll_flag   = false;
var mv_scroll_x      = 0;
var mv_scroll_y      = 0;

function mv_register(object)
{
        if (typeof object != "object") object = document.getElementById(object);
        if (object == null)            return false;

        //event
        object.ondragstart   = new Function("return false;");
        object.onmousedown   = new Function("return false;");
        object.onselectstart = new Function("return false;");

        //add object
        var zindex           = mv_registry.length;
        mv_registry[zindex]  = object;
};
//................................
function mv_get_object()
{
        var object, parent, _object = null,
            x, y, z, _z = 0;

        for (var i = 0; i < mv_registry.length; i++)
        {
                object = mv_registry[i];

                if (
                     ((mv_object == null) || (mv_object != object))
                     &&
                     (fx_mouse_x >= (x = mv_no_cropping ? fx_left(object) : fx_offset_left(object)))
                     &&
                     (fx_mouse_y >= (y = mv_no_cropping ? fx_top(object)  : fx_offset_top(object)))
                     &&

                //check uncropped dimensions first to enhance performance
                     (fx_mouse_x <= (x + fx_width(object, true)))
                     &&
                     (fx_mouse_y <= (y + fx_height(object, true)))
                     &&

                //now check cropped dimensions if required
                     (
                       mv_no_cropping
                       ||
                       (
                         (fx_mouse_x <= (x + fx_width(object)))
                         &&
                         (fx_mouse_y <= (y + fx_height(object)))
                       )
                     )
                   )
                {
                        parent = object;
                        z      = 0;

                        while ((parent = parent.parentNode) != null) z++;

                        if ((_object == null) || (z > _z))
                        {
                                _object = object;
                                _z      = z;
                        };
                };
        };

        return _object;
};
//................................
function mv_scroll()
{
        mv_scroll_flag = true;

        if (mv_object == null)                        { mv_scroll_flag = false; return; };
        if ((mv_scroll_x == 0) && (mv_scroll_y == 0)) { mv_scroll_flag = false; return; };

        //store position
        var left = fx_page_left();
        var top  = fx_page_top();

        //scroll
        window.scrollBy(mv_scroll_x, mv_scroll_y);

        //check position change
        if (fx_page_left() == left) mv_scroll_x = 0;
        if (fx_page_top()  == top)  mv_scroll_y = 0;

        if ((mv_scroll_x == 0) && (mv_scroll_y == 0)) { mv_scroll_flag = false; return; };

        fx_animation_frame(mv_scroll, 100);
};
//................................
function mv_event(event)
{
        switch (event)
        {
        case "mousedown":
                var object = mv_get_object();
                if (object == null) return; //no target

                //global
                mv_left        = fx_mouse_x;
                mv_top         = fx_mouse_y;
                mv_offset_left = fx_mouse_x - fx_left(object, true);
                mv_offset_top  = fx_mouse_y - fx_top(object,  true);
                mv_fixed       = object.style.position == "fixed";

                //adjust fixed element
                if (mv_fixed)
                {
                        mv_offset_left -= fx_page_left();
                        mv_offset_top  -= fx_page_top();
                };

                //set object
                mv_object      = object;
                break;

        case "mousemove":
                if (mv_object == null) return; //no object

                //perform move action
                if (mv_flag)
                {
                        //scroll at page border
                        var page_left = fx_page_left();
                        var page_top  = fx_page_top();

                        if (fx_mouse_x < (page_left + 100))
                                mv_scroll_x = -100 + fx_mouse_x - page_left;
                        else if (fx_mouse_x > (page_left + fx_window_width - 100))
                                mv_scroll_x = 100 - fx_window_width + fx_mouse_x - page_left;
                        else
                                mv_scroll_x = 0;

                        if (fx_mouse_y < (page_top + 100))
                                mv_scroll_y = -100 + fx_mouse_y - page_top;
                        else if (fx_mouse_y > (page_top + fx_window_height - 100))
                                mv_scroll_y = 100 - fx_window_height + fx_mouse_y - page_top;
                        else
                                mv_scroll_y = 0;

                        if (! mv_scroll_flag) mv_scroll();
                }

                //start move action
                else
                {
                        //element has not been moved away from initial position yet
                        if (Math.sqrt(Math.pow(fx_mouse_x - mv_left, 2) +
                                      Math.pow(fx_mouse_y - mv_top,  2)) <= 10.0)
                                return;

                        //prevent content selection
                        mv_onselectstart       = document.onselectstart;
                        document.onselectstart = new Function("return false;");

                        //style
                        document.getElementsByTagName("body")[0].style.cursor = "move";

                        //set move flag
                        mv_flag = true;
                };

                //position
                var x = fx_mouse_x - mv_offset_left;
                var y = fx_mouse_y - mv_offset_top;

                //adjust fixed element
                if (mv_fixed)
                {
                        x -= fx_page_left();
                        y -= fx_page_top();
                };

                fx_move(mv_object, x, y);
                break;

        case "mouseup":
                if (mv_object == null) return; //no object

                //style
                document.getElementsByTagName("body")[0].style.cursor = "auto";

                //clear source
                mv_object = null;

                //clear move flag
                mv_flag   = false;

                //reset content selection
                document.onselectstart = mv_onselectstart;
        };
};

fx_register_callback(mv_event);