<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

//-----------------------------------------------------------------------------------------------------
//===< DESKTOP INTERFACE >=============================================================================
//-----------------------------------------------------------------------------------------------------
cms_load("core_resource", TRUE);

define("CMS_DESKTOP_IMS_STATUS_NONE", 0);
define("CMS_DESKTOP_IMS_STATUS_SENT", 1);

if (empty($filter_user))   $filter_user   = NULL;
if (empty($filter_thread)) $filter_thread = NULL;

//create core object
$ims = new core_resource(CMS_DATA_PATH . "#desktop/ims.core",
                         array("id"       => "string[8]",
                               "thread"   => "string[8]",
                               "time"     => "string[20]",
                               "owner"    => "_string[40]",
                               "receiver" => "_string[40]",
                               "sender"   => "_string[40]",
                               "status"   => "byte",
                               "text"     => "_string[1500]",
                               "hash"     => "string[32]"));

//get receiver list
$list_receiver = NULL;
$data          = new data("#system/permission");

$data->move("first");
while ($key = $data->move("next"))
{
        if (! permission_is_user($key)) continue;
        $_key = substr($key, 5);

        if (
             nstreq($_key, $user)
             &&
             cms_permission(CMS_APPLICATION, NULL, NULL, $_key)
           )
                $list_receiver[$data->get($key, "name")] = $_key;
};

//=====================================================================================================
//   MESSAGE HANDLING / SUB DISPLAY
//=====================================================================================================
switch (CMS_IFC_MESSAGE)
{
case "filter_user":
//.....................................................................................................
        $filter_user = $ifc_param;
        break;

case "filter_thread":
//.....................................................................................................
        $filter_thread = $ifc_param;
        break;

case "filter_reset":
//.....................................................................................................
        $filter_user   = NULL;
        $filter_thread = NULL;
        break;

case "_reply":
case "_send":
//.....................................................................................................
        if (isset($ifc_param1))
        {
                if ($ifc_param2)
                {
                        $time = time();
                        $hash = md5($thread . $time . $user . $ifc_param2);

                        reset($ifc_param1);
                        while (list($key, $value) = each($ifc_param1))
                        {
                                $ims->seek(array("owner" => NULL), TRUE);
                                $ims->set(array("id"       => unique_id(8),
                                                "thread"   => $thread,
                                                "time"     => $time,
                                                "owner"    => $value,
                                                "receiver" => $value,
                                                "sender"   => $user,
                                                "status"   => CMS_DESKTOP_IMS_STATUS_NONE,
                                                "text"     => $ifc_param2,
                                                "hash"     => $hash));

                                $ims->seek(array("owner" => NULL), TRUE);
                                $ims->set(array("id"       => unique_id(8),
                                                "thread"   => $thread,
                                                "time"     => $time,
                                                "owner"    => $user,
                                                "receiver" => $value,
                                                "sender"   => $user,
                                                "status"   => CMS_DESKTOP_IMS_STATUS_SENT,
                                                "text"     => $ifc_param2,
                                                "hash"     => $hash));

                                //create notification file
                                if ($hfile = fopen(CMS_DATA_PATH . "#desktop/" . r($value) . "/ims.txt", "wb"))
                                {
                                        fputs($hfile, 1);
                                        fclose($hfile);
                                };
                        };

                        $ifc_response = CMS_MSG_DONE;
                        break;
                };
        }
        else
        {
                $ifc_param1 = NULL;
        };

        $ifc_response = CMS_MSG_ERROR;

case "reply":
case "send":
//.....................................................................................................
        if (! $list_receiver)
        {
                $ifc_response = CMS_L_DESKTOP_IMS_004;
                break;
        };

        switch (CMS_IFC_MESSAGE)
        {
        case "reply":
//.....................................................................................................
                if ($ims->seek(array("id" => $ifc_param)))
                {
                        $value      = $ims->get();
                        $thread     = $value["thread"];
                        $text       = $value["text"];
                        $ifc_param1 = array($value["sender"]);
                        $ifc_param2 = NULL;
                        $message    = "_" . CMS_IFC_MESSAGE;
                        break;
                };

        case "send":
//.....................................................................................................
                $thread     = unique_id(8);
                $text       = NULL;
                $ifc_param1 = NULL;
                $ifc_param2 = NULL;
                $message    = "_" . CMS_IFC_MESSAGE;
                break;

        case "_reply":
//.....................................................................................................
                if ($ims->seek(array("id" => $ifc_param)))
                {
                        $value   = $ims->get();
                        $text    = $value["text"];
                        $message = CMS_IFC_MESSAGE;
                        break;
                };

        case "_send":
//.....................................................................................................
                $text    = NULL;
                $message = CMS_IFC_MESSAGE;
                break;
        };

        //display
        $ifc = new ifc($ifc_response,
                       NULL,
                       TRUE,
                       array_merge($param,
                                   array("ifc_param"     => $ifc_param,
                                         "filter_user"   => $filter_user,
                                         "filter_thread" => $filter_thread,
                                         "thread"        => $thread)),
                       $message,
                       CMS_L_DESKTOP_IMS_001);

        echo("<table class=\"layout\">");
        if ($text)
                echo("<tr>" .
                     "<td colspan=\"3\" " .
                         "class=\"br\" " .
                         "style=\"BACKGROUND-COLOR:#FFFFCC\">" .
                     x($text) .
                     "</td>" .
                     "</tr>");
        echo("<tr>" .
             "<td>");
        $ifc->set(CMS_L_DESKTOP_IMS_002, "label");
        ksort($list_receiver);
        $ifc->set($list_receiver,        "multiselect 30x15",   $ifc_param1);
        echo("</td>" .
             "<td>" .
             "&nbsp;" .
             "</td>" .
             "<td>");
        $ifc->set(CMS_L_DESKTOP_IMS_003, "textarea 60x20 1500", $ifc_param2);
        echo("</td>" .
             "</tr>" .
             "</table>");

        $ifc->close();

case "delete":
//.....................................................................................................
        if (empty($list)) break; //no selection

        $list   = array_flip($list);
        $filter = array("owner" => $user);
        if ($filter_thread)    $filter["thread"] = $filter_thread;

        $ims->reset();
        while ($ims->next($filter))
        {
                $value = $ims->get();

                if (isset($list[$value["hash"]]))
                {
                        if ($value["status"] & CMS_DESKTOP_IMS_STATUS_SENT)
                        {
                                if (
                                     (! $filter_user)
                                     ||
                                     streq($value["receiver"], $filter_user)
                                   )
                                        $ims->del();
                        }
                        elseif (
                                 (! $filter_user)
                                 ||
                                 streq($value["sender"], $filter_user)
                               )
                                $ims->del();
                };
        };

        $ifc_response = CMS_MSG_DONE;
        break;
};

//remove notification file
$path = CMS_DATA_PATH . "#desktop/" . r($user) . "/ims.txt";
if (is_file($path)) unlink($path);

//menu
$menu = array(CMS_L_COMMAND_REFRESH . "|desktop/command_refresh" => NULL);
if ($list_receiver)                 $menu[CMS_L_DESKTOP_IMS_001  . "|desktop/command_create"] = "send";
$menu[CMS_L_COMMAND_DELETE . "|desktop/command_delete"]  = "delete";
if ($filter_user || $filter_thread) $menu[CMS_L_COMMAND_PREVIOUS . "|desktop/command_cancel"] = "filter_reset";

//display
$ifc = new ifc($ifc_response,
               NULL,
               $menu,
               array_merge($param,
                           array("filter_user"   => $filter_user,
                                 "filter_thread" => $filter_thread)),
               NULL,
               CMS_L_DESKTOP_IMS_012);

//receive messages
$filter    = array("owner" => $user);
if ($filter_thread) $filter["thread"] = $filter_thread;
$message   = NULL;
$list_hash = NULL;
$i         = 0;

$ims->reset();
while ($ims->next($filter))
{
        $value = $ims->get();

        if ($value["status"] & CMS_DESKTOP_IMS_STATUS_SENT)
        {
                if (
                     (! $filter_user)
                     ||
                     streq($value["receiver"], $filter_user)
                   )
                {
                        $hash               = $value["hash"];
                        if (! isset($list_hash[$hash]))
                                $message[$value["time"] . $value["id"]] = $value;
                        $list_hash[$hash][] = $value["receiver"];
                };
        }
        elseif (
                 (! $filter_user)
                 ||
                 streq($value["sender"], $filter_user)
               )
        {
                $message[$value["time"] . $value["id"]] = $value;
        };
};

if (! $message)
{
        $ifc->set(CMS_L_DESKTOP_IMS_005, "info");
        $ifc->close();
};

ifc_table_open();
echo("<colgroup>" .
     "<col style=\"WIDTH:1px\">" .
     "<col style=\"WIDTH:200px\">" .
     "<col>" .
     "</colgroup>");

krsort($message);
reset($message);
while (list($key, $value) = each($message))
{
        $varied = ifc_varied();

        //sent message
        if ($value["status"] & CMS_DESKTOP_IMS_STATUS_SENT)
        {
                if (! isset($list_hash[$value["hash"]])) continue;

                //selection
                echo("<tr>" .
                     "<td class=\"highlight\">");
                $ifc->set(NULL, "checkbox", $value["hash"], FALSE, "list[" . $i++ . "]");
                echo("</td>" .

                //meta data
                     "<td class=\"br\" " .
                         "style=\"BACKGROUND-COLOR:#" . ($varied ? "BFBFFF" : "CCCCFF") . "\">");

                $receiver = $list_hash[$value["hash"]];
                $flag     = NULL;

                reset($receiver);
                while (list($_key, $_value) = each($receiver))
                {
                        if ($flag) echo(", ");
                        else       $flag = TRUE;

                        echo("<a href=\"javascript:ifc_post('filter_user','" . rqx($_value) . "');\" " .
                                "title=\"" . CMS_L_DESKTOP_IMS_009 . "\">" .
                             (($_receiver = $data->get("user.$_value", "name")) ? $_receiver : $_value) .
                             "</a>");
                };

                echo("<br>" .
                     "<small>" .
                     x(friendly_date($value["time"])) .
                     "</small>" .
                     "</td>" .

                //content
                     "<td$varied>" .
                     "<a href=\"javascript:ifc_post('filter_thread','" . rqx($value["thread"]) . "');\" " .
                        "title=\"" . CMS_L_DESKTOP_IMS_010 . "\" " .
                        "class=\"br\">" .
                     image("desktop/icon_message_sent") . "&nbsp;" .
                     nl2br(x($value["text"])) .
                     "</a>" .
                     "</td>" .
                     "</tr>");

                unset($list_hash[$value["hash"]]);
        }

        //received message
        else
        {
                $sender  = $value["sender"];
                $_sender = ($_sender = $data->get("user.$sender", "name")) ? $_sender : $sender;

                //selection
                echo("<tr>" .
                     "<td class=\"highlight\">");
                $ifc->set(NULL, "checkbox", $value["hash"], FALSE, "list[" . $i++ . "]");
                echo("</td>" .

                //meta data
                     "<td style=\"BACKGROUND-COLOR:#" . ($varied ? "BFFFBF" : "CCFFCC") . "\">" .
                     "<a href=\"javascript:ifc_post('filter_user','" . rqx($sender) . "');\" " .
                        "title=\"" . CMS_L_DESKTOP_IMS_009 . "\">" .
                     x($_sender) .
                     "</a><br>" .
                     "<small>" .
                     x(friendly_date($value["time"])) .
                     "</small>" .
                     "</td>" .

                //content
                     "<td$varied>" .
                     "<a href=\"javascript:ifc_post('filter_thread','" . rqx($value["thread"]) . "');\" " .
                        "title=\"" . CMS_L_DESKTOP_IMS_010 . "\" " .
                        "class=\"br\">" .
                     image("desktop/icon_message") . "&nbsp;" .
                     nl2br(x($value["text"])) .
                     "</a>");
                if ($list_receiver)
                        echo("<br>&nbsp;<br>" .
                             "<a href=\"javascript:ifc_post('reply','" . rqx($value["id"]) . "');\">" .
                             "[" . CMS_L_DESKTOP_IMS_011 . "]" .
                             "</a>");
                echo("</td>" .
                     "</tr>");
        };
};

//selection control
echo("<tr>" .
     "<td colspan=\"3\" " .
         "class=\"highlight\" " .
         "style=\"TEXT-ALIGN:right\">");
$ifc->set(CMS_L_DESKTOP_IMS_006, "button", "javascript:ifc_list_activate();");
$ifc->set(CMS_L_DESKTOP_IMS_007, "button", "javascript:ifc_list_invert();");
$ifc->set(CMS_L_DESKTOP_IMS_008, "button", "javascript:ifc_list_deactivate();");
echo("</td>" .
     "</tr>");
ifc_table_close();

$ifc->close();
?>
