<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

//-----------------------------------------------------------------------------------------------------
//===< INTERFACE >=====================================================================================
//-----------------------------------------------------------------------------------------------------
if (! cms_load("filemanager")) ifc_inactive($ifc_page);

//retrieve selected object from permanent cache
if (! isset($object)) $object = (string)cms_cache("filemanager." . CMS_USER . ".object");
if (stre($object))    $object = CMS_ROOT_PATH;

//verify and correct selected object
if (nstreq(substr($object, 0, strlen(CMS_ROOT_PATH)), CMS_ROOT_PATH)) $object = CMS_ROOT_PATH;
while (! file_exists($object))                                        $object = dirname($object) . "/";

if (! isset($list))   $list   = preg_match("/^(?:|ifc_)/", CMS_IFC_MESSAGE) ? array($object) : array();
if (! isset($target)) $target = NULL;
if (! isset($status)) $status = NULL;

//=====================================================================================================
//   MESSAGE HANDLING / SUB DISPLAY
//=====================================================================================================
switch (CMS_IFC_MESSAGE)
{
case "select":
//.....................................................................................................
        $object = stre($ifc_param) ? CMS_ROOT_PATH : $ifc_param;
        $list   = array($object);
        $status = NULL;
        break;

case "save":
//.....................................................................................................
        $ifc_param1 = dirname($object) . "/" . basename($ifc_param1);
        if (is_dir($object)) $ifc_param1 .= "/";

        $access = (isset($ifc_param2) ? 256 : 0) + (isset($ifc_param3) ? 32 : 0) + (isset($ifc_param4)  ? 4 : 0) +
                  (isset($ifc_param5) ? 128 : 0) + (isset($ifc_param6) ? 16 : 0) + (isset($ifc_param7)  ? 2 : 0) +
                  (isset($ifc_param8) ?  64 : 0) + (isset($ifc_param9) ?  8 : 0) + (isset($ifc_param10) ? 1 : 0);
        chmod($object, $access);

        if (rename($object, $ifc_param1))
        {
                $object       = $ifc_param1;
                $list         = array($object);
                $status       = NULL;
                $ifc_response = CMS_MSG_DONE;
                break;
        };

        $ifc_response = CMS_MSG_ERROR;
        break;

case "upload":
//.....................................................................................................
        if (is_array($ifc_file1)) $array = array_combine($ifc_file1, $ifc_file1_name);
        else                      $array = array($ifc_file1 => $ifc_file1_name);

        $path  = is_dir($object) ? $object : dirname($object) . "/";
        $list  = array();
        $error = FALSE;

        filemanager_sort($array);

        reset($array);
        while (list($key, $value) = each($array))
        {
                $value = $path . $value;

                //place upload
                if (move_uploaded_file($key, $value)) $list[] = $value;
                else                                  $error  = TRUE;
        };

        $object       = reset($list);
        $status       = NULL;
        $ifc_response = $error ? CMS_MSG_ERROR : CMS_MSG_DONE;
        break;

case "mkdir":
//.....................................................................................................
        $ifc = new ifc($ifc_response,
                       $ifc_page,
                       TRUE,
                       array("object" => $object,
                             "list"   => $list),
                       "_mkdir",
                       CMS_L_IFC_FILEMANAGER_006);

        $ifc->set(CMS_L_NAME, "text 40 256 b");
        echo(x(is_dir($object) ? $object : dirname($object) . "/"));

        $ifc->close();

case "_mkdir":
//.....................................................................................................
        $_object = (is_dir($object) ? $object : dirname($object) . "/") . "$ifc_param1/";

        if (mkpath($_object))
        {
                $object       = $_object;
                $list         = array($object);
                $ifc_response = CMS_MSG_DONE;
                break;
        };

        $ifc_response = CMS_MSG_ERROR;
        break;

case "mkfile":
//.....................................................................................................
        $ifc = new ifc($ifc_response,
                       $ifc_page,
                       TRUE,
                       array("object" => $object,
                             "list"   => $list),
                       "_mkfile",
                       CMS_L_IFC_FILEMANAGER_007);

        $ifc->set(CMS_L_NAME,                "text 40 256 b");
        echo(x(is_dir($object) ? $object : dirname($object) . "/") . "<br>");
        $ifc->set(CMS_L_IFC_FILEMANAGER_003, "textarea 80x30 n");

        $ifc->close();

case "_mkfile":
//.....................................................................................................
        $_object = (is_dir($object) ? $object : dirname($object) . "/") . $ifc_param1;

        if (is_file($_object) || (! $hfile = fopen($_object, "wb")))
        {
                $ifc_response = CMS_MSG_ERROR;
                break;
        };

        fputs($hfile, $ifc_param2);
        fclose($hfile);

        $object       = $_object;
        $list         = array($object);
        $ifc_response = CMS_MSG_DONE;
        break;

case "_edit":
case "__edit":
//.....................................................................................................
        if ($hfile = fopen($object, "wb"))
        {
                set_file_buffer($hfile, 0);
                fputs($hfile, $ifc_param1);
                fclose($hfile);

                if (streq(CMS_IFC_MESSAGE, "__edit")) ifc_close_external();

                $ifc_response = CMS_MSG_DONE;
        }
        else
        {
                $ifc_response = CMS_MSG_ERROR;
        };

case "edit":
//.....................................................................................................
        if (! is_file($object))             ifc_close_external();
        if (streq(CMS_IFC_MESSAGE, "edit")) $top = $left = 0;

        //display
        $ifc = new ifc($ifc_response,
                       $ifc_page,
                       array(CMS_L_COMMAND_SAVE .         "|filemanager/command_save"
                             => "javascript:filemanager_save();",
                             CMS_L_IFC_FILEMANAGER_004  . "|filemanager/command_save_close"
                             => "__edit"),
                       array("object" => $object,
                             "top"    => $top,
                             "left"   => $left),
                       NULL,
                       CMS_L_COMMAND_EDIT);

        $ifc->set($object, "textarea 80x30 n", file_get_contents($object));

        //javascript
?>
<script>
var filemanager_object = ifc_get_object("ifc_param1");

function filemanager_save()
{
        ifc_change("top",  filemanager_object.scrollTop);
        ifc_change("left", filemanager_object.scrollLeft);
        ifc_post("_edit");
};
//................................
function filemanager_load()
{
        ifc_scroll(filemanager_object,
                   <?php echo((int)$top);?>,
                   <?php echo((int)$left);?>);
};

setTimeout(filemanager_load, 0); //ie workaround
</script>
<?php

        $ifc->close();

case "compress":
//.....................................................................................................
        if (count($list) === 0) break; //no selection

        $_list = array();
        $error = FALSE;

        reset($list);
        while (list(, $value) = each($list))
        {
                if ($value = filemanager_gzcompress($value)) $_list[]  = $value;
                else                                         $error   |= TRUE;
        };

        if (! $error)
        {
                $list         = $_list;
                $status       = NULL;
                $ifc_response = CMS_MSG_DONE;
                break;
        };

        $ifc_response = CMS_MSG_ERROR;
        break;

case "decompress":
//.....................................................................................................
        if (count($list) === 0) break; //no selection

        $_list = array();
        $error = FALSE;

        reset($list);
        while (list(, $value) = each($list))
        {
                if ($value = filemanager_gzdecompress($value)) $_list[]  = $value;
                else                                           $error   |= TRUE;
        };

        if (! $error)
        {
                $list         = $_list;
                $status       = NULL;
                $ifc_response = CMS_MSG_DONE;
                break;
        };

        $ifc_response = CMS_MSG_ERROR;
        break;

case "copy":
case "cut":
//.....................................................................................................
        $target = $object;
        $status = CMS_IFC_MESSAGE;
        break;

case "target":
//.....................................................................................................
        $target = $ifc_param;
        break;

case "copy_insert":
//.....................................................................................................
        if (count($list) === 0) break; //no selection

        if (stre($ifc_param)) $ifc_param = CMS_ROOT_PATH;

        $_list = array();
        $error = FALSE;

        reset($list);
        while (list(, $value) = each($list))
        {
                $_value = basename($value);

                if (is_dir($value))
                {
                        $_value  .= "/";
                        $__value  = $ifc_param . $_value;
                        $i        = 0;

                        while (is_dir($__value))
                                $__value = $ifc_param . str_repeat("_", ++$i) . $_value;
                }
                else
                {
                        $__value  = $ifc_param . $_value;
                        $i        = 0;

                        while (is_file($__value))
                                $__value = $ifc_param . str_repeat("_", ++$i) . $_value;
                };

                if (filemanager_copy($value, $__value)) $_list[] = $__value;
                else                                    $error   = TRUE;
        };

        if (! $error)
        {
                $object       = $ifc_param;
                $list         = $_list;
                $status       = NULL;
                $ifc_response = CMS_MSG_DONE;
                break;
        };

        $ifc_response = CMS_MSG_ERROR;
        break;

case "cut_insert":
//.....................................................................................................
        if (count($list) === 0) break; //no selection

        if (stre($ifc_param)) $ifc_param = CMS_ROOT_PATH;

        $_list = array();
        $error = FALSE;

        reset($list);
        while (list(, $value) = each($list))
        {
                $_value = basename($value);

                if (is_dir($value))
                {
                        $_value  .= "/";
                        $__value  = $ifc_param . $_value;
                        $i        = 0;

                        while (is_dir($__value))
                                $__value = $ifc_param . str_repeat("_", ++$i) . $_value;
                }
                else
                {
                        $__value  = $ifc_param . $_value;
                        $i        = 0;

                        while (is_file($__value))
                                $__value = $ifc_param . str_repeat("_", ++$i) . $_value;
                };

                if (rename($value, $__value)) $_list[] = $__value;
                else                          $error   = TRUE;
        };

        if (! $error)
        {
                $object        = $ifc_param;
                $list          = $_list;
                $status        = NULL;
                $ifc_response  = CMS_MSG_DONE;
                break;
        };

        $ifc_response = CMS_MSG_ERROR;
        break;

case "download":
//.....................................................................................................

        //end output buffering
        ob_implicit_flush(TRUE);
        while (ob_get_level()) ob_end_clean();

        $time = time();

        header("Cache-Control: private, max-age=3600, must-revalidate");
        header("Cache-Control: post-check=3600, pre-check=3600", FALSE); //proprietary internet explorer setting
        header("Content-Disposition: attachment; filename=\"" . basename($object) . "\"");
        header("Content-Length: " . filesize($object));
        header("Content-Type: " . get_mime_type($object));
        header("Date: " . gmdate("D, d M Y H:i:s", $time) . " GMT");
        header("Expires: " . gmdate("D, d M Y H:i:s", $time + 3600) . " GMT");
        header("Last-Modified: " . gmdate("D, d M Y H:i:s", filemtime($object)) . " GMT");

        download($object);
        exit();

case "delete":
//.....................................................................................................
        if (count($list) === 0) break; //no selection

        $error = FALSE;

        reset($list);
        while (list(, $value) = each($list))
                $error |= ! filemanager_delete($value);

        if (! $error)
        {
                $list         = array($object);
                $status       = NULL;
                $ifc_response = CMS_MSG_DONE;
                break;
        };

        $ifc_response = CMS_MSG_ERROR;
        break;

case "transfer":
//.....................................................................................................
        $ifc = new ifc($ifc_response,
                       $ifc_page,
                       TRUE,
                       array("object" => $object,
                             "list"   => $list),
                       "_transfer",
                       CMS_L_IFC_FILEMANAGER_033);

        $ifc->set(CMS_L_URL,  "text 50 b", "http://");
        $ifc->set(CMS_L_NAME, "text 50 256 b");
        echo(x(is_dir($object) ? $object : dirname($object) . "/"));

        $ifc->close();

case "_transfer":
//.....................................................................................................
        $ifc_param1 = utf8_trim($ifc_param1);
        $ifc_param2 = utf8_trim($ifc_param2);
        if (stre($ifc_param2)) $ifc_param2 = basename($ifc_param1);

        $_object    = ((! is_dir($object)) ? dirname($object) . "/" : $object) . $ifc_param2;

        if (($hfile = fopen($ifc_param1, "rb")) !== FALSE)
        {
                if (($_hfile = fopen($_object, "wb")) !== FALSE)
                {
                        do
                        {
                                set_time_limit(600); //10 minutes
                                $buffer = fread($hfile, 524288);
                                fputs($_hfile, $buffer);
                        }
                        while (! feof($hfile));

                        fclose($hfile);
                        fclose($_hfile);

                        $object       = $_object;
                        $list         = array($object);
                        $ifc_response = CMS_MSG_DONE;
                        break;
                };

                fclose($hfile);
        };

        $ifc_response = CMS_MSG_ERROR;
        break;
};

//=====================================================================================================
//   MAIN DISPLAY
//=====================================================================================================
ifc_permission(array("" => CMS_L_ACCESS));

if (CMS_IFC_MESSAGE !== "")
{
        //verify and correct selected object
        if (nstreq(substr($object, 0, strlen(CMS_ROOT_PATH)), CMS_ROOT_PATH)) $object = CMS_ROOT_PATH;
        while (! file_exists($object))                                        $object = dirname($object) . "/";

        //cache selected object permanently
        cms_cache("filemanager." . CMS_USER . ".object", $object, TRUE);
};

$flexview = filemanager_flexview(NULL, $object);
$filename = basename($object);
$url      = substr($object, strlen(CMS_ROOT_PATH));
//                                    -./ 0 - 9  A - Z      _    a - z
$url      = preg_replace_callback("/[^\\x2D-\\x39\\x41-\\x5A\\x5F\\x61-\\x7A]/",
                                  function($value) { return "%" . bin2hex($value[0]); },
                                  $url);
$url      = CMS_ROOT_URL . $url;

//menu
$menu = array(CMS_L_IFC_FILEMANAGER_006 . "|filemanager/command_create_directory" => "mkdir",
              CMS_L_IFC_FILEMANAGER_007 . "|filemanager/command_create_file"      => "mkfile");
if (is_file($object))
{
        if (
             preg_match("/\.dat$/", $object)
             &&
             ifc_available("system")
             &&
             cms_permission("system", TRUE, NULL)
           )
                $menu[CMS_L_COMMAND_EDIT . "|filemanager/command_edit"] =
                        "javascript:load_page('" .
                        q(CMS_ACTIVE_URL) .
                        "?ifc_page=system" .
                        "&ifc_option=external" .
                        "&file=" . rq($object) . "');";
        else
                $menu[CMS_L_COMMAND_EDIT . "|filemanager/command_edit"] =
                        "javascript:load_page('" .
                        q(CMS_ACTIVE_URL) .
                        "?ifc_page=" . rq(CMS_IFC_PAGE) .
                        "&ifc_message=edit" .
                        "&ifc_option=external" .
                        "&object=" . rq($object) . "');";
};
$menu[CMS_L_COMMAND_COPY .        "|filemanager/command_copy"]       = "copy";
$menu[CMS_L_COMMAND_CUT .         "|filemanager/command_cut"]        = "cut";
$menu[CMS_L_COMMAND_DELETE .      "|filemanager/command_delete"]     = "#delete";
$menu[CMS_L_IFC_FILEMANAGER_036 . "|filemanager/command_compress"]   = "compress";
$menu[CMS_L_IFC_FILEMANAGER_037 . "|filemanager/command_decompress"] = "decompress";
$menu[CMS_L_IFC_FILEMANAGER_033 . "|filemanager/command_transfer"]   = "transfer";
if (
     cms_available("directory")
     &&
     cms_permission("directory", TRUE, NULL)
   )
        $menu[CMS_L_IFC_FILEMANAGER_008 . "|filemanager/command_directory"] =
                "javascript:load_page('" .
                q(CMS_ACTIVE_URL) .
                "?ifc_page=directory" .
                "&ifc_message=add_target" .
                "&ifc_option=external" .
                "&ifc_param1=" . rq($filename) .
                "&ifc_param5=" .
                "&ifc_param6=" . rq($url) .
                "&ifc_param7=');";

//display
$ifc = new ifc($ifc_response,
               $ifc_page,
               $menu,
               array("object" => $object,
                     "status" => $status));

echo("<table class=\"layout\" " .
            "style=\"WIDTH:100%\">" .

     "<colgroup>" .
     "<col>");
if ($status)
        echo("<col style=\"WIDTH:1px\">" .
             "<col style=\"WIDTH:200px\">");
echo("<col span=\"2\" " .
          "style=\"WIDTH:1px\">" .
     "</colgroup>" .

     "<tr>" .

//directory
     "<td>");

ifc_table_open();
echo("<tr>" .
     "<td class=\"special\">");

//address
$ifc->set(CMS_L_URL, "description :");
echo("<input name=\"address\" " .
            "type=\"text\" " .
            "value=\"" . x($url) . "\" " .
            "size=\"40\" " .
            "onfocus=\"this.select();\" " .
            "readonly " .
            "class=\"select\">");

//display button
$ifc->set(CMS_L_COMMAND_SHOW,
          "button",
          "javascript:load_page(ifc_value('address'));");

//download button
if (is_file($object))
        $ifc->set(CMS_L_IFC_FILEMANAGER_029,
                  "button",
                  "javascript:load_page('" .
                  q(CMS_ACTIVE_URL) .
                  "?ifc_page=" . rq(CMS_IFC_PAGE) .
                  "&ifc_message=download" .
                  "&object=" . rq($object) . "');");
echo("</td>" .
     "</tr>" .

//shortcuts
     "<tr>" .
     "<td class=\"highlight2\">" .
     "<div class=\"br\">");
$array = array("data", "design", "image", "mp3", "data/#database", "data/#log", "#include", "nuos/#language");
reset($array);
while (list(, $value) = each($array))
    echo("<a href=\"javascript:ifc_post('select','" . qx(CMS_ROOT_PATH . "$value/") . "');\">" .
         image("filemanager/icon_folder") . " " . x(basename($value)) .
         "</a> ");
echo("</div>" .
     "</td>" .
     "</tr>" .

//directory
     "<tr>" .
     "<td class=\"varied2\">" .
     "<div class=\"br\">");
$icon = array_merge(get_mime_list(),
                    array("file"       => "filemanager/icon_file",
                          "container"  => "filemanager/icon_folder",
                          "+container" => "filemanager/icon_folderopen"));
$flexview->set_encoding_function(__NAMESPACE__ . "\\qr");
$flexview->set_checkbox_identifier("list");
$flexview->set_checkbox_list($list);
$flexview->show_column($object,                                    //index
                       "javascript:ifc_post('select','%index%');", //action
                       "name",                                     //name key
                       NULL,                                       //mark
                       $icon);                                     //icon
echo("</div>" .
     "</td>" .
     "</tr>" .

//path
     "<tr>" .
     "<td class=\"highlight2\">" .
     x($object) .
     "</td>" .
     "</tr>" .

//selection control
     "<tr>" .
     "<td class=\"highlight\" " .
         "style=\"TEXT-ALIGN:right\">");
$ifc->set(CMS_L_IFC_FILEMANAGER_024, "button", "javascript:ifc_list_activate();");
$ifc->set(CMS_L_IFC_FILEMANAGER_025, "button", "javascript:ifc_list_invert();");
$ifc->set(CMS_L_IFC_FILEMANAGER_026, "button", "javascript:ifc_list_deactivate();");
echo("</td>" .
     "</tr>");
ifc_table_close();

echo("</td>");

if ($status !== NULL)
{
        //spacer
        echo("<td>" .
             "&nbsp;" .
             "</td>" .

        //target
             "<td>");

        ifc_table_open();
        echo("<tr>" .
             "<th>" .
             (streq($status, "copy") ? CMS_L_COMMAND_COPY : CMS_L_COMMAND_CUT) .
             "</th>" .
             "</tr>" .
             "<tr>" .
             "<td class=\"target\">");
        $flexview = filemanager_flexview_directory(NULL, $target);
        $flexview->set_encoding_function(__NAMESPACE__ . "\\qr");
        $flexview->show_target($target,
                               "javascript:ifc_post('target','%index%');",
                               "javascript:ifc_post('" . $status . "_insert','%index%');");
        echo("</td>" .
             "</tr>");
        ifc_table_close();

        echo("</td>");
};

//spacer
echo("<td>" .
     "&nbsp;" .
     "</td>" .

//data
     "<td>");

ifc_table_open();
echo("<tr>" .
     "<th>" .
     CMS_L_IFC_FILEMANAGER_009 .
     "</th>" .
     "</tr>" .
     "<tr>" .
     "<td class=\"varied\">");
if (nstreq($object, CMS_ROOT_PATH))
{
        $ifc->set(CMS_L_NAME,         "text 30",  $filename);
        $ifc->set(CMS_L_COMMAND_SAVE, "button b", "save");
};

$ifc->set(CMS_L_IFC_FILEMANAGER_010, "description");

//access permission
$access = fileperms($object);

ifc_table_open();
echo("<tr>" .

//owner
     "<th>" .
     CMS_L_IFC_FILEMANAGER_011 .
     "</th>" .

//group
     "<th>" .
     CMS_L_IFC_FILEMANAGER_012 .
     "</th>" .

//all
     "<th>" .
     CMS_L_IFC_FILEMANAGER_013 .
     "</th>" .
     "</tr>" .
     "<tr>" .

//uid
     "<td>" .
     "UID: " .
     "<strong>" .
     x(fileowner($object)) .
     "</strong>" .
     "</td>" .

//gid
     "<td>" .
     "GID: " .
     "<strong>" .
     x(filegroup($object)) .
     "</strong>" .
     "</td>" .

//spacer
     "<td>" .
     "&nbsp;" .
     "</td>" .
     "</tr>" .
     "<tr>" .

//owner read
     "<td>");
$ifc->set(CMS_L_IFC_FILEMANAGER_014,        "checkbox", TRUE, $access & 0400);
echo("</td>" .

//group read
     "<td>");
$ifc->set(CMS_L_IFC_FILEMANAGER_014,        "checkbox", TRUE, $access & 040);
echo("</td>" .

//all read
     "<td>");
$ifc->set(CMS_L_IFC_FILEMANAGER_014,        "checkbox", TRUE, $access & 04);
echo("</td>" .
     "</tr>" .
     "<tr>" .

//owner write
     "<td>");
$ifc->set(CMS_L_IFC_FILEMANAGER_015 . " *", "checkbox", TRUE, $access & 0200);
echo("</td>" .

//group write
     "<td>");
$ifc->set(CMS_L_IFC_FILEMANAGER_015,        "checkbox", TRUE, $access & 020);
echo("</td>" .

//all write
     "<td>");
$ifc->set(CMS_L_IFC_FILEMANAGER_015,        "checkbox", TRUE, $access & 02);
echo("</td>" .
     "</tr>" .
     "<tr>" .

//owner execute
     "<td>");
$ifc->set(CMS_L_IFC_FILEMANAGER_016,        "checkbox", TRUE, $access & 0100);
echo("</td>" .

//group execute
     "<td>");
$ifc->set(CMS_L_IFC_FILEMANAGER_016,        "checkbox", TRUE, $access & 010);
echo("</td>" .

//all execute
     "<td>");
$ifc->set(CMS_L_IFC_FILEMANAGER_016,        "checkbox", TRUE, $access & 01);
echo("</td>" .
     "</tr>");
ifc_table_close();
echo("<small>" .
     "* " . CMS_L_IFC_FILEMANAGER_017 .
     "</small>");

//information
$filesize         = sprintf("%u", filesize($object));
$transtime        = $filesize / 8000; //64 * 1000 / 8;
if ($transtime < 1) $transtime = 1;

$disk_total_space = disk_total_space(CMS_ROOT_PATH);
$disk_free_space  = disk_free_space(CMS_ROOT_PATH);
$percentage_used  = 100 / $disk_total_space * $disk_free_space;

ifc_table_open();

//type
echo("<tr>" .
     "<td>" .
     CMS_L_IFC_FILEMANAGER_018 . ":" .
     "</td>" .
     "<td>" .
     filetype($object) .
     "</td>" .
     "</tr>" .

//size
     "<tr>" .
     "<td>" .
     CMS_L_IFC_FILEMANAGER_019 . ":" .
     "</td>" .
     "<td>" .
     format_bytesize($filesize) .
     "</td>" .
     "</tr>" .

//transmission time
     "<tr>" .
     "<td>" .
     CMS_L_IFC_FILEMANAGER_023 . ":" .
     "</td>" .
     "<td>" .
     gmdate(CMS_L_TIME_FORMAT, $transtime) .
     "</td>" .
     "</tr>" .

//modified time
     "<tr>" .
     "<td>" .
     CMS_L_IFC_FILEMANAGER_020 . ":" .
     "</td>" .
     "<td>" .
     x(friendly_date(filemtime($object))) .
     "</td>" .
     "</tr>" .

//accessed time
     "<tr>" .
     "<td>" .
     CMS_L_IFC_FILEMANAGER_021 . ":" .
     "</td>" .
     "<td>" .
     x(friendly_date(fileatime($object))) .
     "</td>" .
     "</tr>" .

//total space
     "<tr>" .
     "<td>" .
     CMS_L_IFC_FILEMANAGER_038 . ":" .
     "</td>" .
     "<td>" .
     format_bytesize($disk_total_space) .
     "</td>" .
     "</tr>" .

//free space
     "<tr>" .
     "<td>" .
     CMS_L_IFC_FILEMANAGER_022 . ":" .
     "</td>" .
     "<td>" .
     format_bytesize($disk_free_space) .
     " (" . number_format($percentage_used, 1, CMS_L_DECIMAL_SEPARATOR, NULL) . "%)" .
     "</td>" .
     "</tr>");
ifc_table_close();

echo("</td>" .
     "</tr>" .

//file upload
     "<tr>" .
     "<td class=\"highlight\">");
$ifc->set(CMS_L_IFC_FILEMANAGER_002, "multifile");
$ifc->set(CMS_L_IFC_FILEMANAGER_001, "button b", "upload");
$ifc->set(CMS_L_IFC_FILEMANAGER_027, "info");
echo(jscript("ifc_autopost(\"ifc_file1[]\", \"upload\");") .

     "</td>" .
     "</tr>");
ifc_table_close();

echo("</td>" .
     "</tr>" .
     "</table>");

$ifc->close();
?>
