<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

//-----------------------------------------------------------------------------------------------------
//===< MODULE >========================================================================================
//-----------------------------------------------------------------------------------------------------
global $comment_message,
       $comment_index,
       $comment_page,
       $comment_add_name,
       $comment_add_email,
       $comment_add_url,
       $comment_add_text,
       $comment_add_message,
       $comment_add_captcha_key,
       $comment_add_captcha_code,
       $comment_edit_name,
       $comment_edit_email,
       $comment_edit_url,
       $comment_edit_text,
       $comment_edit_message;

if (! cms_load("comment"))
{
        echo(CMS_MSG_UNAVAILABLE);
        return;
};

//instance
if (isset($GLOBALS["comment"]["instance"]) && nstre($GLOBALS["comment"]["instance"]))
        $instance = $GLOBALS["comment"]["instance"];
else
        $instance = defined("CMS_CONTENT_INDEX") ? CMS_CONTENT_INDEX : "";

$comment = new comment($instance);
if (! $comment->enabled)
{
        echo(CMS_MSG_UNAVAILABLE);
        return;
};

//validate access permissions
if (! ($comment->reader || $comment->writer || $comment->operator)) return;

//default status
if (isset($GLOBALS["comment"]["default_status"]) && nstre($GLOBALS["comment"]["default_status"]))
        $default_status = (bool)$GLOBALS["comment"]["default_status"] ?
                          CMS_DB_COMMENT_STATUS_ACTIVE :
                          CMS_DB_COMMENT_STATUS_INACTIVE;
else
        $default_status = CMS_DB_COMMENT_STATUS_INACTIVE;

//notification e-mail
$notification_email = (isset($GLOBALS["comment"]["notification_email"]) &&
                       nstre($GLOBALS["comment"]["notification_email"])) ?
                      $GLOBALS["comment"]["notification_email"] : "";
if (! verify_email($notification_email)) $notification_email = "";

//display order
$display_order = (isset($GLOBALS["comment"]["display_order"]) &&
                  nstre($GLOBALS["comment"]["display_order"])) ?
                 (bool)$GLOBALS["comment"]["display_order"] : FALSE;

//spam threshold
$spam_threshold = (isset($GLOBALS["comment"]["spam_threshold"]) &&
                   nstre($GLOBALS["comment"]["spam_threshold"])) ?
                  (int)$GLOBALS["comment"]["spam_threshold"] : 95;
if ($spam_threshold < 0)       $spam_threshold = 0;
elseif ($spam_threshold > 100) $spam_threshold = 100;

//use captcha
$use_captcha = (isset($GLOBALS["comment"]["use_captcha"]) &&
                nstre($GLOBALS["comment"]["use_captcha"])) ?
               (bool)$GLOBALS["comment"]["use_captcha"] : TRUE;

//configuration
$comment->default_status = $default_status;
$comment->spam_threshold = $spam_threshold;

//process messages
$error   = array();
$success = "";
switch ($comment_message)
{
case "add":
//.....................................................................................................
        if (! $comment->writer) break;

        if (nstreq(CMS_USER, "anonymous"))
        {
                $comment_add_name = CMS_NAME;
                if (verify_email(CMS_EMAIL)) $comment_add_email = CMS_EMAIL;
        };

        //validate input data
        $comment_add_name = stripspaces($comment_add_name);
        if ($comment_add_name == "")
                $error[] = CMS_L_MOD_COMMENT_001;

        if (! verify_email($comment_add_email))
                $error[] = CMS_L_MOD_COMMENT_002;

        if (
             nstre($comment_add_url)
             &&
             (! preg_match("/^https?:\/\//", $comment_add_url))
           )
                $comment_add_url = "http://$comment_add_url";

        $comment_add_text = stripspaces($comment_add_text);
        if ($comment_add_text == "")
                $error[] = CMS_L_MOD_COMMENT_004;

        if ($use_captcha && cms_load("captcha"))
        {
                $captcha = new captcha();
                if (! $captcha->verify($comment_add_captcha_key, $comment_add_captcha_code))
                        $error[] = CMS_L_MOD_COMMENT_045;
        };

        if (count($error)) break; //abort on error

        //add comment
        $index = $comment->add($comment_add_name,
                               $comment_add_email,
                               $comment_add_url,
                               $comment_add_text);

        //error handling
        switch ($index)
        {
        case -1: //spam
                $error[] = CMS_L_MOD_COMMENT_038;
                break 2;

        case -2: //double post
                $error[] = CMS_L_MOD_COMMENT_040;
                break 2;

        case FALSE: //error
                $error[] = CMS_L_MOD_COMMENT_022;
                break 2;
        };

        //success

        //e-mail notification
        if (nstre($notification_email) && cms_load("smtp"))
        {
                $url  = CMS_ACTIVE_URL . cms_param() . "#comment";
                $_url = CMS_MODULES_URL . "identification.php?location=" .
                        r(CMS_ACTIVE_URL .
                          cms_param(array("comment_message" => "\x1B%s",
                                          "comment_index"   => $index)) .
                          "#comment");

                //requires approval
                if ($comment->default_status == CMS_DB_COMMENT_STATUS_INACTIVE)
                {
                        smtp_send($notification_email,
                                  CMS_L_MOD_COMMENT_027,
                                  sprintf(CMS_L_MOD_COMMENT_029,
                                          $url,
                                          $comment_add_text,
                                          str_replace("%25s", "enable", $_url),
                                          str_replace("%25s", "delete", $_url),
                                          str_replace("%25s", "spam",   $_url)));
                        $success = CMS_L_MOD_COMMENT_032;
                }

                //instantly visible
                else
                {
                        smtp_send($notification_email,
                                  CMS_L_MOD_COMMENT_028,
                                  sprintf(CMS_L_MOD_COMMENT_029,
                                          $url,
                                          $comment_add_text,
                                          str_replace("%25s", "disable", $_url),
                                          str_replace("%25s", "delete",  $_url),
                                          str_replace("%25s", "spam",    $_url)));
                        $success = CMS_L_MOD_COMMENT_031;
                };
        };

        //reset input data
        $comment_add_name  = "";
        $comment_add_email = "";
        $comment_add_url   = "";
        $comment_add_text  = "";

        break;

case "rate_good":
//.....................................................................................................
        if (! $comment->rate_good($comment_index))
                $error[] = CMS_L_MOD_COMMENT_024;
        break;

case "rate_bad":
//.....................................................................................................
        if (! $comment->rate_bad($comment_index))
                $error[] = CMS_L_MOD_COMMENT_024;
        break;

case "_edit":
//.....................................................................................................
        if (! $comment->operator) break;

        //editing cancelled
        if (streq($comment_edit_message, CMS_L_COMMAND_CANCEL))
        {
                $comment_message = "";
                break;
        };

        //validate input data
        $comment_edit_name = stripspaces($comment_edit_name);
        if ($comment_edit_name == "")
                $error[] = CMS_L_MOD_COMMENT_005;

        if (! verify_email($comment_edit_email))
                $error[] = CMS_L_MOD_COMMENT_002;

        if (
             nstre($comment_edit_url)
             &&
             (! preg_match("/^https?:\/\//", $comment_edit_url))
           )
                $comment_edit_url = "http://$comment_edit_url";

        $comment_edit_text = stripspaces($comment_edit_text);
        if ($comment_edit_text == "")
                $error[] = CMS_L_MOD_COMMENT_006;

        if (count($error) == 0) //no error
        {
                //edit comment
                if ($comment->edit($comment_index,
                                   $comment_edit_name,
                                   $comment_edit_email,
                                   $comment_edit_url,
                                   $comment_edit_text))
                {
                        $comment_message = "";
                        $success         = CMS_L_MOD_COMMENT_033;
                        break;
                };

                $error[] = CMS_L_MOD_COMMENT_023;
        };

        break;

case "disable":
//.....................................................................................................
        if (! $comment->operator) break;
        if (! $comment->status($comment_index, CMS_DB_COMMENT_STATUS_HIDDEN))
                $error[] = CMS_L_MOD_COMMENT_024;
        break;

case "enable":
//.....................................................................................................
        if (! $comment->operator) break;

        if (
             ($result = mysql_query("SELECT " . CMS_DB_COMMENT_STATUS . ", " .
                                                CMS_DB_COMMENT_EMAIL . " " .
                                    "FROM " .   CMS_DB_COMMENT . " " .
                                    "WHERE " .  CMS_DB_COMMENT_INDEX .
                                    "='" .      sqlesc($comment_index) . "' " .
                                    "LIMIT 1"))
             &&
             ($resultrow = mysql_fetch_assoc($result))
             &&
             $comment->status($comment_index, CMS_DB_COMMENT_STATUS_ACTIVE)
           )
        {
                //send approval notification on initial activation
                if (
                     ($resultrow[CMS_DB_COMMENT_STATUS] == CMS_DB_COMMENT_STATUS_INACTIVE)
                     &&
                     verify_email($resultrow[CMS_DB_COMMENT_EMAIL])
                     &&
                     cms_load("smtp")
                   )
                        smtp_send($resultrow[CMS_DB_COMMENT_EMAIL],
                                  CMS_L_MOD_COMMENT_041,
                                  sprintf(CMS_L_MOD_COMMENT_042,
                                          CMS_ACTIVE_URL . cms_param() . "#comment"));
                break;
        };

        $error[] = CMS_L_MOD_COMMENT_024;
        break;

case "delete":
//.....................................................................................................
        if (! $comment->operator) break;
        if (! $comment->delete($comment_index))
                $error[] = CMS_L_MOD_COMMENT_024;
        break;

case "spam":
//.....................................................................................................
        if (! $comment->operator) break;
        if (! $comment->delete($comment_index, TRUE))
                $error[] = CMS_L_MOD_COMMENT_024;
        break;
};

insert("top");

//get number of comments
if (! ($result = mysql_query("SELECT COUNT(" . CMS_DB_COMMENT_INDEX . ") " .
                             "FROM " .         CMS_DB_COMMENT . " " .
                             "WHERE " .        CMS_DB_COMMENT_INSTANCE .
                             "='" .            sqlesc($comment->instance) . "'" .
                             ($comment->operator ? "" :
                            " AND " .          CMS_DB_COMMENT_STATUS .
                             "='" .            sqlesc(CMS_DB_COMMENT_STATUS_ACTIVE) . "'"))))
        return;

$count  = mysql_result($result, 0);
$_count = ceil($count / 10) - 1;
if ($_count < 0)         $_count = 0;
$page   = (int)$comment_page;
if ($page < 0)           $page   = 0;
elseif ($page > $_count) $page   = $_count;

//title
echo("<h3>" .
     "<a id=\"comment\" " .
        "href=\"" . x(CMS_ACTIVE_URL .
                      cms_param(array("comment_page" => $page))) .
                    "#comment\">" .
     sprintf(CMS_L_MOD_COMMENT_012, $count) .
     "</a>" .
     "</h3>");

//display errors
if (count($error))
        echo("<p class=\"comment-error\">" .
             "<strong>" .
             implode("<br>", $error) .
             "</strong>" .
             "</p>");

//display success
if ($success != "")
        echo("<p class=\"comment-success\">" .
             "<strong>" .
             $success .
             "</strong>" .
             "</p>");

//process messages
switch ($comment_message)
{
case "edit":
case "_edit":
//.....................................................................................................
        if (! $comment->operator) break;

        //save comment data
        if (
             streq($comment_message, "edit")
             &&
             ($result = mysql_query("SELECT * " .
                                    "FROM " .  CMS_DB_COMMENT . " " .
                                    "WHERE " . CMS_DB_COMMENT_INDEX .
                                    "='" .     sqlesc($comment_index) . "' " .
                                    "LIMIT 1"))
             &&
             ($resultrow = mysql_fetch_assoc($result))
           )
        {
                $comment_edit_name  = $resultrow[CMS_DB_COMMENT_NAME];
                $comment_edit_email = $resultrow[CMS_DB_COMMENT_EMAIL];
                $comment_edit_url   = $resultrow[CMS_DB_COMMENT_URL];
                $comment_edit_text  = $resultrow[CMS_DB_COMMENT_TEXT];
        };

        //display edit form
        echo("<form name=\"comment_edit\" " .
                   "action=\"" . x(CMS_ACTIVE_URL .
                                   cms_param(array("comment_message" => "_edit",
                                                   "comment_index"   => $comment_index,
                                                   "comment_page"    => $page))) .
                                 "#comment_" . x($comment_index) . "\" " .
                   "method=\"post\">" .

             "<table class=\"comment-edit\">" .

        //title
             "<tr>" .
             "<th colspan=\"2\">" .
             CMS_L_MOD_COMMENT_007 .
             "</th>" .
             "</tr>" .

        //name
             "<tr>" .
             "<td>" .
             "*&nbsp;" . CMS_L_NAME . ":" .
             "</td>" .
             "<td style=\"TEXT-ALIGN:right\">" .
             "<input name=\"comment_edit_name\" " .
                    "type=\"text\" " .
                    "size=\"40\" " .
                    "maxlength=\"40\" " .
                    "value=\"" . x($comment_edit_name) . "\">" .
             "</td>" .
             "</tr>" .

        //e-mail
             "<tr>" .
             "<td>" .
             "*&nbsp;" . CMS_L_EMAIL . ":" .
             "</td>" .
             "<td style=\"TEXT-ALIGN:right\">" .
             "<input name=\"comment_edit_email\" " .
                    "type=\"text\" " .
                    "size=\"40\" " .
                    "maxlength=\"40\" " .
                    "value=\"" . x($comment_edit_email) . "\"><br>" .
             "<small>" .
             CMS_L_MOD_COMMENT_008 .
             "</small>" .
             "</td>" .
             "</tr>" .

        //homepage url
             "<tr>" .
             "<td>" .
             CMS_L_MOD_COMMENT_009 . ":" .
             "</td>" .
             "<td style=\"TEXT-ALIGN:right\">" .
             "<input name=\"comment_edit_url\" " .
                    "type=\"text\" " .
                    "size=\"40\" " .
                    "maxlength=\"40\" " .
                    "value=\"" . x($comment_edit_url) . "\">" .
             "</td>" .
             "</tr>" .

        //text
             "<tr>" .
             "<td colspan=\"2\">" .
             "*&nbsp;" . CMS_L_MOD_COMMENT_010 . ":<br>" .
             "<textarea name=\"comment_edit_text\" " .
                       "cols=\"60\" " .
                       "rows=\"10\">\n" .
             x($comment_edit_text) .
             "</textarea>" .
             "</td>" .
             "</tr>" .

        //buttons
             "<tr>" .
             "<td colspan=\"2\" " .
                 "class=\"varied\" " .
                 "style=\"TEXT-ALIGN:right\">" .

        //cancel button
             "<input name=\"comment_edit_message\" " .
                    "type=\"submit\" " .
                    "value=\"" . CMS_L_COMMAND_CANCEL . "\"> " .

        //save button
             "<input name=\"comment_edit_message\" " .
                    "type=\"submit\" " .
                    "value=\"" . CMS_L_COMMAND_SAVE . "\">" .
             "</td>" .
             "</tr>" .

             "</table>" .

             "</form>");

        insert("bottom");
        return;
};

if ($comment->writer && (($success == "") || ($comment_message == "")))
{
        //display add form
        echo("<form name=\"comment_add\" " .
                   "action=\"" . x(CMS_ACTIVE_URL .
                                   cms_param(array("comment_message" => "add",
                                                   "comment_page"    => $page))) .
                                 "#comment\" " .
                   "method=\"post\">" .
             "<table class=\"comment-add\">" .

        //title
             "<tr>" .
             "<th colspan=\"2\">" .
             CMS_L_MOD_COMMENT_011 .
             "</th>" .
             "</tr>");

        if ($flag = streq(CMS_USER, "anonymous"))

                //name
                echo("<tr>" .
                     "<td>" .
                     "*&nbsp;" . CMS_L_NAME . ":" .
                     "</td>" .
                     "<td style=\"TEXT-ALIGN:right\">" .
                     "<input name=\"comment_add_name\" " .
                            "type=\"text\" " .
                            "size=\"40\" " .
                            "maxlength=\"40\" " .
                            "value=\"" . x($comment_add_name) . "\">" .
                     "</td>" .
                     "</tr>");

        if ($flag || (! verify_email(CMS_EMAIL)))
        {
                if (stre($comment_add_email)) $comment_add_email = CMS_EMAIL;

                //e-mail
                echo("<tr>" .
                     "<td>" .
                     "*&nbsp;" . CMS_L_EMAIL . ":" .
                     "</td>" .
                     "<td style=\"TEXT-ALIGN:right\">" .
                     "<input name=\"comment_add_email\" " .
                            "type=\"text\" " .
                            "size=\"40\" " .
                            "maxlength=\"40\" " .
                            "value=\"" . x($comment_add_email) . "\"><br>" .
                     "<small>" .
                     CMS_L_MOD_COMMENT_008 .
                     "</small>" .
                     "</td>" .
                     "</tr>");
        };

        //homepage url
        echo("<tr>" .
             "<td>" .
             CMS_L_MOD_COMMENT_009 . ":" .
             "</td>" .
             "<td style=\"TEXT-ALIGN:right\">" .
             "<input name=\"comment_add_url\" " .
                    "type=\"text\" " .
                    "size=\"40\" " .
                    "maxlength=\"40\" " .
                    "value=\"" . x($comment_add_url) . "\">" .
             "</td>" .
             "</tr>" .

        //text
             "<tr>" .
             "<td colspan=\"2\">" .
             "*&nbsp;" . CMS_L_MOD_COMMENT_010 . ":<br>" .
             "<textarea name=\"comment_add_text\" " .
                       "cols=\"60\" " .
                       "rows=\"10\">\n" .
             x($comment_add_text) .
             "</textarea>" .
             "</td>" .
             "</tr>");

        //captcha
        if ($use_captcha && cms_load("captcha"))
        {
                $captcha = new captcha();
                $code    = $captcha->create();

                if ($code !== FALSE)
                        echo("<tr>" .
                             "<td>" .
                             CMS_L_MOD_COMMENT_043 . ":" .
                             "</td>" .
                             "<td style=\"TEXT-ALIGN:right\">" .
                             "<input name=\"comment_add_captcha_key\" " .
                                    "type=\"text\" " .
                                    "size=\"40\" " .
                                    "maxlength=\"40\" " .
                                    "value=\"\"><br>" .
                             "<small>" .
                             CMS_L_MOD_COMMENT_044 .
                             "</small><br>" .
                             "<input name=\"comment_add_captcha_code\" " .
                                    "type=\"hidden\" " .
                                    "value=\"" . x($code) . "\">" .
                             image(CMS_DATA_URL . "captcha/$code.png", NULL, NULL, CMS_L_MOD_COMMENT_043) .
                             "</td>" .
                             "</tr>");
        };

        //buttons
        echo("<tr>" .
             "<td colspan=\"2\" " .
                 "class=\"varied\" " .
                 "style=\"TEXT-ALIGN:right\">" .

        //save button
             "<input name=\"comment_add_message\" " .
                    "type=\"submit\" " .
                    "value=\"" . CMS_L_COMMAND_SAVE . "\"> " .
             "</td>" .
             "</tr>" .

             "</table>" .
             "</form>");
};

if ($count > 0)
{
        echo(jscript("
function comment_rate_bad(index)
{
        var text = \"" . q(CMS_L_MOD_COMMENT_034) . "\";
        var url  = \"" . q(CMS_ACTIVE_URL .
                           cms_param(array("comment_message" => "rate_bad",
                                           "comment_index"   => "\x1B%s",
                                           "comment_page"    => $page))) .
                         "#comment\".replace(/%s/, index);
        if (confirm(text)) window.location.href = url;
};
//................................
function comment_rate_good(index)
{
        var text = \"" . q(CMS_L_MOD_COMMENT_035) . "\";
        var url  = \"" . q(CMS_ACTIVE_URL .
                           cms_param(array("comment_message" => "rate_good",
                                           "comment_index"   => "\x1B%s",
                                           "comment_page"    => $page))) .
                         "#comment\".replace(/%s/, index);
        if (confirm(text)) window.location.href = url;
};
//................................
function comment_delete(index)
{
        var text = \"" . q(CMS_L_MOD_COMMENT_013) . "\";
        var url  = \"" . q(CMS_ACTIVE_URL .
                           cms_param(array("comment_message" => "delete",
                                           "comment_index"   => "\x1B%s",
                                           "comment_page"    => $page))) .
                         "#comment\".replace(/%s/, index);
        if (confirm(text)) window.location.href = url;
};
//................................
function comment_spam(index)
{
        var text = \"" . q(CMS_L_MOD_COMMENT_021) . "\";
        var url  = \"" . q(CMS_ACTIVE_URL .
                           cms_param(array("comment_message" => "spam",
                                           "comment_index"   => "\x1B%s",
                                           "comment_page"    => $page))) .
                         "#comment\".replace(/%s/, index);
        if (confirm(text)) window.location.href = url;
};"));

        if (
             ($result = mysql_query("SELECT * " .
                                    "FROM " .     CMS_DB_COMMENT . " " .
                                    "WHERE " .    CMS_DB_COMMENT_INSTANCE .
                                    "='" .        sqlesc($comment->instance) . "' " .
                                    ($comment->operator ? "" :
                                    "AND " .      CMS_DB_COMMENT_STATUS .
                                    "='" .        sqlesc(CMS_DB_COMMENT_STATUS_ACTIVE) . "' ") .
                                    "ORDER BY " . CMS_DB_COMMENT_TIME . " " .
                                                  ($display_order ? "" : "DESC ") .
                                    "LIMIT " .    sqlesc($page * 10) . ", 10"))
             &&
             (mysql_num_rows($result) > 0)
           )
        {
                echo("<ol start=\"" . ($page * 10 + 1) . "\" " .
                         "class=\"comment-display\">");

                while ($resultrow = mysql_fetch_assoc($result))
                {
                        $index  = $resultrow[CMS_DB_COMMENT_INDEX];
                        $_index = x($index);
                        $name   = x($resultrow[CMS_DB_COMMENT_NAME]);
                        $url    = $resultrow[CMS_DB_COMMENT_URL];
                        $rating = (int)$resultrow[CMS_DB_COMMENT_RATING_VALUE];

                        echo("<li>" .

                        //head
                             "<div class=\"comment-display-head\">" .

                        //name
                             "<a id=\"comment_$_index\"" .
                             (preg_match("/^https?:\/\//", $url) ?
                               " href=\"" . x($url) . "\"" .
                               " title=\"" . CMS_L_MOD_COMMENT_018 . "\"" .
                               " rel=\"nofollow\">" : ">") .
                             $name .
                             "</a> " .
                             CMS_L_MOD_COMMENT_014 . "\xE2\x80\xA6" .
                             "</div>" .

                        //body
                             "<div class=\"comment-display-body\"" .
                                   (($resultrow[CMS_DB_COMMENT_STATUS] != CMS_DB_COMMENT_STATUS_ACTIVE) ?
                                   " style=\"FONT-STYLE:italic\">" : ">") .
                             x($resultrow[CMS_DB_COMMENT_TEXT]) .
                             "</div>" .

                        //foot
                             "<div class=\"comment-display-foot\">" .

                        //timestamp
                             "<a href=\"" . x(CMS_ACTIVE_URL .
                                              cms_param(array("comment_page" => $page))) .
                                            "#comment_$_index\" " .
                                "title=\"" . sprintf(CMS_L_MOD_COMMENT_017, $_index) . "\">" .
                             x(friendly_date($resultrow[CMS_DB_COMMENT_TIME])) .
                             "</a>" .

                        //display rating
                             " | " .
                             x(sprintf(CMS_L_MOD_COMMENT_019, $resultrow[CMS_DB_COMMENT_RATING_COUNT])) . ": " .
                             (($rating == 0) ? "<strong style=\"COLOR:grey\">"   :
                             (($rating > 0)  ? "<strong style=\"COLOR:green\">+" :
                                               "<strong style=\"COLOR:red\">")) .
                             $rating .
                             "</strong>");

                        //edit rating
                        if (strpos($resultrow[CMS_DB_COMMENT_RATING_USERID], "/" . CMS_USERID . "/") === FALSE)
                                echo(" [" .

                                //rate bad
                                     "<a href=\"#\" " .
                                        "title=\"" . CMS_L_MOD_COMMENT_036 . "\" " .
                                        "onclick=\"comment_rate_bad('$_index');return false;\">" .
                                     " - " .
                                     "</a> " .

                                     "|" .

                                //rate good
                                     "<a href=\"#\" " .
                                        "title=\"" . CMS_L_MOD_COMMENT_037 . "\" " .
                                        "onclick=\"comment_rate_good('$_index');return false;\">" .
                                     " + " .
                                     "</a>" .

                                     "]");

                        echo("</div>");

                        //command
                        if ($comment->operator)
                        {
                                echo("<div class=\"comment-display-command\">" .

                                //edit command
                                     "<a href=\"" . x(CMS_ACTIVE_URL .
                                                      cms_param(array("comment_message" => "edit",
                                                                      "comment_index"   => $index,
                                                                      "comment_page"    => $page))) .
                                                    "#comment\">" .
                                     "[" . CMS_L_COMMAND_EDIT . "]" .
                                     "</a> ");

                                //disable command
                                if ($resultrow[CMS_DB_COMMENT_STATUS] == CMS_DB_COMMENT_STATUS_ACTIVE)
                                        echo("<a href=\"" . x(CMS_ACTIVE_URL .
                                                              cms_param(array("comment_message" => "disable",
                                                                              "comment_index"   => $index,
                                                                              "comment_page"    => $page))) .
                                                            "#comment_$_index\">" .
                                             "[" . CMS_L_MOD_COMMENT_015 . "]" .
                                             "</a> ");

                                //enable command
                                else
                                        echo("<strong>" .
                                             "<a href=\"" . x(CMS_ACTIVE_URL .
                                                              cms_param(array("comment_message" => "enable",
                                                                              "comment_index"   => $index,
                                                                              "comment_page"    => $page))) .
                                                            "#comment_$_index\">" .
                                             "[" . CMS_L_MOD_COMMENT_016 . "]" .
                                             "</a>" .
                                             "</strong> ");

                                //delete command
                                echo("<a href=\"#\" " .
                                        "onclick=\"comment_delete('$_index');return false;\">" .
                                     "[" . CMS_L_COMMAND_DELETE . "]" .
                                     "</a> " .

                                //spam command
                                     "<a href=\"#\" " .
                                        "onclick=\"comment_spam('$_index');return false;\">" .
                                     "[" . CMS_L_MOD_COMMENT_020 . "]" .
                                     "</a> " .
                                     "<span title=\"" . CMS_L_MOD_COMMENT_039 . "\">" .
                                     "(" . x($resultrow[CMS_DB_COMMENT_SPAM_PROBABILITY]) . "%)" .
                                     "</span>" .
                                     "</div>");
                        };

                        echo("</li>");
                };

                echo("</ol>");
        };

        //page selection
        echo("<div style=\"TEXT-ALIGN:right\">");

        //previous command
        if ($page > 0)
                echo("<a href=\"" . x(CMS_ACTIVE_URL .
                                      cms_param(array("comment_page" => $page - 1))) .
                                    "#comment\">" .
                     "\xC2\xAB" .
                     "</a> ");

        //page number command
        for ($i = 0; $i <= $_count; $i++)
                echo("<a href=\"" . x(CMS_ACTIVE_URL .
                                      cms_param(array("comment_page" => $i))) .
                                    "#comment\">" .
                     (($i === $page) ? "<strong>" . ($i + 1) . "</strong>" : $i + 1) .
                     "</a>" .
                     (($i < $_count) ? " | " : ""));

        //next command
        if ($page < $_count)
                echo(" | " .
                     "<a href=\"" . x(CMS_ACTIVE_URL .
                                      cms_param(array("comment_page" => $page + 1))) .
                                    "#comment\">" .
                     CMS_L_MOD_COMMENT_003 . "&nbsp;\xC2\xBB" .
                     "</a>");

        echo("</div>");
};

insert("bottom");
permission(array(CMS_COMMENT_PERMISSION_READER . ".$instance"   => CMS_L_READ,
                 CMS_COMMENT_PERMISSION_WRITER . ".$instance"   => CMS_L_WRITE,
                 CMS_COMMENT_PERMISSION_OPERATOR . ".$instance" => CMS_L_OPERATOR));
?>