<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

//-----------------------------------------------------------------------------------------------------
//===< MODULE >========================================================================================
//-----------------------------------------------------------------------------------------------------
global $location,
       $identification_message,
       $identification_user,
       $identification_email,
       $identification_captcha_key,
       $identification_captcha_code,
       $identification_code;

$recover_message = "<p>" .
                   CMS_L_MOD_IDENTIFICATION_010 .
                   "</p>";

switch ($identification_message)
{
case "__recover":
//.....................................................................................................
        if (stre($identification_code)) //no code
        {
                //fail
                echo("<p class=\"identification-error\">" .
                     CMS_L_MOD_IDENTIFICATION_017 .
                     "</p>");
                return; //exit
        };

        //retrieve recovery data
        $data = new data("#system/identification.recover");
        $user = $data->get($identification_code, "user");
        $type = $data->get($identification_code, "type");

        //erase recovery data
        $data->del($identification_code);
        $data->save();

        $password = unique_id();
        $success  = FALSE;

        switch ($type)
        {
        case "permission":
//.....................................................................................................

                //verify user
                $data  = new data("#system/permission");
                $email = $data->get("user.$user", "email");
                if ($email === NULL) break; //fail

                //set new password
                $data->set(md5($password), "user.$user", "password");
                if (! $data->save()) break; //fail

                $success = TRUE;
                break;

        case "profile":
//.....................................................................................................

                //verify user
                if (! cms_load("profile"))   break;
                $profile      = new profile(TRUE);
                $profile_data = $profile->get($user, CMS_DB_PROFILE_USER);
                if ($profile_data === FALSE) break; //fail
                $email        = $profile_data->email;

                //set new password
                $profile_data->password = $password;
                if (! $profile->set($profile_data)) break; //fail

                $success = TRUE;
                break;
        };

        if ($success) //successfully wrote new password
        {
                //success
                echo("<p class=\"identification-success\">" .
                     CMS_L_MOD_IDENTIFICATION_018 .
                     "</p>");

                //send password mail
                if (
                     cms_load("smtp")
                     &&
                     smtp_send($email,
                               CMS_L_MOD_IDENTIFICATION_022,
                               sprintf(CMS_L_MOD_IDENTIFICATION_021,
                                       $user,
                                       CMS_ROOT_URL,
                                       $password))
                   )
                        //success
                        echo("<p class=\"identification-success\">" .
                             CMS_L_MOD_IDENTIFICATION_019 .
                             "</p>");
                else
                        //fail
                        echo("<p class=\"identification-error\">" .
                             CMS_L_MOD_IDENTIFICATION_020 .
                             "</p>");
        }
        else
        {
                //fail
                echo("<p class=\"identification-error\">" .
                     CMS_L_MOD_IDENTIFICATION_017 .
                     "</p>");
        };

        return; //exit

case "_recover":
//.....................................................................................................

        //verify captcha
        if (cms_load("captcha"))
        {
                $captcha = new captcha();

                if (! $captcha->verify($identification_captcha_key, $identification_captcha_code)) //captcha mismatch
                {
                        //fail
                        $recover_message = "<p class=\"identification-mismatch\">" .
                                           CMS_L_MOD_IDENTIFICATION_013 .
                                           "</p>";
                        break;
                };
        };

        //fail as default
        $recover_message = "<p class=\"identification-mismatch\">" .
                           CMS_L_MOD_IDENTIFICATION_011 .
                           "</p>";

        if (stre($identification_user)) break; //missing user id, fail

        //seek user in permissions
        $data  = new data("#system/permission");
        $email = $data->get("user.$identification_user", "email");
        $type  = "permission";

        if (($email === NULL) && cms_load("profile")) //fail
        {
                //alternatively seek user in profiles
                $profile      = new profile(TRUE);
                $profile_data = $profile->get($identification_user, CMS_DB_PROFILE_USER);

                if (($profile_data !== FALSE) && nstre($profile_data->email))
                {
                        $email = $profile_data->email;
                        $type  = "profile";
                };
        };

        //user has valid email address
        if (verify_email($email))
        {
                //store recovery data
                $data = new data("#system/identification.recover");
                $code = unique_id(16);

                $data->set(array("user"    => $identification_user,
                                 "type"    => $type,
                                 "#expire" => time() + 3600), //1 hour
                           $code);

                if (

                //save recovery data
                     $data->save()
                     &&

                //send recovery mail
                     cms_load("smtp")
                     &&
                     smtp_send($email,
                               CMS_L_MOD_IDENTIFICATION_009,
                               sprintf(CMS_L_MOD_IDENTIFICATION_016,
                                       $identification_user,
                                       CMS_ROOT_URL,
                                       CMS_ACTIVE_URL .
                                       cms_param(array("identification_message" => "__recover",
                                                       "identification_code"    => $code))))
                   )
                {
                        //success
                        echo("<p class=\"identification-success\">" .
                             CMS_L_MOD_IDENTIFICATION_012 .
                             "</p>");
                        return; //exit
                };
        };
};

switch ($identification_message)
{
case "_recover":
case "recover":
//.....................................................................................................
        insert("recover_top");

        echo("<form id=\"identification\" " .
                   "action=\"" . x(CMS_ACTIVE_URL . cms_param()) . "#identification\" " .
                   "method=\"post\">" .
             "<section class=\"identification\">" .

        //title
             "<h2>" .
             CMS_L_MOD_IDENTIFICATION_009 .
             "</h2>" .

        //instruction
             $recover_message .

        //username
             "<fieldset>" .

             "<legend>" .
             "<label for=\"identification_user\">" .
             CMS_L_MOD_IDENTIFICATION_002 .
             "</label>" .
             "</legend>" .

             "<input name=\"identification_user\" " .
                    "type=\"text\" " .
                    "value=\"" . x($identification_user) . "\" " .
                    "size=\"40\" " .
                    "maxlength=\"40\">" .

             "</fieldset>");

        //captcha
        if (cms_load("captcha"))
        {
                //create captcha
                $captcha = new captcha();
                $code    = $captcha->create();

                //start output
                if ($code !== FALSE)
                        echo("<fieldset>" .

                             "<legend>" .
                             "<label for=\"identification-captcha\">" .
                             CMS_L_MOD_IDENTIFICATION_014 .
                             "</label>" .
                             "</legend>" .

                        //captcha input
                             "<input id=\"identification-captcha\" " .
                                    "name=\"identification_captcha_key\" " .
                                    "type=\"text\" " .
                                    "size=\"40\" " .
                                    "maxlength=\"40\"><br>" .

                             "<small>" .
                             CMS_L_MOD_IDENTIFICATION_015 .
                             "</small><br>" .

                        //verification code
                             "<input name=\"identification_captcha_code\" " .
                                    "type=\"hidden\" " .
                                    "value=\"" . x($code) . "\">" .
                             image(CMS_DATA_URL . "captcha/$code.png", NULL, NULL, CMS_L_MOD_IDENTIFICATION_014) .

                             "</fieldset>");
        };

        //submit button
        echo("<fieldset class=\"identification-command\">" .

             "<button name=\"identification_message\" " .
                     "type=\"submit\" " .
                     "value=\"_recover\">" .
             CMS_L_COMMAND_CONFIRM .
             "</button>" .

             "</fieldset>" .

             "</section>" .
             "</form>");

        insert("recover_bottom");

        return; //exit
};

$flag_continue = FALSE;
if (stre($location)) $location      = CMS_ACTIVE_URL . cms_param();
else                 $flag_continue = TRUE;
$location      = x($location);

//generate obfuscation string
$secret1 = cms_cache("cms.5c080f9d6448aa7514a2842e1ae732d7");
$secret2 = "11f544779d678530603c344c5381d2b1";
$string  = md5("$secret1|$secret2");

?>
<script src="<?php echo(x(CMS_JAVASCRIPT_URL));?>md5.js"></script>
<script>
function identification_submit()
{
        with (document.getElementById("identification-password"))
                value = md5("<?php echo($string);?>" + "|" + md5(value));
};
</script>
<?php
insert("top");

echo("<form id=\"identification\" " .
           "action=\"$location\" " .
           "method=\"post\" " .
           "onsubmit=\"identification_submit();\">" .
     "<section class=\"identification\">" .

     "<noscript>" .
     "<input name=\"cms_login_noscript\" " .
            "type=\"hidden\" " .
            "value=\"1\">" .
     "</noscript>" .

//title
     "<h2>" .
     CMS_L_MOD_IDENTIFICATION_004 .
     "</h2>");

//display current user
if (nstreq(CMS_USER, "anonymous"))
{
        echo("<p>" .
             sprintf(CMS_L_MOD_IDENTIFICATION_001, x(CMS_NAME), x(CMS_USER)) . "<br>");
        if ($flag_continue)
                echo("<a href=\"$location\">" .
                     CMS_L_MOD_IDENTIFICATION_008 .
                     "</a> " .
                     "| ");
        echo("<a href=\"" . x(CMS_MODULES_URL) . "identification.php?cms_logout=1\">" .
             CMS_L_MOD_IDENTIFICATION_003 .
             "</a>" .
             "</p>");
};

//username
echo("<fieldset>" .

     "<legend>" .
     "<label for=\"identification-user\">" .
     CMS_L_MOD_IDENTIFICATION_002 .
     "</label>" .
     "</legend>" .

     "<input id=\"identification-user\" " .
            "name=\"cms_login_user\" " .
            "type=\"text\" " .
            option(nstreq(CMS_USER, "anonymous"), "value=\"" . x(CMS_USER) . "\" ") .
            "size=\"40\" " .
            "maxlength=\"40\">" .

     "</fieldset>" .

//password
     "<fieldset>" .

     "<legend>" .
     "<label for=\"identification-password\">" .
     CMS_L_PASSWORD .
     "</label>" .
     "</legend>" .

     "<input id=\"identification-password\" " .
            "name=\"cms_login_password\" " .
            "type=\"password\" " .
            "size=\"40\" " .
            "maxlength=\"40\"><br>" .

//recover password
     "<a href=\"" . x(CMS_ACTIVE_URL . cms_param(array("identification_message" => "recover"))) . "#identification\">" .
     CMS_L_MOD_IDENTIFICATION_006 .
     "</a>" .

     "</fieldset>" .

//submit button
     "<fieldset class=\"identification-command\">" .

     "<button type=\"submit\">" .
     CMS_L_MOD_IDENTIFICATION_005 .
     "</button>" .

     "</fieldset>" .

     "</section>" .
     "</form>");
?>
<script>
with (document.getElementById("identification-user"))
{
        select();
        focus();
};
</script>
<?php
insert("bottom");
?>
