<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

//-----------------------------------------------------------------------------------------------------
//===< MODULE >========================================================================================
//-----------------------------------------------------------------------------------------------------
global $mailform_form,
       $mailform_form_active,
       $mailform_page,
       $mailform_message,
       $mailform_captcha_code,
       $mailform_captcha_key;

if (! cms_load("smtp"))
{
        echo(CMS_MSG_UNAVAILABLE);
        return;
};

$data = new data("#system/mailform");

//display form selection
if (stre($mailform_form))
{
        $buffer = "";
        $count  = 0;

        //list all valid forms
        $data->move("first");
        while (($key = $data->move("next")) !== NULL)
        {
                if (
                     $data->is_container($key)
                     &&
                     (
                       nstre(l($data->get($key, "email")))
                       ||
                       nstre(l($data->get($key, "url")))
                     )
                   )
                {
                        $mailform_form  = $key;
                        $count         += 1;
                        $name           = l($data->get($key, "name"));
                        $description    = l($data->get($key, "description"));

                        $buffer        .= "<li>" .
                                          "<a href=\"" . x(CMS_ACTIVE_URL .
                                                           cms_param(array("mailform_form" => $key))) . "\">" .
                                          x($name) .
                                          "</a>" .
                                          option($description, "<p>" . parse_text($description) . "</p>") .
                                          "</li>";
                };
        };

        //no or more than one form
        if ($count <> 1)
        {
                echo("<section>");

                //more than one form => display list and exit
                if ($count > 1)
                {
                        insert("top");

                        echo("<h2>" .
                             CMS_L_MOD_MAILFORM_011 .
                             "</h2>" .
                             "<ul>" .
                             $buffer .
                             "</ul>");

                        insert("bottom");
                        permission(array("" => CMS_L_ACCESS));
                }

                //no form => display message and exit
                else
                {
                        echo(CMS_MSG_UNAVAILABLE);
                };

                echo("</section>");
                return;
        };

        //otherwise display the one existing form
};

//check if current form has been submitted
$flag_active = streq($mailform_form, $mailform_form_active);

if ($flag_active)
{
        $_mailform_page = (int)$mailform_page;

        //process message
        switch ($mailform_message)
        {
        case CMS_L_COMMAND_PREVIOUS: //go to previous page
//.....................................................................................................
                if (--$_mailform_page < 0) $_mailform_page = 0;
                break;

        default:
        case CMS_L_COMMAND_NEXT:     //go to next page or submit form
//.....................................................................................................
                $_mailform_page++;
                break;
        };
}

//otherwise go to first page
else
{
        $_mailform_page = 0;
};

//process data
$array          = array();
$form_name      = l($data->get($mailform_form, "name"));
$_form_name     = x($form_name);
$confirm        = NULL;
$page           = 0;
$flag_mismatch  = FALSE;
$flag_pagebreak = FALSE;
$flag_captcha   = FALSE;
$i              = 1;

//seek current form and run through elements
$data->move("to", $mailform_form);
while (($key = $data->move("next")) !== NULL)
{
        //exit loop at end of current form
        $type = $data->get($key, "#type");
        if (streq($type, "/container")) break;

        //check for page break
        if (streq($type, "pagebreak"))
        {
                //overwrite element data
                $array[$key] = TRUE;

                //exit loop at end of current page
                if (++$page > $_mailform_page)
                {
                        $flag_pagebreak = TRUE;
                        break;
                };

                continue;
        };

        //set element id
        $id = $data->get($key, "id");
        if (stre($id)) $id = "mailform_value$i";
        $i++;

        //get submitted data
        $_id   = preg_replace("/[\\x20\\x2E\\x5B\\x80-\\x9F]/", "_", $id);
        $value = isset($GLOBALS[$_id]) ? $GLOBALS[$_id] : NULL;
        if (is_array($value)) $value = implode("\n", $value);

        //add data to element list
        $array[$key] = array("name"  => l($data->get($key, "name")),
                             "id"    => $id,
                             "value" => $value);

        if ($flag_active) //form has been submitted
        {
                if ($flag_mismatch || ($page < $_mailform_page)) //invalid input or final submission
                {
                        $_flag_mismatch = FALSE;

                        if (nstre($value)) //value is set
                        {
                                //check input against regular expression
                                if (
                                     nstre($match = $data->get($key, "match"))
                                     &&
                                     (preg_match($match, $value) !== 1)
                                   )
                                        $_flag_mismatch = TRUE; //failed

                                //add input to confirmation receiver list
                                elseif (
                                         ($data->get($key, "confirm") !== NULL)
                                         &&
                                         verify_email($value)
                                       )
                                        $confirm[] = $value;
                        }

                        //check if input is required
                        elseif ($data->get($key, "required") !== NULL)
                        {
                                $_flag_mismatch = TRUE;
                        };

                        //set display form flag on mismatch
                        if ($_flag_mismatch)
                        {
                                $array[$key]["mismatch"] = TRUE;
                                $flag_mismatch           = TRUE;
                                $_mailform_page          = $page;
                        };
                };
        };
};

if (! $flag_pagebreak) //last page
{
        if (
             (($captcha = $data->get($mailform_form, "captcha")) !== NULL) //use captcha
             &&
             $flag_active                                                  //form has been submitted
             &&
             ($flag_mismatch || ($page < $_mailform_page))                 //invalid input or final submission
             &&
             cms_load("captcha")                                           //load library
           )
        {
                //verify captcha
                $_captcha = new captcha();

                //captcha mismatch
                if (! $_captcha->verify($mailform_captcha_key, $mailform_captcha_code))
                {
                        $flag_captcha   = TRUE;
                        $flag_mismatch  = TRUE;
                        $_mailform_page = $page;
                };
        };

        $page++;
};

echo("<section id=\"mailform-" . x($mailform_form) . "\">");

//display form
if ($page > $_mailform_page)
{
        insert("form_top");

        echo("<form action=\"" . x(CMS_ACTIVE_URL . cms_param()) . "#mailform-" . rx($mailform_form) . "\" " .
                   "method=\"post\">" .
             "<section class=\"mailform\">" .

        //title
             "<h2>" .
             $_form_name .
             "</h2>" .

        //hidden data fields
             "<input name=\"mailform_form\" " .
                    "type=\"hidden\" " .
                    "value=\"" . x($mailform_form) . "\">" .
             "<input name=\"mailform_form_active\" " .
                    "type=\"hidden\" " .
                    "value=\"" . x($mailform_form) . "\">" .
             "<input name=\"mailform_page\" " .
                    "type=\"hidden\" " .
                    "value=\"$_mailform_page\">");

        //display mismatch message
        if ($flag_mismatch)
                echo("<p class=\"mailform-mismatch\">" .
                     CMS_L_MOD_MAILFORM_008 .
                     "</p>");

        $_page = 0;

        reset($array);
        while (list($key, $value) = each($array))
        {
                //skip page break
                if ($value === TRUE)
                {
                        $_page++;
                        continue;
                };

                //field name
                $name     = x($value["name"]);
                if ($data->get($key, "required") !== NULL) $name = "* $name";

                //field id
                $id       = x($value["id"]);
                $_id      = x("mailform-$mailform_form-$key");

                //mismatch flag
                $mismatch = isset($value["mismatch"]);

                //field value
                $value    = $value["value"];

                //hide input of precedent pages
                if ($_page < $_mailform_page)
                {
                        echo("<input name=\"$id\" " .
                                    "type=\"hidden\" " .
                                    "value=\"" . x($value) . "\">");
                        continue;
                };

                //display element
                $type = $data->get($key, "#type");

                switch ($type)
                {
                case "checkbox":
                case "radio":
//.....................................................................................................

                        //option values
                        $option  = preg_split("/\r?\n/", l($data->get($key, "option")),
                                              -1, PREG_SPLIT_NO_EMPTY);
                        if (count($option) === 0) break;

                        //selected values
                        $novalue = stre($value);
                        $value   = explode("\n", $value);

                        //formatting
                        $column  = (int)$data->get($key, "column");
                        if ($column < 1)          $column = 1;
                        $width   = 100 / $column; //100% width divided by number of columns

                        //start output
                        echo("<fieldset class=\"mailform-$type" . option($mismatch, " mailform-mismatch") . "\">" .

                             "<legend" . option($mismatch, " style=\"COLOR:#ff0000\"") . ">" .
                             $name .
                             "</legend>");

                        //description
                        $description = l($data->get($key, "description"));
                        if (nstre($description))
                                echo("<p class=\"mailform-description\">" .
                                     parse_text($description) .
                                     "</p>");

                        while (list($_key, $_value) = each($option))
                        {
                                //preselection
                                $selected  = FALSE;
                                if (streq(substr($_value, 0, 1), "+"))
                                {
                                        $selected = $novalue && ($page > $_mailform_page);
                                        $_value   = substr($_value, 1);
                                };
                                $selected |= in_array($_value, $value);

                                //output option
                                echo("<div style=\"DISPLAY:inline-block;WIDTH:$width%\">" .

                                     "<input id=\"$_id-$_key\" " .
                                            "name=\"$id" . option(streq($type, "checkbox"), "[$_key]") . "\" " .
                                            "type=\"$type\" " .
                                            "value=\"" . x($_value) . "\"" .
                                            option($selected, " checked") . ">" .

                                     "<label for=\"$_id-$_key\">" .
                                     x($_value) .
                                     "</label>" .

                                     "</div>");
                        };

                        echo("</fieldset>");

                        break;

                case "select":
//.....................................................................................................

                        //option values
                        $option  = preg_split("/\r?\n/", l($data->get($key, "option")),
                                              -1, PREG_SPLIT_NO_EMPTY);
                        if (count($option) === 0) break;

                        $novalue = stre($value);

                        //start output
                        echo("<fieldset class=\"mailform-select" . option($mismatch, " mailform-mismatch") . "\">" .

                             "<legend" . option($mismatch, " style=\"COLOR:#ff0000\"") . ">" .
                             "<label for=\"$_id\">" .
                             $name .
                             "</label>" .
                             "</legend>");

                        //description
                        $description = l($data->get($key, "description"));
                        if (nstre($description))
                                echo("<p class=\"mailform-description\">" .
                                     parse_text($description) .
                                     "</p>");

                        echo("<select id=\"$_id\" " .
                                     "name=\"$id\">");

                        reset($option);
                        while (list($_key, $_value) = each($option))
                        {
                                //preselection
                                $selected  = FALSE;
                                if (streq(substr($_value, 0, 1), "+"))
                                {
                                        $selected = stre($value) && ($page > $_mailform_page);
                                        $_value   = substr($_value, 1);
                                };
                                $selected |= streq($_value, $value);

                                //output option
                                $_value    = x($_value);
                                echo("<option value=\"$_value\"" . option($selected, " selected") . ">" .
                                     whitechartostring($_value, "&nbsp;") .
                                     "</option>");
                        };

                        echo("</select>" .

                             "</fieldset>");

                        break;

                case "text":
//.....................................................................................................

                        //start output
                        echo("<fieldset class=\"mailform-text" . option($mismatch, " mailform-mismatch") . "\">" .

                             "<legend" . option($mismatch, " style=\"COLOR:#ff0000\"") . ">" .
                             "<label for=\"$_id\">" .
                             $name .
                             "</label>" .
                             "</legend>");

                        //description
                        $description = l($data->get($key, "description"));
                        if (nstre($description))
                                echo("<p class=\"mailform-description\">" .
                                     parse_text($description) .
                                     "</p>");

                        //preset value
                        if (stre($value) && ($page > $_mailform_page))
                                $value = l($data->get($key, "default"));

                        //parameters
                        $width  = (int)$data->get($key, "width");
                        $length = (int)$data->get($key, "length");

                        echo("<input id=\"$_id\" " .
                                    "name=\"$id\" " .
                                    "type=\"text\" " .
                                    "value=\"" . x($value) . "\"" .
                                    option($width  > 0, " size=\"" . x($width) . "\"") .
                                    option($length > 0, " maxlength=\"" . x($length) . "\"") . ">" .

                             "</fieldset>");

                        break;

                case "textarea":
//.....................................................................................................

                        //start output
                        echo("<fieldset class=\"mailform-textarea" . option($mismatch, " mailform-mismatch") . "\">" .

                             "<legend" . option($mismatch, " style=\"COLOR:#ff0000\"") . ">" .
                             "<label for=\"$_id\">" .
                             $name .
                             "</label>" .
                             "</legend>");

                        //description
                        $description = l($data->get($key, "description"));
                        if (nstre($description))
                                echo("<p class=\"mailform-description\">" .
                                     parse_text($description) .
                                     "</p>");

                        //preset value
                        if (stre($value) && ($page > $_mailform_page))
                                $value = l($data->get($key, "default"));

                        //parameters
                        $width  = (int)$data->get($key, "width");
                        $height = (int)$data->get($key, "height");

                        echo("<textarea id=\"$_id\" " .
                                       "name=\"$id\" " .
                                       option($width  > 0, " cols=\"" . x($width) . "\"") .
                                       option($height > 0, " rows=\"" . x($height) . "\"") . ">" .
                             x($value) .
                             "</textarea>" .

                             "</fieldset>");

                        break;

                case "code":
//.....................................................................................................

                        //start output
                        if ($flag_codeonly = $data->get($key, "codeonly"))
                        {
                            echo("<div class=\"mailform-code" . option($mismatch, " mailform-mismatch") . "\">");
                        }
                        else
                        {
                            echo("<fieldset class=\"mailform-code" . option($mismatch, " mailform-mismatch") . "\">" .

                                 "<legend" . option($mismatch, " style=\"COLOR:#ff0000\"") . ">" .
                                 "<label for=\"$_id\">" .
                                 $name .
                                 "</label>" .
                                 "</legend>");

                            //description
                            $description = l($data->get($key, "description"));
                            if (nstre($description))
                                    echo("<p class=\"mailform-description\">" .
                                         parse_text($description) .
                                         "</p>");
                        };

                        //preset value
                        if (stre($value) && ($page > $_mailform_page))
                                $value = l($data->get($key, "default"));

                        //code
                        $code        = l($data->get($key, "code"));

                        //replace placeholder
                        $placeholder = array("label" => $name,
                                             "id"    => $_id,
                                             "name"  => $id,
                                             "value" => x($value));

                        reset($placeholder);
                        while (list($_key, $_value) = each($placeholder))
                                $code = stre($_value) ?
                                        preg_replace("/(?:\\[[^\\[\\]]*)?%$_key%(?:[^\\[\\]]*\\])?/",
                                                     "",
                                                     $code) :
                                        preg_replace("/(\\[([^\\[\\]]*))?%$_key%(([^\\[\\]]*)\\])?/",
                                                     "\${2}$_value\\4",
                                                     $code);

                        echo($code .

                             ($flag_codeonly ? "</div>" : "</fieldset>"));

                        break;

                case "hidden":
//.....................................................................................................

                        //preset value
                        if ((! $value) && ($page > $_mailform_page))
                                $value = l($data->get($key, "default"));

                        echo("<input name=\"$id\" " .
                                    "type=\"hidden\" " .
                                    "value=\"" . x($value) . "\">");

                        break;
                };
        };

        if (
             (! $flag_pagebreak) //last page
             &&
             $captcha            //use captcha
             &&
             cms_load("captcha") //load library
           )
        {
                //create captcha
                $_captcha = new captcha();
                $code     = $_captcha->create();

                //start output
                if ($code !== FALSE)
                        echo("<fieldset class=\"mailform-captcha" . option($flag_captcha, " mailform-mismatch") . "\">" .

                             "<legend" . option($flag_captcha, " style=\"COLOR:#ff0000\"") . ">" .
                             "<label for=\"mailform-captcha\">" .
                             "* " . CMS_L_MOD_MAILFORM_009 .
                             "</label>" .
                             "</legend>" .

                        //captcha input
                             "<input id=\"mailform-captcha\" " .
                                    "name=\"mailform_captcha_key\" " .
                                    "type=\"text\" " .
                                    "size=\"40\" " .
                                    "maxlength=\"40\" " .
                                    "value=\"\"><br>" .

                             "<small>" .
                             CMS_L_MOD_MAILFORM_010 .
                             "</small><br>" .

                        //verification code
                             "<input name=\"mailform_captcha_code\" " .
                                    "type=\"hidden\" " .
                                    "value=\"" . x($code) . "\">" .
                             image(CMS_DATA_URL . "captcha/$code.png", NULL, NULL, CMS_L_MOD_MAILFORM_009) .

                             "</fieldset>");
        };

        //form commands
        echo("<fieldset class=\"mailform-command\">");

        //button back
        if ($_mailform_page)
                echo("<input name=\"mailform_message\" " .
                            "type=\"submit\" " .
                            "value=\"" . CMS_L_COMMAND_PREVIOUS . "\"> ");

        //button next/submit
        if ($flag_pagebreak)
            $value = CMS_L_COMMAND_NEXT;
        elseif (stre($value = l($data->get($mailform_form, "submit"))))
            $value = CMS_L_MOD_MAILFORM_003;
        else
            $value = x($value);

        echo("<button name=\"mailform_message\" " .
                     "type=\"submit\" " .
                     "value=\"" . CMS_L_COMMAND_NEXT . "\">" .
             $value .
             "</button>" .

             "</fieldset>" .

             "</section>" .
             "</form>");

        insert("form_bottom");
        permission(array("" => CMS_L_ACCESS));

        echo("</section>");
        return;
};

//transmit form
$subject      = isset($GLOBALS["mailform_subject"]) ? $GLOBALS["mailform_subject"] : $form_name;
$confirmation = l($data->get($mailform_form, "confirmation"));
$confirmation = stre($confirmation) ? CMS_L_MOD_MAILFORM_002 : parse_text($confirmation);
$flag_success = FALSE;

if (($email = l($data->get($mailform_form, "email"))) || $confirm)
{
        //stylesheet
        $buffer   = read_file(CMS_PATH . "stylesheet/mail.css");
        $buffer   = stripspaces($buffer);
        $buffer   = str_replace("%", "%%", $buffer);

        //html
        $buffer   = CMS_DOCTYPE_HTML .
                    "<html>" .
                    "<head>" .
                    "<title>" .

        //title
                    "%s" .
                    "</title>" .
                    "<meta name=\"generator\" " .
                          "content=\"" . x(CMS) . "\">" .
                    "<meta http-equiv=\"content-type\" " .
                          "content=\"text/html; charset=utf-8\">" .
                    "<style>" .
                    $buffer .
                    "</style>" .
                    "</head>" .
                    "<body>" .

        //confirmation
                    "<p>" .
                    "%s" .
                    "</p>" .
                    "<div style=\"TEXT-ALIGN:right\">" .
                    date(CMS_L_DATE_FORMAT . ", " . CMS_L_SHORT_TIME_FORMAT) .
                    "</div>" .
                    "<table>" .
                    "<tr>" .
                    "<th>" .
                    CMS_L_MOD_MAILFORM_005 .
                    "</th>" .
                    "<th>" .
                    CMS_L_MOD_MAILFORM_006 .
                    "</th>" .
                    "</tr>" .

        //content
                    "%s" .
                    "</table>" .
                    "</body>" .
                    "</html>";

        //content
        $content  = NULL;
        $_content = NULL;

        reset($array);
        while (list($key, $value) = each($array))
        {
                if ($value === TRUE) continue;

                $_buffer   = "<tr>" .
                             "<td>" .
                             x($value["name"]) . ":" .
                             "</td>" .
                             "<td>" .
                             nl2br(x($value["value"])) .
                             "</td>" .
                             "</tr>";
                $content  .= $_buffer;
                if ($data->get($key, "secret")) continue;
                $_content .= $_buffer;

        };

        //send confirmation
        if ($confirm)
        {
                $_buffer = sprintf($buffer,
                                   CMS_L_MOD_MAILFORM_001 . ": $_form_name",
                                   $confirmation,
                                   $_content);

                reset($confirm);
                while (list($key, $value) = each($confirm))
                        smtp_send($value,                                //to
                                  CMS_L_MOD_MAILFORM_001 . ": $subject", //subject
                                  $_buffer,                              //body
                                  TRUE);                                 //html
        };

        //send form
        if ($email)
        {
                $_buffer      = sprintf($buffer, $_form_name, "", $content);
                $flag_success = smtp_send($email,                           //to
                                          $subject,                         //subject
                                          $_buffer,                         //body
                                          TRUE,                             //html
                                          $confirm ? end($confirm) : NULL); //reply_to
        };
};

//http post
if (nstre($url = l($data->get($mailform_form, "url"))))
{
        $_array  = array("mailform_form"    => $mailform_form,
                         "mailform_subject" => $subject);

        reset($array);
        while (list(, $value) = each($array))
                $_array[$value["id"]] = $value["value"];

        $__array = preg_split("/[\p{C}\p{Z}]+/u", $url);

        reset($__array);
        while (list(, $value) = each($__array))
                $flag_success |= http_post(translate_url($value), $_array) !== FALSE;
};

//display confirmation
if ($flag_success)
{
        $log = new log();
        $log->access(sprintf(CMS_L_MOD_MAILFORM_004, $form_name));

        insert("confirmation_top");

        echo("<p class=\"mailform-success\">" .
             $confirmation .
             "</p>");

        insert("confirmation_bottom");
}
else
{
        echo("<p class=\"mailform-error\">" .
             CMS_L_MOD_MAILFORM_012 .
             "</p>");
};

echo("</section>");
?>
