<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

//-----------------------------------------------------------------------------------------------------
//===< MODULE >========================================================================================
//-----------------------------------------------------------------------------------------------------
global $profile_message,
       $profile_param,
       $profile_user,
       $profile_email,
       $profile_captcha_key,
       $profile_captcha_code,
       $profile_code;

//load library
if (! cms_load("profile"))
{
        echo(CMS_MSG_UNAVAILABLE);
        return;
};

//instantiate
$profile = new profile();
if (! $profile->enabled)
{
        echo(CMS_MSG_UNAVAILABLE);
        return;
};

//operator status required for some operations
$profile->operator = TRUE;

//activation
if (streq($profile_message, "activate"))
{
        //seek registration data
        $data = new data("#system/profile.registration");
        if (stre($profile_code) || ($data->get($profile_code) === NULL))
        {
                //not found, invalid code
                echo("<p class=\"profile-error\">" .
                     CMS_L_MOD_PROFILE_047 .
                     "</p>");
                return;
        };

        //set profile data
        $profile_data = new profile_data();
        $array        = get_object_vars($profile_data);

        reset($array);
        while (list($key) = each($array))
        {
                $value = $data->get($profile_code, $key);
                eval("\$profile_data->$key = \$value;");
        };

        $profile_data->superuser = "profile";
        $profile_data->enabled   = TRUE;

        //create profile
        if ($profile->add($profile_data))
        {
                //log
                $log = new log();
                $log->access(sprintf(CMS_L_MOD_PROFILE_050, $profile_data->user));
                $log->user($profile_data->user, NULL, NULL, NULL, NULL, $email);

                //remove registration data
                $data->del($profile_code);
                $data->save();

                if (headers_sent())
                {
                        //success
                        echo("<p class=\"profile-success\">" .
                             CMS_L_MOD_PROFILE_048 .
                             "</p>");
                        return;
                };

                //obfuscate password
                $secret1  = cms_cache("cms.5c080f9d6448aa7514a2842e1ae732d7");
                $secret2  = "11f544779d678530603c344c5381d2b1";
                $password = md5(md5("$secret1|$secret2") . "|" . md5($profile_data->password));

                //redirect and log on
                $location = CMS_ACTIVE_URL .
                            cms_param(array("profile_message"    => "_activate",
                                            "cms_login_user"     => $profile_data->user,
                                            "cms_login_password" => $password));
                header("Location: $location");
                exit();
        };

        //error
        echo("<p class=\"profile-error\">" .
             CMS_MSG_ERROR .
             "</p>");
        return;
}

//retrieve profile
elseif (($profile_data = $profile->get(CMS_USER, CMS_DB_PROFILE_USER)) !== FALSE)
{
        if (stre($profile_message)) $profile_message = "edit";
}

//no profile
else
{
        //create new profile
        $profile_data = new profile_data();
        if (stre($profile_message)) $profile_message = "register";
};

//build field configuration
$data = new data("#system/profile");

//categories
$title = array(1 => CMS_L_MOD_PROFILE_002,
               2 => CMS_L_MOD_PROFILE_003,
               3 => CMS_L_MOD_PROFILE_004,
               4 => CMS_L_MOD_PROFILE_005,
               5 => CMS_L_MOD_PROFILE_006,
               6 => CMS_L_MOD_PROFILE_007,
               7 => CMS_L_MOD_PROFILE_011);

//default fields
$field = array(1 => array(CMS_DB_PROFILE_CODE                  => CMS_L_MOD_PROFILE_008,
                          CMS_DB_PROFILE_USER                  => CMS_L_MOD_PROFILE_009,
                          CMS_DB_PROFILE_PASSWORD              => CMS_L_PASSWORD),
               2 => array(CMS_DB_PROFILE_COMPANY               => CMS_L_MOD_PROFILE_012,
                          CMS_DB_PROFILE_PRENAME               => CMS_L_MOD_PROFILE_013,
                          CMS_DB_PROFILE_SURNAME               => CMS_L_MOD_PROFILE_014,
                          CMS_DB_PROFILE_STREET                => CMS_L_MOD_PROFILE_015,
                          CMS_DB_PROFILE_ZIPCODE               => CMS_L_MOD_PROFILE_016,
                          CMS_DB_PROFILE_CITY                  => CMS_L_MOD_PROFILE_017,
                          CMS_DB_PROFILE_COUNTRY               => CMS_L_MOD_PROFILE_018),
               3 => array(CMS_DB_PROFILE_PHONE_1               => CMS_L_MOD_PROFILE_019,
                          CMS_DB_PROFILE_PHONE_2               => CMS_L_MOD_PROFILE_020,
                          CMS_DB_PROFILE_MOBILE                => CMS_L_MOD_PROFILE_021,
                          CMS_DB_PROFILE_FAX                   => CMS_L_MOD_PROFILE_022,
                          CMS_DB_PROFILE_EMAIL                 => CMS_L_EMAIL,
                          CMS_DB_PROFILE_URL                   => CMS_L_URL),
               4 => array(CMS_DB_PROFILE_ACCOUNT_NUMBER        => CMS_L_MOD_PROFILE_023,
                          CMS_DB_PROFILE_FINANCIAL_INSTITUTION => CMS_L_MOD_PROFILE_024,
                          CMS_DB_PROFILE_BANKCODE              => CMS_L_MOD_PROFILE_025,
                          CMS_DB_PROFILE_ACCOUNT_HOLDER        => CMS_L_MOD_PROFILE_026),
               5 => array(CMS_DB_PROFILE_CREDIT_CARD_NUMBER    => CMS_L_MOD_PROFILE_027,
                          CMS_DB_PROFILE_CREDIT_INSTITUTE      => CMS_L_MOD_PROFILE_028,
                          CMS_DB_PROFILE_CREDIT_CARD_HOLDER    => CMS_L_MOD_PROFILE_029,
                          CMS_DB_PROFILE_CREDIT_CARD_VALIDITY  => CMS_L_MOD_PROFILE_030),
               6 => array(CMS_DB_PROFILE_COMMENT               => CMS_L_MOD_PROFILE_031));

//custom fields
for ($i = 1; $i <= 20; $i++)
{
        if (! $_title = $data->get(CMS_DB_PROFILE_CUSTOM_FIELD . $i, "title"))
                $_title = CMS_L_MOD_PROFILE_010 . " $i";

        $field[7][CMS_DB_PROFILE_CUSTOM_FIELD . $i] = $_title;
};

//remove fields used in registration
if ($flag = in_array($profile_message, array("register", "_register")))
{
        unset($field[1]);
        unset($field[3][CMS_DB_PROFILE_EMAIL]);
};

$visible  = array();
$editable = array();
$required = array();
$mismatch = array();

//read configuration
reset($field);
while (list($key, $value) = each($field))
{
        reset($value);
        while (list($_key) = each($value))
        {
                if ($data->get($_key, "visible"))
                {
                        $visible[$key][$_key] = TRUE;
                        if ($data->get($_key, "editable"))
                        {
                                $editable[$key][$_key] = TRUE;
                                if ($data->get($_key, "required")) $required[$key][$_key] = TRUE;
                        };
                };
        };
};

$user_equals_email = ! (bool)$data->get(CMS_DB_PROFILE_USER, "required");

//process form data
while (in_array($profile_message, array("_register", "_edit")))
{
        //set field data
        if (count($editable) > 0)
        {
                reset($editable);
                while (list($key, $value) = each($editable))
                {
                        reset($value);
                        while (list($_key, $_value) = each($value))
                        {
                                if (! isset($profile_param[$_key])) continue;

                                switch ($_key)
                                {
                                case CMS_DB_PROFILE_CODE:
                                case CMS_DB_PROFILE_USER:
                                case CMS_DB_PROFILE_COMPANY:
                                case CMS_DB_PROFILE_PRENAME:
                                case CMS_DB_PROFILE_SURNAME:
                                case CMS_DB_PROFILE_STREET:
                                case CMS_DB_PROFILE_ZIPCODE:
                                case CMS_DB_PROFILE_CITY:
                                case CMS_DB_PROFILE_COUNTRY:
                                case CMS_DB_PROFILE_PHONE_1:
                                case CMS_DB_PROFILE_PHONE_2:
                                case CMS_DB_PROFILE_MOBILE:
                                case CMS_DB_PROFILE_FAX:
                                case CMS_DB_PROFILE_URL:
                                case CMS_DB_PROFILE_ACCOUNT_NUMBER:
                                case CMS_DB_PROFILE_FINANCIAL_INSTITUTION:
                                case CMS_DB_PROFILE_BANKCODE:
                                case CMS_DB_PROFILE_ACCOUNT_HOLDER:
                                case CMS_DB_PROFILE_CREDIT_INSTITUTE:
                                case CMS_DB_PROFILE_CREDIT_CARD_HOLDER:
                                case CMS_DB_PROFILE_COMMENT:
                                default:
//.....................................................................................................
                                        $_value = $profile_param[$_key];
                                        break;

                                case CMS_DB_PROFILE_EMAIL:
//.....................................................................................................

                                        $_value = $profile_param[$_key];
                                        if (stre($_value)) break;

                                        //verify email
                                        if (! verify_email($_value))
                                        {
                                                $mismatch[$key][$_key] = CMS_L_MOD_PROFILE_044;
                                                continue 2;
                                        };

                                        break;

                                case CMS_DB_PROFILE_PASSWORD:
//.....................................................................................................

                                        //keep password
                                        if (stre($profile_param[$_key][0])) continue 2;

                                        //password and confirmation do not match
                                        if (nstreq($profile_param[$_key][0],
                                                   $profile_param[$_key][1]))
                                        {
                                                $mismatch[$key][$_key] = CMS_L_MOD_PROFILE_032;
                                                continue 2;
                                        };

                                        //password is too short
                                        if (strlen($profile_param[$_key][0]) < 4)
                                        {
                                                $mismatch[$key][$_key] = CMS_L_MOD_PROFILE_033;
                                                continue 2;
                                        };

                                        $_value = $profile_param[$_key][0];
                                        break;

                                case CMS_DB_PROFILE_CREDIT_CARD_NUMBER:
//.....................................................................................................
                                        $_value = implode("-", $profile_param[$_key]);
                                        break;

                                case CMS_DB_PROFILE_CREDIT_CARD_VALIDITY:
//.....................................................................................................
                                        $_value = implode("/", $profile_param[$_key]);
                                        break;
                                };

                                //check if required data is given
                                if (isset($required[$key][$_key]) && stre(utf8_trim($_value)))
                                {
                                        $mismatch[$key][$_key] = CMS_L_MOD_PROFILE_051;
                                        continue;
                                };

                                eval("\$profile_data->$_key = \$_value;");
                        };
                };

                //save
                if (
                     (count($mismatch) === 0)
                     &&
                     streq($profile_message, "_edit")
                   )
                {
                        if ($profile->set($profile_data))
                                echo("<p class=\"profile-success\">" .
                                     CMS_L_MOD_PROFILE_034 .
                                     "</p>");
                        break;
                };
        };

        //registration

        //verify user
        if (! $user_equals_email)
        {
                $profile_user = stripspaces($profile_user);
                $profile_user = utf8_substr($profile_user, 0, 40);
                $profile_user = utf8_trim($profile_user);
                if (stre($profile_user))
                {
                        $mismatch[0]["user"] = CMS_L_MOD_PROFILE_051;
                }
                else
                {
                        $permission   = new data("#system/permission");
                        $registration = new data("#system/profile.registration");

                        if (
                             ($profile->get($profile_user, CMS_DB_PROFILE_USER) !== FALSE)
                             ||
                             ($permission->get("user.$profile_user") !== NULL)
                             ||
                             ($registration->seek(array("user" => $profile_user)) !== FALSE)
                           )
                                $mismatch[0]["user"] = CMS_L_MOD_PROFILE_045;
                };
        };

        //verify email
        if (stre($profile_email))
                $mismatch[0]["email"] = CMS_L_MOD_PROFILE_051;

        elseif (! verify_email($profile_email))
                $mismatch[0]["email"] = CMS_L_MOD_PROFILE_044;

        elseif ($user_equals_email)
        {
                $profile_user = $profile_email;
                $permission   = new data("#system/permission");
                $registration = new data("#system/profile.registration");

                if (
                     ($profile->get($profile_user, CMS_DB_PROFILE_USER) !== FALSE)
                     ||
                     ($permission->get("user.$profile_user") !== NULL)
                     ||
                     ($registration->seek(array("user" => $profile_user)) !== FALSE)
                   )
                        $mismatch[0]["email"] = CMS_L_MOD_PROFILE_052;
        };

        //verify captcha
        if (cms_load("captcha"))
        {
                $captcha = new captcha();

                //captcha mismatch
                if (! $captcha->verify($profile_captcha_key, $profile_captcha_code))
                        $mismatch[0]["captcha"] = CMS_L_MOD_PROFILE_046;
        };

        //mismatch found
        if (isset($mismatch[0])) break;

        //set registration data
        $code     = unique_id();
        $password = unique_id();
        $registration->set(array("user"     => $profile_user,
                                 "email"    => $profile_email,
                                 "password" => $password,
                                 "#expire"  => time() + 86400) +
                           get_object_vars($profile_data),
                           $code);

        if (

        //save registration data
             $registration->save()
             &&

        //send confirmation mail
             cms_load("smtp")
             &&
             smtp_send($profile_email,
                       CMS_L_MOD_PROFILE_041,
                       sprintf(CMS_L_MOD_PROFILE_042,
                               CMS_ROOT_URL,
                               $profile_user,
                               $password,
                               CMS_ACTIVE_URL .
                               cms_param(array("profile_message" => "activate",
                                               "profile_code"    => $code))))
           )
        {
                //success
                echo("<p class=\"profile-success\">" .
                     CMS_L_MOD_PROFILE_043 .
                     "<p>");
                return;
        };

        //error
        echo("<p class=\"profile-error\">" .
             CMS_MSG_ERROR .
             "</p>");

        break;
};

//display
insert("top");

//display activation message
if (streq($profile_message, "_activate"))
        echo("<p class=\"profile-success\">" .
             CMS_L_MOD_PROFILE_048 .
             "</p>");

echo("<form id=\"profile\" " .
           "action=\"" . x(CMS_ACTIVE_URL . cms_param()) . "#profile\" " .
           "method=\"post\">" .
     "<section class=\"profile\">");

if ($flag)
{
        //title
        echo("<h2>" .
             CMS_L_MOD_PROFILE_039 .
             "</h2>");

        //display mismatch message
        if (count($mismatch) > 0)
                echo("<p class=\"profile-mismatch\">" .
                     CMS_L_MOD_PROFILE_036 .
                     "</p>");

        //user
        if (! $user_equals_email)
        {
                $_mismatch = isset($mismatch[0]["user"]);
                echo("<fieldset" . option($_mismatch, " class=\"profile-mismatch\"") . ">" .

                     "<legend" . option($_mismatch, " style=\"COLOR:#ff0000\"") . ">" .
                     "<label for=\"profile-user\">" .
                     "* " . CMS_L_MOD_PROFILE_009 .
                     "</label>" .
                     "</legend>");

                if ($_mismatch)
                        echo("<p class=\"profile-mismatch\">" .
                             x($mismatch[0]["user"]) .
                             "</p>");

                echo("<input id=\"profile-user\" " .
                            "name=\"profile_user\" " .
                            "type=\"text\" " .
                            "value=\"" . x($profile_user) . "\" " .
                            "size=\"40\" " .
                            "maxlength=\"40\">" .

                     "</fieldset>");
        };

        //email
        $_mismatch = isset($mismatch[0]["email"]);
        echo("<fieldset" . option($_mismatch, " class=\"profile-mismatch\"") . ">" .

             "<legend" . option($_mismatch, " style=\"COLOR:#ff0000\"") . ">" .
             "<label for=\"profile-email\">" .
             "* " . CMS_L_EMAIL .
             "</label>" .
             "</legend>");

        if ($_mismatch)
                echo("<p class=\"profile-mismatch\">" .
                     x($mismatch[0]["email"]) .
                     "</p>");

        echo("<input id=\"profile-email\" " .
                    "name=\"profile_email\" " .
                    "type=\"text\" " .
                    "value=\"" . x($profile_email) . "\" " .
                    "size=\"40\">" .

             "</fieldset>");

        //captcha
        if (cms_load("captcha"))
        {
                //create captcha
                $captcha = new captcha();
                $code    = $captcha->create();

                //start output
                if ($code !== FALSE)
                {
                        $_mismatch = isset($mismatch[0]["captcha"]);
                        echo("<fieldset" . option($_mismatch, " class=\"profile-mismatch\"") . ">" .

                             "<legend" . option($_mismatch, " style=\"COLOR:#ff0000\"") . ">" .
                             "<label for=\"profile-captcha\">" .
                             "* " . CMS_L_MOD_PROFILE_037 .
                             "</label>" .
                             "</legend>");

                        if ($_mismatch)
                                echo("<p class=\"profile-mismatch\">" .
                                     x($mismatch[0]["captcha"]) .
                                     "</p>");

                        //captcha input
                        echo("<input id=\"profile-captcha\" " .
                                    "name=\"profile_captcha_key\" " .
                                    "type=\"text\" " .
                                    "value=\"\" " .
                                    "size=\"40\" " .
                                    "maxlength=\"40\"><br>" .

                             "<small>" .
                             CMS_L_MOD_PROFILE_038 .
                             "</small><br>" .

                        //verification code
                             "<input name=\"profile_captcha_code\" " .
                                    "type=\"hidden\" " .
                                    "value=\"" . x($code) . "\">" .
                             image(CMS_DATA_URL . "captcha/$code.png", NULL, NULL, CMS_L_MOD_PROFILE_037) .

                             "</fieldset>");
                };
        };
};

//generate input form
reset($visible);
while (list($key, $value) = each($visible))
{
        //title
        echo("<h2>" .
             $title[$key] .
             "</h2>");

        //display mismatch message
        if ((! $flag) && (count($mismatch) > 0))
        {
                echo("<p class=\"profile-mismatch\">" .
                     CMS_L_MOD_PROFILE_036 .
                     "</p>");
                $flag = TRUE;
        };

        //content
        reset($value);
        while (list($_key) = each($value))
        {
                $id        = "profile-" . preg_replace("/[^0-9a-zA-Z]+/", "-", $_key);
                $name      = isset($editable[$key][$_key]) ? "profile_param[" . x($_key) . "]" : "";
                $_mismatch = isset($mismatch[$key][$_key]);

                eval("\$_value = \$profile_data->$_key;");

                echo("<fieldset" . option($_mismatch, " class=\"profile-mismatch\"") . ">" .
                     "<legend" . option($_mismatch, " style=\"COLOR:#ff0000\"") . ">" .
                     "<label for=\"$id\">" .
                     option(isset($required[$key][$_key]), "* ") . x($field[$key][$_key]) .
                     "</label>" .
                     "</legend>");

                if ($_mismatch)
                        echo("<p class=\"profile-mismatch\">" .
                             x($mismatch[$key][$_key]) .
                             "</p>");

                switch ($_key)
                {
                case CMS_DB_PROFILE_CODE:
                case CMS_DB_PROFILE_USER:
                case CMS_DB_PROFILE_COMPANY:
                case CMS_DB_PROFILE_PRENAME:
                case CMS_DB_PROFILE_SURNAME:
                case CMS_DB_PROFILE_STREET:
                case CMS_DB_PROFILE_ZIPCODE:
                case CMS_DB_PROFILE_CITY:
                case CMS_DB_PROFILE_COUNTRY:
                case CMS_DB_PROFILE_PHONE_1:
                case CMS_DB_PROFILE_PHONE_2:
                case CMS_DB_PROFILE_MOBILE:
                case CMS_DB_PROFILE_FAX:
                case CMS_DB_PROFILE_EMAIL:
                case CMS_DB_PROFILE_URL:
                case CMS_DB_PROFILE_ACCOUNT_NUMBER:
                case CMS_DB_PROFILE_FINANCIAL_INSTITUTION:
                case CMS_DB_PROFILE_BANKCODE:
                case CMS_DB_PROFILE_ACCOUNT_HOLDER:
                case CMS_DB_PROFILE_CREDIT_INSTITUTE:
                case CMS_DB_PROFILE_CREDIT_CARD_HOLDER:
                default:
//.....................................................................................................

                        //fixed
                        if (stre($name))
                                echo(stre($_value) ? CMS_L_MOD_PROFILE_001 : x($_value));

                        //editable
                        else
                                echo("<input id=\"$id\" " .
                                            "name=\"$name\" " .
                                            "type=\"text\" " .
                                            "value=\"" . x($_value) . "\" " .
                                            "size=\"40\">");

                        break;

                case CMS_DB_PROFILE_PASSWORD:
//.....................................................................................................

                        //fixed
                        if (stre($name))
                                echo("****");

                        //editable
                        else
                                if (! $mismatch)
                                        echo("<p class=\"profile-description\">" .
                                             CMS_L_MOD_PROFILE_049 .
                                             "</p>");
                                echo("<input id=\"$id\" " .
                                            "name=\"" . $name . "[0]\" " .
                                            "type=\"password\" " .
                                            "size=\"40\"><br>" .

                                     "</fieldset>" .
                                     "<fieldset" . option($_mismatch, " class=\"profile-mismatch\"") . ">" .

                                     "<legend>" .
                                     "<label for=\"$id-confirmation\">" .
                                     "* " . CMS_L_MOD_PROFILE_035 .
                                     "</label>" .
                                     "</legend>" .

                                     "<input id=\"$id-confirmation\" " .
                                            "name=\"" . $name . "[1]\" " .
                                            "type=\"password\" " .
                                            "size=\"40\">");

                        break;

                case CMS_DB_PROFILE_CREDIT_CARD_NUMBER:
//.....................................................................................................

                        //fixed
                        if (stre($name))
                        {
                                echo(stre($_value) ? CMS_L_MOD_PROFILE_001 : x($_value));
                        }

                        //editable
                        else
                        {
                                $_value = explode("-", $_value);

                                echo("<input id=\"$id\" " .
                                            "name=\"" . $name . "[0]\" " .
                                            "type=\"text\" " .
                                            "value=\"" . (isset($_value[0]) ? x($_value[0]) : "") . "\" " .
                                            "size=\"4\" " .
                                            "maxlength=\"4\">" .
                                     " - " .
                                     "<input name=\"" . $name . "[1]\" " .
                                            "type=\"text\" " .
                                            "value=\"" . (isset($_value[1]) ? x($_value[1]) : "") . "\" " .
                                            "size=\"4\" " .
                                            "maxlength=\"4\">" .
                                     " - " .
                                     "<input name=\"" . $name . "[2]\" " .
                                            "type=\"text\" " .
                                            "value=\"" . (isset($_value[2]) ? x($_value[2]) : "") . "\" " .
                                            "size=\"4\" " .
                                            "maxlength=\"4\">" .
                                     " - " .
                                     "<input name=\"" . $name . "[3]\" " .
                                            "type=\"text\" " .
                                            "value=\"" . (isset($_value[3]) ? x($_value[3]) : "") . "\" " .
                                            "size=\"4\" " .
                                            "maxlength=\"4\">");
                        };

                        break;

                case CMS_DB_PROFILE_CREDIT_CARD_VALIDITY:
//.....................................................................................................

                        //fixed
                        if (stre($name))
                        {
                                echo(stre($_value) ? CMS_L_MOD_PROFILE_001 : x($_value));
                        }

                        //editable
                        else
                        {
                                $_value = explode("/", $_value);

                                select(array("01" => "01", "02" => "02", "03" => "03", "04" => "04",
                                             "05" => "05", "06" => "06", "07" => "07", "08" => "08",
                                             "09" => "09", "10" => "10", "11" => "11", "12" => "12"),
                                       isset($_value[0]) ? $_value[0] : date("m"),
                                       $name . "[0]");
                                echo(" / " .
                                     "<input id=\"$id\" " .
                                            "name=\"" . $name . "[1]\" " .
                                            "type=\"text\" " .
                                            "value=\"" . (isset($_value[1]) ? $_value[1] : date("Y")) . "\" " .
                                            "size=\"4\">");
                        };

                        break;

                case CMS_DB_PROFILE_COMMENT:
//.....................................................................................................

                        //fixed
                        if (stre($name))
                                echo(stre($_value) ? CMS_L_MOD_PROFILE_001 : x($_value));

                        //editable
                        else
                                echo("<textarea id=\"$id\" " .
                                               "name=\"$name\" " .
                                               "cols=\"40\" " .
                                               "rows=\"8\">\n" .
                                     x($_value) .
                                     "</textarea>");

                        break;
                };

                echo("</fieldset>");
        };
};

switch ($profile_message)
{
case "register":
case "_register":
//.....................................................................................................

        //submit button
        echo("<fieldset class=\"profile-command\">" .

             "<button name=\"profile_message\" " .
                     "type=\"submit\" " .
                     "value=\"_register\">" .
             CMS_L_MOD_PROFILE_040 .
             "</button>" .

             "</fieldset>");
        break;

default:
case "_activate":
case "edit":
case "_edit":
//.....................................................................................................

        //save button
        if (count($editable) > 0)
                echo("<fieldset class=\"profile-command\">" .

                     "<button name=\"profile_message\" " .
                             "type=\"submit\" " .
                             "value=\"_edit\">" .
                     CMS_L_COMMAND_SAVE .
                     "</button>" .

                     "</fieldset>");
        break;
};

echo("</section>" .
     "</form>");

insert("bottom");

//permission control
permission(array("" => CMS_L_ACCESS));
?>
