<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

//-----------------------------------------------------------------------------------------------------
//===< MODULE >========================================================================================
//-----------------------------------------------------------------------------------------------------
require(dirname(dirname(__FILE__)) . "/nuos.inc");

function mod_content() {

//memorize start time
$time_start = microtime(TRUE);

global $content_display,
       $content_index,
       $content_directory_index,
       $content_option,
       $content_user,
       $content_message,
       $range,
       $type,
       $value,
       $left,
       $top,
       $id;

if (isset($_GET["content_directory_index"])) unset($_GET["content_directory_index"]);
if (isset($_GET["content_display"]))         unset($_GET["content_display"]);

//get reference from directory
if (
     ($flag = stre($content_index))
     ||
     streq($content_display, "directory")
   )
{
        $data = new data("#system/directory");

        //select first entry if none is given
        if (! $content_directory_index)
        {
                $content_directory_index = $data->move("first");
                $url                     = translate_url("directory://$content_directory_index");
                header($_SERVER["SERVER_PROTOCOL"] . " 301 Moved Permanently");
                header("Location: $url");
                exit();
        };

        //retrieve url
        $url = utf8_trim($data->get($content_directory_index, "url"));

        if ($url !== "")
        {
                //directory dereferencing
                if (streq(substr($url, 0, 12), "directory://"))
                {
                        do
                        {
                                import_querystring($url);
                                $url                     = analyze_url($url);
                                $content_directory_index = $url["host"];
                                $url                     = $data->get($url["host"], "url");
                        }
                        while (streq(substr($url, 0, 12), "directory://"));
                };

                //internal reference
                if (streq(substr($url, 0, 10), "content://"))
                {
                        import_querystring($url);
                        $url           = substr($url, 10);
                        $content_index = substr($url, 0, strspn($url, "1234567890"));
                }

                //external reference
                else
                {
                        $url = absolute_path(CMS_ROOT_URL, $url);
                        header($_SERVER["SERVER_PROTOCOL"] . " 301 Moved Permanently");
                        header("Location: $url");
                        exit();
                };
        }

        //missing reference
        elseif ($flag)
        {
                header($_SERVER["SERVER_PROTOCOL"] . " 404 Not Found");
                exit();
        };

        define("CMS_CONTENT_DIRECTORY_INDEX", $content_directory_index);
};

//copy partial content
if (streq($type, "#buffer"))
{
        /*
         * this section is called by a javascript function
         * using an image object to transfer query data
         */

        if (
             cms_load("content")
             &&
             cms_load("document")
           )
        {
                $content = new content($content_user);

                if (
                     $content->enabled
                     &&
                     $content->test_update($content_index)
                     &&
                     ($result = mysql_query("SELECT " . CMS_DB_CONTENT_BUFFER_TEXT . ", " .
                                                        CMS_DB_CONTENT_BUFFER_TEMPLATE . " " .
                                            "FROM " .   CMS_DB_CONTENT . " " .
                                            "WHERE " .  CMS_DB_CONTENT_INDEX . "='" . sqlesc($content_index) . "' " .
                                            "LIMIT 1"))
                     &&
                     ($resultrow = mysql_fetch_assoc($result))
                   )
                {
                        $document = new document($resultrow[CMS_DB_CONTENT_BUFFER_TEXT],
                                                 $resultrow[CMS_DB_CONTENT_BUFFER_TEMPLATE]);
                        $document = $document->extract($range);
                        $buffer   = $document->export();

                        //permanent caching
                        cms_cache("content." . CMS_USER . ".buffer", $buffer, TRUE);
                };
        };

        exit();
};

//load libraries
if (
     (! cms_load("content"))
     ||
     (! cms_load("template"))
   )
{
        echo(CMS_MSG_UNAVAILABLE);
        return;
};

//instantiate content object
$content = new content($content_user);
if (! $content->enabled)
{
        echo(CMS_MSG_UNAVAILABLE);
        exit();
};

//verify read permission
if (
     (! ($content->writer || $content->editor || $content->publisher))     //inherent
     &&
     (! cms_permission(CMS_CONTENT_PERMISSION_READER . ".$content_index")) //explicit
   )
{
        //no access
        if (headers_sent()) return FALSE;

        if ($query_string = getenv("QUERY_STRING"))
                $query_string = "?$query_string";

        header("Location: " . CMS_MODULES_URL . "identification.php?" .
               "location=" . r(CMS_ACTIVE_URL . $query_string));
        exit();
};

//retrieve metadata
if (
     ! (
         ($result = mysql_query("SELECT " . CMS_DB_CONTENT_STATUS . ", " .
                                            CMS_DB_CONTENT_TITLE . " " .
                                "FROM " .   CMS_DB_CONTENT . " " .
                                "WHERE " .  CMS_DB_CONTENT_INDEX . "='" . sqlesc($content_index) . "' " .
                                "LIMIT 1"))
         &&
         mysql_num_rows($result)
       )
   )
{
        //not found
        header($_SERVER["SERVER_PROTOCOL"] . " 404 Not Found");
        exit();
};

//set global query values
if (! defined("CMS_CONTENT_EXTERNAL"))
{
        cms_param($content_index,           "content_index");
        cms_param($content_directory_index, "content_directory_index");
};

//read mode
if (! $content->test_update($content_index))
{
        //not published
        if (mysql_result($result, 0, 0) != CMS_CONTENT_STATUS_PUBLICATION)
        {
                header($_SERVER["SERVER_PROTOCOL"] . " 410 Gone"); //assuming it was published before
                exit();
        };

        //log access
        $log = new log();
        $log->access(sprintf(CMS_L_MOD_CONTENT_001, l(mysql_result($result, 0, 1))));

        //generate output
        echo(content_parse($content, $content_index));

        //memorize end time
        $time_end = microtime(TRUE);

        //display processing time
        echo("\n<!-- page generated in " . ($time_end - $time_start) . " seconds -->");
        exit();
};

//edit mode
cms_param($content_option, "content_option");
cms_param($content_user,   "content_user");

$active_url     = rq(CMS_ACTIVE_URL);
$modules_url    = rq(CMS_MODULES_URL);
$javascript_url = x(CMS_JAVASCRIPT_URL);
$interface_url  = $modules_url . "interface.php";
$desktop_url    = $modules_url . "desktop.php";
$_content_index = rq($content_index);
$header         = jscript("document.documentElement.style.display = \"none\";\n" .
                          "var CMS_L_COMMAND_PASTE   = \"" . q(CMS_L_COMMAND_PASTE) . "\";\n" .
                          "var CMS_L_COMMAND_DELETE  = \"" . q(CMS_L_COMMAND_DELETE) . "\";\n" .
                          "var CMS_L_MOD_CONTENT_002 = \"" . q(CMS_L_MOD_CONTENT_002) . "\";\n" .
                          "var CMS_L_MOD_CONTENT_003 = \"" . q(CMS_L_MOD_CONTENT_003) . "\";\n" .
                          "var CMS_L_MOD_CONTENT_004 = \"" . q(CMS_L_MOD_CONTENT_004) . "\";\n" .
                          "var CMS_L_MOD_CONTENT_005 = \"" . q(CMS_L_MOD_CONTENT_005) . "\";\n" .
                          "var CMS_L_MOD_CONTENT_009 = \"" . q(CMS_L_MOD_CONTENT_009) . "\";\n" .
                          "var CMS_L_MOD_CONTENT_010 = \"" . q(CMS_L_MOD_CONTENT_010) . "\";\n" .
                          "var CMS_L_MOD_CONTENT_011 = \"" . q(CMS_L_MOD_CONTENT_011) . "\";\n" .
                          "var CMS_L_MOD_CONTENT_012 = \"" . q(CMS_L_MOD_CONTENT_012) . "\";\n" .
                          "var CMS_L_MOD_CONTENT_013 = \"" . q(CMS_L_MOD_CONTENT_013) . "\";\n") .
                  "\n<script src=\"" . $javascript_url . "asr.js\">" .
                  "</script>" .
                  "\n<script src=\"" . $javascript_url . "content.js\">" .
                  "</script>";

//process command
if (stre($range))
{
        $flag_redirect = TRUE;

        switch ($content_message)
        {
        case "apply":
//.....................................................................................................
                $content->apply($content_index);
                break;

        case "revert":
//.....................................................................................................
                $content->revert($content_index);
                break;

        case "undo":
//.....................................................................................................
                $content->step_undo($content_index);
                break;

        case "redo":
//.....................................................................................................
                $content->step_redo($content_index);
                break;

        default:
//.....................................................................................................
                $flag_redirect = FALSE;
        };

        if ($flag_redirect)
        {
                header("Location: " . CMS_ACTIVE_URL . cms_param(array("left" => $left, "top" => $top)));
                exit();
        };
}

//write changes
else
{
        content_set_range($content, $content_index, $range, $type, $value);
};

//setup option select function
$action[CMS_TEMPLATE_CONTROL][CMS_TEMPLATE_SWITCH][CMS_TEMPLATE_CODE] =
        "javascript:tp_ctrl_opt_apply('" .
        $active_url .
        cms_param(array("content_option" => "%value%",
                        "left"           => "%left%",
                        "top"            => "%top%")) . "');";

$action[CMS_TEMPLATE_CONTROL][CMS_TEMPLATE_COMMAND] =

//setup change apply function
        array(CMS_L_MOD_CONTENT_003 => "javascript:content_edit_apply('" .
                                       $active_url .
                                       cms_param(array("content_message" => "apply",
                                                       "left"            => "%left%",
                                                       "top"             => "%top%")) . "');",

//setup change revert function
              CMS_L_MOD_CONTENT_004 => "javascript:content_edit_revert('" .
                                       $active_url .
                                       cms_param(array("content_message" => "revert",
                                                       "left"            => "%left%",
                                                       "top"             => "%top%")) . "');");

//setup action undo function
if ($content->test_step_undo($content_index))
{
        $name = CMS_L_MOD_CONTENT_016 . " (" . $content->step_undo_depth($content_index) . ")";
        $action[CMS_TEMPLATE_CONTROL][CMS_TEMPLATE_COMMAND][$name] =
                "javascript:content_edit_command('" .
                $active_url .
                cms_param(array("content_message" => "undo",
                                "left"            => "%left%",
                                "top"             => "%top%")) . "');";
};

//setup action redo function
if ($content->test_step_redo($content_index))
{
        $name = CMS_L_MOD_CONTENT_017 . " (" . $content->step_redo_depth($content_index) . ")";
        $action[CMS_TEMPLATE_CONTROL][CMS_TEMPLATE_COMMAND][$name] =
                "javascript:content_edit_command('" .
                $active_url .
                cms_param(array("content_message" => "redo",
                                "left"            => "%left%",
                                "top"             => "%top%")) . "');";
};

//content
if (is_file(CMS_MODULES_PATH . "#interface/ifc.content.inc"))
{
        //setup hyperlink edit function
        if ($content_option & CMS_TEMPLATE_OPTION_HREF)
                $action[CMS_TEMPLATE_ACTION][CMS_TEMPLATE_TYPE_HREF] =
                        array(CMS_TEMPLATE_CODE  => "javascript:content_edit_open('" .
                                                    $interface_url .
                                                    "?ifc_page=content" .
                                                    "&ifc_message=edit_href" .
                                                    "&ifc_option=external" .
                                                    "&ifc_select=value" .
                                                    "&ifc_select_action=" .
                                                    "javascript:content_edit_href_save();" .
                                                    "&object=$_content_index" .
                                                    "&range=%index%" .
                                                    "&id=%id%');",
                              CMS_TEMPLATE_IMAGE => CMS_IMAGES_URL . "content/button_href.png");

        //setup plugin edit function
        if ($content_option & CMS_TEMPLATE_OPTION_PLUGIN)
                $action[CMS_TEMPLATE_ACTION][CMS_TEMPLATE_TYPE_PLUGIN] =
                        array(CMS_TEMPLATE_CODE  => "javascript:content_edit_open('" .
                                                    $interface_url .
                                                    "?ifc_page=content" .
                                                    "&ifc_message=edit_plugin" .
                                                    "&ifc_option=external" .
                                                    "&object=$_content_index" .
                                                    "&range=%index%" .
                                                    "&id=%id%');",
                              CMS_TEMPLATE_IMAGE => CMS_IMAGES_URL . "content/button_plugin.png");

        //setup text edit function
        if ($content_option & CMS_TEMPLATE_OPTION_TEXT)
                $action[CMS_TEMPLATE_ACTION][CMS_TEMPLATE_TYPE_TEXT] =
                        array(CMS_TEMPLATE_CODE  => "javascript:content_edit_open('" .
                                                    $interface_url .
                                                    "?ifc_page=content" .
                                                    "&ifc_message=edit_range" .
                                                    "&ifc_option=external" .
                                                    "&object=$_content_index" .
                                                    "&range=%index%" .
                                                    "&id=%id%');",
                              CMS_TEMPLATE_IMAGE => CMS_IMAGES_URL . "content/button_text.png");

        //setup value edit function
        if ($content_option & CMS_TEMPLATE_OPTION_VALUE)
                $action[CMS_TEMPLATE_ACTION][CMS_TEMPLATE_TYPE_VALUE] =
                        array(CMS_TEMPLATE_CODE  => "javascript:content_edit_open('" .
                                                    $interface_url .
                                                    "?ifc_page=content" .
                                                    "&ifc_message=edit_value" .
                                                    "&ifc_option=external" .
                                                    "&object=$_content_index" .
                                                    "&range=%index%" .
                                                    "&id=%id%');",
                              CMS_TEMPLATE_IMAGE => CMS_IMAGES_URL . "content/button_value.png");

        //setup pool open function
        $action[CMS_TEMPLATE_CONTROL][CMS_TEMPLATE_COMMAND][CMS_L_MOD_CONTENT_014] =
                "javascript:load_page('','" .
                "$interface_url" .
                "?ifc_page=content&" .
                "&ifc_message=pool" .
                "&ifc_option=external');";

        //setup text analysis function
        $action[CMS_TEMPLATE_CONTROL][CMS_TEMPLATE_COMMAND][CMS_L_MOD_CONTENT_006] =
                "javascript:load_page('','" .
                $interface_url .
                "?ifc_page=content" .
                "&ifc_message=analyze" .
                "&ifc_option=external" .
                "&object=$_content_index');";
};

//download
if (
     ($content_option & CMS_TEMPLATE_OPTION_DOWNLOAD)
     &&
     cms_available("download")
     &&
     is_file(CMS_MODULES_PATH . "#interface/ifc.download.inc")
     &&
     cms_permission("interface.download", NULL, NULL)
   )
{
        //setup download edit function
        $action[CMS_TEMPLATE_ACTION][CMS_TEMPLATE_TYPE_DOWNLOAD] =
                array(CMS_TEMPLATE_CODE  => "javascript:content_edit_open('" .
                                            $interface_url .
                                            "?ifc_page=download" .
                                            "&ifc_option=external" .
                                            "&object=%value%" .
                                            "&ifc_select=object" .
                                            "&ifc_select_action='+encodeURIComponent('" .
                                            $active_url .
                                            cms_param(array("range" => "\x1B%index%",
                                                            "type"  => "download",
                                                            "value" => "%return%",
                                                            "left"  => "%left%",
                                                            "top"   => "%top%",
                                                            "id"    => "\x1B%id%")) . "'));",
                      CMS_TEMPLATE_IMAGE => CMS_IMAGES_URL . "content/button_download.png");
};

//image
if (
     cms_available("image")
     &&
     is_file(CMS_MODULES_PATH . "#interface/ifc.image.inc")
     &&
     cms_permission("interface.image", NULL, NULL)
   )
{
        $buffer = "javascript:content_edit_open('" .
                  $interface_url .
                  "?ifc_page=image" .
                  "&ifc_option=external" .
                  "&object=%value%" .
                  "&ifc_select=object" .
                  "&ifc_select_action='+encodeURIComponent('" .
                  $active_url .
                  cms_param(array("range" => "\x1B%index%",
                                  "type"  => "image",
                                  "value" => "%return%",
                                  "left"  => "%left%",
                                  "top"   => "%top%",
                                  "id"    => "\x1B%id%")) . "'));";

        //setup image edit function
        if ($content_option & CMS_TEMPLATE_OPTION_IMAGE)
                $action[CMS_TEMPLATE_ACTION][CMS_TEMPLATE_TYPE_IMAGE] =
                        array(CMS_TEMPLATE_CODE  => $buffer,
                              CMS_TEMPLATE_IMAGE => CMS_IMAGES_URL . "content/button_image.png");

        //setup thumbnail edit function
        if ($content_option & CMS_TEMPLATE_OPTION_THUMBNAIL)
                $action[CMS_TEMPLATE_ACTION][CMS_TEMPLATE_TYPE_THUMBNAIL] =
                        array(CMS_TEMPLATE_CODE  => $buffer,
                              CMS_TEMPLATE_IMAGE => CMS_IMAGES_URL . "content/button_thumbnail.png");
};

//media
if (
     ($content_option & CMS_TEMPLATE_OPTION_MEDIA)
     &&
     cms_available("media")
     &&
     cms_available("media_type")
     &&
     is_file(CMS_MODULES_PATH . "#interface/ifc.media.inc")
     &&
     cms_permission("interface.media", NULL, NULL)
   )
{
        //setup media edit function
        $action[CMS_TEMPLATE_ACTION][CMS_TEMPLATE_TYPE_MEDIA] =
                array(CMS_TEMPLATE_CODE  => "javascript:content_edit_open('" .
                                            $interface_url .
                                            "?ifc_page=media" .
                                            "&ifc_option=external" .
                                            "&object=%value%" .
                                            "&ifc_select=object" .
                                            "&ifc_select_action='+encodeURIComponent('" .
                                            $active_url .
                                            cms_param(array("range" => "\x1B%index%",
                                                            "type"  => "media",
                                                            "value" => "%return%",
                                                            "left"  => "%left%",
                                                            "top"   => "%top%",
                                                            "id"    => "\x1B%id%")) . "'));",
                      CMS_TEMPLATE_IMAGE => CMS_IMAGES_URL . "content/button_media.png");
};

//template
if (
     ($content_option & CMS_TEMPLATE_OPTION_TEMPLATE)
     &&
     is_file(CMS_MODULES_PATH . "#interface/ifc.template.inc")
     &&
     cms_permission("interface.template", NULL, NULL)
   )
{
        //setup template edit function
        $action[CMS_TEMPLATE_ACTION][CMS_TEMPLATE_TYPE_TEMPLATE] =
                array(CMS_TEMPLATE_CODE  => "javascript:content_edit_open('" .
                                            $interface_url .
                                            "?ifc_page=template" .
                                            "&ifc_option=external" .
                                            "&object=%value%" .
                                            "&ifc_select=object" .
                                            "&ifc_select_action='+encodeURIComponent('" .
                                            $active_url .
                                            cms_param(array("range" => "\x1B%index%",
                                                            "type"  => "template",
                                                            "value" => "%return%",
                                                            "left"  => "%left%",
                                                            "top"   => "%top%",
                                                            "id"    => "\x1B%id%")) . "'));",
                      CMS_TEMPLATE_IMAGE => CMS_IMAGES_URL . "content/button_template.png");

        //setup template export function
        if (cms_permission("interface.template.operator", NULL, NULL))
        {
                $action[CMS_TEMPLATE_COMMAND][CMS_TEMPLATE_COMMAND_EXPORT] =
                        "javascript:content_edit_open('" .
                        $interface_url .
                        "?ifc_page=template" .
                        "&ifc_message=export" .
                        "&ifc_option=external" .
                        "&content_index=$_content_index" .
                        "&content_range=%index%" .
                        "&ifc_select=object" .
                        "&ifc_select_action='+encodeURIComponent('" .
                        $active_url .
                        cms_param(array("range" => "\x1B%index%",
                                        "type"  => "template",
                                        "value" => "%return%",
                                        "left"  => "%left%",
                                        "top"   => "%top%",
                                        "id"    => "\x1B%id%")) . "'));";
        };
};

//group
if ($content_option & CMS_TEMPLATE_OPTION_GROUP)
{
        //setup template edit function
        $action[CMS_TEMPLATE_ACTION][CMS_TEMPLATE_TYPE_GROUP] =
                array(CMS_TEMPLATE_CODE  => "javascript:return false;",
                      CMS_TEMPLATE_IMAGE => CMS_IMAGES_URL . "content/button_group.png");
};

//repeat
if ($content_option & CMS_TEMPLATE_OPTION_REPEAT)
{
        //setup repeat edit function
        $action[CMS_TEMPLATE_ACTION][CMS_TEMPLATE_TYPE_REPEAT] =
                array(CMS_TEMPLATE_CODE  => "javascript:content_edit_repeat('" .
                                            $active_url .
                                            cms_param(array("range" => "\x1B%index%",
                                                            "type"  => "repeat",
                                                            "value" => "%return%",
                                                            "left"  => "%left%",
                                                            "top"   => "%top%",
                                                            "id"    => "\x1B%id%")) . "'," .
                                            "'%value%');",
                      CMS_TEMPLATE_IMAGE => CMS_IMAGES_URL . "content/button_repeat.png");
};

//shift
if ($content_option & CMS_TEMPLATE_OPTION_SHIFT)
{
        //setup shift edit function
        $action[CMS_TEMPLATE_ACTION][CMS_TEMPLATE_TYPE_SHIFT] =
                array(CMS_TEMPLATE_CODE  => "javascript:content_edit_shift('" .
                                            $active_url .
                                            cms_param(array("range" => "\x1B%index%",
                                                            "type"  => "shift",
                                                            "value" => "%return%",
                                                            "left"  => "%left%",
                                                            "top"   => "%top%",
                                                            "id"    => "\x1B%id%")) . "'," .
                                            "'%value%');",
                      CMS_TEMPLATE_IMAGE => CMS_IMAGES_URL . "content/button_shift.png");
};

if ($content_option)
{
        $unique_id = "_" . unique_id(); //used to prevent caching

        //setup copy function
        $action[CMS_TEMPLATE_COMMAND][CMS_TEMPLATE_COMMAND_BUFFER] =
                "javascript:content_edit_copy('" .
                $active_url .
                cms_param(array("range"    => "\x1B%path%",
                                "type"     => "\x1B'+encodeURIComponent('#')+'buffer",
                                "value"    => "none",
                                $unique_id => TRUE)) .
                "','%index%');";

        //setup paste function
        $action[CMS_TEMPLATE_COMMAND][CMS_TEMPLATE_COMMAND_PASTE] =
                "javascript:content_edit_paste('" .
                $active_url .
                cms_param(array("range"    => "\x1B%path%",
                                "type"     => "\x1B'+encodeURIComponent('#')+'paste",
                                "value"    => "none",
                                "left"     => "%left%",
                                "top"      => "%top%",
                                "id"       => "\x1B%id%",
                                $unique_id => TRUE)) . "');";

        //setup swap function
        $action[CMS_TEMPLATE_COMMAND][CMS_TEMPLATE_COMMAND_SWAP] =
                "javascript:content_edit_swap('" .
                $active_url .
                cms_param(array("range"    => "\x1B%path%",
                                "type"     => "\x1B'+encodeURIComponent('#')+'swap",
                                "value"    => "\x1B'+encodeURIComponent(content_buffer)+'",
                                "left"     => "%left%",
                                "top"      => "%top%",
                                "id"       => "\x1B%id%",
                                $unique_id => TRUE)) . "');";

        //setup kick down function
        $action[CMS_TEMPLATE_COMMAND][CMS_TEMPLATE_COMMAND_KICK1] =
                "javascript:content_edit_kick1('" .
                $active_url .
                cms_param(array("range"    => "\x1B%path%",
                                "type"     => "\x1B'+encodeURIComponent('#')+'kick1",
                                "value"    => "%return%",
                                "left"     => "%left%",
                                "top"      => "%top%",
                                "id"       => "\x1B%id%",
                                $unique_id => TRUE)) . "');";

        //setup kick up function
        $action[CMS_TEMPLATE_COMMAND][CMS_TEMPLATE_COMMAND_KICK2] =
                "javascript:content_edit_kick2('" .
                $active_url .
                cms_param(array("range"    => "\x1B%path%",
                                "type"     => "\x1B'+encodeURIComponent('#')+'kick2",
                                "value"    => "%return%",
                                "left"     => "%left%",
                                "top"      => "%top%",
                                "id"       => "\x1B%id%",
                                $unique_id => TRUE)) . "');";

        //setup drop up function
        $action[CMS_TEMPLATE_COMMAND][CMS_TEMPLATE_COMMAND_DROP1] =
                "javascript:content_edit_command('" .
                $active_url .
                cms_param(array("range"    => "\x1B%path%",
                                "type"     => "\x1B'+encodeURIComponent('#')+'drop1",
                                "value"    => "1",
                                "left"     => "%left%",
                                "top"      => "%top%",
                                "id"       => "\x1B%id%",
                                $unique_id => TRUE)) . "');";

        //setup drop down function
        $action[CMS_TEMPLATE_COMMAND][CMS_TEMPLATE_COMMAND_DROP2] =
                "javascript:content_edit_command('" .
                $active_url .
                cms_param(array("range"    => "\x1B%path%",
                                "type"     => "\x1B'+encodeURIComponent('#')+'drop2",
                                "value"    => "-1",
                                "left"     => "%left%",
                                "top"      => "%top%",
                                "id"       => "\x1B%id%",
                                $unique_id => TRUE)) . "');";

        if (is_file(CMS_MODULES_PATH . "#interface/ifc.content.inc"))
        {
                //setup release pool function
                if (cms_permission("interface.content.pool.operator", NULL, NULL))
                {
                        $action[CMS_TEMPLATE_COMMAND][CMS_TEMPLATE_COMMAND_RELEASE] =
                                "javascript:content_edit_open('" .
                                $interface_url .
                                "?ifc_page=content" .
                                "&ifc_message=pool_add" .
                                "&ifc_option=external" .
                                "&pool_type=%type%" .
                                "&content_index=$_content_index" .
                                "&content_range=%path%');";
                };

                //setup reference pool function
                $action[CMS_TEMPLATE_COMMAND][CMS_TEMPLATE_COMMAND_REFERENCE] =
                        "javascript:content_edit_open('" .
                        $interface_url .
                        "?ifc_page=content" .
                        "&ifc_message=pool" .
                        "&ifc_option=external" .
                        "&pool_object=%reference%" .
                        "&pool_type=%type%" .
                        "&ifc_select=pool_object" .
                        "&ifc_select_action='+encodeURIComponent('" .
                        $active_url .
                        cms_param(array("range" => "\x1B%index%",
                                        "type"  => "\x1B'+encodeURIComponent('#')+'reference",
                                        "value" => "%return%",
                                        "left"  => "%left%",
                                        "top"   => "%top%",
                                        "id"    => "\x1B%id%")) . "'));";
        };

        //setup clear function
        $action[CMS_TEMPLATE_COMMAND][CMS_TEMPLATE_COMMAND_CLEAR] =
                "javascript:content_edit_clear('" .
                $active_url .
                cms_param(array("range"    => "\x1B%path%",
                                "type"     => "\x1B'+encodeURIComponent('#')+'clear",
                                "value"    => "none",
                                "left"     => "%left%",
                                "top"      => "%top%",
                                "id"       => "\x1B%id%",
                                $unique_id => TRUE)) . "');";

        //setup dragdrop swap function
        $action[CMS_TEMPLATE_COMMAND][CMS_TEMPLATE_COMMAND_DRAGDROP1] =
                "javascript:content_edit_command('" .
                $active_url .
                cms_param(array("range"    => "\x1B%path%",
                                "type"     => "\x1B'+encodeURIComponent('#')+'dragdrop1",
                                "value"    => "\x1B%value%",
                                "left"     => "%left%",
                                "top"      => "%top%",
                                "id"       => "\x1B%id%",
                                $unique_id => TRUE)) . "');";

        //setup dragdrop copy function
        $action[CMS_TEMPLATE_COMMAND][CMS_TEMPLATE_COMMAND_DRAGDROP2] =
                "javascript:content_edit_command('" .
                $active_url .
                cms_param(array("range"    => "\x1B%path%",
                                "type"     => "\x1B'+encodeURIComponent('#')+'dragdrop2",
                                "value"    => "\x1B%value%",
                                "left"     => "%left%",
                                "top"      => "%top%",
                                "id"       => "\x1B%id%",
                                $unique_id => TRUE)) . "');";

        //set conditional block flag
        if ($content_option & CMS_TEMPLATE_OPTION_CBLOCK)
                $action[CMS_TEMPLATE_ACTION][CMS_TEMPLATE_TYPE_CBLOCK] = TRUE;

        //set alternative block flag
        if ($content_option & CMS_TEMPLATE_OPTION_CALT)
                $action[CMS_TEMPLATE_ACTION][CMS_TEMPLATE_TYPE_CALT]   = TRUE;

        //set debug flag
        if ($content_option & CMS_TEMPLATE_OPTION_DEBUG)
                $action[CMS_TEMPLATE_ACTION][CMS_TEMPLATE_TYPE_DEBUG]  = TRUE;
};

//establish edit mode
if (! isset($action[CMS_TEMPLATE_ACTION]))
        $action[CMS_TEMPLATE_ACTION] = TRUE;

//setup interface open function
if (cms_permission("interface", FALSE, FALSE))
        $action[CMS_TEMPLATE_CONTROL][CMS_TEMPLATE_COMMAND][CMS_L_MOD_CONTENT_007] =
                "javascript:content_load('$interface_url');";

//setup desktop open function
if (cms_permission("desktop", FALSE, FALSE))
        $action[CMS_TEMPLATE_CONTROL][CMS_TEMPLATE_COMMAND][CMS_L_MOD_CONTENT_008] =
                "javascript:content_load('$desktop_url');";

//setup logout function
$action[CMS_TEMPLATE_CONTROL][CMS_TEMPLATE_COMMAND][CMS_L_MOD_CONTENT_015] =
        x(CMS_MODULES_URL) . "identification.php?cms_logout=1";

//generate output
echo(content_parse($content, $content_index, $action, $header));

?>
<script>

(function()
{
        tp_flp_restore("<?php echo(qx($content_index)); ?>");

        fx_register_callback(tp_event);

        var func = function(object)
        {
                var type   = object.getAttribute("data-tp-dd-type");
                var accept = object.getAttribute("data-tp-dd-accept");
                dd_register(object.id, type, accept);
        };

        Array.prototype.forEach.call(document.getElementsByClassName("tp-dd"), func);
        Array.prototype.forEach.call(document.getElementsByClassName("tp-dd100"), func);

        document.documentElement.style.display = "block";
        fx_update_page_size();

        if (document.URL.indexOf("#") != -1) return;
        var left = fx_page_left();
        if (left == 0) left = <?php echo((int)$left); ?>;
        var _top = fx_page_top();
        if (_top == 0) _top = <?php echo((int)$top); ?>;
        var object = document.getElementById("tp-dd-<?php echo(qx($id)); ?>");
        if (object != null) object.scrollIntoView();
        window.scrollTo(left, _top);

}());

</script>
<?php

//display processing time
$time_end = microtime(TRUE);
echo("\n<!-- page generated in " . ($time_end - $time_start) . " seconds -->");

}; mod_content();
?>
