<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

//-----------------------------------------------------------------------------------------------------
//===< MODULE >========================================================================================
//-----------------------------------------------------------------------------------------------------
require("../nuos.inc");

if (empty($desktop_display))                                    $desktop_display = NULL;
elseif (in_array($desktop_display, array("interface", "link"))) $ifc_option      = "external";

//load libraries
cms_load("ifc",     TRUE);
cms_load("desktop", TRUE);

//set user
if (
     empty($user)
     ||
     (
       nstreq($user, CMS_SUPERUSER)
       &&
       (! cms_permission(CMS_APPLICATION . ".$user", NULL, NULL))
     )
   )
        $user = CMS_SUPERUSER;

switch ($desktop_display)
{
default:
//-----------------------------------------------------------------------------------------------------
        $desktop = new desktop($user);

        if (! isset($object)) $object = (string)cms_cache("desktop.$user.object");
        if (! isset($parent)) $parent = (string)cms_cache("desktop.$user.parent");
        if (! isset($load))   $load   = FALSE;

        //message handling
        switch (CMS_IFC_MESSAGE)
        {
        case "activate":
//.....................................................................................................
                if (
                     (! $ifc_param)
                     ||
                     streq($desktop->object_type($ifc_param), "container")
                   )
                {
                        $parent = $ifc_param;
                }
                else
                {
                        $object = $ifc_param;
                        $load   = TRUE;
                };

                break;

        case "select":
//.....................................................................................................
                $object = $ifc_param;
                break;

        case "create":
//.....................................................................................................
                $ifc_param = explode(",", $ifc_param);

                if ($_object = $desktop->create_object($parent, $ifc_param[0], $ifc_param[1]))
                {
                        $object       = $_object;
                        $load         = nstreq($ifc_param[0], "container");
                        $ifc_response = CMS_MSG_DONE;
                        break;
                };

                $ifc_response = CMS_MSG_ERROR;
                break;

        case "quickaccess":
//.....................................................................................................
                $desktop->object_set($object,
                                     "quickaccess",
                                     ! $desktop->object_get($object, "quickaccess"));

                $ifc_response = $desktop->save() ? CMS_MSG_DONE : CMS_MSG_ERROR;
                break;

        case "rename":
//.....................................................................................................
                if ($ifc_param)
                {
                        $desktop->object_set($object, "name", $ifc_param);

                        if ($desktop->save())
                        {
                                $ifc_response = CMS_MSG_DONE;
                                break;
                        };
                };

                $ifc_response = CMS_MSG_ERROR;
                break;

        case "drop":
//.....................................................................................................
                $ifc_param = explode(",", $ifc_param);

                $desktop->object_set($ifc_param[0], "x", $ifc_param[1]);
                $desktop->object_set($ifc_param[0], "y", $ifc_param[2]);

                if ($desktop->save())
                {
                        $object       = $ifc_param[0];
                        $ifc_response = CMS_MSG_DONE;
                        break;
                };

                $ifc_response = CMS_MSG_ERROR;
                break;

        case "move":
//.....................................................................................................
                if ($ifc_param)
                {
                        $ifc_param = explode(",", $ifc_param);
                        $object    = $ifc_param[0];
                        $parent    = $ifc_param[1];
                };

                if ($_object = $desktop->move_object($object, $parent))
                {
                        $desktop->object_set($_object, "x", NULL);
                        $desktop->object_set($_object, "y", NULL);

                        $object       = $_object;
                        $ifc_response = CMS_MSG_DONE;
                        break;
                };

                $ifc_response = CMS_MSG_ERROR;
                break;

        case "delete":
//.....................................................................................................
                if ($desktop->delete_object($ifc_param))
                {
                        if (streq($ifc_param, $object)) $object = NULL;

                        $ifc_response = CMS_MSG_DONE;
                        break;
                };

                $ifc_response = CMS_MSG_ERROR;
                break;

        case "send":
//.....................................................................................................

                //generate receiver list
                $object = $ifc_param;
                $array  = NULL;
                $data   = new data("#system/permission");

                $data->move("first");
                while ($key = $data->move("next"))
                {
                        if (! permission_is_user($key)) continue;
                        $_key = substr($key, 5);

                        if (
                             nstreq($_key, $user)
                             &&
                             cms_permission(CMS_APPLICATION, NULL, NULL, $_key)
                           )
                                $array[$data->get($key, "name")] = $_key;
                };

                if (! $array)
                {
                        $ifc_response = CMS_MSG_ERROR . "<br>" .
                                        CMS_L_MOD_DESKTOP_010;
                        break;
                };

                ksort($array);

                //display
                $ifc   = new ifc(NULL,
                                 NULL,
                                 TRUE,
                                 array("user"   => $user,
                                       "object" => $object,
                                       "parent" => $parent),
                                 "_send",
                                 CMS_L_MOD_DESKTOP_011);

                $ifc->set(CMS_L_MOD_DESKTOP_012, "label");
                $ifc->set($array,                "multiselect 30x15");

                $ifc->close();

        case "_send":
//.....................................................................................................
                if (empty($ifc_param1)) { $ifc_response = CMS_MSG_ERROR; break; };

                $ifc_response = TRUE;
                $desktop->data->copy($object);

                if (is_array($ifc_param1))
                {
                        reset($ifc_param1);
                        while (list($key, $value) = each($ifc_param1))
                        {
                                $_desktop     = new desktop($value);

                                $_desktop->data->set_buffer(array_merge(array(array("name"  => date(CMS_L_DATE_FORMAT . ", " .
                                                                                                    CMS_L_SHORT_TIME_FORMAT) .
                                                                                               " " . CMS_NAME,
                                                                                   "#type"  => "container")),
                                                                        $desktop->data->buffer,
                                                                        array(array("#type" => "/container"))));
                                $key          = $_desktop->data->seek(array("#subtype" => "mailbox"));
                                $_desktop->data->insert($key);

                                $ifc_response = $_desktop->save() && $ifc_response;
                        };
                };

                $ifc_response = $ifc_response ? CMS_MSG_DONE : CMS_MSG_ERROR;
                break;

        case "config":
//.....................................................................................................

                //display
                $ifc = new ifc(NULL,
                               NULL,
                               TRUE,
                               array("user"   => $user,
                                     "object" => $object,
                                     "parent" => $parent),
                               "_config",
                               CMS_L_MOD_DESKTOP_026);

                $ifc->set(CMS_L_MOD_DESKTOP_027, "title");
                $ifc->set(CMS_L_MOD_DESKTOP_014, "password 40 40 b");
                $ifc->set(CMS_L_MOD_DESKTOP_015, "password 40 40 b");
                $ifc->set(CMS_L_MOD_DESKTOP_016, "password 40 40 b");

                //timezone
                $array = timezone_identifiers_list();
                $array = array_combine($array, $array);
                $ifc->set(CMS_L_MOD_DESKTOP_028, "title");
                $ifc->set(CMS_L_MOD_DESKTOP_025, "label");
                $ifc->set($array,                "select 40", CMS_TIMEZONE);

                $ifc->close();

        case "_config":
//.....................................................................................................
                $data         = new data("#system/permission");
                $password     = $data->get("user." . CMS_SUPERUSER, "password");
                $superuser    = "user." . CMS_SUPERUSER;

                //password
                if (
                     streq(md5($ifc_param1), $password)
                     &&
                     $ifc_param2
                     &&
                     streq($ifc_param2, $ifc_param3)
                   )
                {
                        $ifc_param2 = md5($ifc_param2);
                        $data->set($ifc_param2, $superuser, "password");

                        //generate obfuscation string
                        $secret1    = cms_cache("cms.5c080f9d6448aa7514a2842e1ae732d7");
                        $secret2    = "11f544779d678530603c344c5381d2b1";
                        $ifc_param2 = md5(md5("$secret1|$secret2") . "|$ifc_param2");

                        setcookie("cms_password",   //name
                                  $ifc_param2,      //value
                                  NULL,             //expire
                                  CMS_RELATIVE_URL, //path
                                  NULL,             //domain
                                  FALSE,            //secure
                                  TRUE);            //httponly
                };

                //timezone
                $data->set($ifc_param4, $superuser, "timezone");

                $ifc_response = $data->save() ? CMS_MSG_DONE : CMS_MSG_ERROR;
        };

        //cache selected object and parent permanently
        if (CMS_IFC_MESSAGE !== "")
        {
                cms_cache("desktop.$user.object", $object, TRUE);
                cms_cache("desktop.$user.parent", $parent, TRUE);
        };

        //icon
        $desktop_icon   = array("link"        => "desktop/icon_webpage",
                                "note"        => "desktop/icon_note",
                                "appointment" => "desktop/icon_appointment",
                                "address"     => "desktop/icon_address",
                                "container"   => "desktop/icon_container",
                                "+container"  => "desktop/icon_containeropen",
                                "mailbox"     => "desktop/icon_mailbox");

        //type
        $desktop_type   = array("link"        => CMS_DESKTOP_TYPE_LINK,
                                "note"        => CMS_DESKTOP_TYPE_NOTE,
                                "appointment" => CMS_DESKTOP_TYPE_APPOINTMENT,
                                "address"     => CMS_DESKTOP_TYPE_ADDRESS,
                                "container"   => CMS_DESKTOP_TYPE_CONTAINER,
                                "mailbox"     => CMS_DESKTOP_TYPE_MAILBOX);

        //accept
        $desktop_accept = array("link"        => CMS_DESKTOP_TYPE_NONE,
                                "note"        => CMS_DESKTOP_TYPE_NONE,
                                "appointment" => CMS_DESKTOP_TYPE_NONE,
                                "address"     => CMS_DESKTOP_TYPE_NONE,
                                "container"   => CMS_DESKTOP_TYPE_ALL,
                                "mailbox"     => CMS_DESKTOP_TYPE_NONE);

        //menu
        $menu = NULL;
        if ($object)
        {
                $menu[CMS_L_MOD_DESKTOP_009 . "|desktop/command_rename"] = "javascript:desktop_rename();";
                if (! $desktop->data->is_child($parent, $object))
                        $menu[CMS_L_MOD_DESKTOP_017 . "|desktop/command_cut"] = "move";
        };
        $menu[CMS_L_COMMAND_REFRESH . "|desktop/command_refresh"]       = TRUE;
        $menu[CMS_L_MOD_DESKTOP_026 . "|desktop/command_configuration"] = "config";

        //display
        $ifc = new ifc(NULL,
                       NULL,
                       $menu,
                       array("user" => $user),
                       NULL,
                       NULL,
                       "desktop-content");

        //rename object
        if ($object)
        {
?>
<script>
function desktop_rename()
{
        var value = prompt("<?php echo(q(CMS_L_MOD_DESKTOP_009));?>",
                           "<?php echo(q($desktop->object_get($object, "name")));?>");
        if (value) ifc_post("rename", value);
};
</script>
<?php
        };

        //desktop control
        echo("<div id=\"desktop-control\">" .

             "<a id=\"desktop-logo\" " .
                "href=\"http://" . x(CMS_HOMEPAGE) . "/\" " .
                "title=\"" . x(CMS) . "\">" .
             image(CMS_IFC_EDITION . "/logo") .
             "</a>" .

        //desktop control head
             "<div id=\"desktop-control-head\">");

        //interface
        if (cms_permission("interface", NULL, NULL))
                echo("<a href=\"" . x(CMS_MODULES_URL) . "interface.php\">" .
                     image(CMS_IFC_EDITION . "/icon_interface") . " " . CMS_L_MOD_DESKTOP_029 .
                     "</a>");

        echo("<a href=\"" . x(CMS_ROOT_URL) . "\">" .
             image(CMS_IFC_EDITION . "/icon_website") . " " . CMS_L_MOD_DESKTOP_030 .
             "</a>" .
             "</div>" .

        //desktop control body
             "<div id=\"desktop-control-body\">" .

        //user selection
             "<div id=\"desktop-user\">");

        $array = NULL;
        $data  = new data("#system/permission");

        //permission
        ifc_permission(array(""    => CMS_L_ACCESS,
                             $user => $data->get("user.$user", "name") . ": " . CMS_L_ACCESS));

        $data->move("first");
        while ($key = $data->move("next"))
        {
                if (! permission_is_user($key)) continue;
                $_key = substr($key, 5);

                if (
                     nstreq($_key, CMS_SUPERUSER)
                     &&
                     cms_permission(CMS_APPLICATION, NULL, NULL, $_key)
                     &&
                     cms_permission($_key)
                   )
                        $array[$data->get($key, "name")] = $_key;
        };

        if ($array)
        {
?>
<script>
function desktop_user_select(value)
{
        ifc_change("user", value);
        ifc_post();
};
</script>
<?php
                echo("<select onchange=\"if (this.selectedIndex != -1) desktop_user_select(this[this.selectedIndex].value);\">" .
                     "<option value=\"" . x(CMS_SUPERUSER) . "\"" .
                              (streq($user, CMS_SUPERUSER)  ? " selected>" : ">") .
                     x(CMS_NAME) .
                     "</option>");

                ksort($array);
                reset($array);

                while (list($key, $value) = each($array))
                        echo("<option value=\"" . x($value) . "\"" .
                                      (streq($user, $value) ? " selected>" : ">") .
                             x($key) .
                             "</option>");

                echo("</select>");
        }
        else
        {
                echo(x(CMS_NAME));
        };

        echo("</div>" .

        //user favourites
             "<div id=\"desktop-link-title\">" .
             ($object ? "<a href=\"javascript:ifc_post('quickaccess');\">[" : "") .
             CMS_L_MOD_DESKTOP_018 .
             ($object ? " +/-]</a>"                                         : "") .
             "</div>" .

             "<div id=\"desktop-link\">");

        $buffer      = NULL;
        $_buffer     = NULL;
        $offset_x    = 290;
        $offset_y    = 115;
        $mod_x       = $offset_x % 100;
        $mod_y       = $offset_y %  50;
        $depth       = $parent ? 0                                            : 1;
        $raster      = $parent ? array($offset_x => array($offset_y => TRUE)) : NULL;
        $path        = NULL;
        $_path       = array();
        $appointment = NULL;
        $time        = time();
        $time_start  = $time - 3600;
        $time_end    = $time + 3600;

        $desktop->data->move("first");
        while ($key = $desktop->data->move("next"))
        {
                $name = x($desktop->object_get($key, "name"));
                if (streq($key, $object))
                        $name = sprintf("<strong>%s</strong>", $name);

                if ($desktop->object_get($key, "quickaccess"))
                        echo("<a href=\"javascript:ifc_post('activate','" . rqx($key) . "');\">" .
                             image($desktop_icon[$desktop->object_type($key)]) . "&nbsp;$name" .
                             "</a>");

                switch ($type = $desktop->object_type($key))
                {
                default:
//.....................................................................................................
                        if ($depth == 1)
                        {
                                $x  = (int)$desktop->object_get($key, "x") - 50;
                                if ($x < $offset_x) $x = $offset_x;
                                elseif ($x > 1500)  $x = 1500;
                                $x -= $x % 100 - $mod_x;

                                $y  = (int)$desktop->object_get($key, "y") - 25;
                                if ($y < $offset_y) $y = $offset_y;
                                elseif ($y > 1500)  $y = 1500;
                                $y -= $y % 50  - $mod_y;

                                if (isset($raster[$x][$y]))
                                {
                                        for (; $x < 1500; $x += 100)
                                        {
                                                for (; $y < 1500; $y += 50)
                                                {
                                                        if (! isset($raster[$x][$y])) break 2;
                                                };
                                        };
                                };

                                $raster[$x][$y]  = TRUE;

                                $buffer         .= "<div id=\"" . x("dd-$key") . "\" " .
                                                        "class=\"icon\" " .
                                                        "style=\"LEFT:" . $x . "px;TOP:" . $y . "px;\">" .
                                                   image($desktop_icon[$desktop->object_type($key)]) . "<br>$name" .
                                                   "</div>";
                                $_buffer        .= "dd_register(\"dd-" . q($key) . "\", " .
                                                                $desktop_type[$type] . ", " .
                                                                $desktop_accept[$type] . ");";
                        };

                        if (streq($type, "container"))
                        {
                                if (nstreq($path, $parent))
                                {
                                        $path = $key;
                                        array_push($_path, $path);
                                };

                                if (
                                     ($depth > 0)
                                     ||
                                     streq($key, $parent)
                                   )
                                        $depth++;
                        };

                        break;

                case "/container":
//.....................................................................................................
                        if (nstreq($path, $parent)) array_pop($_path);
                        if ($depth)                 $depth--;
                };

                if (
                     streq($type, "appointment")
                     &&
                     (($_time = $desktop->object_get($key, "time")) >= $time_start)
                   )
                        $appointment[$_time] = $key;
        };
        echo("</div>" .

        //create
             "<div id=\"desktop-create-title\">" .
             CMS_L_MOD_DESKTOP_001 .
             "</div>" .

             "<div id=\"desktop-create\">");
?>
<script>
function desktop_create(type)
{
        var value;

        switch (type)
        {
        case "link":
                value = "<?php echo(q(CMS_L_MOD_DESKTOP_003));?>";
                break;

        case "note":
                value = "<?php echo(q(CMS_L_MOD_DESKTOP_004));?>";
                break;

        case "appointment":
                value = "<?php echo(q(CMS_L_MOD_DESKTOP_005));?>";
                break;

        case "address":
                value = "<?php echo(q(CMS_L_MOD_DESKTOP_006));?>";
                break;

        case "container":
                value = "<?php echo(q(CMS_L_MOD_DESKTOP_007));?>";
                break;

        case "mailbox":
                value = "<?php echo(q(CMS_L_MOD_DESKTOP_008));?>";
        };

        value = prompt("<?php echo(q(CMS_L_MOD_DESKTOP_023));?>".replace(/%s/, value), "");
        if (value) ifc_post("create", type + "," + value);
};
</script>
<?php
        //link
        echo("<a href=\"javascript:desktop_create('link');\">" .
             image($desktop_icon["link"]) . "&nbsp;" . CMS_L_MOD_DESKTOP_003 .
             "</a>" .

        //note
             "<a href=\"javascript:desktop_create('note');\">" .
             image($desktop_icon["note"]) . "&nbsp;" . CMS_L_MOD_DESKTOP_004 .
             "</a>" .

        //appointment
             "<a href=\"javascript:desktop_create('appointment');\">" .
             image($desktop_icon["appointment"]) . "&nbsp;" . CMS_L_MOD_DESKTOP_005 .
             "</a>" .

        //address
             "<a href=\"javascript:desktop_create('address');\">" .
             image($desktop_icon["address"]) . "&nbsp;" . CMS_L_MOD_DESKTOP_006 .
             "</a>" .

        //container
             "<a href=\"javascript:desktop_create('container');\">" .
             image($desktop_icon["container"]) . "&nbsp;" . CMS_L_MOD_DESKTOP_007 .
             "</a>" .

        //mailbox
             "<a href=\"javascript:desktop_create('mailbox');\">" .
             image($desktop_icon["mailbox"]) . "&nbsp;" . CMS_L_MOD_DESKTOP_008 .
             "</a>" .

             "</div>");

        //show current appointments
        $active_url = rqx(CMS_ACTIVE_URL);
        $_user      = rqx($user);

        if ($appointment)
        {
                echo("<div id=\"desktop-appointment-title\">" .
                     CMS_L_MOD_DESKTOP_002 .
                     "</div>" .

                     "<div id=\"desktop-appointment\">");

                $i = 0;
                ksort($appointment);

                reset($appointment);
                while (
                        (list($key, $value) = each($appointment))
                        &&
                        (
                          ($key <= $time)
                          ||
                          (++$i <= 3)
                        )
                      )
                {
                        $name = x($desktop->object_get($value, "name"));

                        //lapsed
                        if ($key <= $time)
                                $name = "<span style=\"FONT-STYLE:italic\">" .
                                        $name .
                                        "</span>";

                        //imminent
                        elseif ($key <= $time_end)
                                $name = "<strong>" .
                                        $name .
                                        "</strong>";

                        echo("<a href=\"javascript:load_page('','" .
                                 $active_url .
                                "?desktop_display=interface" .
                                "&amp;user=$_user" .
                                "&amp;object=" . rqx($value) . "');\" " .
                                "title=\"" . x(friendly_date($key)) . "\">" .
                             image("desktop/icon_appointment") . "&nbsp;$name" .
                             "</a>");
                };

                echo("</div>");
        };

        //receiver
        echo("<div id=\"dd-receiver\">" .
             image("desktop/icon_receiver", NULL, NULL, CMS_L_MOD_DESKTOP_011) .
             "</div>" .

        //trashbin
             "<div id=\"dd-trashbin\">" .
             image("desktop/icon_trashbin", NULL, NULL, CMS_L_COMMAND_DELETE) .
             "</div>" .

             "</div>" .

        //desktop control foot
             "<div id=\"desktop-control-foot\">" .

        //logout
             "<a href=\"" . x(CMS_MODULES_URL) . "identification.php?cms_logout=1\">" .
             image(CMS_IFC_EDITION . "/icon_logout") . " " . CMS_L_IFC_007 .
             "</a>" .
             "</div>" .

        //date / time
             "<div id=\"desktop-date\">" .
             date(CMS_L_DATE_FORMAT . ", " . CMS_L_TIME_FORMAT) .
             "</div>" .

             "</div>" .

        //shortlinks
             "<div id=\"desktop-link-quick\">" .

        //instant messenger
             "<a href=\"javascript:load_page('','" .
                 $active_url .
                "?desktop_display=interface" .
                "&amp;desktop_interface=ims" .
                "&amp;user=$_user');\" " .
                "title=\"" . CMS_L_MOD_DESKTOP_020 . "\">" .
             (is_file(CMS_DATA_PATH . "#desktop/" . r($user) . "/ims.txt") ?
             image("desktop/icon_ims_highlight") .
             "</a> " .
             "<audio src=\"" . CMS_SOUNDS_URL . "desktop/ims_alert.wav\" " .
                    "autoplay>" :
             image("desktop/icon_ims") .
             "</a> ") .

        //calendar
             "<a href=\"javascript:load_page('','" .
                 $active_url .
                "?desktop_display=interface" .
                "&amp;desktop_interface=appointment" .
                "&amp;user=$_user');\" " .
                "title=\"" . CMS_L_MOD_DESKTOP_022 . "\">" .
             image("desktop/icon_appointment") .
             "</a> " .

        //address book
             "<a href=\"javascript:load_page('','" .
                 $active_url .
                "?desktop_display=interface" .
                "&amp;desktop_interface=address" .
                "&amp;user=$_user');\" " .
                "title=\"" . CMS_L_MOD_DESKTOP_021 . "\">" .
             image("desktop/icon_address") .
             "</a>" .
             "</div>");



        //path
        if (count($_path))
        {
                echo("<div id=\"desktop-path\">" .
                     "<a href=\"javascript:ifc_post('activate');\">" .
                     "<strong>" .
                     CMS_L_MOD_DESKTOP_024 .
                     "</strong>" .
                     "</a>");

                reset($_path);
                while (list($key, $value) = each($_path))
                {
                        echo(" / <a href=\"javascript:ifc_post('activate','" . rqx($value) . "');\">" .
                             x($desktop->object_get($value, "name")) .
                             "</a>");
                };

                echo("</div>");
        };

        if ($parent)
        {
                $_parent = $desktop->data->move("parent", $parent);

                echo("<div id=\"dd-" . x($_parent) . "\" " .
                          "class=\"icon\" " .
                          "style=\"LEFT:" . $offset_x . "px;TOP:" . $offset_y . "px;\">" .
                     image("desktop/icon_containeropen") . "<br>" .
                     ($_parent ? x($desktop->object_get($_parent, "name")) : CMS_L_MOD_DESKTOP_024) .
                     "</div>");
        };

        echo($buffer);
?>
<script>
function desktop_event(event,
                       source,
                       target)
{
        source_id = (source == null) ? "" : source.id.substr(3, 100);
        target_id = (target == null) ? "" : target.id.substr(3, 100);

        switch (event)
        {
        case "activate":
                if (source_id == "trashbin")      break;
                if (source_id == "receiver")      break;
                ifc_post("activate", source_id);
                break;

        case "select":
                if (source_id == "")              break;
                if (source_id == "trashbin")      break;
                if (source_id == "receiver")      break;
                ifc_post("select", source_id);
                break;

        case "dropon":
                if (target_id == "trashbin")      ifc_post("delete", source_id);
                else if (target_id == "receiver") ifc_post("send", source_id);
                else                              ifc_post("move", source_id + "," + target_id);
                break;

        case "drop":
                ifc_post("drop", source_id + "," + fx_mouse_x + "," + fx_mouse_y);
                break;
        };
};
dd_no_cropping = true;
dd_set_callback(desktop_event);
dd_register("dd-receiver", 0, 255, true, true);
dd_register("dd-trashbin", 0, 255, true, true);
<?php
        if ($parent)
                echo("dd_register(\"dd-" . q($_parent) . "\", " .
                                  $desktop_type["container"] . ", " .
                                  $desktop_accept["container"] . ", " .
                                  "true);");
        echo($_buffer);
?>
</script>
<?php

        //activate
        if ($load && ($type = $desktop->object_type($object)))
        {
                $active_url = q(CMS_ACTIVE_URL);

                switch ($type)
                {
                case "note":
//.....................................................................................................
                        echo(jscript("load_page(\"\", \"" .
                                     $active_url .
                                     "?desktop_display=interface" .
                                     "&user=$_user" .
                                     "&object=" . rq($object) . "\");"));
                        break;

                default:
//.....................................................................................................
                        echo(jscript("load_page(\"\", \"" .
                                     $active_url .
                                     "?desktop_display=interface" .
                                     "&user=$_user" .
                                     "&object=" . rq($object) . "\");"));
                        break;
                };
        };

        $ifc->close();

case "interface":
//-----------------------------------------------------------------------------------------------------
        $desktop = new desktop($user);

        if (empty($object))
        {
                $object            = 0;
                if (empty($desktop_interface)) $desktop_interface = NULL;
        }
        else
        {
                $desktop_interface = $desktop->object_type($object);
        };

        $param = array("desktop_interface" => $desktop_interface,
                       "user"              => $user,
                       "object"            => $object);

        switch ($desktop_interface)
        {
        case "mailbox":
//.....................................................................................................
                require(CMS_DESKTOP_PATH . "desktop.mailbox.inc");

        case "address":
//.....................................................................................................
                require(CMS_DESKTOP_PATH . "desktop.address.inc");

        case "note":
//.....................................................................................................
                require(CMS_DESKTOP_PATH . "desktop.note.inc");

        case "appointment":
//.....................................................................................................
                require(CMS_DESKTOP_PATH . "desktop.appointment.inc");

        case "ims":
//.....................................................................................................
                require(CMS_DESKTOP_PATH . "desktop.ims.inc");

        case "link":
//.....................................................................................................
                echo(CMS_DOCTYPE_FRAMESET .
                     "<html>" .
                     "<head>" .
                     CMS_HTML_HEADER .
                     "</head>" .
                     "<frameset rows=\"*,75\">" .
                     "<frame name=\"desktop_link_output\" " .
                            "src=\"" . x(CMS_URL) . "blank.htm\" " .
                            "noresize>" .
                     "<frame name=\"desktop_link_input\" " .
                            "src=\"" . x(CMS_ACTIVE_URL) .
                            "?desktop_display=link" .
                            "&amp;user=" . r($user) .
                            "&amp;object=" . r($object) . "\" " .
                            "scrolling=\"no\" " .
                            "noresize>" .
                     "</frameset>" .
                     "</html>");

                exit;

        default:
//.....................................................................................................
                ifc_close_external();
        };

case "link":
//-----------------------------------------------------------------------------------------------------
        $desktop = new desktop($user);

        if (empty($object)) $object = NULL;

        switch (CMS_IFC_MESSAGE)
        {
        case "reload":
//.....................................................................................................
                $ifc_param1 = NULL;
                break;

        case "save":
//.....................................................................................................
                $desktop->object_set($object, "url", $ifc_param1);
                $ifc_response = $desktop->save() ? CMS_MSG_DONE : CMS_MSG_ERROR;
        };

        $param   = array("user"   => $user,
                         "object" => $object);
        $url     = empty($ifc_param1) ? $desktop->object_get($object, "url") : $ifc_param1;

        //display
        $ifc     = new ifc(NULL,
                           $ifc_page,
                           array(CMS_L_COMMAND_SHOW . "|desktop/command_view"
                                 => "javascript:parent.desktop_link_output.location.replace(" .
                                    "document.ifc.ifc_param1.value);",
                                 CMS_L_MOD_DESKTOP_013 . "|desktop/command_readout"
                                 => "javascript:ifc_change(" .
                                    "'ifc_param1'," .
                                    "parent.desktop_link_output.location.href);",
                                 CMS_L_COMMAND_REFRESH . "|desktop/command_refresh"
                                 => "reload",
                                 CMS_L_COMMAND_SAVE .    "|desktop/command_save"
                                 => "save"),
                           $param,
                           NULL,
                           CMS_L_MOD_DESKTOP_003);

        if ($url) echo(jscript("parent.desktop_link_output.location.replace(\"" . q($url) . "\");"));
        $ifc->set(CMS_L_URL, "text 60 256 :", $url);

        $ifc->close();
};
?>