<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

//-----------------------------------------------------------------------------------------------------
//===< MODULE >========================================================================================
//-----------------------------------------------------------------------------------------------------
require("../nuos.inc");

function mod_messenger() {

global $messenger_display,
       $messenger_name,
       $core_control_object,
       $core_control_command,
       $core_control_value,
       $messenger_data;

cms_load("core", TRUE);

define("CMS_MESSENGER_TIMEOUT",                30); //timeout in seconds
define("CMS_MESSENGER_REFRESH",              5000); //refresh rate in milliseconds
define("CMS_MESSENGER_AWAY_TIMER_TIMEOUT", 600000); //away timeout in milliseconds
define("CMS_MESSENGER_AWAY_TIMER_REFRESH",  30000); //away timer refresh rate in milliseconds

cms_param(TRUE, "_" . unique_id()); //random parameter to force update from server

switch ($messenger_display)
{
default:
//-----------------------------------------------------------------------------------------------------
        $core = new core(CMS_MESSENGER_TIMEOUT);
        $core->update_index(TRUE); //update user list removing oneself on timeout

        //error - probably banned - or still connected
        if ((! $core->enabled) || $core->index->seek(array("guid" => $core->guid)))
        {
                //display message and
                echo(CMS_DOCTYPE_HTML .
                     "<html>" .
                     "<head>" .
                     CMS_HTML_HEADER . CMS_JAVASCRIPT . CMS_STYLESHEET .
                     "</head>" .
                     "<body class=\"" . x(CMS_CLASS) . "\">" .
                     "<div>" .

                //title
                     "<h1>" .
                     CMS_L_MOD_MESSENGER_014 .
                     "</h1>" .

                //message
                     ($core->enabled ?
                     CMS_L_MOD_MESSENGER_001 .
                     jscript("setTimeout(\"location.replace(\\\"" .
                             q(q(CMS_ACTIVE_URL . cms_param())) . "\\\");\", 10000);") :
                     CMS_L_MOD_MESSENGER_019) .
                     "</div>" .
                     "</body>" .
                     "</html>");

                exit();
        };

        //try to log in
        if ($core->connect($messenger_name))
        {
                $profile = $core->get_profile();

                //log
                $log = new log();
                $log->access(sprintf(CMS_L_MOD_MESSENGER_010, $profile["name"]));
                $log->user($profile["name"], NULL, NULL, NULL, NULL, $profile["email"]);

                header("Location: " . CMS_ACTIVE_URL . cms_param(array("messenger_display" => "frame")));
                exit();
        };

        //determine number of connected users and channels
        $channel = array();
        $count   = 0;

        $core->index->reset();
        while ($core->index->next(array("guid" => TRUE)))
        {
                $guid     = $core->index->get("guid");
                $_channel = $core->index->get("channel");
                $status   = $core->get_status($guid, $channel);

                if (! flag($status, CMS_CORE_STATUS_INVISIBLE)) //not invisible
                {
                        $count++;
                        $channel[$_channel] = TRUE;
                };
        };

        $_count = count($channel);
        if ($_count === 0) $_count = 1;

        echo(CMS_DOCTYPE_HTML .
             "<html>" .
             "<head>" .
             CMS_HTML_HEADER . CMS_JAVASCRIPT . CMS_STYLESHEET .
             "</head>" .
             "<body class=\"" . x(CMS_CLASS) . "\">" .
             "<div>");

        //insert
        insert("top");

        echo("<noscript>" .
             "<div>" .
             CMS_L_NOSCRIPT .
             "</div>" .
             "</noscript>" .

        //title
             "<h1>" .
             CMS_L_MOD_MESSENGER_014 .
             "</h1>" .

        //number of connected users and channels
             "<p>" .
             sprintf(CMS_L_MOD_MESSENGER_007, $count, $_count) .
             "</p>" .

             "<form name=\"messenger_login\" " .
                   "method=\"post\" " .
                   "action=\"" . x(CMS_ACTIVE_URL . cms_param()) . "\">" .
             "<div>" .

        //name taken message
             (stre($messenger_name) ? "" : info(CMS_L_MOD_MESSENGER_003)) .
             CMS_L_MOD_MESSENGER_008 . "<br>" .

        //name
             "<label>" .
             "<strong>" .
             CMS_L_NAME .
             "</strong><br>" .
             "<input name=\"messenger_name\" " .
                    "type=\"text\" " .
                    "value=\"" . x($messenger_name) . "\" " .
                    "size=\"20\" " .
                    "maxlength=\"20\">" .
             "</label> " .

        //connect button
             "<input name=\"messenger_submit\" " .
                    "type=\"submit\" " .
                    "value=\"" . CMS_L_COMMAND_CONNECT . "\">" .
             "</div>" .
             "</form>" .

        //identification
             "<hr>" .
             CMS_L_MOD_MESSENGER_013);

        cms_application("identification");
        echo(jscript("document.messenger_login.messenger_name.select();" .
                     "document.messenger_login.messenger_name.focus();"));

        //insert
        insert("bottom");

        echo("</div>" .
             "</body>" .
             "</html>");

        exit();

case "disconnect":
//-----------------------------------------------------------------------------------------------------
        $core = new core(CMS_MESSENGER_TIMEOUT);
        $core->update_index();

        //disconnect
        if ($core->enabled && $core->index->seek(array("guid" => $core->guid))) $core->disconnect();

case "disconnected":
//-----------------------------------------------------------------------------------------------------
        echo(CMS_DOCTYPE_HTML .
             "<html>" .
             "<head>" .
             CMS_HTML_HEADER . CMS_JAVASCRIPT . CMS_STYLESHEET .
             "</head>" .
             "<body class=\"" . x(CMS_CLASS) . "-disconnected\">" .
             "<div>" .

        //title
             "<h1>" .
             CMS_L_MOD_MESSENGER_014 .
             "</h1>" .

        //message
             "<p>" .
             "<strong>" .
             CMS_L_MOD_MESSENGER_004 .
             "</strong>" .
             "</p>" .

        //reconnect button
             "<a href=\"" . x(CMS_ACTIVE_URL . cms_param()) . "\">" .
             CMS_L_MOD_MESSENGER_015 .
             "</a>" .
             "</div>" .
             "</body>" .
             "</html>");

        exit();

case "frame":
//-----------------------------------------------------------------------------------------------------
        $core = new core(CMS_MESSENGER_TIMEOUT);
        $core->update_index();

        //error
        if ((! $core->enabled) || (! $core->index->seek(array("guid" => $core->guid))))
        {
                header("Location: " . CMS_ACTIVE_URL . cms_param());
                exit();
        };

        echo(CMS_DOCTYPE_FRAMESET .
             "<html>" .
             "<head>" .
             CMS_HTML_HEADER .
             "</head>" .
             "<frameset rows=\"*,150,0\">");
        if (cms_permission(CMS_CORE_PERMISSION_CONTROL))
                echo("<frameset cols=\"*,330\">" .

                //display
                     "<frame name=\"messenger_display\" " .
                            "src=\"" . x(CMS_URL) . "blank.htm\">" .

                //control
                     "<frame name=\"messenger_control\" " .
                            "src=\"" . x(CMS_ACTIVE_URL . cms_param(array("messenger_display" => "control"))) . "\">" .
                     "</frameset>");
        else

                //display
                echo("<frame name=\"messenger_display\" " .
                            "src=\"" . x(CMS_URL) . "blank.htm\">");

        //interface
        echo("<frame name=\"messenger_interface\" " .
                    "src=\"" . x(CMS_ACTIVE_URL . cms_param(array("messenger_display" => "interface"))) . "\">" .
             "<frameset cols=\"50%,50%\">" .

        //send
             "<frame name=\"messenger_send\" " .
                    "src=\"" . x(CMS_URL) . "blank.htm\" " .
                    "scrolling=\"no\" " .
                    "noresize>" .

        //receive
             "<frame name=\"messenger_receive\" " .
                    "src=\"" . x(CMS_URL) . "blank.htm\" " .
                    "scrolling=\"no\" " .
                    "noresize>" .
             "</frameset>" .
             "</frameset>" .
             "</html>");

        exit();

case "control":
//-----------------------------------------------------------------------------------------------------
        if (! cms_permission(CMS_CORE_PERMISSION_CONTROL)) exit();

        cms_load("core_control", TRUE);

        $core = new core(CMS_MESSENGER_TIMEOUT);
        $core->update_index();

        //error
        if ((! $core->enabled) || (! $core->index->seek(array("guid" => $core->guid)))) exit();

        echo(CMS_DOCTYPE_HTML .
             "<html>" .
             "<head>" .
             CMS_HTML_HEADER . CMS_JAVASCRIPT . CMS_STYLESHEET .
             "</head>" .
             "<body class=\"" . x(CMS_CLASS) . "-control\">" .
             "<div>");

        //insert
        insert("control_top");

        //load control element
        core_control($core,
                     $core_control_object,
                     $core_control_command,
                     $core_control_value,
                     array("cms_instance"      => CMS_INSTANCE,
                           "messenger_display" => "control"));

        //insert
        insert("control_bottom");

        //permission
        permission(array(""                           => CMS_L_ACCESS,
                         CMS_CORE_PERMISSION_CONTROL  => CMS_L_MOD_MESSENGER_012,
                         CMS_CORE_PERMISSION_OPERATOR => CMS_L_OPERATOR));

        echo("</div>" .
             "</body>" .
             "</html>");

        exit();

case "interface":
//-----------------------------------------------------------------------------------------------------
        echo(CMS_DOCTYPE_HTML .
             "<html>" .
             "<head>" .
             CMS_HTML_HEADER . CMS_JAVASCRIPT . CMS_STYLESHEET .
             "</head>" .
             "<body class=\"" . x(CMS_CLASS) . "-interface\" " .
                   "onload=\"messenger_onload();\">" .
             "<form name=\"messenger_interface\" " .
                   "method=\"post\" " .
                   "action=\"" . x(CMS_ACTIVE_URL . cms_param(array("messenger_display" => "send"))) . "\" " .
                   "target=\"messenger_send\" " .
                   "onsubmit=\"return messenger_send();\">" .
             "<div>" .

        //input
             "<input name=\"messenger_data\" " .
                    "type=\"text\" " .
                    "size=\"65\" " .
                    "maxlength=\"1024\"><br>" .

        //scroll switch
             "<label>" .
             "<input name=\"messenger_scroll_enabled\" " .
                    "type=\"checkbox\" " .
                    "checked>" .
             CMS_L_MOD_MESSENGER_009 .
             "</label> " .

        //focus switch
             "<label>" .
             "<input name=\"messenger_focus_enabled\" " .
                    "type=\"checkbox\">" .
             CMS_L_MOD_MESSENGER_011 .
             "</label> " .

        //sound switch
             "<label>" .
             "<input name=\"messenger_sound_enabled\" " .
                    "type=\"checkbox\" " .
                    "checked>" .
             CMS_L_MOD_MESSENGER_006 .
             "</label> " .

        //monkey switch
             "<label>" .
             "<input name=\"messenger_monkey_enabled\" " .
                    "type=\"checkbox\" " .
                    "checked>" .
             CMS_L_MOD_MESSENGER_018 .
             "</label> " .

        //away switch/button
             "<input name=\"messenger_away_timer_enabled\" " .
                    "type=\"checkbox\" " .
                    "checked>" .
             "<a href=\"#\" " .
                "title=\"/away\" " .
                "onclick=\"return messenger_replace('/away',1);\">" .
             CMS_L_MOD_MESSENGER_016 .
             "</a> " .
             "| " .

        //clear button
             "<a href=\"#\" " .
                "onclick=\"return messenger_init();\">" .
             CMS_L_MOD_MESSENGER_020 .
             "</a> " .
             "| " .

        //disconnect button
             "<a href=\"#\" " .
                "onclick=\"return messenger_disconnect();\">" .
             CMS_L_MOD_MESSENGER_017 .
             "</a><br>");

        //emoticons
        $list = array();
        $path = CMS_IMAGES_PATH . "emoticon";

        if (is_dir($path) && ($hdir = opendir($path)))
        {
                while (($file = readdir($hdir)) !== FALSE)
                {
                        if (! is_file("$path/$file"))                     continue; //no file
                        if (! preg_match("/\.(gif|jpe?g|png)$/i", $file)) continue; //no image

                        $name        = base64_decode(preg_replace("/\.[^\.]+$/", "", $file));
                        $_name       = x($name);
                        $list[$name] = $file;

                        //emoticon button
                        echo("<a href=\"#\" " .
                                "onclick=\"return messenger_append(' " . qx($name) . " ');\" " .
                                "title=\"$_name\">" .
                             "<img src=\"" . x(CMS_IMAGES_URL . "emoticon/" . r($file)) . "\" " .
                                  "alt=\"$_name\">" .
                             "</a>");
                };

                closedir($hdir);
        };

        //sound
        echo("</div>" .
             "</form>");
?>
<script>
function messenger_onload()
{
        messenger_init();
        messenger_receive();
        messenger_away_timer();
};
//--------------------------------
var messenger_receive_time = 0;
function messenger_receive()
{
        var delay = <?php echo(CMS_MESSENGER_REFRESH);?> - new Date().getTime() + messenger_receive_time;
        if (delay > 0)
        {
                setTimeout(messenger_receive, delay);
                return;
        };
        parent.messenger_receive.location.replace(
                "<?php echo(q(CMS_ACTIVE_URL . cms_param(array("messenger_display" => "receive"))));?>");
        setTimeout(messenger_receive, <?php echo(CMS_MESSENGER_REFRESH);?>);
};
//--------------------------------
function messenger_receive_ok()
{
        messenger_receive_time = new Date().getTime();
};
//--------------------------------
function messenger_send()
{
        document.messenger_interface.submit();
        setTimeout(messenger_clear, 50);
        messenger_away_timer_reset();
        return false;
};
//--------------------------------
function messenger_clear()
{
        with (document.messenger_interface)
        {
                messenger_data.value = "";
                messenger_data.focus();
        };
};
//--------------------------------
function messenger_append(text)
{
        with (document.messenger_interface.messenger_data)
        {
                value += text;
                focus();
        };
        return false;
};
//--------------------------------
function messenger_replace(text) //arguments: 1 => post
{
        with (document.messenger_interface.messenger_data)
        {
                value = text;
                focus();
        };
        if (arguments.length > 1) messenger_send();
        return false;
};
//--------------------------------
function messenger_write(text)
{
        var date  = new Date();
        var value = 0;
        parent.messenger_display.document.write(
                "<div>" +
                "<div style=\"FLOAT:right\">" +
                "<small>" +
                date.getFullYear() +                                          "-" +
                (((value = date.getMonth())   < 10) ? "0" + value :  value) + "-" +
                (((value = date.getDate())    < 10) ? "0" + value :  value) + " " +
                (((value = date.getHours())   < 10) ? "0" + value :  value) + ":" +
                (((value = date.getMinutes()) < 10) ? "0" + value :  value) + ":" +
                (((value = date.getSeconds()) < 10) ? "0" + value :  value) +
                "<\/small>" +
                "<\/div>" +
                text +
                "<br style=\"CLEAR:both\">" +
                "<\/div>");
        messenger_scroll();
};
//--------------------------------
function messenger_message(name,
                           color,
                           text,
                           image)
{
        if (text.match(/(^|\s):\(\|\)(\s|$)/)) messenger_monkey();
        var _name = htmlspecialchars(name);
        text      = htmlspecialchars(text);
        text      = messenger_link(text);
        text      = messenger_emoticon(text);
        messenger_write(
                ((image == "") ? "" :
                "<div style=\"FLOAT:left;" +
                             "HEIGHT:auto;" +
                             "MAX-HEIGHT:30px;" +
                             "OVERFLOW:hidden\">" +
                "<img src=\"" + htmlspecialchars(image) + "\" " +
                     "alt=\"" + _name + "\" " +
                     "style=\"MARGIN:0 5px 5px 0;" +
                             "WIDTH:35px\">" +
                "<\/div>") +
                "<span" + ((color == "") ? "" : " style=\"COLOR:" + htmlspecialchars(color) + "\"") + ">" +
                "<a href=\"#\" " +
                   "onclick=\"return parent.messenger_interface.messenger_append('@" + addslashes(_name) + " ');\">" +
                _name +
                "<\/a>" +
                ": " + text +
                "<\/span>");
        messenger_focus();
        messenger_notify("<?php echo(q(CMS_L_MOD_MESSENGER_005));?>".replace(/%s/, name));
        messenger_sound("<?php echo(q(CMS_SOUNDS_URL));?>messenger/message.wav");
};
//--------------------------------
function messenger_private(name,
                           color,
                           text,
                           image)
{
        if (text.match(/(^|\s):\(\|\)(\s|$)/)) messenger_monkey();
        var _name = htmlspecialchars(name);
        text      = htmlspecialchars(text);
        text      = messenger_link(text);
        text      = messenger_emoticon(text);
        messenger_write(
                ((image == "") ? "" :
                "<div style=\"FLOAT:left;" +
                             "HEIGHT:auto;" +
                             "MAX-HEIGHT:30px;" +
                             "OVERFLOW:hidden\">" +
                "<img src=\"" + htmlspecialchars(image) + "\" " +
                     "alt=\"" + _name + "\" " +
                     "style=\"MARGIN:0 5px 5px 0;" +
                             "WIDTH:35px\">" +
                "<\/div>") +
                "<span style=\"" + ((color == "") ? "" : "COLOR:" + htmlspecialchars(color) + ";") +
                              "FONT-WEIGHT:bold\">" +
                _name + " (<?php echo(q(CMS_L_MOD_MESSENGER_002));?>): " + text +
                "<\/span>");
        messenger_focus();
        messenger_notify("<?php echo(q(CMS_L_MOD_MESSENGER_005));?>".replace(/%s/, name));
        messenger_sound("<?php echo(q(CMS_SOUNDS_URL));?>messenger/private.wav");
};
//--------------------------------
function messenger_system(text)
{
        messenger_write(
                "<span style=\"FONT-FAMILY:monospace\">" +
                htmlspecialchars(text) +
                "<\/span>");
        messenger_focus();
        messenger_notify(text);
        messenger_sound("<?php echo(q(CMS_SOUNDS_URL));?>messenger/system.wav");
};
//--------------------------------
function messenger_info(text)
{
        messenger_write(
                "<small>" +
                htmlspecialchars(text) +
                "<\/small>");
};
//--------------------------------
function messenger_link(text)
{
        text = text.replace(
                /(^|\s)(www\.\S+)(?=\s|$)/gi,
                "$1http://$2");
        text = text.replace(
                /(^|\s)((?:ftp|https?):\/\/\S+\.(?:gif|jpg|jpeg|png))(?=\s|$)/gi,
                "$1" +
                "<a href=\"$2\" " +
                   "onclick=\"window.open(this.href);return false;\">" +
                "<img src=\"$2\" " +
                     "alt=\"\" " +
                     "style=\"MAX-HEIGHT:200px;" +
                             "MAX-WIDTH:200px\">" +
                "<\/a>");
        text = text.replace(
                /(^|\s)((?:ftp|https?):\/\/[^\s"]+)(?=\s|$)/gi,
                "$1" +
                "<a href=\"$2\" " +
                   "onclick=\"window.open(this.href);return false;\">" +
                "$2" +
                "<\/a>");
        return text;
};
//--------------------------------
function messenger_emoticon(text)
{
        return text<?php
        reset($list);
        while (list($key, $value) = each($list))
                echo(".replace(/(^|\\s)" . preg_quote(x($key), "/") . "(?=\\s|$)/g, " .
                     "\"$1<img src=\\\"" . qx(CMS_IMAGES_URL . "emoticon/" . r($value)) . "\\\" " .
                              "alt=\\\"" . qx($key) . "\\\">\")");?>;
};
//--------------------------------
function messenger_scroll()
{
        if (document.messenger_interface.messenger_scroll_enabled.checked)
                setTimeout("parent.messenger_display.scrollBy(0, 99999999);", 50);
};
//--------------------------------
function messenger_focus()
{
        if (! document.messenger_interface.messenger_focus_enabled.checked) return;
        {
                top.focus();
                document.messenger_interface.messenger_data.focus();
        };
};
//--------------------------------
var messenger_notify_text  = "";
var messenger_notify_count = 0;
function messenger_notify(text)
{
        messenger_notify_text = text;
        if (messenger_notify_count == 0)
        {
                messenger_notify_count = 4;
                _messenger_notify();
                return;
        };
        messenger_notify_count = 4;
};
//--------------------------------
function _messenger_notify()
{
        top.document.title = messenger_notify_text;
        if (--messenger_notify_count > 0)
        {
                setTimeout("top.document.title=\"...\";", 250);
                setTimeout(_messenger_notify, 750);
        };
};
//--------------------------------
function messenger_sound(file)
{
        if (document.messenger_interface.messenger_sound_enabled.checked)
        {
                var audio = new Audio();
                audio.setAttribute("src", file);
                audio.load();
                audio.oncanplay = function() { audio.play() };
        };
};
//--------------------------------
function messenger_monkey()
{
        if (document.messenger_interface.messenger_monkey_enabled.checked)
                messenger_sound("<?php echo(q(CMS_SOUNDS_URL));?>messenger/monkey.wav");
};
//--------------------------------
var messenger_away_timer_value = <?php echo(CMS_MESSENGER_AWAY_TIMER_TIMEOUT);?>;
function messenger_away_timer()
{
        with (document.messenger_interface)
        {
                if (messenger_away_timer_enabled.checked)
                {
                        if (messenger_away_timer_value >= 0)
                        {
                                messenger_away_timer_value -= <?php echo(CMS_MESSENGER_AWAY_TIMER_REFRESH);?>;
                                if (messenger_away_timer_value < 0)
                                {
                                        messenger_data.value = "away";
                                        submit();
                                        setTimeout(messenger_clear, 50);
                                };
                        };
                }
                else
                {
                        messenger_away_timer_value = <?php echo(CMS_MESSENGER_AWAY_TIMER_TIMEOUT);?>;
                };
        };
        setTimeout(messenger_away_timer, <?php echo(CMS_MESSENGER_AWAY_TIMER_REFRESH);?>);
};
//--------------------------------
function messenger_away_timer_reset()
{
        messenger_away_timer_value = <?php echo(CMS_MESSENGER_AWAY_TIMER_TIMEOUT);?>;
};
//--------------------------------
function messenger_init()
{
        with (parent.messenger_display.document)
        {
                close();
                open("text/html", "replace");
                write("<?php echo(str_replace("</", "<\\/", q(CMS_DOCTYPE_HTML)));?>" +
                      "<html>" +
                      "<head>" +
                      "<?php echo(str_replace("</", "<\\/", q(CMS_HTML_HEADER)));?>" +
                      "<?php echo(str_replace("</", "<\\/", q(CMS_STYLESHEET)));?>" +
                      "<\/head>" +
                      "<body class=\"<?php echo(qx(CMS_CLASS));?>-display\">" +
                      "<h1>" +
                      "<?php echo(q(CMS_L_MOD_MESSENGER_014));?>" +
                      "<\/h1>");
        };
        return false;
};
//--------------------------------
function messenger_disconnect()
{
        parent.location.replace("<?php echo(q(CMS_ACTIVE_URL . cms_param(array("messenger_display" => "disconnect"))));?>");
        return false;
};
</script>
<?php
        echo("</body>" .
             "</html>");

        exit();

case "send":
case "receive":
//-----------------------------------------------------------------------------------------------------
        $core = new core(CMS_MESSENGER_TIMEOUT);
        $core->update_index();

        echo(CMS_DOCTYPE_HTML .
             "<html>" .
             "<head>" .
             "<title>" .
             "</title>" .
             "<script>" .
             "with (parent.messenger_interface)" .
             "{");

        //error
        if ((! $core->enabled) || (! $core->index->seek(array("guid" => $core->guid))))
        {
                echo("parent.location.replace(\"" . q(CMS_ACTIVE_URL .
                     cms_param(array("messenger_display" => "disconnected"))) . "\");");
        }

        //data exchange
        else
        {
                //send
                if (
                     streq($messenger_display, "send")
                     &&
                     nstre($messenger_data = utf8_trim($messenger_data))
                   )
                {
                        //input commands
                        switch ($messenger_data)
                        {
                        case "afk":
                        case "away":
                        case "brb":
                        case "/afk":
                        case "/away":
                        case "/brb":
                                $core->status(CMS_CORE_STATUS_ABSENT);
                                break;

                        case "/bye":
                        case "/disconnect":
                        case "/exit":
                        case "/logout":
                        case "/off":
                        case "/quit":
                                if ($core->index->seek(array("guid" => $core->guid))) $core->disconnect();
                                break;

                        default:
                                $core->send($messenger_data);
                        };
                };

                //receive
                if ($data = $core->receive())
                {
                        reset($data);
                        while (list($key, $value) = each($data))
                        {
                                //system message
                                if (flag($value["status"], CMS_CORE_DATA_SYSTEM))
                                {
                                        echo("messenger_system(" .
                                             "\"" . q($value["data"]) . "\");");
                                }

                                //private message
                                elseif (flag($value["status"], CMS_CORE_DATA_PRIVATE))
                                {
                                        //get spy receivers
                                        if (flag($value["status"], CMS_CORE_DATA_SPY_RECEIVER))
                                        {
                                                $profile  = $core->get_profile($value["guid"]);
                                                $info     = (stre($profile["name"]) ? CMS_L_UNKNOWN : $profile["name"]) . " @[";

                                                $receiver = explode("\n", $value["data"]);

                                                reset($receiver);
                                                while (list(, $guid) = each($receiver))
                                                {
                                                        $profile  = $core->get_profile($guid);
                                                        $info    .= (stre($profile["name"]) ? CMS_L_UNKNOWN : $profile["name"]) . ", ";
                                                };

                                                $info = substr($info, 0, -2) . "]: ";
                                        }

                                        //get spy data
                                        elseif (flag($value["status"], CMS_CORE_DATA_SPY_DATA))
                                        {
                                                echo("messenger_info(\"(" . q($info . $value["data"]) . ")\");");
                                        }

                                        //display
                                        else
                                        {
                                                $profile = $core->get_profile($value["guid"]);
                                                $image   = stre($profile["image"]) ? "" : image_process(CMS_DATA_URL . "core/" . r($profile["image"]), 35);
                                                echo("messenger_private(" .
                                                     "\"" . q(stre($profile["name"]) ? CMS_L_UNKNOWN : $profile["name"]) . "\", " .
                                                     "\"" . q($profile["color"]) . "\", " .
                                                     "\"" . q($value["data"]) . "\", " .
                                                     "\"" . q($image) . "\");");
                                        };
                                }

                                //default message
                                else
                                {
                                        $profile = $core->get_profile($value["guid"]);
                                        $image   = stre($profile["image"]) ? "" : image_process(CMS_DATA_URL . "core/" . r($profile["image"]), 35);
                                        echo("messenger_message(" .
                                             "\"" . q(stre($profile["name"]) ? CMS_L_UNKNOWN : $profile["name"]) . "\", " .
                                             "\"" . q($profile["color"]) . "\", " .
                                             "\"" . q($value["data"]) . "\", " .
                                             "\"" . q($image) . "\");");
                                };
                        };
                };
                echo("messenger_receive_ok();");
        };
        echo("};" .
             "</script>" .
             "</head>" .
             "<body>" .
             "<div>" .
             "</div>" .
             "</body>" .
             "</html>");

        exit();
};

}; mod_messenger();
?>