<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

//-----------------------------------------------------------------------------------------------------
//===< MODULE >========================================================================================
//-----------------------------------------------------------------------------------------------------
require("../nuos.inc");

//load libraries
cms_load("content",   TRUE);
cms_load("directory", TRUE);
cms_load("rss",       TRUE);

//send rss content type header
header("Content-Type: application/rss+xml; charset=utf-8");

$rss = new rss();

//selected channel
if (empty($rss_channel))             return; //no channel
if (! $rss->data->get($rss_channel)) return; //invalid channel

//number of items to display
$rss_limit = empty($rss_limit) ? 0 : (int)$rss_limit;
if ($rss_limit < 0)                  $rss_limit = FALSE;

//sorting order
if (empty($rss_order))               $rss_order = "published";
switch($rss_order)
{
default:
case "modified":
        $rss_order = CMS_DB_CONTENT_TIME;
        break;

case "published":
        $rss_order = CMS_DB_CONTENT_PUBLISHER_TIME;
        break;
};

//caching
$cache_key  = "rss.$rss_channel.$rss_limit.$rss_order." . CMS_LANGUAGE;
$cache_time = cms_cache_time($cache_key);
$time       = time();

if (($cache_time !== FALSE) && ($cache_time > ($time - 60))) //60 seconds delay
{
        echo(cms_cache($cache_key));
        exit();
};

//start output
$buffer  = "<?xml version=\"1.0\" " .
                 "encoding=\"utf-8\"?>" .
           "<rss version=\"2.0\">" .
           "<channel>";

//title
$title   = l($rss->data->get($rss_channel, "name"));
$buffer .= "<title>" . x($title) . "</title>";

//link
$link    = l($rss->data->get($rss_channel, "link"));
$buffer .= "<link>" . x($link) . "</link>";

//description
$value   = l($rss->data->get($rss_channel, "description"));
$buffer .= "<description>" . x($value) . "</description>";

//language
$buffer .= "<language>" . x(CMS_LANGUAGE) . "</language>";

//publication date
$mysql  = new mysql();
$result = mysql_query("SELECT " .   CMS_DB_CONTENT_PUBLISHER_TIME . " " .
                      "FROM " .     CMS_DB_CONTENT . " " .
                      "WHERE " .    CMS_DB_CONTENT_STATUS .
                      "='" .        CMS_CONTENT_STATUS_PUBLICATION . "' " .
                      "AND " .      CMS_DB_CONTENT_CHANNEL . " " .
                      "LIKE '%/" .  sqlesc($rss_channel) . "/%' " .
                      "ORDER BY " . CMS_DB_CONTENT_PUBLISHER_TIME . " DESC " .
                      "LIMIT 1");
if (mysql_num_rows($result))
{
        $value   = date("r", mysql_result($result, 0));
        $buffer .= "<pubDate>" . x($value) . "</pubDate>";
};

//last build date
$value   = date("r");
$buffer .= "<lastBuildDate>" . x($value) . "</lastBuildDate>";

//category
if ($value = l($rss->data->get($rss_channel, "category")))
        $buffer .= "<category>" . x($value) . "</category>";

//generator
$buffer .= "<generator>" . x(CMS_SOFTWARE . " " . CMS_VERSION) . "</generator>";

//image
$value   = ($value = l($rss->data->get($rss_channel, "image"))) ?
           translate_url($value) : CMS_IMAGES_URL . "content/rss.jpg";
$buffer .= "<image>" .
           "<url>" .   x($value) . "</url>" .
           "<title>" . x($title) . "</title>" .
           "<link>" .  x($link) .  "</link>" .
           "</image>";

//items
$directory = new directory();
$result    = mysql_query("SELECT " .   CMS_DB_CONTENT_INDEX . ", " .
                                       CMS_DB_CONTENT_TIME . ", " .
                                       CMS_DB_CONTENT_PUBLISHER_TIME . ", " .
                                       CMS_DB_CONTENT_TITLE . ", " .
                                       CMS_DB_CONTENT_DESCRIPTION . " " .
                         "FROM " .     CMS_DB_CONTENT . " " .
                         "WHERE " .    CMS_DB_CONTENT_STATUS .
                         "='" .        CMS_CONTENT_STATUS_PUBLICATION . "' " .
                         "AND " .      CMS_DB_CONTENT_CHANNEL . " " .
                         "LIKE '%/" .  sqlesc($rss_channel) . "/%' " .
                         "ORDER BY " . $rss_order . " DESC " .
                         (($rss_limit > 0) ? " LIMIT " . sqlesc($rss_limit) : ""));
$_time     = $time - 259200; //3 days

while ($resultrow = mysql_fetch_assoc($result))
{
        $index   = $resultrow[CMS_DB_CONTENT_INDEX];

        $buffer .= "<item>";

        //title
        $value   = l($resultrow[CMS_DB_CONTENT_TITLE]);

        //new
        if (($__time = $resultrow[CMS_DB_CONTENT_PUBLISHER_TIME]) > $_time)
                $value = CMS_L_MOD_RSS_001 . ": $value";

        //update
        if ($resultrow[CMS_DB_CONTENT_TIME] > $__time)
        {
                $__time = $resultrow[CMS_DB_CONTENT_TIME];
                if ($__time > $_time) $value .= " [" . CMS_L_MOD_RSS_002 . "]";
        };

        //time
        if ($__time > $_time)
                $value .= " (" . friendly_date($__time) . ")";

        $buffer .= "<title>" . x($value) . "</title>";

        //description
        $value   = parse_text(l($resultrow[CMS_DB_CONTENT_DESCRIPTION]));
        $buffer .= "<description>" . x($value) . "</description>";

        //category
        $array      = array();
        $stack_key  = array();
        $stack_name = array();
        $link       = FALSE;
        $length     = strlen($index) + 10;

        $directory->data->move("first");
        while ($key = $directory->data->move("next"))
        {
                switch ($directory->data->get($key, "#type"))
                {
                case "container":
//.....................................................................................................
                        $url = $directory->data->get($key, "url");

                        if (
                             streq(substr($url, 0, $length), "content://$index")
                             &&
                             (
                               stre($url = substr($url, $length, 1))
                               ||
                               ($url === "?")
                               ||
                               ($url === "#")
                             )
                           )
                        {
                                $_key = end($stack_key);
                                if (! isset($array[$_key]))
                                        $array[$_key] = ($_key === FALSE) ?
                                                        CMS_L_MOD_RSS_007 :
                                                        implode("/", $stack_name);

                                if ($link === FALSE)
                                        $link         = translate_url("directory://$key");
                        };

                        array_push($stack_key,  $key);
                        array_push($stack_name, l($directory->data->get($key, "name")));
                        break;

                case "/container":
//.....................................................................................................
                        array_pop($stack_key);
                        array_pop($stack_name);
                        break;
                };
        };

        reset($array);
        while (list($key, $value) = each($array))
        {
                $key     = ($key === 0) ? CMS_ROOT_URL : translate_url("directory://$key");
                $buffer .= "<category domain=\"" . x($key) . "\">" .
                           x($value) .
                           "</category>";
        };

        //guid
        $guid    = CMS_MODULES_URL . "content.php" .
                   querystring(array("content_index" => $index));
        $buffer .= "<guid isPermaLink=\"true\">" . x($guid) . "</guid>";

        //link
        if ($link === FALSE) $link = $guid;
        $buffer .= "<link>" . x($link) . "</link>";

        //publication date
        $value   = date("r", $resultrow[CMS_DB_CONTENT_PUBLISHER_TIME]);
        $buffer .= "<pubDate>" . x($value) . "</pubDate>";

        $buffer .= "</item>";
};

$buffer .= "</channel>" .
           "</rss>";

//cache permanently
cms_cache($cache_key, $buffer, TRUE);

//output
echo($buffer);
?>