<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

//-----------------------------------------------------------------------------------------------------
//===< MODULE >========================================================================================
//-----------------------------------------------------------------------------------------------------
require("../nuos.inc");

//load libraries
cms_load("content", TRUE);

//send sitemap content type header
header("Content-Type: application/xml; charset=utf-8");

//caching
$cache_key  = "sitemap";
$cache_time = cms_cache_time($cache_key);
$time       = time();

if (($cache_time !== FALSE) && ($cache_time > ($time - 60))) //60 seconds delay
{
        echo(cms_cache($cache_key));
        exit();
};

//retrieve language specific mapping
$data     = new data("#system/directory");
$map      = array();
$language = (CMS_LANGUAGE_ENABLED != "") ? explode(",", CMS_LANGUAGE_ENABLED) : array(0);

reset($language);
while (list(, $value) = each($language))
{
        $_value = ($value !== 0) ? "$value." : "";
        $map[$value]  = new map("#system/" . $_value . "directory.canonical");
};

//address
$list = array();

$data->move("first");
while ($key = $data->move("next"))
{
        if (nstreq($data->get($key, "#type"), "container")) continue;

        $url = $data->get($key, "url");

        if (streq(substr($url, 0, 10), "content://"))
        {
                //get content index
                $index = substr($url, 10);
                $index = substr($index, 0, strspn($index, "1234567890"));

                $list[$index] = $key;
        };
};

//start output
$buffer = "<?xml version=\"1.0\" " .
                "encoding=\"utf-8\"?>" .
          "<urlset xmlns=\"http://www.sitemaps.org/schemas/sitemap/0.9\" " .
                  "xmlns:xhtml=\"http://www.w3.org/1999/xhtml\">";

//retrieve publications
$mysql  = new mysql();
$result = mysql_query("SELECT " .   CMS_DB_CONTENT_INDEX . ", " .
                                    CMS_DB_CONTENT_PUBLISHER_TIME . " " .
                      "FROM " .     CMS_DB_CONTENT . " " .
                      "WHERE " .    CMS_DB_CONTENT_STATUS .
                      "='" .        CMS_CONTENT_STATUS_PUBLICATION . "' " .
                      "AND NOT " .  CMS_DB_CONTENT_FLAG . " & " . CMS_CONTENT_FLAG_SITEMAP_EXCLUDE . " " .
                      "ORDER BY " . CMS_DB_CONTENT_PUBLISHER_TIME . " DESC");

//process publications
while ($resultrow = mysql_fetch_assoc($result))
{
        $index = $resultrow[CMS_DB_CONTENT_INDEX];

        if (isset($list[$index]))
        {
                $key     = $list[$index];
                $buffer .= "<url>";
                $flag    = TRUE;

                reset($language);
                while (list(, $value) = each($language))
                {
                        $address = $map[$value]->get_value($key);

                        //primary address
                        if ($flag)
                        {
                                $buffer .= "<loc>" .
                                           x($address) .
                                           "</loc>" .
                                           "<lastmod>" .
                                           x(date("Y-m-d", $resultrow[CMS_DB_CONTENT_PUBLISHER_TIME])) .
                                           "</lastmod>";
                                $flag    = FALSE;
                        };

                        //alternate address
                        if ($value !== 0)
                                $buffer .= "<xhtml:link rel=\"alternate\" " .
                                                       "hreflang=\"" . x($value) . "\" " .
                                                       "href=\"" . x($address) . "\"/>";
                };

                $buffer .= "</url>";
        };
};

$buffer .= "</urlset>";

//cache permanently
cms_cache($cache_key, $buffer, TRUE);

//output
echo($buffer);
?>