<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

//overload constant functions to automatically include namespaces
function constant($name)
{
        return \constant(__NAMESPACE__ . "\\$name");
};
//-----------------------------------------------------------------------------------------------------
function define($name,
                $value,
                $case_insensitive = FALSE)
{
        return \define(__NAMESPACE__ . "\\$name", $value, $case_insensitive);
};
//-----------------------------------------------------------------------------------------------------
function defined($name)
{
        return \defined(__NAMESPACE__ . "\\$name");
};

//product information
define("CMS_SOFTWARE",  "nuos CMS");
define("CMS_VERSION",   is_file(dirname(__FILE__) . "/version.txt") ?
                        file_get_contents(dirname(__FILE__) . "/version.txt") : "?");
define("CMS_COPYRIGHT", "Copyright \xC2\xA9 2018 Patrick Heyer");
define("CMS_HOMEPAGE",  "www.nuos-cms.org");
define("CMS",           CMS_SOFTWARE . " " . CMS_VERSION . " " . CMS_COPYRIGHT . ", " . CMS_HOMEPAGE);

//requirements check
if ((! function_exists("version_compare")) || version_compare(PHP_VERSION, "5.3.0", "<"))
        die("PHP5 >= 5.3.0 is required.");

if (! (extension_loaded("mysqli") || extension_loaded("mysql")))
        die("MySQL or MySQLi (recommended) support is required.");

if (! extension_loaded("pcre"))
        die("PCRE support is required.");

if (! preg_match("/\pL/u", "a"))
        die("PCRE UTF-8 support is required.");

ob_implicit_flush(FALSE);
ob_start();

define("CMS_TIMEZONE_DEFAULT", "UTC");
date_default_timezone_set(CMS_TIMEZONE_DEFAULT);

$cms_time = time();

header("Cache-Control: private, max-age=0, must-revalidate");
header("Cache-Control: post-check=0, pre-check=0", FALSE); //proprietary internet explorer setting
header("Content-Type: text/html; charset=utf-8");
header("Date: " . gmdate("D, d M Y H:i:s", $cms_time) . " GMT");
header("Expires: " . gmdate("D, d M Y H:i:s", $cms_time + 3600) . " GMT");
header("Last-Modified: " . gmdate("D, d M Y H:i:s", $cms_time) . " GMT");

umask(0000);
if (version_compare(PHP_VERSION, "5.3.0", "<")) set_magic_quotes_runtime(FALSE);
error_reporting(E_ALL);

//error handler
function cms_error($code,
                   $message,
                   $path    = NULL,
                   $line    = NULL,
                   $display = NULL)
{
        if (($code & (E_ALL & ~E_DEPRECATED & ~E_STRICT)) === 0) return;

        //error types
        $array = array(E_ERROR             => array("Error",             "#FF0000"),
                       E_WARNING           => array("Warning",           "#FF7F00"),
                       E_PARSE             => array("Parse error",       "#FF0000"),
                       E_NOTICE            => array("Notice",            "#007FFF"),
                       E_CORE_ERROR        => array("Core error",        "#FF0000"),
                       E_CORE_WARNING      => array("Core warning",      "#FF7F00"),
                       E_COMPILE_ERROR     => array("Compile error",     "#FF0000"),
                       E_COMPILE_WARNING   => array("Compile warning",   "#FF7F00"),
                       E_USER_ERROR        => array("User error",        "#FF0000"),
                       E_USER_WARNING      => array("User warning",      "#FF7F00"),
                       E_USER_NOTICE       => array("User notice",       "#007FFF"),
                       E_STRICT            => array("Strict",            "#777777"),
                       E_RECOVERABLE_ERROR => array("Recoverable Error", "#FF0000"),
                       E_DEPRECATED        => array("Deprecated",        "#777777"),
                       E_USER_DEPRECATED   => array("User deprecated",   "#777777"),
                       100                 => array("Unknown error",     "#777777"));

        if (! isset($array[$code])) $code = 100;

        //write to log
        if (
             defined("CMS_L_DATE_FORMAT")
             &&
             defined("CMS_L_TIME_FORMAT")
             &&
             defined("CMS_DATA_PATH")
             &&
             function_exists("mkpath")
             &&
             mkpath(CMS_DATA_PATH . "#log")
             &&
             function_exists("_fopen")
             &&
             function_exists("_unlink")
             &&
             function_exists("_rename")
           )
        {
                //compose message
                $buffer = date(CMS_L_DATE_FORMAT . ", " . CMS_L_TIME_FORMAT) . "\n" .
                          $array[$code][0] . ": " .
                          $message .
                          ($path ? " in $path"      : "") .
                          ($line ? " on line $line" : "");

                //prepend to existing log
                if (
                     is_file(CMS_DATA_PATH . "#log/error.txt")
                     &&
                     ($hfile = fopen(CMS_DATA_PATH . "#log/error.txt", "rb"))
                   )
                {
                        $flag = FALSE;

                        //create temporary file
                        if ($_hfile = fopen(CMS_DATA_PATH . "#log/error.tmp", "wb"))
                        {
                                //write new message
                                fputs($_hfile, "$buffer\n\n");

                                //append 100 lines from existing log
                                for ($i = 0; ($i < 100) && (! feof($hfile)); $i++)
                                        fputs($_hfile, fgets($hfile));

                                fclose($_hfile);

                                $flag = TRUE;
                        };

                        fclose($hfile);

                        //swap files
                        if ($flag)
                        {
                                unlink(CMS_DATA_PATH . "#log/error.txt");
                                rename(CMS_DATA_PATH . "#log/error.tmp", CMS_DATA_PATH . "#log/error.txt");
                        };
                }

                //create new log
                elseif ($hfile = fopen(CMS_DATA_PATH . "#log/error.txt", "wb"))
                {
                        fputs($hfile, $buffer);
                        fclose($hfile);
                };
        };

        //direct output
        if (
             ($display === TRUE)
             ||
             (strcmp(getenv("HTTP_HOST"), "192.168.115.120") === 0)
             ||
             (defined("CMS_SUPERUSER") === FALSE)
             ||
             (strcmp(constant("CMS_SUPERUSER"), "admin") === 0)
           )
        {
                $id  = "error-" . htmlspecialchars(uniqid());
                $_id = addslashes($id);

                echo("\n\n" .
                     "<div class=\"error\">" .
                     "<a href=\"#\" " .
                        "style=\"FLOAT:right\" " .
                        "onclick=\"this.parentNode.parentNode.removeChild(this.parentNode);return false;\">" .
                     "[Close]" .
                     "</a>\n" .
                     "<strong style=\"COLOR:" . $array[$code][1] . "\">" . $array[$code][0] . "</strong><br>" .
                     "<br>\n" .
                     htmlspecialchars($message) . "<br>\n");
                if ($path) echo("<small>in " . htmlspecialchars($path) . "</small><br>\n");
                if ($line) echo("<small>on line " . htmlspecialchars($line) . "</small><br>\n");
                echo("<br>" .

                //display details
                     "<a href=\"#\" " .
                        "onclick=\"document.getElementById('$_id').style.display=(document.getElementById('$_id').style.display=='')?'none':'';return false;\">" .
                     "[Details]" .
                     "</a>\n" .
                     "<pre id=\"$id\" " .
                          "style=\"DISPLAY:none\">\n");

                $array = debug_backtrace();
                array_shift($array); //skip current function

                reset($array);
                while (list($key, $value) = each($array))
                {
                        //output stack level
                        $file     = (isset($value["file"])     && ($value["file"]     !== "")) ?
                                    $value["file"]     : "unknown";
                        $line     = (isset($value["line"])     && ($value["line"]     !== "")) ?
                                    $value["line"]     : "unknown";
                        $function = (isset($value["function"]) && ($value["function"] !== "")) ?
                                    $value["function"] : "unknown";
                        $class    = (isset($value["class"])    && ($value["class"]    !== "")) ?
                                    $value["class"]    : "";
                        $type     = (isset($value["type"])     && ($value["type"]     !== "")) ?
                                    $value["type"]     : "";

                        echo(htmlspecialchars(
                                "$key: file $file, line $line\n" .
                                "  $class$type$function\n" .
                                "    (\n"));

                        //arguments
                        if (isset($value["args"]) && is_array($value["args"]))
                        {
                                reset($value["args"]);
                                while (list($_key, $_value) = each($value["args"]))
                                {
                                        $type = gettype($_value);

                                        switch ($type)
                                        {
                                        case "boolean":
                                                $buffer = "(boolean)" . ($_value ? "TRUE" : "FALSE");
                                                break;

                                        case "integer":
                                                $buffer = "(integer)$_value";
                                                break;

                                        case "double":
                                                $buffer = "(float)$_value";
                                                break;

                                        case "string":

                                                $buffer   = "(string[" . strlen($_value) . "])";
                                                $_value   = preg_replace(array("/^[\p{C}\p{Z}]+/u",
                                                                               "/[\p{C}\p{Z}]+$/u",
                                                                               "/[\p{C}\p{Z}]+/u"),
                                                                         array("",
                                                                               "",
                                                                               " "),
                                                                         $_value);
                                                $__value  = preg_match("/^(.{0,20})/su", $_value, $__value) ?
                                                            $__value[1] : $_value;
                                                if (strlen($__value) < strlen($_value)) $__value .= "\xE2\x80\xA6";
                                                $buffer  .= "\"$__value\"";
                                                break;

                                        case "array":
                                                $buffer = "(array[" . count($_value) . "])";
                                                break;

                                        case "object":
                                                $buffer = "(object)" . get_class($_value);
                                                break;

                                        case "resource":
                                                $buffer = $args[] = "(resource)" . get_resource_type($_value);
                                                break;

                                        case "NULL":
                                                $buffer = "NULL";
                                                break;

                                        default:
                                                $buffer = "($type)";
                                        };

                                        echo("      " . htmlspecialchars("$_key: $buffer") . "\n");
                                };
                        };

                        echo("    )\n");
                };

                echo("</pre>" .
                     "</div>" .
                     "\n\n");
        };

        return TRUE;
};

set_error_handler(__NAMESPACE__ . "\\cms_error");

//-----------------------------------------------------------------------------------------------------
//===< SYSTEM FUNCTIONS >==============================================================================
//-----------------------------------------------------------------------------------------------------
if (get_magic_quotes_gpc())
{
        function cms_strip_magic_quotes($value)
        {
                return is_array($value) ?
                       array_map(__NAMESPACE__ . "\\cms_strip_magic_quotes", $value) :
                       stripslashes($value);
        };

        $_POST    = array_map(__NAMESPACE__ . "\\cms_strip_magic_quotes", $_POST);
        $_GET     = array_map(__NAMESPACE__ . "\\cms_strip_magic_quotes", $_GET);
        $_COOKIE  = array_map(__NAMESPACE__ . "\\cms_strip_magic_quotes", $_COOKIE);
        $_REQUEST = array_map(__NAMESPACE__ . "\\cms_strip_magic_quotes", $_REQUEST);
};
//-----------------------------------------------------------------------------------------------------
function cms_utf8_normalize($value)
{
        return is_array($value) ?
               array_map(__NAMESPACE__ . "\\cms_utf8_normalize", $value) :
               utf8_normalize(preg_replace("/\r\n?/", "\n", $value));
};
//-----------------------------------------------------------------------------------------------------
function cms_initialize_globals()
{
        if (defined("CMS_INITIALIZE_GLOBALS_EXECUTED")) return;
        define("CMS_INITIALIZE_GLOBALS_EXECUTED", TRUE);

        reset($_COOKIE);
        while (list($key, $value) = each($_COOKIE))
                $GLOBALS[$key] = cms_utf8_normalize($value);

        reset($_GET);
        while (list($key, $value) = each($_GET))
                $GLOBALS[$key] = cms_utf8_normalize($value);

        reset($_POST);
        while (list($key, $value) = each($_POST))
                $GLOBALS[$key] = cms_utf8_normalize($value);

        reset($_FILES);
        while (list($key, $value) = each($_FILES))
        {
                reset($value);
                while (list($_key, $_value) = each($value))
                {
                        $__key           = $key . (strcmp($_key, "tmp_name") ? "_$_key" : NULL);
                        $GLOBALS[$__key] = cms_utf8_normalize($_value);
                };
        };
};
//-----------------------------------------------------------------------------------------------------
function cms_generate_id()
{
        if (defined("CMS_USERID")) return;

        $id = getenv("HTTP_ACCEPT") .
              getenv("HTTP_ACCEPT_CHARSET") .
              getenv("HTTP_ACCEPT_ENCODING") .
              getenv("HTTP_ACCEPT_LANGUAGE") .
              getenv("HTTP_USER_AGENT") .
              getenv("HTTP_X_FORWARDED_FOR") .
              getenv("REMOTE_ADDR") .
              getenv("REMOTE_IDENT") .
              getenv("REMOTE_USER");

        define("CMS_USERID", md5($id));
};
//-----------------------------------------------------------------------------------------------------
function cms_identification()
{
        //security check
        if (defined("CMS_USER"))      exit();
        if (defined("CMS_PASSWORD"))  exit();
        if (defined("CMS_SUPERUSER")) exit();
        if (defined("CMS_NAME"))      exit();
        if (defined("CMS_EMAIL"))     exit();
        if (defined("CMS_TIMEZONE"))  exit();
        if (defined("CMS_PROFILE"))   exit();

        global $cms_user,
               $cms_password,
               $cms_login_user,
               $cms_login_password,
               $cms_login_noscript,
               $cms_logout;

        //generate random string used for password obfuscation
        if (cms_cache("cms.5c080f9d6448aa7514a2842e1ae732d7") === NULL)
                cms_cache("cms.5c080f9d6448aa7514a2842e1ae732d7",
                          md5(mt_rand() . "|" . microtime()),
                          TRUE);

        //logout
        if ($cms_logout)
        {
                //delete cookies
                setcookie("cms_user",       //name
                          NULL,             //value
                          NULL,             //expire
                          CMS_RELATIVE_URL, //path
                          NULL,             //domain
                          FALSE,            //secure
                          TRUE);            //httponly

                setcookie("cms_password",   //name
                          NULL,             //value
                          NULL,             //expire
                          CMS_RELATIVE_URL, //path
                          NULL,             //domain
                          FALSE,            //secure
                          TRUE);            //httponly

                header("Location: " . CMS_ROOT_URL);
                exit();
        };

        //login
        if ((string)$cms_login_user !== "") $cms_user = (string)$cms_login_user;
        elseif ((string)$cms_user === "")   $cms_user = "anonymous";

        if ((string)$cms_login_password !== "")
        {
                $cms_password = $cms_login_noscript ?
                                md5(md5(cms_cache("cms.5c080f9d6448aa7514a2842e1ae732d7") . "|" .
                                        "11f544779d678530603c344c5381d2b1") .               "|" .
                                    md5($cms_login_password)) :
                                $cms_login_password;
        };

        $login_try = (strcmp($cms_login_user, "anonymous") && ((string)$cms_login_password !== "")) ?
                     $cms_user : FALSE;

        //verify user
        $permission = new permission();

        if (($array = $permission->verify_user($cms_user, $cms_password)) !== FALSE)
        {
                $superuser    = $array["superuser"];
                $name         = $array["name"];
                $email        = $permission->data->get("user.$superuser", "email");
                $timezone     = $permission->data->get("user.$superuser", "timezone");
                $profile      = $array["profile"];
        }
        else
        {
                $cms_user     = "anonymous";
                $cms_password = "";
                $superuser    = $cms_user;
                $name         = $permission->data->get("user.$cms_user", "name");
                $email        = $permission->data->get("user.$cms_user", "email");
                $timezone     = $permission->data->get("user.$cms_user", "timezone");
                $profile      = "";
        };

        //set timezone
        if (! in_array($timezone, timezone_identifiers_list()))
                $timezone     = CMS_TIMEZONE_DEFAULT;
        date_default_timezone_set($timezone);

        //set constants
        define("CMS_USER",      $cms_user);
        define("CMS_PASSWORD",  $cms_password);
        define("CMS_SUPERUSER", $superuser);
        define("CMS_NAME",      $name);
        define("CMS_EMAIL",     $email);
        define("CMS_TIMEZONE",  $timezone);
        define("CMS_PROFILE",   $profile);

        //verify permission
        if (! $permission->given())
        {
                //log failed access attempt
                if ($login_try !== FALSE)
                {
                        $log = new log();
                        $log->access(sprintf(CMS_002, $login_try), NULL, TRUE);
                };

                //redirect to identification prompt
                if (($query_string = (string)getenv("QUERY_STRING")) !== "")
                        $query_string = "?$query_string";

                header("Location: " . CMS_MODULES_URL . "identification.php?" .
                       "location=" . rawurlencode(CMS_ACTIVE_URL . $query_string));
                exit();
        };

        //at this point we are sure the user is valid
        //and permitted to access the current application

        //set cookies
        if (CMS_USER !== "anonymous")
        {
                $expire = time() + 3600;
                $secure = CMS_PROTOCOL === "https";

                setcookie("cms_user",       //name
                          CMS_USER,         //value
                          $expire,          //expire
                          CMS_RELATIVE_URL, //path
                          NULL,             //domain
                          $secure,          //secure
                          TRUE);            //httponly

                setcookie("cms_password",   //name
                          CMS_PASSWORD,     //value
                          $expire,          //expire
                          CMS_RELATIVE_URL, //path
                          NULL,             //domain
                          $secure,          //secure
                          TRUE);            //httponly
        };

        //on initial login
        if ($login_try !== FALSE)
        {
                //clear interface page
                setcookie("cms_ifc_page",   //name
                          "",               //value
                          NULL,             //expire
                          CMS_RELATIVE_URL, //path
                          NULL,             //domain
                          FALSE,            //secure
                          TRUE);            //httponly

                //log successful identification
                $log = new log();
                $log->access(CMS_001, NULL, TRUE);

                //redirect administrator to setup
                if (strcmp(CMS_USER, "admin") === 0)
                {
                        $system = new system();
                        if ($system->getval("setup", "done") === NULL)
                        {
                                header("Location: " . CMS_MODULES_URL . "interface.php?ifc_page=setup");
                                exit();
                        };
                }

                //set account expiration if applicable
                elseif (
                         (strcmp(CMS_USER, "anonymous") !== 0)
                         &&
                         (($expire = (int)$permission->data->get("user." . CMS_USER, "expire")) > 0)
                       )
                {
                        $permission->data->set(time() + $expire, "user." . CMS_USER, "#expire");
                        $permission->save();
                };
        };
};
//-----------------------------------------------------------------------------------------------------
function cms_language_extract($language_selected,
                              $language_supported)
{
        $language_selected  = strtolower($language_selected);
        $language_supported = strtolower($language_supported);
        $array              = explode(",", $language_selected);
        $_array             = array();
        $__array            = array();

        reset($array);
        while (list(, $value) = each($array))
        {
                if (! preg_match(

                //[CFWS] 1*8ALPHA * ( "-" 1*8ALPHA )
                                 "/^\s*([a-z]{1,8}(?:-[a-z]{1,8})*)" .

                //[CFWS] ";" [CFWS] "q" [CFWS] "=" ( "0" [ "." 0*3DIGIT ] ) | ( "1" [ "." 0*3("0") ] )
                                 "\s*(?:;\s*q\s*=\s*(0(?:\.[0-9]{0,3})?|1(?:\.0{0,3})?))?\s*$/",
                                 $value, $value))
                        continue;

                if (
                     isset($value[2])
                     &&
                     ($value[2] < 1.0)
                   )
                        $_array[$value[1]]  = $value[2];
                else
                        $__array[$value[1]] = 1.0;
        };

        //order by quality value
        arsort($_array, SORT_NUMERIC);

        $__array += $_array;
        $_array   = array();

        while (count($__array))
        {
                reset($__array);
                while (list($key, $value) = each($__array))
                {
                        //search for match
                        if (strpos("$language_supported,", "$key,") !== FALSE) return $key;

                        //strip hindmost subtag
                        if (($i = strrpos($key, "-")) !== FALSE)
                        {
                                $key = substr($key, 0, $i);

                                if (
                                     (! isset($_array[$key]))
                                     ||
                                     ($value < $_array[$key])
                                   )
                                        $_array[$key] = $value;
                        };
                };

                $__array = $_array;
                $_array  = array();
        };

        return preg_match("/^[^,]+/", CMS_LANGUAGE_ENABLED, $match) ? $match[0] : "";
};
//-----------------------------------------------------------------------------------------------------
function cms_language()
{
        if (defined("CMS_LANGUAGE_ENABLED")) return;
        if (defined("CMS_LANGUAGE_DEFAULT")) return;
        if (defined("CMS_LANGUAGE"))         return;

        global $cms_select_language,
               $cms_language;

        $system   = new system();
        $language = strtolower($system->getval("language", "default"));

        define("CMS_LANGUAGE_ENABLED", $language);

        //retrieve selected language
        if ((string)$cms_language === "") $cms_language = cms_cache("language." . CMS_USERID);

        //test validity
        if (strpos(CMS_LANGUAGE_ENABLED . ",", "$cms_select_language,") === FALSE) $cms_select_language = "";
        if (strpos(CMS_LANGUAGE_ENABLED . ",", "$cms_language,")        === FALSE) $cms_language        = "";

        if (

        //new language requested by user
             ((string)$cms_select_language !== "")
             ||

        //no language set
             ((string)$cms_language === "")
           )
        {
                $cms_language = cms_language_extract("$cms_select_language," . getenv("HTTP_ACCEPT_LANGUAGE"),
                                                     CMS_LANGUAGE_ENABLED);

                define("CMS_LANGUAGE", $cms_language);

                //store selected language
                cms_cache("language." . CMS_USERID, CMS_LANGUAGE, TRUE);
        }

        //language previously set
        else
        {
                define("CMS_LANGUAGE", $cms_language);
        };

        if (CMS_LANGUAGE !== "") header("Content-Language: " . CMS_LANGUAGE);

        require(((CMS_LANGUAGE != "") && is_file(CMS_PATH . "#language/" . CMS_LANGUAGE . ".language.inc")) ?
                CMS_PATH . "#language/" . CMS_LANGUAGE . ".language.inc" :
                CMS_PATH . "#language/language.inc");
};
//-----------------------------------------------------------------------------------------------------
function cms_email_agent()
{
        if (defined("CMS_EMAIL_AGENT")) return;

        $system = new system();
        if (($address = $system->getval("email", "address")) == "")
                $address = "mailagent@" . CMS_DOMAIN;
        define("CMS_EMAIL_AGENT", $address);
};
//-----------------------------------------------------------------------------------------------------
function cms_load_system()
{
        if (defined("CMS_LOAD_SYSTEM_EXECUTED")) return;
        define("CMS_LOAD_SYSTEM_EXECUTED", TRUE);

        if (! $hdir = opendir(CMS_SYSTEM_PATH))
                die("Path '" . CMS_SYSTEM_PATH . "' cannot be opened.");

        while ($file_object = readdir($hdir))
        {
                if (
                     (! is_file(CMS_SYSTEM_PATH . $file_object))
                     ||
                     (! preg_match("/^sys\.([^\.]+)\.inc$/", $file_object))
                   )
                        continue;

                require(CMS_SYSTEM_PATH . $file_object);
        };

        closedir($hdir);
};
//-----------------------------------------------------------------------------------------------------
function cms_load($library,
                  $exit_on_error = FALSE,
                  $test          = FALSE)
{
        static $loaded = array();

        if (! isset($loaded[$library]))
        {
                $_library = CMS_SYSTEM_PATH . "lib.$library.inc";

                if (! is_file($_library))
                {
                        if ($exit_on_error)
                                die("Library '$_library' cannot be opened.");

                        $loaded[$library] = FALSE;
                }
                else
                {
                        if (! $test)
                        {
                                require($_library);
                                $loaded[$library] = TRUE;
                        };

                        return TRUE;
                };
        };

        return $loaded[$library];
};
//-----------------------------------------------------------------------------------------------------
function cms_available($library)
{
        return cms_load($library, FALSE, TRUE);
};
//-----------------------------------------------------------------------------------------------------
function cms_application($application = NULL,
                         $instance    = NULL,
                         $permission  = NULL,
                         $user        = NULL)
{
        static $_application         = CMS_APPLICATION,
               $_instance            = CMS_INSTANCE,
               $application_instance = "",
               $_permission          = NULL,
               $initialize           = TRUE;

        if ($initialize)
        {
                $application_instance = CMS_APPLICATION . ((CMS_INSTANCE != "") ? "." . CMS_INSTANCE : "");
                $_permission          = new permission();
                $initialize           = FALSE;
        };

        if ($application === TRUE)
        {
                if ($instance === TRUE)
                {
                        if ($permission != "")
                        {
                                if ($user != "") return $_permission->test("$application_instance.$permission", $user);
                                return $_permission->given("$application_instance.$permission");
                        };
                        return $application_instance;
                }
                if ($instance == "")
                {
                        if ($permission != "")
                        {
                                if ($user != "") return $_permission->test("$_application.$permission", $user);
                                return $_permission->given("$_application.$permission");
                        };
                        return $_application;
                };
        }
        elseif ($application == "")
        {
                if ($instance === TRUE)
                {
                        if ($permission != "")
                        {
                                if ($user != "") return $_permission->test("$_instance.$permission", $user);
                                return $_permission->given("$_instance.$permission");
                        }
                        return $_instance;
                }
                elseif ($instance == "")
                {
                        if ($permission != "")
                        {
                                if ($user != "") return $_permission->test($permission, $user);
                                return $_permission->given($permission);
                        }
                        return $application_instance;
                };
        };

        $file = CMS_MODULES_PATH . "#module/mod.$application.inc";
        if (is_file($file))
        {
                $__application         = $_application;
                $__instance            = $_instance;
                $_application         .= ".$application";
                $_instance             = preg_replace("/[^_a-z0-9]+/", "-", strtolower($instance));
                $application_instance  = $_application . (($_instance != "") ? ".$_instance" : "");

                if ($_permission->given($application_instance)) require($file);

                $_application          = $__application;
                $_instance             = $__instance;
                $application_instance  = $_application . (($_instance != "") ? ".$_instance" : "");
        };
};
//-----------------------------------------------------------------------------------------------------
function cms_instance()
{
        return cms_application(NULL, TRUE);
};
//-----------------------------------------------------------------------------------------------------
function cms_permission($permission,
                        $application = TRUE,
                        $instance    = TRUE,
                        $user        = NULL)
{
        return cms_application((bool)$application, (bool)$instance, $permission, $user);
};
//-----------------------------------------------------------------------------------------------------
function cms_param($value = NULL,
                   $key   = NULL)
{
        static $param = array();

        if ($key != NULL)
        {
                $param[(string)$key] = (string)$value;
                return TRUE;
        };

        $value  = is_array($value) ? $value : array();
        $value  = array_merge($param, $value);
        $return = "";

        //recursively retrieve field names and values
        $array = array();
        $func  = function($value, $key = "") use (&$func, &$array)
        {
                if (is_array($value))
                {
                    ksort($value);
                    reset($value);
                    while (list($_key, $_value) = each($value))
                        $func($_value, ((string)$key === "") ? $_key : $key . "[$_key]");
                }
                else
                {
                    $array[$key] = $value;
                };
        };

        $func($value);

        reset($array);
        while (list($key, $value) = each($array))
        {
                if ((string)$value == "") continue;

                $value   = (substr($value, 0, 1) === "\x1B") ?
                           substr($value, 1) :
                           rawurlencode($value);
                $return .= (($return == "") ? "?" : "&") .
                           rawurlencode($key) . "=$value";
        };

        return $return;
};
//-----------------------------------------------------------------------------------------------------
function cms_cache($key       = NULL,
                   $value     = NULL,
                   $permanent = FALSE)
{
        static $cache = array();

        if (($value !== NULL) || ($permanent !== FALSE))
        {
                //delete
                if ($value === "")
                {
                        if ($key === NULL) return TRUE;
                        if ($permanent)
                        {
                                $code = md5($key);
                                $path = CMS_DATA_PATH . "#cache/" . substr($code, 0, 4) . "/" . substr($code, 4);
                                if (is_file($path)) unlink($path);
                        };
                        if (isset($cache[$key])) unset($cache[$key]);
                        return TRUE;
                };

                //store
                if ($key === NULL)
                {
                        $key = 0;
                        while (isset($cache[$key])) $key++;
                }
                elseif ($permanent)
                {
                        $code = md5($key);
                        $path = CMS_DATA_PATH . "#cache/" . substr($code, 0, 4) . "/";
                        $code = substr($code, 4);

                        if (mkpath($path) && ($hfile = fopen($path . $code, "wb")))
                        {
                                set_file_buffer($hfile, 0);
                                fputs($hfile, $value);
                                fclose($hfile);
                        };
                };

                $cache[$key] = $value;
                return TRUE;
        };

        //retrieve
        if (isset($cache[$key])) return $cache[$key];
        if ($key !== NULL)
        {
                $code        = md5($key);
                $path        = CMS_DATA_PATH . "#cache/" . substr($code, 0, 4) . "/" . substr($code, 4);
                $value       = read_file($path);
                $cache[$key] = $value;
                if ($value !== FALSE)
                {
                        touch($path);
                        return $value;
                };
        };
        return NULL;
};
//-----------------------------------------------------------------------------------------------------
function cms_cache_time($key)
{
        $code = md5($key);
        $path = CMS_DATA_PATH . "#cache/" . substr($code, 0, 4) . "/" . substr($code, 4);

        if (is_file($path)) return filemtime($path);
        return FALSE;
};
//-----------------------------------------------------------------------------------------------------
function cms_cache_touch($key)
{
        $code = md5($key);
        $path = CMS_DATA_PATH . "#cache/" . substr($code, 0, 4) . "/" . substr($code, 4);

        if (is_file($path)) return touch($path);
        return FALSE;
};
//-----------------------------------------------------------------------------------------------------
function cms_cache_clean($path)
{
        $depth        = 0;
        $hdir[$depth] = opendir($path);
        if ($hdir[$depth] === FALSE) return FALSE;
        $path         = array($depth => $path);
        $flag[$depth] = FALSE;
        $time         = time();

        do
        {
                while (($file_object = readdir($hdir[$depth])) !== FALSE)
                {
                        if (in_array($file_object, array(".", "..", ".htaccess"), TRUE)) continue;

                        //directory
                        if (is_dir($path[$depth] . $file_object))
                        {
                                $depth++;
                                $path[$depth] = $path[$depth - 1] . "$file_object/";
                                $hdir[$depth] = opendir($path[$depth]);
                                if ($hdir[$depth] === FALSE) $depth--;
                                else                         $flag[$depth] = TRUE;
                                continue;
                        };

                        //file
                        $atime = fileatime($path[$depth] . $file_object);
                        if (($time - $atime) > CMS_CACHE_TTL) unlink($path[$depth] . $file_object);
                        else                                  $flag[$depth] = FALSE;
                };

                closedir($hdir[$depth]);
                if ($flag[$depth]) rmdir($path[$depth]);
                $depth--;
        }
        while ($depth >= 0);

        return TRUE;
};
//-----------------------------------------------------------------------------------------------------
function cms_path_urlencode($string)
{
        /*
         * encode all non-alphanumerical characters but $-_.+!*'(),
         * according to rfc 1738
         * additionally excluding path separator /
         * this function can be used to make file paths safe
         */

        return preg_replace_callback(
                "/[^0-9A-Za-z\\x21\\x24\\x27-\\x2F\\x5F]/",
                function($value) { return "%" . bin2hex($value[0]); },
                $string);
};

$cms_application = get_required_files();
if (count($cms_application) <= 1)
        die("nuos.inc may not be executed.");

$cms_php_self     = $_SERVER["PHP_SELF"];
$cms_http_host    = getenv("HTTP_HOST");

//base url
reset($cms_application);
$cms_base_url     = current($cms_application);
$cms_base_url     = substr($cms_base_url, 0, -strlen($cms_php_self));
$cms_base_url     = dirname(substr(__FILE__, strlen($cms_base_url)));
$cms_base_url     = str_replace("\\", "/", $cms_base_url) . "/";
$cms_base_url     = cms_path_urlencode($cms_base_url);

$cms_relative_url = dirname($cms_base_url) . "/";
$cms_relative_url = str_replace("\\", "/", $cms_relative_url);
$cms_relative_url = str_replace("//", "/", $cms_relative_url);
$cms_php_self     = cms_path_urlencode($cms_php_self);

//application
end($cms_application);
$cms_application  = prev($cms_application);
$cms_application  = strtolower(basename($cms_application));
$cms_application  = preg_replace("/\..+$/", "", $cms_application);

//instance
$cms_instance     = "";

//interface
if (! strcmp($cms_application, "interface"))
{
        if (
             isset($_REQUEST["ifc_message"])
             &&
             (! strcmp($_REQUEST["ifc_message"], "ifc_select_page"))
             &&
             isset($_REQUEST["ifc_param"])
           )
        {
                $cms_instance = $_REQUEST["ifc_param"];
                setcookie("cms_ifc_page",    //name
                          $cms_instance,     //value
                          NULL,              //expire
                          $cms_relative_url, //path
                          NULL,              //domain
                          FALSE,             //secure
                          TRUE);             //httponly
        }

        elseif (isset($_REQUEST["ifc_page"]))
        {
                $cms_instance = $_REQUEST["ifc_page"];
        }

        else
        {
                $cms_instance = isset($_COOKIE["cms_ifc_page"]) ? $_COOKIE["cms_ifc_page"] : "";
        };
};

//instance
if ($cms_instance != "") $cms_instance = preg_replace("/[^_a-z0-9]+/", "-", strtolower($cms_instance));

//-----------------------------------------------------------------------------------------------------
//===< SYSTEM CONSTANTS >==============================================================================
//-----------------------------------------------------------------------------------------------------
define("CMS_APPLICATION",               $cms_application);
define("CMS_INSTANCE",                  $cms_instance);
define("CMS_DB_PREFIX",                 "cms_");
define("CMS_IFC_EDITION",               "ifc");

define("CMS_PATH",                      str_replace("\\", "/", dirname(__FILE__)) . "/");
define("CMS_SYSTEM_PATH",               CMS_PATH . "#system/");
define("CMS_MODULES_PATH",              CMS_PATH . "module/");
define("CMS_INTERFACE_PATH",            CMS_MODULES_PATH  . "#interface/");
define("CMS_DESKTOP_PATH",              CMS_MODULES_PATH . "#desktop/");
define("CMS_IMAGES_PATH",               CMS_PATH . "image/");
define("CMS_ROOT_PATH",                 dirname(CMS_PATH) . "/");
define("CMS_DATA_PATH",                 CMS_ROOT_PATH . "data/");

define("CMS_PROTOCOL",                  ((! preg_match("/^(?:|off)$/i", getenv("HTTPS"))) ||
                                        (strcasecmp(getenv("HTTP_X_FORWARDED_PROTO"), "https") === 0)) ?
                                        "https" : "http");
define("CMS_HOST",                      CMS_PROTOCOL . "://$cms_http_host");
define("CMS_DOMAIN",                    preg_replace("/^www\.([^:]+).*/i", "\\1", $cms_http_host));
define("CMS_ACTIVE_URL",                CMS_HOST . $cms_php_self);
define("CMS_URL",                       CMS_HOST . $cms_base_url);
define("CMS_MODULES_URL",               CMS_URL . "module/");
define("CMS_IMAGES_URL",                CMS_URL . "image/");
define("CMS_JAVA_URL",                  CMS_URL . "java/");
define("CMS_JAVASCRIPT_URL",            CMS_URL . "javascript/");
define("CMS_SOUNDS_URL",                CMS_URL . "sound/");
define("CMS_ROOT_URL",                  dirname(CMS_URL) . "/");
define("CMS_RELATIVE_URL",              $cms_relative_url);
define("CMS_DATA_URL",                  CMS_ROOT_URL . "data/");

define("CMS_DOCTYPE_HTML",              "<!DOCTYPE HTML>");
define("CMS_DOCTYPE_FRAMESET",          "<!DOCTYPE HTML>");
define("CMS_HTML_HEADER",               "<title>" .
                                        htmlspecialchars(ucfirst(CMS_APPLICATION) . " " . CMS_DOMAIN) .
                                        "</title>" .
                                        "<meta name=\"generator\" " .
                                              "content=\"" . htmlspecialchars(CMS) . "\">" .
                                        "<meta name=\"robots\" " .
                                              "content=\"index, follow\">" .
                                        "<meta http-equiv=\"content-type\" " .
                                              "content=\"text/html; charset=utf-8\">" .
                                        "<base href=\"" . htmlspecialchars(CMS_ROOT_URL) . "\">");
define("CMS_JAVASCRIPT",                "<script src=\"" . htmlspecialchars(CMS_JAVASCRIPT_URL) . "asr.js\">" .
                                        "</script>" .
                                        "<script src=\"" . htmlspecialchars(CMS_JAVASCRIPT_URL) . "common.js\">" .
                                        "</script>" .
                                        "<script src=\"" . htmlspecialchars(CMS_JAVASCRIPT_URL) . "fx.js\">" .
                                        "</script>" .
                                        "<script src=\"" . htmlspecialchars(CMS_JAVASCRIPT_URL) . "dragdrop.js\">" .
                                        "</script>");
define("CMS_STYLESHEET",                "<link rel=\"stylesheet\" " .
                                              "href=\"" . htmlspecialchars(CMS_DATA_URL) . "stylesheet.css\" " .
                                              "media=\"not all\" " .
                                              "onload=\"this.media='all';\">" .
                                        "<noscript>" .
                                        "<link rel=\"stylesheet\" " .
                                              "href=\"" . htmlspecialchars(CMS_DATA_URL) . "stylesheet.css\">" .
                                        "</noscript>");
define("CMS_CLASS",                     "cms-" . str_replace(".", "-", CMS_APPLICATION));

define("CMS_APACHE",                    function_exists("apache_get_version"));
define("CMS_CACHE_TTL",                 86400);

//generate user id
cms_generate_id();

//load system libraries
cms_load_system();

//register global variables
//apply unicode normalization
cms_initialize_globals();

//initialize language
cms_language();

//initialize email agent
cms_email_agent();

//clean cache
if ((time() - cms_cache("cms.0G1JZSJYq7NcmS1oLXQ7YRB2a1VWw6Fm")) > CMS_CACHE_TTL)
{
        cms_cache_clean(CMS_DATA_PATH . "#cache/");
        cms_cache_clean(CMS_DATA_PATH . "#content/cache/");
        cms_cache("cms.0G1JZSJYq7NcmS1oLXQ7YRB2a1VWw6Fm", time(), TRUE);
};

//user identification
cms_identification();

cms_param(CMS_INSTANCE, "cms_instance");

//clean up
unset($cms_time);
unset($cms_php_self);
unset($cms_http_host);
unset($cms_relative_url);
unset($cms_instance);
unset($cms_application);
unset($cms_base_url);
unset($cms_user);
unset($cms_password);
unset($cms_login_user);
unset($cms_login_password);
?>